#include "stdafx.h"
#pragma warning(disable: 4786)


/*#include "afxwin.h" [REMOVING IN THIS (DOS/LINUX) VERSION] */
#include "ExpressionEvaluator.h"

#include "CSentence.h"
#include "CLexicalEntry.h"
#include "CChart.h"
#include "FCPair.h"
#include "Mapper.h"

#include "LogManager.h"
/*#include ".\cchart.h" REMOVED IN THIS (LINUX) VERSION */

#define BREAK  0
#define CONTINUE 1
#define USUAL 2

using namespace std;

short CChart::WordIndexId=0;
short CChart::PredId=0;

//##ModelId=40987EA80306
CChart::CChart(CGrammar*grmr,string SubCatFileName)
{
	Chart=NULL;
	Grammar=grmr;
	ElementCounter=1;

	LexiconPtr=Grammar->GetLexicon();

	LoadSubCatFrameList(SubCatFileName);

	WordIndexId=CSymbolTableManager::IndexId;
	PredId=CSymbolTableManager::PredId;	
}

CChart::CChart(string SubCatFileName)
{
	Chart=NULL;
	LoadSubCatFrameList(SubCatFileName);
}

bool CChart::LoadIntoChart(CProduction*prod,
			 size_t chart_index,
			 size_t start_index,
			 bool CheckBeforeLoading)
//Loads Production into Chart Column Stack
{
	//check the validatity of chart_index
 	if( chart_index>=ChartSize || chart_index<0 || 
		start_index>=ChartSize || start_index<0) 
		return false;	

	//check if the production is already loaded then it is not needed to load it again 

	//added......on july 12,2005...to fix problem
	 //if(prod->IsNullProduction())
	//	CheckBeforeLoading=false;
		
		
	/////////////////////////////
		
	if(CheckBeforeLoading && SearchProductionInChartColumn(prod,chart_index,start_index)==true)
	{
		//printf("\nAlready inserted...............\n");
		//prod->PrintProduction();
		//printf(".................................\n");
		return false;
	}

	//prod->PrintProduction();

	//create a new chart item and push into chart column stack
	CChartItem* citem=new CChartItem(prod,start_index,chart_index,false);
	
	if(!citem)
		return false;
	
	Chart[chart_index].ItemVector.push_back(citem);
	
return true;
}

bool CChart::GetSourceStructures(bool CheckCompletenessCoherence,list<CFSCSPairInfo*> & result)
{
	using namespace std;
		
	CChartItem*citem=NULL;
	
	size_t last_index=ChartSize-1;
	size_t csize=Chart[last_index].ItemVector.size();
	
	if(csize<=0)	
		return false;						
	
	CFStructureItem*nroot=NULL;
	CCStructureItem*croot=NULL;

	wstring constraint;
	size_t constraint_len=0;
	
	bool passed=false;
	bool done=false;

	CFSCSPairInfo * fscsinfo=NULL;
	result.clear();	
	
	for(vector<CChartItem*>::iterator it=Chart[last_index].ItemVector.begin();
		it!=Chart[last_index].ItemVector.end();it++)
		{			
			citem=*it;
			
			if(citem->Prod->LHS->Label==L"S" && citem->Prod->IsComplete())
			{	
				#ifdef GenLog
				CLogManager::AddMessage(L"Executing Constraints in Language Parser \r\n");
				#endif
				
				if(citem->Prod->LHS->FSPtr)
				{
					//passed = CheckLexConstraints(citem,nroot);
					nroot=citem->Prod->LHS->FSPtr->Root;
					passed=true;

					for(vector<wstring>::iterator it=nroot->ConstraintList.begin();
					it!=nroot->ConstraintList.end();it++)
					{
						constraint=*it;						
						if(!EvaluateExpression(constraint,nroot,nroot))
						{	
							#ifdef GenLog
							CLogManager::AddMessage(L"Constraint : "+constraint+L" - failed\r\n");
							#endif
							passed=false;
							break;							
						}
					}
					if (passed)
						passed = CheckLexConstraints(nroot);

					if(passed)
					{	
						//converting [ PRED 'go<SUBJ> ] to [ PRED,PRED.VALUE='go',PRED.GF='SUBJ' ]
						CMapper::AdjustFStructure(citem->Prod->LHS->FSPtr);
						
						if(CheckCompletenessCoherence)
						{
							if(!CheckForwardCompleteness(nroot))
							{
								#ifdef GenLog
								CLogManager::AddMessage(L"Completeness-Coherence Check failed \r\n");
								#endif
								continue;
							}						

						}						

						if(!FS_ExistsInList(result,nroot))
						{
							fscsinfo=new CFSCSPairInfo(new CFStructure(nroot,1.0),NULL);
							/////////added on August 04,2005/////
							fscsinfo->FSRef->SetParentPointers();
							/////////////////////////////////////
							croot=RetrieveCStructure(citem->Prod,&fscsinfo->SenWords);				
							fscsinfo->CSRef = new CCStructure(croot);
							result.push_back(fscsinfo);
						}						
					}		
				}

			}

		}
	
		if(result.size()==0)
			return false;
	
	return true;
}

bool CChart::EarleyParse(vector<vector<CLexicalEntry*> * > * word_list, 
						 list<CFSCSPairInfo*> & result_list,
						 bool CheckCompletenessCoherence)
{
	// parses sentence words on chart
		
		CFStructureItem::ResetNodeId();
		ElementCounter=1;		

		ChartSize=word_list->size()+1;
		Chart=new CChartColumn[ChartSize];

		if(!Chart)
			return false;
		
		/*for(size_t k=0;k<ChartSize-1;k++) CHANGING FOR THIS (LINUX VERSION)*/
		size_t k;
		for(k=0;k<ChartSize-1;k++)
			Chart[k].WordLexEntries=word_list->at(k);				
		
		Chart[k].WordLexEntries=NULL;				

		//Get the Starting production from Grammar and load
		//into the chart's first column
		short StartId=Grammar->GetNum(L"S");
		
		vector<CProduction*> plist;
		Grammar->GetProductionsExtended(StartId,plist);
		
		CProduction*sprod=NULL;
		
		if(plist.size()==0)
		{
			#ifdef GenLog
			CLogManager::AddMessage(L" No production was found for symbol 'S' \r\n");
			#endif
			return false;		
		}
					
		for(vector<CProduction*>::iterator it=plist.begin();it!=plist.end();it++)
		{
			sprod=*it;
			sprod=sprod->GetClone();
			
			#ifdef GenLog
			sprod->PrintProductionF(CLogManager::GetLogFileRef(),L"S Productions are Loaded: ");
			#endif
			
			if(!LoadIntoChart(sprod,0,0,false))
				return false; /*CHANGED FROM NULL TO FALSE IN THIS (LINUX) VERSION*/
		}						

		CChartItem*citem=NULL;
		
		CProduction * curr_prod=NULL;
		vector <CLexicalEntry*> * le_list=NULL;		
		CLexicalEntry*le=NULL;
			
		//AfxMessageBox(L"hello-1",0,1);

		for (size_t i =0;i<ChartSize;i++)
		//iteration on each word in a sentence
		{			
			le_list=Chart[i].WordLexEntries;						

			//Inititializing current_ptr of chart[i] to the first production
			Chart[i].InitCurrPtr();			

			//GetCurrent function will return the chart item pointed by current_ptr and 
			//move the the current ptr to next production						

			for(unsigned short n=0;n<Chart[i].ItemVector.size();n++)			
			{								
				citem=Chart[i].ItemVector.at(n);
				
				curr_prod=citem->Prod;
				
				//le=le_list->at(0);				
				//consume the word if there is a Terminal on TOS

				if(curr_prod->IsNullProduction() || curr_prod->IsComplete())
				{					
					Completer(citem,n);		
				}
				else if(curr_prod->GetCurrentSymbol()->isTerminal)
				{					
					if(!Scanner(citem,le_list))					
						citem->IsFailed=true;									
				}
				else
				{					
					Predictor(citem);	
					//load the productions of non-terminal on the TOS					
				}

			}//end for

		}//end for
		
	GetSourceStructures(CheckCompletenessCoherence,result_list);
	Sort(result_list);
	size_t list_size=result_list.size();

	if(list_size>0)
	{
		#ifdef GenLog
		CLogManager::AddMessage(L"Parser has parsed the sentence and generated "+GetWSTR(list_size)+L" f-structure(s).\r\n");
		#endif
		return true;
	}
	
	#ifdef GenLog
	CLogManager::AddMessage(L"Parser could not generate any structure.\r\n");		
	#endif

	return false;
}

bool CChart::Sort(list<CFSCSPairInfo*> & result)
{
	list<CFSCSPairInfo*>::iterator it;
	list<CFSCSPairInfo*> toplist;
	short max_subcat_length=0;

	it=result.begin();
	CFStructureItem *tempfs;

	if(it!=result.end())
	{	
		tempfs=(*it)->FSRef->Root->SearchEx(CSymbolTableManager::PredId);
		if(tempfs)
			tempfs=tempfs->SearchEx(CSymbolTableManager::GFId);

		if(tempfs)
		{
			max_subcat_length=tempfs->ValueList.size();
			toplist.push_back(*it);
			it++;
		}
	}

	for(;it!=result.end();it++)
	{
		tempfs=(*it)->FSRef->Root->SearchEx(CSymbolTableManager::PredId);
		
		if(tempfs)
			tempfs=tempfs->SearchEx(CSymbolTableManager::GFId);

		if(tempfs)
		{
			if(max_subcat_length<=tempfs->ValueList.size())
			{
				if(max_subcat_length<tempfs->ValueList.size())
				{
					max_subcat_length=tempfs->ValueList.size();
					toplist.clear();
				}
				toplist.push_back(*it);
			}
		}
	}

	if(toplist.size()==0)
		toplist=result;

	list<CFSCSPairInfo*> besttoplist;
	it=toplist.begin();

	if(it!=toplist.end())
	{	
		besttoplist.push_back(*it);
		it++;
	}

	for(;it!=toplist.end();it++)
	{
		/* CHANGED IN THIS (LINUX) VERSION
		for(int i=0; i<(ChartSize-1) && (*it)->SenWords.Words[i]->POS==(*besttoplist.begin())->SenWords.Words[i]->POS;i++);
		*/
		int i;
		for(i=0; i<(ChartSize-1) && (*it)->SenWords.Words[i]->POS==(*besttoplist.begin())->SenWords.Words[i]->POS;i++);
		if(i==(ChartSize-1))
		{
			besttoplist.push_back(*it);
		}
		else if( (*it)->SenWords.Words[i]->POS < (*besttoplist.begin())->SenWords.Words[i]->POS)
		{
			besttoplist.clear();
			besttoplist.push_back(*it);
		}
	}

	if(besttoplist.size()>0)
		result=besttoplist;

	return true;
}

CChartColumn  * CChart::EarleyParse(vector<vector<CLexicalEntry*> * > * word_list)
// parses sentence words on chart
{	
		
		CFStructureItem::ResetNodeId();

		ElementCounter=1;

		//initializing chart
		//associating each word with the chart columns

		/*
		if(Chart)
		{
			ClearChart();			
		}		
		*/

		//debug_fw=new CFileWriter("data\\debug.txt");
		//if(!debug_fw)
		//	return NULL;


		//debug_fw->Write(L"\r\n ##################### ENTERED ##################\r\n");			


		ChartSize=word_list->size()+1;
		Chart=new CChartColumn[ChartSize];

		if(!Chart)
			return NULL;

		/*for(size_t k=0;k<ChartSize-1;k++) CHANGED IN THIS (LINUX) VERSION*/
		size_t k;
		for(k=0;k<ChartSize-1;k++)
			Chart[k].WordLexEntries=word_list->at(k);				
		
		Chart[k].WordLexEntries=NULL;				

		//Get the Starting production from Grammar and load
		//into the chart's first column
		
		vector<CProduction*> * plist=Grammar->GetProductions(L"S");
		CProduction*sprod=NULL;
		
		if(!plist)
			return NULL;		
					
		for(vector<CProduction*>::iterator it=plist->begin();it!=plist->end();it++)
		{
			sprod=*it;
			sprod=sprod->GetClone();
			
			#ifdef GenLog
			sprod->PrintProductionF(CLogManager::GetLogFileRef(),L"S Production Loaded: ");
			#endif
			
			if(!LoadIntoChart(sprod,0,0,false))
				return NULL;
		}				

		try{
		delete plist;
		}
		catch(...)
		{
			#ifdef GenLog
			CLogManager::AddMessage(L"#### plist");
			#endif
		}


		CChartItem*citem=NULL;
		
		CProduction * curr_prod=NULL;
		vector <CLexicalEntry*> * le_list=NULL;		
		CLexicalEntry*le=NULL;
			
		//AfxMessageBox(L"hello-1",0,1);

		for (size_t i =0;i<ChartSize;i++)
		//iteration on each word in a sentence
		{			
			le_list=Chart[i].WordLexEntries;						

			//Inititializing current_ptr of chart[i] to the first production
			Chart[i].InitCurrPtr();			

			//GetCurrent function will return the chart item pointed by current_ptr and 
			//move the the current ptr to next production						

			for(unsigned short n=0;n<Chart[i].ItemVector.size();n++)			
			{								
				citem=Chart[i].ItemVector.at(n);
				
				curr_prod=citem->Prod;
				
				//le=le_list->at(0);				
				//consume the word if there is a Terminal on TOS

				if(curr_prod->IsNullProduction() || curr_prod->IsComplete())
				{					
					Completer(citem,n);		
				}
				else if(curr_prod->GetCurrentSymbol()->isTerminal)
				{					
					if(!Scanner(citem,le_list))					
						citem->IsFailed=true;									
				}
				else
				{					
					Predictor(citem);	
					//load the productions of non-terminal on the TOS					
				}

			}//end for

		}//end for
	
		
	//debug_fw->Write(L"\r\n############################################\r\n");
			
	return Chart;
}

bool  CChart::AddIntoDebugLog(wstring header,
							  CProduction * prod, 
							  short chart_index, 
							  short start_index)							  
{		
	//return true;

	if(!prod)
		return false;

	#ifdef GenLog

	wstring cindex=GetWSTR(chart_index);
	wstring sindex=GetWSTR(start_index);

	wstring title=header;			
			title+=L": [";
			title+=cindex;
			title+=L",";
			title+=sindex;
			title+=L"] ";
	
	prod->PrintProductionF(CLogManager::GetLogFileRef(),title);
	
	#endif

	return true;
}

bool CChart::Predictor(CChartItem*citem)
//Gets Productions against the non-temrinal and
//loads into the chart column stack
{		
	using namespace std;

	if(!citem)
		return false;

	//debug_fw->Write(L"##### Predictor IN \r\n");

	vector<CLexicalEntry*> * lex_ent = Chart[citem->CurrentIndex].WordLexEntries;

	CProductionSymbol * psymb=citem->Prod->GetCurrentSymbol();
	
	vector<CProduction*> prod_list;
	Grammar->GetProductionsExExtended(psymb->LabelID,lex_ent,prod_list);
	
	if(prod_list.size()==0)
	{	
		#ifdef GenLog
		
		wstring str=L"Suitable Production Not Found in Grammar for Symbol '";
				str+=psymb->Label;
				str+=L"'\r\n";
		
		CLogManager::AddMessage(str);			
		
		#endif

		return false;	
	}	

	CProduction*tprod=NULL;
	CProduction*clone_prod=NULL;
	wstring title;	
	bool isNullProduction=false;
		
	for(vector<CProduction*>::iterator it=prod_list.begin();it!=prod_list.end();it++)
	{	
		tprod=*it;
		
		if(tprod->IsNullProduction() && 
			SearchProductionInChartColumn(tprod,citem->CurrentIndex,citem->StartIndex))
		{
			#ifdef GenLog
			tprod->PrintProductionF(CLogManager::GetLogFileRef(),L"Predictor: Production Found Already in Chart");
			#endif
			continue;
		}

		tprod->InitCurrentIndex();		

		//tprod->LHS->FSPtr = new CFStructure(0,NULL);	
		//number_str=GetWSTR(citem->CurrentIndex);
		
		clone_prod=tprod->GetClone();
		
		//////////////MM/////////////////////////////////////////////
		/////////// if the clone_prod->LHS is available then delete it too
		//	if(clone_prod->LHS->FSPtr)
		//	{
		//		delete clone_prod->LHS->FSPtr
		//		clone_prod->LHS->FSPtr=NULL;
		//	}
		////////////////////////////////////////////////////////////

		clone_prod->LHS->FSPtr = new CFStructure(0,0);/*ARG 2 CHANGED FROM NULL IN THIS (LINUX) VERSION*/
		isNullProduction=clone_prod->IsNullProduction();

		if(!clone_prod)
		{
			#ifdef GenLog
			CLogManager::AddMessage(L"Clone Not Created\r\n");
			#endif
		}
		else if(LoadIntoChart(clone_prod,citem->CurrentIndex,citem->CurrentIndex,true))	
			AddIntoDebugLog(L"Predictor",tprod,citem->CurrentIndex,citem->CurrentIndex);
		/*else if(isNullProduction)
		{	
			//*************
			//added in last days of release........
			//prone to problems...
			/*************
			//
			//try{
			//	delete clone_prod;
			//}catch(...){}
			

			if(clone_prod)
			{
				delete clone_prod;
				clone_prod=NULL;
			}

			clone_prod=citem->Prod->GetClone();
			
			clone_prod->MoveDot();			

			if(LoadIntoChart(clone_prod,citem->CurrentIndex,citem->CurrentIndex,true))
				AddIntoDebugLog(L"Last Loading Successful",clone_prod,citem->CurrentIndex,citem->CurrentIndex);
			else						
				AddIntoDebugLog(L"Last Loading Failed",clone_prod,citem->CurrentIndex,citem->CurrentIndex);
			
			//CLogManager::AddMessage(L"Executed Something....\r\n");
			//debug_fw->Write(L"Executed Something....\r\n");
		}*/
		else
		{
			AddIntoDebugLog(L"Not Loaded",tprod,citem->CurrentIndex,citem->CurrentIndex);			
			
			/*
			try{
				delete clone_prod;
			}
			catch(...)
			{
				CLogManager::AddMessage(L"#### cloned_prod");
			}
			*/

			//delete tprod->LHS->FSPtr;
			//tprod->LHS->FSPtr=NULL;
			//delete tprod;
		}
		
	}			

	return true;
}

bool CChart::Scanner(CChartItem*citem,vector<CLexicalEntry*> * word_list)
//consumes words in the sentence if there is a terminal on the TOS
{	
	using namespace std;

	if(!citem || !word_list)
		return false;

//	debug_fw->Write(L"##### Scanner IN \r\n");

	vector<CLexicalEntry*>* le_list=GetLexEntriesOnPOSAndFeatureMatch(word_list,citem->Prod->GetCurrentSymbol());

	if(!le_list)
		return false;	

	short len=le_list->size();

	CProduction * tprod=NULL;
	CLexicalEntry* le=NULL;
	wstring title;
	
	for(vector<CLexicalEntry*>::iterator it=le_list->begin();it!=le_list->end();it++)
	{
		le=*it;
		tprod = citem->Prod->GetClone();					
		   
		if(!tprod)
		{
			#ifdef GenLog
			CLogManager::AddMessage(L"Clone Not Created\r\n");
			#endif
		}
		else if(BuildFStructure(tprod,le,citem->CurrentIndex))	
		{			
			//////added for mapper enhancement/////////
			//tprod->
			///////////////////////////////////////////

			tprod->GetCurrentSymbol()->SenWord=le->Word;
			
			if(!BuildFStructureEx(tprod,tprod))
			{
				//debug_fw->Write(L"Scanner: Failed Production , "+le->Predicate+L" , "+tprod->LHS->Label+L" : "+tprod->GetCurrentSymbol()->Label+L"\r\n");
				//le->PrintW(

				/*
				try{
					if(tprod)
						delete tprod;
				}
				catch(...)
				{
					CLogManager::AddMessage(L"#### tprod");
				}
				*/

				continue;
			}
		
			tprod->MoveDot();
												
			if(LoadIntoChart(tprod, citem->CurrentIndex+1, citem->StartIndex,false))
				AddIntoDebugLog(L"Scanner",tprod,citem->CurrentIndex+1,citem->StartIndex);
			else
			{
				AddIntoDebugLog(L"Not Loaded",tprod,citem->CurrentIndex+1,citem->StartIndex);			
				
				/*try{
					delete tprod;
				}
				catch(...)
				{
					CLogManager::AddMessage(L"#### tprod");
				}*/

				tprod=NULL;
			}
		}
		else
		{
			/*
			try{
				delete tprod;
				tprod=NULL;
			}
			catch(...)
			{
				CLogManager::AddMessage(L"#### tprod");
			}
			*/
		}
	}

	///////////////////////MM/////////////11.08.2005//////////
	le_list->clear();
	delete le_list;
	le_list=NULL;
	/////////////////////////////////////////////////////////
//	debug_fw->Write(L"##### Scanner OUT \r\n");

	return true;
}

short CChart::FindandLoadProdOnChart(CChartItem*chart_item, CProduction * prod2, wstring str1, CChartItem*temp, bool NullCompletion){
	
		


	if (prod2->IsNullProduction() )
	{
		if (NullCompletion)
		{
			if(chart_item->Prod->LHS->Label == prod2->LHS->Label && prod2->IsNullProduction() ) //change this cond
					return BREAK;	
			else
				return CONTINUE;
		}
		else
			return CONTINUE;
	}
	else //if production is not NULL
	{
		if(chart_item->Prod==prod2)
			return BREAK;	
	}


	if(chart_item->IsFailed || prod2->IsComplete())
		return CONTINUE;

	if(prod2->GetCurrentSymbol()->Label==str1)
	{
		//if(str1==L"Sdecl" && prod2->GetCurrentSymbol()->Label==L"Sdecl")
		//	str1=str1;

		CProduction * cloned = prod2->GetClone();											

		if(!cloned)
		{
			#ifdef GenLog
			CLogManager::AddMessage(L"Clone Not Created\r\n");
			#endif
		}
		else if(BuildFStructureEx(cloned,chart_item->Prod)){			

			cloned->GetCurrentSymbol()->ChartItemPtr=chart_item;												
			cloned->MoveDot();

			if(LoadIntoChart(cloned,chart_item->CurrentIndex,temp->StartIndex,false))
				AddIntoDebugLog(L"Completer",cloned,chart_item->CurrentIndex,temp->StartIndex);
			else
			{
				AddIntoDebugLog(L"Not Loaded",cloned,chart_item->CurrentIndex,temp->StartIndex);					
				cloned=NULL;
				/*try{					
					delete cloned;
					cloned=NULL;
				}
				catch(...)
				{
					CLogManager::AddMessage(L"#### ");
				}*/
			}
		}	
		else
		{
			cloned=NULL;
			/*try{
				delete cloned;
				cloned=NULL;
			}
			catch(...)
			{
				CLogManager::AddMessage(L"#### cloned");
			}*/

		}
	}
return CONTINUE;
}
bool CChart::Completer(CChartItem*chart_item, unsigned short item_no)			
//It moves the dot forward in all the parent productions after the
//completion of a child production
{	
	
	if(!chart_item)
		return false;

//	debug_fw->Write(L"##### Completer IN \r\n");
	
	CChartItem* citem=NULL;
	
	short len=0;

	size_t limit=chart_item->StartIndex;

	if(limit>=ChartSize)
		return false;	

	//for(int p=0;p<=limit;p++)
	//{

	//if(chart_item->StartIndex<ChartSize)
	len=Chart[limit].ItemVector.size();
	//else
	//	return false;

	CProduction * prod2=NULL;
	CProduction * cloned=NULL;

	wstring str1=chart_item->Prod->LHS->Label;	
//	CProductionSymbol*ps2=NULL;	

	CChartItem*temp=NULL;
	wstring title=L"";
	
	//new code written on July 14, 2005 to

	if (chart_item->Prod->IsNullProduction())
	{
		// iterate from current position, to top of the col or same production is found
		// if dot symbol is found, move the dot and load the cloned prod on the chart
		// 
		#ifdef GenLog
		CLogManager::AddMessage(L" IN NULL Production 'S' \r\n");
		#endif

		for (int i=item_no-1; i >= 0; i--){
			temp=Chart[limit].ItemVector.at(i);
			prod2 = temp->Prod;

			if (FindandLoadProdOnChart(chart_item, prod2, str1, temp,true)==BREAK)
				break;

		}
	return true;
	}

	for(short k=0;k<len;k++)
	{	
		temp=Chart[limit].ItemVector.at(k);
		prod2 = temp->Prod;

		if (FindandLoadProdOnChart(chart_item, prod2, str1, temp,false)== BREAK)
			break;
			
/*	
		if(chart_item->Prod==prod2)
			break;		
		
		if(chart_item->IsFailed || prod2->IsNullProduction()||prod2->IsComplete())
			continue;
	
		if(prod2->GetCurrentSymbol()->Label==str1)
		{
			//if(str1==L"Sdecl" && prod2->GetCurrentSymbol()->Label==L"Sdecl")
			//	str1=str1;

			cloned=prod2->GetClone();											
			
			if(!cloned)
			{
				CLogManager::AddMessage(L"Clone Not Created\r\n");
			}
			else if(BuildFStructureEx(cloned,chart_item->Prod)){			
				
				cloned->GetCurrentSymbol()->ChartItemPtr=chart_item;												
				cloned->MoveDot();

				if(LoadIntoChart(cloned,chart_item->CurrentIndex,temp->StartIndex,false))
					AddIntoDebugLog(L"Completer",cloned,chart_item->CurrentIndex,temp->StartIndex);
				else
				{
					AddIntoDebugLog(L"Not Loaded",cloned,chart_item->CurrentIndex,temp->StartIndex);					
					
					try{					
						delete cloned;
						cloned=NULL;
					}
					catch(...)
					{
						CLogManager::AddMessage(L"#### ");
					}
				}
			}	
			else
			{
				try{
					delete cloned;
					cloned=NULL;
				}
				catch(...)
				{
					CLogManager::AddMessage(L"#### cloned");
				}

			}
		}
*/
	}
	//}//outer FOR

//	debug_fw->Write(L"##### Completer OUT \r\n");

	return true;
}


bool CChart::BuildFStructureEx(CProduction * ProdDest,CProduction * ProdSrc)
	//F-Structures are created after the following operations:
	//unification is applied
	//constraints are checked
	//set operations are applied
{
	using namespace std;

	if(!ProdDest||!ProdSrc)
		return false;	

	if(ProdSrc->IsNullProduction())
		return true;				
	
	if(!ProdDest->LHS->FSPtr)
	{
		ProdDest->LHS->FSPtr=new CFStructure();
		
		if(!ProdDest->LHS->FSPtr)
			return false;

		ProdDest->LHS->FSPtr->Root=new CFStructureItem();
		
		if(!ProdDest->LHS->FSPtr->Root)
			return false;

		ProdDest->LHS->FSPtr->Root->Label=CSymbolTableManager::RootId;
	}

	if(!ProdSrc->LHS->FSPtr)
		return true;
	
	CProductionSymbol * symb = ProdDest->GetCurrentSymbol();		
	
	CFStructure* FSPtr_up=ProdDest->LHS->FSPtr;
	CFStructure* FSPtr_down=NULL;
	
	if(ProdDest==ProdSrc)
		FSPtr_down=ProdSrc->GetCurrentSymbol()->FSPtr;
	else
		FSPtr_down=ProdSrc->LHS->FSPtr;
	
	
	CFStructureItem*temp_FSPtr=NULL;
	
	CFStructureItem*FSPtr1=NULL;
	CFStructureItem*FSPtr2=NULL;
	CFStructureItem*FSPtr3=NULL;
	
	CFStructureItem*FSPtrRet=NULL;

	vector<CFStructureItem*> * ValueSet=NULL;
	CFDExpression * val_list=NULL;
		
	vector<wstring> * constr_list=&symb->constraints;	
	vector<CFDescription*> * cond_list=&symb->conditions;
	
	short len=cond_list->size();

	bool flag=true;
	bool flag_constraint=true;
	wstring err_desc=L"";

	wstring fdesc_str=L"";
	CFDescription*fdesc=NULL;	

	bool created_fs=false;
	short element_id=CSymbolTableManager::ElementId;								

	//if(ProdDest->LHS->Label==L"S" /* && ProdSrc->RHS && ProdSrc->RHS->at(0)->Label==L"VPmain"*/)
	//{		
	//	MessageBox(NULL,ProdSrc->LHS->Label.c_str(),L"",MB_ICONINFORMATION);
	//}
	
	wstring msg=L"";

	for(short k=0;k<len && flag ;k++)
	{
		fdesc=cond_list->at(k);		

		/*
		if(fdesc->Label->list.size()!=0)
		{ 
			msg=LexiconPtr->GetConstantName(fdesc->Label->list.at(0));
			//CLogManager::AddMessage(msg);
			MessageBox(NULL,msg.c_str(),L"msg",1);
		} 
		*/		

		switch(fdesc->Operator)
		{
			//checks if there is a unification operator (=)

		case UNIFY_OP:

				FSPtrRet=NULL;
				
				if(fdesc->Label->op==UP_OP)
				{
					CFStructureItem * fL=NULL;
					CFStructureItem * fR=NULL;
					char created=0;
					bool error=false;

					if(FSPtr_up->Root==NULL)
					{
						FSPtr_up->Root=new CFStructureItem();
						
						if(!FSPtr_up->Root)
							error=true;
						else
							FSPtr_up->Root->Label=-1;
					}
						


					if(fdesc->Label->list.size()==0)
					{
						fL=FSPtr_up->Root;
					}
					else
						fL=FSPtr_up->Root->SearchAndCreate(&fdesc->Label->list,created);

					//fL->Reference
					
					if(fdesc->Value && fdesc->Value->op==DOWN_OP)
					{						
						if(fdesc->Value->list.size()==0)
						{
							fR=FSPtr_down->Root;
						}
						else
						{
							fR=FSPtr_down->Root->Search(&fdesc->Value->list);
							if(!fR)
							{								
								error=true;
								err_desc=L"Value Not Found From Down F-Structure";																
							}
						}						
						
						if(fR && !UnifySpecial(fL,fR,NULL))
						{							
							error=true;
							err_desc=L"Unification failed UnifySpecial(fL,fR,NULL)";						
						}						

					}
					else if(fdesc->Value && fdesc->Value->op==UP_OP)
					{
						if(fdesc->Value->list.size()==0)
						{							
							err_desc=L"Second is also UP, ^ Something=^ , is given";
							error=true;						
						}
						else		
						{						

							fR=FSPtr_up->Root->SearchAndCreate(&fdesc->Value->list,created);

							if(!fR)							
								error=true;							
							else
							{
								if(fL->Reference)
								{
									//fL->Reference;
									if(fR && !UnifySpecial(fL,fR,NULL))
									{
										error=true;
										err_desc=L"^aa=^bb Case: Unification failed UnifySpecial(fL,fR,NULL)";								
									}
								}
								else/* if(fL->ChildList.size()==0)*/
								{
									fL->Reference=fR->ItemNo;
									fR->IsReferred=true;
									fL->ValueList.clear();
								}
								/*else
								{
									error=true;
									err_desc=L"Logical Error , ^ Something having children= ^ something , is given";
								}*/
							}														
						}
					}
					else if (fdesc->Value && !UnifySpecial(fL,NULL,&fdesc->Value->list))
					{																							
						error=true;			
						err_desc=L"^AA=val Case: UnifySpecial(fL,NULL,&fdesc->Value->list)";
					}

					if(error)
					{
						PrintFDescW(fdesc,err_desc);
						//ProdDest->PrintProductionW(L"Error");
						return false;																		
					}
							
				}
				
				break;			
	 	
		case SET_OP:
		
			//checks if there is a set operator ($)
			created_fs=false;

			if(fdesc->Label->op==DOWN_OP)
			{
				if(fdesc->Label->list.size()==0)				
					FSPtr2 = FSPtr_down->Root;														
				else
					FSPtr2 = FSPtr_down->Search(&fdesc->Label->list);						

				if(fdesc->Value->op==UP_OP)
				{						
					//FSPtr1=FSPtr_up->Search(&fdesc->Value->list);
					FSPtr1=FSPtr_up->Search(&fdesc->Value->list);
					
					if(!FSPtr1)
					{
						if(!FSPtr_up->Root)
							FSPtr_up->Root=new CFStructureItem(-1,NULL);

						char created;
						FSPtr1=FSPtr_up->Root->SearchAndCreate(&fdesc->Value->list,created);
					}
				}
			
			}
			
			if(FSPtr2)
			{	
				ElementCounter++;
				wstring str = L"ELEMENT" + GetWSTR(ElementCounter);
				
				
				FSPtr2->Label=CSymbolTableManager::GetNum(str);

				if(FSPtr1)
					FSPtr1->AddElementInSet(FSPtr2);																
				else
					ProdDest->LHS->FSPtr->AddNewSetElement(FSPtr2);				
			}

			break;

		}//end switch
		
			
	}//end for

	//////////////////////////////////////////////////////////////////////
	///////////		Checking Constraints				//////////////////
	//////////////////////////////////////////////////////////////////////
	
	len=constr_list->size();
	wstring cstr=L"";
	int result = 0;
	for(short n=0;n<len;n++)
	{
		cstr=constr_list->at(n);
		result = EvaluateExpressionEx(cstr,ProdDest->LHS->FSPtr->Root,FSPtr_down->Root);
		//if(!EvaluateExpression(cstr,ProdDest->LHS->FSPtr->Root,FSPtr_down->Root/*  down ProdSrc->LHS->FSPtr->Root*/))
		if(!result)
		{
			#ifdef GenLog
			cstr=L"Constraint Failed : ";
			cstr+=constr_list->at(n);
			cstr+=L"\r\n";
			//debug_fw->Write(cstr);
			CLogManager::AddMessage (cstr);
			#endif

			return false;
		}	
		else
			if(result < 0)
			{
				
				if(!ProdDest->LHS->FSPtr)
				{
					ProdDest->LHS->FSPtr=new CFStructure();
		
					if(!ProdDest->LHS->FSPtr)
						return false;
				}
				if(!ProdDest->LHS->FSPtr->Root)
				{
					ProdDest->LHS->FSPtr->Root=new CFStructureItem();		

					if(!FSPtr_up->Root)
						return false;
				}

				ProdDest->LHS->FSPtr->Root->ConstraintList.push_back(cstr);

				
			}
	}

	return flag;

}

bool CChart::BuildFStructure(CProduction * Prod,CLexicalEntry*le,int WordIndex)
	//F-Structures are created after the following operations:
	//unification is applied
	//constraints are checked
	//set operations are applied
{
	
	using namespace std;
	
	CProductionSymbol*ProdSymb=Prod->GetCurrentSymbol();

	if(!Prod || !ProdSymb || !le)
		return false;
	
	CFStructure * fsptr=NULL;

	if(!ProdSymb->FSPtr)
	{
		ProdSymb->FSPtr=new CFStructure();
		
		if(!ProdSymb->FSPtr)
			return false;

		ProdSymb->FSPtr->Root=new CFStructureItem();		

		if(!ProdSymb->FSPtr->Root)
			return false;

	}
		
	fsptr=ProdSymb->FSPtr;	
	
	CFStructureItem * fs_item=fsptr->Root;
	CFStructureItem * fstemp=NULL;
	
	bool ret=true;
	
	CFDescription*fdesc1=NULL;	
	CFDescription*fdesc2=NULL;
	vector<short>*vlist=NULL;	
	char created;

	CFStructureItem * WIFSItem=NULL;
	short NumId=0;	
	wstring WordIndexStr=L"";

	for(list<CFDescription*>::iterator it=le->ConditionList.begin();
		it!=le->ConditionList.end();it++)
	{
		fdesc1=*it;						
			
		fstemp=fs_item->SearchAndCreate(&fdesc1->Label->list,created);		
		
		if(fstemp->Label==PredId)
		{
			WIFSItem=new CFStructureItem(WordIndexId,0);
					
			WordIndexStr=GetWSTR(WordIndex);
			NumId=CSymbolTableManager::GetNum(WordIndexStr,true);
			WIFSItem->ValueList.push_back(NumId);
			
			fstemp->AppendFSItem(WIFSItem,false);
			//fs_item->AppendFSItem(WIFSItem,false);
		}

		if(created)		
		{
			try
			{
				fstemp->ValueList=fdesc1->Value->list;					
			}
			catch(...)
			{
				#ifdef GenLog
				CLogManager::AddMessage(L"error: fstemp->ValueList=fdesc1->Value->list; \r\n");
				#endif
			}
		}
		else if(fstemp)
		{			
			vlist=GetSetIntersection(&fstemp->ValueList,&fdesc1->Value->list);
			if(vlist)
			{
				fstemp->ValueList=*vlist;			
				vlist->clear();
				
				try{
					delete vlist;
					vlist=NULL;
				}
				catch(...)
				{
					#ifdef GenLog
					CLogManager::AddMessage(L"#### cloned");
					#endif
				}
			}
			else
			{
				#ifdef GenLog
				CLogManager::AddMessage(L"PARSER:  Unification Failed:\r\n");				
				#endif
				return false;				
			}			
		}		
					
	}
	
	if(le->ConstraintList.size()>0)
	{
		wstring tstr=L"";
		
		#ifdef GenLog
		CLogManager::AddMessage(L"Copying Constraints From Lex -> F-Structure Node :\r\n");
		#endif
		
		for(list<wstring>::iterator temp=le->ConstraintList.begin();temp!=le->ConstraintList.end();temp++)
		{
			tstr=*temp;		
			fsptr->Root->ConstraintList.push_back(tstr);
			#ifdef GenLog
			CLogManager::AddMessage(tstr+L"\r\n");
			#endif
		}
	}


	
	
	return ret;
}

vector<CLexicalEntry*> * CChart::GetLexEntriesOnPOSAndFeatureMatch(
			 vector<CLexicalEntry*> * word_list,
			 CProductionSymbol*psymb)
{
	using namespace std;

	vector<CLexicalEntry*> * req_list=NULL;

	if(!word_list || !psymb)
		return req_list;

	CLexicalEntry*le=NULL;
	CLexicon*lexicon=Grammar->GetLexicon();	

	if(!lexicon)
		return NULL;

	short terminal_id=CSymbolTableManager::GetNum(psymb->Label,false);		

	if(terminal_id<0)
		return req_list;

	req_list=new vector<CLexicalEntry*>;

	if(!req_list)
		return req_list;

	short matched=0;
	for(vector<CLexicalEntry*>::iterator it=word_list->begin();it!=word_list->end();it++)
	{
		le=*it;				
		if(le->POS==terminal_id)		
		{
			req_list->push_back(le);			
			matched++;
		}
	}

	if(matched>0)
		return req_list;

	if(req_list)
	{
		try{
		delete req_list;
		}
		catch(...)
		{
			#ifdef GenLog
			CLogManager::AddMessage(L"#### req_list");
			#endif
		}

	}

	return NULL;
}

bool CChart::SearchProductionInChartColumn(CProduction *prod,size_t chart_index,size_t start_index )
{
	using namespace std;

	if(!prod || chart_index<0 || chart_index>=ChartSize || 
		start_index<0 || start_index>=ChartSize)
		return false;			

	CChartItem*citem=NULL;
	CProduction *cprod=NULL;
	
	bool matched=false;
	short plen=0;
	
	CProductionSymbol*psymb1=NULL;
	CProductionSymbol*psymb2=NULL;

	for(vector<CChartItem*>::iterator it=Chart[chart_index].ItemVector.begin();
		it!=Chart[chart_index].ItemVector.end();it++)
	{
		citem=*it;
		cprod=citem->Prod;
		plen=cprod->RHS.size();

		if( cprod->LHS->Label == prod->LHS->Label &&			
			citem->StartIndex == start_index &&
			cprod->curr_index == prod->curr_index &&
			plen == prod->RHS.size())
		{
			//compare right hand sides (RHS) of productions
			matched=true;
			
			if(!cprod->LHS->isEqual(prod->LHS))
				return false;
									
			for(short n=0;n<plen;n++)
			{
				psymb1= prod->RHS.at(n);
				psymb2= cprod->RHS.at(n);
		
				if(!psymb1->isEqual(psymb2))
				{
					matched=false;
					break;
				}
			}
			
			if(matched)
				return true;
		}		
	}

	return false;
}

void CChart::PrintChart()
{
	using namespace std;
	CChartItem*citem=NULL;
	
	for(size_t i=0;i<ChartSize;i++)
	{		
		for(vector<CChartItem*>::iterator it=Chart[i].ItemVector.begin();
			it!=Chart[i].ItemVector.end();it++)
			{			
				citem=*it;
				printf("[%d,%d] %c ",citem->StartIndex,citem->CurrentIndex,citem->IsFailed);
				//citem->Prod->PrintProduction();
			}		
	}
	
}

void CChart::PrintTree()
{
	using namespace std;
	CChartItem*citem=NULL;
	
	short last_index=ChartSize-1;
	short csize=Chart[last_index].ItemVector.size();
	
	if(csize<=0)	
		return;
	
	for(vector<CChartItem*>::iterator it=Chart[last_index].ItemVector.begin();
		it!=Chart[last_index].ItemVector.end();it++)
		{			
			citem=*it;
			if(citem->Prod->LHS->Label==L"S" && citem->Prod->IsComplete())
			{
				printf("[%d,%d] %c ",citem->StartIndex,citem->CurrentIndex,citem->IsFailed);
				//citem->Prod->PrintProduction();
				TraverseProduction(citem->Prod);   
			}
		}		
}

void CChart::TraverseProduction(CProduction *prod,short tab_count)
{
	using namespace std;

	if(!prod)
		return;
	
	short ind=0;
	wstring tab_str=L"\t";
	CProductionSymbol*psymb=NULL;	
	
	for(vector<CProductionSymbol*>::iterator it=prod->RHS.begin();	
		it!=prod->RHS.end();it++)
	{
		psymb=*it;											
		
		wprintf(L"%s\n",psymb->Label.c_str());
		
		for(ind=0;ind<tab_count;ind++)
			wprintf(tab_str.c_str());

		if(psymb->Label==L".")			
			wprintf(L".");			
		else if(!psymb->isTerminal && psymb->ChartItemPtr)
			TraverseProduction(psymb->ChartItemPtr->Prod,tab_count+1);								
			
		printf("\n");				
		
		for(ind=0;ind<tab_count-1;ind++)
			wprintf(tab_str.c_str());						
	}							
}

vector<short>* CChart::GetSetIntersection(vector<short> * list1, vector<short> * list2)
{
	if(!list1||!list2)
		return NULL;	

	short len1= list1->size();
	short len2= list2->size();

	vector<short>*list3=new vector<short>;

	if(!list3)
		return NULL;

	short val;
	short val2;
	
	if(len1==0)
	{
		*list3=*list2;
		return list3;
	}

	for(short k=0;k<len2;k++)
	{		
		val=list2->at(k);								
		
		for(short p=0;p<len1;p++)	
		{			
			val2=list1->at(p);									

			if(val==val2)
				list3->push_back(val);	
		}
	}

	if(list3->size()==0)
	{
		try{
			delete list3;
			list3=NULL;
		}
		catch(...)
		{
			#ifdef GenLog
			CLogManager::AddMessage(L"#### list3");
			#endif
		}

		return NULL;
	}
			
	return list3;
}

/* REMOVED IN THIS (DOS/LINUX) VERSION
void CChart::ShowCStructure(CTreeCtrl *m_TreeCtrl)
{
	using namespace std;

	if(!m_TreeCtrl)
		return;
	
	CChartItem*citem=NULL;
	
	short last_index=ChartSize-1;
	short csize=Chart[last_index].ItemVector.size();
	
	if(csize<=0)
	{		
		//MessageBox(NULL,L"The sentence could not be parsed.",L"LFG Parser",MB_ICONEXCLAMATION);		
		return;
	}		

	short counter=49;	
	HTREEITEM hitem;
	wstring cstr;
	
	
	for(vector<CChartItem*>::iterator it=Chart[last_index].ItemVector.begin();
		it!=Chart[last_index].ItemVector.end();it++)
		{			
			citem=*it;
			if(citem->Prod->LHS->Label==L"S" && citem->Prod->IsComplete())
			{						
				cstr=L"C-Structure # ";
				cstr+=counter;				
				hitem=m_TreeCtrl->InsertItem(cstr.c_str());
				ShowCStructureRec(citem->Prod,m_TreeCtrl,hitem,true);
				m_TreeCtrl->Expand(hitem,TVE_EXPAND);
				counter++;
			}
		}		  

}
REMOVED IN THIS (DOS/LINUX) VERSION */

/* REMOVED IN THIS (DOS/LINUX) VERSION
void CChart::ShowCStructureRec(CProduction *prod,CTreeCtrl *m_TreeCtrl, HTREEITEM parent, bool use_parent)
{	
	using namespace std;

	if(!prod)
		return;
	
	CProductionSymbol*psymb=NULL;
	HTREEITEM new_parent=NULL;	
	
	wstring word_info=L"";
	wstring label=L"";
	wstring npstr=Grammar->GetNewProdName();

	short len=prod->RHS.size();
	
	for(short p=0;p<len;p++)
	{
		psymb=prod->RHS.at(p);	
		label=psymb->Label;
		

		if(label.substr(0,npstr.size())==npstr || label[0]=='.')
		{
			new_parent=parent;
		}
		else
		{
			if(psymb->isTerminal)
			{							
					label+=L" ----- ";					
					label+=psymb->SenWord;						
			}
			
			if(psymb->Label==L"." && len!=1)
				len=len;
			else if(use_parent)
				new_parent=m_TreeCtrl->InsertItem(label.c_str(),0,0, parent);
			else
				new_parent=m_TreeCtrl->InsertItem(label.c_str());			
		}
					
		if(!psymb->isTerminal && psymb->ChartItemPtr && psymb->Label!=L".")
		{
			ShowCStructureRec(psymb->ChartItemPtr->Prod,m_TreeCtrl,new_parent,true);				
			m_TreeCtrl->Expand(new_parent,TVE_EXPAND);
		}
		
	}						
}
REMOVED IN THIS (DOS/LINUX) VERSION */

CCStructureItem * CChart::RetrieveCStructure(CProduction * prod, CSentenceInfo * sen_info)
{	
	using namespace std;

	if(!prod)
		return NULL;

	CCStructureItem * csitem = new CCStructureItem(prod->LHS->Label,prod->LHS->SenWord,NULL,NULL,NULL);
	CCStructureItem * temp=NULL;

	CProductionSymbol*psymb=NULL;
			
	short id = 0;

	for(vector<CProductionSymbol*>::iterator psit=prod->RHS.begin();psit!=prod->RHS.end();psit++)
	{
		psymb=*psit;
									
		if(psymb->Label.at(0)=='.')
			continue;

		if(psymb->isTerminal && psymb->SenWord!=L"")
		{
			temp = new CCStructureItem(psymb->Label,psymb->SenWord,csitem,NULL,NULL);			
			id = CSymbolTableManager::GetNum(psymb->Label,true);
			sen_info->Words.push_back(new CWordInfo(psymb->SenWord,id));
		}													
		else if(psymb->ChartItemPtr)
		{
			temp = RetrieveCStructure(psymb->ChartItemPtr->Prod,sen_info);							
		}
		else
		{
			#ifdef GenLog
			CLogManager::AddMessage(L"psymb->ChartItemPtr is NULL ??? \r\n");
			#endif
		}	

		csitem->ChildList.push_back(temp);
	}			
	
	return csitem;
}

/* REMOVING IN THIS (DOS/LINUX) VERSION
void CChart::GenerateCStructureRecXML(CProduction *prod,CFileWriter * fw)
{	
	using namespace std;

	if(!prod || !fw)
		return;
	
	CProductionSymbol*psymb=NULL;
		
	wstring word_info=L"";
	wstring label=L"";
	wstring value=L"";
	wstring npstr=Grammar->GetNewProdName();
	bool show=false;

	short len=prod->RHS.size();
	
	for(short p=0;p<len;p++)
	{
		psymb=prod->RHS.at(p);
		show=true;
		
		label=psymb->Label;
		label=MakeXMLCompatible(label);
		
		if(label.at(0)=='.' || label.substr(0,npstr.size())==npstr)
			show=false;

		if(show)
			fw->Write(L"<"+label+L">");

		if(psymb->isTerminal && show)
		{											
			value=MakeXMLCompatible(psymb->SenWord);				
			fw->Write(value);				
		}											
		
		if(!psymb->isTerminal && psymb->ChartItemPtr && psymb->Label.at(0)!='.')			
			GenerateCStructureRecXML(psymb->ChartItemPtr->Prod,fw);											
		
		if(show)
			fw->Write(L"</"+label+L">");
	}			
		
}
REMOVING IN THIS (DOS/LINUX) VERSION */

/* REMOVED IN THIS (DOS/LINUX) VERSION
void CChart::ShowFStructure(CTreeCtrl *m_TreeCtrl)
{
	using namespace std;

	if(!m_TreeCtrl)
		return;
	
	CChartItem*citem=NULL;
	
	short last_index=ChartSize-1;
	short csize=Chart[last_index].ItemVector.size();
	
	if(csize<=0)
	{			
		return;
	}	
	
	short counter=49;	
	HTREEITEM hitem;
	wstring cstr;
	
	CFStructureItem*nroot=NULL;
	short constraint_len=0;
	wstring constraint;
	bool passed=false;
	
	for(vector<CChartItem*>::iterator it=Chart[last_index].ItemVector.begin();
		it!=Chart[last_index].ItemVector.end();it++)
		{			
			citem=*it;
			if(citem->Prod->LHS->Label==L"S" && citem->Prod->IsComplete())
			{																						
				nroot=citem->Prod->LHS->FSPtr->Root;
				passed=true;

				for(vector<wstring>::iterator it=nroot->ConstraintList.begin();
					it!=nroot->ConstraintList.end();it++)
				{
					constraint=*it;
					if(!EvaluateExpression(constraint,nroot,nroot))
					{			
						passed=false;
						break;
					}					
				}

				if(passed && CheckForwardCompleteness(nroot))
				{					
					cstr=L"F-Structure # ";
					cstr+=counter;				
					hitem=m_TreeCtrl->InsertItem(cstr.c_str());
					ShowFStructureRec(nroot,m_TreeCtrl,hitem,true);
					m_TreeCtrl->Expand(hitem,TVE_EXPAND);
					counter++;
				}		

				if(!passed)
					MessageBox(NULL,L"Sentence could not be parsed",L"LFG Parser",MB_ICONINFORMATION);								

			}
		}		  

}
REMOVED IN THIS (DOS/LINUX) VERSION */

/* REMOVED IN THIS (DOS/LINUX) VERSION
void CChart::ShowFStructureRec(CFStructureItem * fsitem,
							   CTreeCtrl *m_TreeCtrl,
							   HTREEITEM parent,
							   bool use_parent)
{	
	using namespace std;

	if(!fsitem)
		return;
	
	CFStructureItem * fstemp=NULL;	
	CLexicon*lex=Grammar->GetLexicon();

	if(!lex)
	{	
		return;
	}
	
	wstring label;
	wstring value;
	wstring node_str;
	
	short vsize=0;
	short chsize=0;
	
	HTREEITEM hitem;
	HTREEITEM phitem;
		
	chsize=fsitem->ChildList.size();		
	vsize=fsitem->ValueList.size();
	
	label=CSymbolTableManager::GetString(fsitem->Label);
	value=L"";

	/////////// supress the m-structure f-descriptions

	for(short k=0;k<vsize;k++)
	{
		value+=CSymbolTableManager::GetString(fsitem->ValueList.at(k));
		value+=L" ";
	}
	
	node_str=label;
	
	if(vsize>0)
	{
		node_str+=L" ----- ";
		node_str+=value;			
	}

	if(!use_parent)
		hitem=m_TreeCtrl->InsertItem(node_str.c_str());			
	else
		hitem=m_TreeCtrl->InsertItem(node_str.c_str(),0,0,parent);			


	for(vector<CFStructureItem*>::iterator it=fsitem->ChildList.begin();
		it!=fsitem->ChildList.end();it++)
	{
		ShowFStructureRec(*it,m_TreeCtrl,hitem,true);
		m_TreeCtrl->Expand(hitem,TVE_EXPAND);
	}			

	if(!fsitem->SetElements.empty())
	{
		//temporary changes to display set

		phitem=m_TreeCtrl->InsertItem(L"SET",0,0,hitem);
		
		for(vector<CFStructureItem*>::iterator it2=fsitem->SetElements.begin();
			it2!=fsitem->SetElements.end();it2++)
		{
			ShowFStructureRec(*it2,m_TreeCtrl,phitem,true);
			m_TreeCtrl->Expand(phitem,TVE_EXPAND);
		}			

		m_TreeCtrl->Expand(hitem,TVE_EXPAND);
	}
}
REMOVED IN THIS (DOS/LINUX) VERSION */

/* REMOVING IN THIS (DOS/LINUX) VERSION
void CChart::GenerateFStructureRecXML(CFStructureItem * fsitem,CFileWriter*fw)
{	
	using namespace std;

	if(!fsitem)
		return;
	
	CFStructureItem * fstemp=NULL;	
	
	wstring label;
	wstring value;
	wstring node_str;
	
	short vsize=0;
	short chsize=0;		

	chsize=fsitem->ChildList.size();		
	vsize=fsitem->ValueList.size();
	
	label=CSymbolTableManager::GetString(fsitem->Label);

	if(label.substr(0,1)==L"_")
		return;

	label=MakeXMLCompatible(label);	
	
	/*
	short npos=label.find(L"ELEMENT");
	wstring elem=label.substr(0,7);

	if(elem==L"Element")		
		label=elem;		
	*/
/*
	fw->Write(L"<"+label+L">");
	value=L" ";
			
	for(short k=0;k<vsize;k++)
	{
		fw->Write(L"<Value>");
		value=CSymbolTableManager::GetString(fsitem->ValueList.at(k));
		value=MakeXMLCompatible(value);
		fw->Write(value);			
		fw->Write(L"</Value>");
	}														

	
	for(vector<CFStructureItem*>::iterator it=fsitem->ChildList.begin();it!=fsitem->ChildList.end();it++)
	{
		GenerateFStructureRecXML(*it,fw);				
	}			
	
	
	for(vector<CFStructureItem*>::iterator it2=fsitem->SetElements.begin();
		it2!=fsitem->SetElements.end();it2++)
	{
		GenerateFStructureRecXML(*it2,fw);
	}			

	fw->Write(L"</"+label+L">");
			
}
REMOVING IN THIS (DOS/LINUX) VERSION */

bool CChart::EvaluateExpression(wstring &wstr_in, CFStructureItem * FSPtr_up,CFStructureItem * FSPtr_down)
{
	if(!FSPtr_up)
		return false;

	/* REPLACING IN THIS (DOS/LINUX) VERSION
	CString cstr=wstr_in.c_str();			
	cstr.Replace(L"=c",L"=="); 

	wstring wstr=cstr.GetBuffer(512);
	END REPLACING IN THIS (DOS/LINUX) VERSION */
	
	//MessageBox(NULL,wstr.c_str(),L"In Evaluate Expression",MB_ICONINFORMATION);
	
	/* REPLACING WITH */
	wstring wstr= L"" ; 

	if(wstr_in.find(L"=c")!=wstring::npos)
		wstr=wstr_in.replace(wstr_in.find(L"=c"),2,L"==");
	else
		wstr=wstr_in;
	/* END REPLACING WITH */

	wstring	left=L"";
	wstring	right=L"";
	wstring	str=L"";	

	string bool_exp="";	
	
	short cond_len= wstr.size();		
	short last_op=0;
	short result=0;	
	
	for(short n=0;n<cond_len;n++)
	{
		switch(wstr[n])
		{
		   case ']':
												
				if(left==L"")
				{
					bool_exp+=")";
					last_op=')';
					break;
				}
								
				right=str;
				result=CheckConstraint(FSPtr_up,FSPtr_down,left,right);				
				bool_exp += (result+48);
				
				bool_exp+=")";
				last_op=')';

				left=right=str=L"";
				break;

		   case '&':
		   case '|':

				if(last_op==wstr[n])
					break;

				if(last_op==')' && left==L"")
				{					
					last_op= wstr[n];	
					bool_exp +=(char) wstr[n];
					break;
				}

				last_op= wstr[n];

				right=str;

				result=CheckConstraint(FSPtr_up,FSPtr_down,left,right);
						
				bool_exp += (result+48);
				bool_exp += (char)wstr[n];
				left=right=str=L"";
				
				break;

		   case '[':
				
				bool_exp+="(";		
				break;
   		   			
		   case '=':

				if(last_op=='=')
					break;

				last_op='=';
				left=str;
				str=L"";
				break;
					
			case '~':

				bool_exp += "!";	
				break;
					
  			default:
		
				str+= wstr[n];					

		}//end switch					
	
	}//end for
	
	long rs=EvaluateBooleanExpressionEx(bool_exp);

	if(rs>0)
		return true;
	
	return false;
}

int CChart::EvaluateExpressionEx(wstring &wstr_in, CFStructureItem * FSPtr_up,CFStructureItem * FSPtr_down)
{
	bool leave = false;

	if(!FSPtr_up)
		return false;

	/* REPLACING IN THIS (DOS/LINUX) VERSION
	CString cstr=wstr_in.c_str();			
	cstr.Replace(L"=c",L"=="); 

	wstring wstr=cstr.GetBuffer(512);
	END REPLACING IN THIS (DOS/LINUX) VERSION */
	//MessageBox(NULL,wstr.c_str(),L"In Evaluate Expression",MB_ICONINFORMATION);

	/* REPLACING WITH */
	wstring wstr= L"" ; 

	if(wstr_in.find(L"=c")!=wstring::npos)
		wstr=wstr_in.replace(wstr_in.find(L"=c"),2,L"==");
	else
		wstr=wstr_in;
	/* END REPLACING WITH */
	
	wstring	left=L"";
	wstring	right=L"";
	wstring	str=L"";	

	string bool_exp="";	
	
	short cond_len= wstr.size();		
	short last_op=0;
	short result=0;	
	
	for(short n=0;n<cond_len;n++)
	{
		switch(wstr[n])
		{
		   case ']':
												
				if(left==L"")
				{
					bool_exp+=")";
					last_op=')';
					break;
				}
								
				right=str;
				result=CheckConstraintEx(FSPtr_up,FSPtr_down,left,right);		
				if(result < 0)
				{
					leave = true;
					result = 1;
				}
				bool_exp += (result+48);
				
				bool_exp+=")";
				last_op=')';

				left=right=str=L"";
				break;

		   case '&':
		   case '|':

				if(last_op==wstr[n])
					break;

				if(last_op==')' && left==L"")
				{					
					last_op= wstr[n];	
					bool_exp +=(char) wstr[n];
					break;
				}

				last_op= wstr[n];

				right=str;

				result=CheckConstraintEx(FSPtr_up,FSPtr_down,left,right);
				if(result < 0)
				{
					leave = true;
					result = 1;
				}		
				bool_exp += (result+48);
				bool_exp += (char)wstr[n];
				left=right=str=L"";
				
				break;

		   case '[':
				
				bool_exp+="(";		
				break;
   		   			
		   case '=':

				if(last_op=='=')
					break;

				last_op='=';
				left=str;
				str=L"";
				break;
					
			case '~':

				bool_exp += "!";	
				break;
					
  			default:
		
				str+= wstr[n];					

		}//end switch					
	
	}//end for
	
	long rs=EvaluateBooleanExpressionEx(bool_exp);

	if(rs>0 && !leave)
		return 1;
	if(rs>0 && leave)
		return -1;
	return 0;
}

bool CChart::UnifySpecial(CFStructureItem * & fL, 
						  CFStructureItem *fR, 
						  vector<short> *vR,
						  bool ApplySetUnification)
{
	using namespace std;
	wstring msg;

	CFStructureItem*temp = NULL;
	CFStructureItem*fL_element = NULL;
	CFStructureItem*fR_element = NULL;
	vector<short> *result=NULL;	
		
 	if(!fR && fL && vR)
	{		
		if(fL->ChildList.size()!=0)
		{
			//msg=L"fL->ChildList.size()!=0";
			//MessageBox(NULL,msg.c_str(),msg.c_str(),2);
			return false;
		}
				
		if(!fL->ValueList.empty())
		{
			wstring old_val=CSymbolTableManager::GetString(fL->Label);
				
			if(old_val.at(0)!='#')
			{
				result=GetSetIntersection(&fL->ValueList,vR);
			}

			if(!result)
			{
				//msg=L"!result";
				//MessageBox(NULL,msg.c_str(),msg.c_str(),2);
				return false;
			}
			
			fL->ValueList=*result;
			result->clear();
			try{
				delete result;
				result=NULL;
				}
			catch(...)
			{
				#ifdef GenLog
				CLogManager::AddMessage(L"#### ");
				#endif
			}
		}		
		else
		{
			for(vector<short>::iterator it=vR->begin();it!=vR->end();it++)
				fL->ValueList.push_back(*it);						
		}		

	}
	else if(!vR)
	{
		short len=fR->ChildList.size();
 		short size=fR->SetElements.size();

		char created=0;		
				
		//calculate size of Set
		//check len==0 and size==0
		
		if( len==0 && ( (ApplySetUnification && size==0) || !ApplySetUnification ))
		{			
			if(!fR->ValueList.empty())
			{			
				if(fL->ValueList.empty())
					fL->ValueList= fR->ValueList;
				else
				{
					result=GetSetIntersection(&fR->ValueList,&fL->ValueList);
					if(!result)
					{						
						//msg=L"!vR ==> !result";
						//MessageBox(NULL,msg.c_str(),msg.c_str(),2);
						return false;
					}
					fL->ValueList=*result;
					
					///////////////// added for LDD //////////////////////////
					//fL->ItemNo=fR->ItemNo;
					//fL->IsReferred=fR->IsReferred;
					//fL->Reference=fR->Reference; 
					///////////////// added for LDD //////////////////////////

					result->clear();
					
					try{					
						delete result;
						result=NULL;
					}
					catch(...)
					{
						#ifdef GenLog
						CLogManager::AddMessage(L"#### result");
						#endif
					}

				}
			}			
		}
		else
		{		
			if(ApplySetUnification && size!=0)
			{
				CFStructureItem*nfs=new CFStructureItem();
				
				if(!nfs)
					return false;

				nfs->Label=CSymbolTableManager::SpecialSetId;
				CFStructureItem*rs=NULL;
				
				for(vector<CFStructureItem*>::iterator it=fR->SetElements.begin();
					it!=fR->SetElements.end();it++)
					{
						temp=*it;
						rs=fL->GetClone();
						
						if(!UnifySpecial(rs,temp,NULL))
						{		
							try{
								delete rs;
								rs=NULL;
							}
							catch(...)
							{
								#ifdef GenLog
								CLogManager::AddMessage(L"#### rs");
								#endif
							}

							//MessageBox(NULL,L"Set Unification Failed",L"Set Unification Failed",MB_ICONERROR);
							return false;					
						}
					
						AppendConstraints(rs,temp);							
						nfs->SetElements.push_back(rs);																
					}
							
				*fL=*nfs;
			}
		
			for(short p=0;p<len;p++)
			{
				fR_element=fR->ChildList.at(p);				

				fL_element=fL->SearchEx(fR_element->Label);			
				
				if(fL_element)
				{
					if(!UnifySpecial(fL_element,fR_element,NULL))
					{
						//msg=L"!UnifySpecial(fL_element,fR_element,NULL)";
						//MessageBox(NULL,msg.c_str(),msg.c_str(),2);
						return false;				
					}
				}
				else				
					fL->AppendFSItem(fR_element->GetClone(),false);
			}				
		}
	}		
		

	if(!ApplySetUnification && fR)
	{			
		for(vector<CFStructureItem*>::iterator it=fR->SetElements.begin();
			it!=fR->SetElements.end();it++)
			{
				temp=*it;
				fL->SetElements.push_back(temp->GetClone());
			}
		
		AppendConstraints(fL,fR);		
	}			
	
return true;
}


bool CChart::UnifySpecial_OLD(CFStructureItem * & fL, CFStructureItem *fR, vector<short> *vR)
{
	using namespace std;
	wstring msg;

	CFStructureItem*temp = NULL;
	CFStructureItem*fL_element = NULL;
	CFStructureItem*fR_element = NULL;
	vector<short> *result=NULL;
		
 	if(!fR && fL && vR)
	{		
		if(fL->ChildList.size()!=0)
		{
			msg=L"fL->ChildList.size()!=0";
			/*MessageBox(NULL,msg.c_str(),msg.c_str(),2); REMOVING IN THIS (DOS/LINUX) VERSION */
			return false;
		}
				
		if(!fL->ValueList.empty())
		{
			wstring old_val=CSymbolTableManager::GetString(fL->Label);
				
			if(old_val.at(0)!='#')
			{
				result=GetSetIntersection(&fL->ValueList,vR);
			}

			if(!result)
			{
				msg=L"!result";
				/*MessageBox(NULL,msg.c_str(),msg.c_str(),2); REMOVING IN THIS (DOS/LINUX) VERSION */
				return false;
			}
			
			fL->ValueList=*result;						
			result->clear();

			try{
				delete result;
				result=NULL;
			}
			catch(...)
			{
				#ifdef GenLog
				CLogManager::AddMessage(L"#### result");
				#endif
			}
		}		
		else
		{

			for(vector<short>::iterator it=vR->begin();it!=vR->end();it++)
				fL->ValueList.push_back(*it);						
		}		

	}
	else if(!vR)
	{
		short len=fR->ChildList.size();
 		char created=0;		
		
		
		//calculate size of Set
		//check len==0 and size==0
		if(len==0)
		{			
			if(!fR->ValueList.empty())
			{			
				if(fL->ValueList.empty())
					fL->ValueList= fR->ValueList;
				else
				{
					result=GetSetIntersection(&fR->ValueList,&fL->ValueList);
					if(!result)
					{						
						msg=L"!vR ==> !result";
						/*MessageBox(NULL,msg.c_str(),msg.c_str(),2); REMOVING IN THIS (DOS/LINUX) VERSION */
						return false;
					}
					fL->ValueList=*result;
					result->clear();
					
					try{
						delete result;
						result=NULL;
					}
					catch(...)
					{
						#ifdef GenLog
						CLogManager::AddMessage(L"#### ");
						#endif
					}
				}
			}			
		}
		else
		{						
			//if size!=0
				//.....
			//else following
			for(short p=0;p<len;p++)
			{
				fR_element=fR->ChildList.at(p);				

				fL_element=fL->SearchEx(fR_element->Label);			
				
				if(fL_element)
				{
					if(!UnifySpecial(fL_element,fR_element,NULL))
					{
						msg=L"!UnifySpecial(fL_element,fR_element,NULL)";
						/*MessageBox(NULL,msg.c_str(),msg.c_str(),2); REMOVING IN THIS (DOS/LINUX) VERSION */
						return false;				
					}
				}
				else				
					fL->AppendFSItem(fR_element->GetClone(),false);
			}	
		}
	}		
	
	bool ApplySetUnification=false;

	if(fR)
	{			
		if(!ApplySetUnification)
		{
			for(vector<CFStructureItem*>::iterator it=fR->SetElements.begin();
				it!=fR->SetElements.end();it++)
				{
					temp=*it;
					fL->SetElements.push_back(temp->GetClone());
				}
			
			AppendConstraints(fL,fR);		
		}
		else
		{		
			//Set Unification
		
			short size=fR->SetElements.size();

			if(size>0)
			{
				CFStructureItem*nfs=new CFStructureItem();
		
				if(!nfs)
					return false;

				nfs->Label=CSymbolTableManager::SpecialSetId;

				CFStructureItem*rs=NULL;

				for(vector<CFStructureItem*>::iterator it=fR->SetElements.begin();
					it!=fR->SetElements.end();it++)
					{
						temp=*it;
						rs=fL->GetClone();
						
						if(!UnifySpecial(rs,temp,NULL))
						{		
							try{						
								delete rs;
								rs=NULL;
							}
							catch(...)
							{
								#ifdef GenLog
								CLogManager::AddMessage(L"#### RS");
								#endif
							}

							/*MessageBox(NULL,L"Set Unification Failed",L"Set Unification Failed",MB_ICONERROR); REMOVING IN THIS (DOS/LINUX) VERSION */
							return false;					
						}
					
						AppendConstraints(rs,temp);		
						nfs->SetElements.push_back(rs);																
					}
							
				fL=nfs;
			}
	
		}//end else
	
	}//end if(fR)
	
return true;
}



short CChart::CheckConstraint(CFStructureItem*up,CFStructureItem*down,
							  wstring &left,wstring &right)
{
	if(!down || !up)
		return 0;	

	CFDExpression * left_exp=LexiconPtr->GenerateFDExpression(left);
	CFDExpression * right_exp=LexiconPtr->GenerateFDExpression(right);

	if(!left_exp||!right_exp)
	{
		CFDExpression::DeleteFDExpressions(left_exp,right_exp);
		return 0;
	}

	CFStructureItem*fs1=NULL;
	CFStructureItem*fs2=NULL;

	if(left_exp->op==UP_OP)
	{
		fs1=up->Search(&left_exp->list);

		if(right_exp->op==DOWN_OP)
		{
			fs2=down->Search(&right_exp->list);		
			
			if(!fs2||!fs1)
			{
				CFDExpression::DeleteFDExpressions(left_exp,right_exp);
				return 0;						
			}
		
			if(!IsIntersectionEmpty(&fs2->ValueList,&fs1->ValueList)) 
			{
	
				CFDExpression::DeleteFDExpressions(left_exp,right_exp);
				return 1;
			}

			CFDExpression::DeleteFDExpressions(left_exp,right_exp);

			return 0;
		}
								
		if(fs1 && !IsIntersectionEmpty(&right_exp->list,&fs1->ValueList))	
		{
			CFDExpression::DeleteFDExpressions(left_exp,right_exp);
			return 1;
		}
	}
	else if(left_exp->op==DOWN_OP)
	{
		fs2=down->Search(&left_exp->list);	
		
		if(right_exp->op==UP_OP)
		{
			fs1=up->Search(&right_exp->list);							
			
			if(!fs1)
			{
				CFDExpression::DeleteFDExpressions(left_exp,right_exp);
				return 0;						
			}
			
			if(fs2 && !IsIntersectionEmpty(&fs2->ValueList,&fs1->ValueList)) 
			{
				CFDExpression::DeleteFDExpressions(left_exp,right_exp);
				return 1;
			}

			CFDExpression::DeleteFDExpressions(left_exp,right_exp);
			return 0;
		}

	
		if(fs2)
		{							
			if(!IsIntersectionEmpty(&right_exp->list,&fs2->ValueList))
			{
				CFDExpression::DeleteFDExpressions(left_exp,right_exp);
				return 1;
			}
		}

	}

	CFDExpression::DeleteFDExpressions(left_exp,right_exp);	
	return 0;	
}
short CChart::CheckConstraintEx(CFStructureItem*up,CFStructureItem*down,
							  wstring &left,wstring &right)
{
	if(!down || !up)
		return 0;	

	CFDExpression * left_exp=LexiconPtr->GenerateFDExpression(left);
	CFDExpression * right_exp=LexiconPtr->GenerateFDExpression(right);

	if(!left_exp||!right_exp)
	{
		CFDExpression::DeleteFDExpressions(left_exp,right_exp);
		return 0;
	}

	CFStructureItem*fs1=NULL;
	CFStructureItem*fs2=NULL;

	if(left_exp->op==UP_OP)
	{
		fs1=up->Search(&left_exp->list);
		if(!fs1)
		{
			CFDExpression::DeleteFDExpressions(left_exp,right_exp);
			return -1;						
		}
		if(right_exp->op==DOWN_OP)
		{
			fs2=down->Search(&right_exp->list);		
			
			if(!fs2||!fs1)
			{
				CFDExpression::DeleteFDExpressions(left_exp,right_exp);
				return 0;						
			}
		
			if(!IsIntersectionEmpty(&fs2->ValueList,&fs1->ValueList)) 
			{
	
				CFDExpression::DeleteFDExpressions(left_exp,right_exp);
				return 1;
			}

			CFDExpression::DeleteFDExpressions(left_exp,right_exp);

			return 0;
		}
								
		if(fs1 && !IsIntersectionEmpty(&right_exp->list,&fs1->ValueList))	
		{
			CFDExpression::DeleteFDExpressions(left_exp,right_exp);
			return 1;
		}
	}
	else if(left_exp->op==DOWN_OP)
	{
		fs2=down->Search(&left_exp->list);	
		
		if(right_exp->op==UP_OP)
		{
			fs1=up->Search(&right_exp->list);							
			
			if(!fs1)
			{
				CFDExpression::DeleteFDExpressions(left_exp,right_exp);
				return 0;						
			}
			
			if(fs2 && !IsIntersectionEmpty(&fs2->ValueList,&fs1->ValueList)) 
			{
				CFDExpression::DeleteFDExpressions(left_exp,right_exp);
				return 1;
			}

			CFDExpression::DeleteFDExpressions(left_exp,right_exp);
			return 0;
		}

	
		if(fs2)
		{							
			if(!IsIntersectionEmpty(&right_exp->list,&fs2->ValueList))
			{
				CFDExpression::DeleteFDExpressions(left_exp,right_exp);
				return 1;
			}
		}

	}

	CFDExpression::DeleteFDExpressions(left_exp,right_exp);	
	return 0;	
}

bool CChart::CheckListEquality(vector<short> *list1,vector<short> *list2)
{	
	if(!list1 || !list2)
		return false;

	short p=0;
	short len1=list1->size();
	short len2=list2->size();

	if(len1!=len2)
		return false;

	bool found=false;

	short num1=0;
	short num2=0;

	for(vector<short>::iterator it1=list1->begin();it1!=list1->end();it1++)
	{
		num1=*it1;

		found=false;

		for(vector<short>::iterator it2=list2->begin();it2!=list2->end();it2++)
		{
			num2=*it2;
			
			if(num1==num2)
			{
				found=true;
				break;
			}
		}

		if(!found)
			return false;
	}
		
	return true;		
}

bool CChart::IsIntersectionEmpty(vector<short> *list1,vector<short> *list2)
{
	///////////////////////////////////////////////////////
	//added during gen update....
	///////////////////////////////////////////////////////

	if(list1->size()==0 && list2->size()==0)
		return false;

	//////////////////////////////////////////////////////

	vector<short>*iset=GetSetIntersection(list1,list2);
	
	bool EmptySet=true;

	if(iset)
	{
		 if(iset->size()>0)
			EmptySet=false;

		try{
			delete iset;	
			iset=NULL;
		}
		catch(...)
		{
			#ifdef GenLog
			CLogManager::AddMessage(L"#### iset");
			#endif
		}		
	}

	return EmptySet;
}

void CChart::AppendConstraints(CFStructureItem *fL, CFStructureItem *fR)
{
	using namespace std;

	if(!fL||!fR)
		return;

	for(vector<wstring>::iterator it=fR->ConstraintList.begin(); it!=fR->ConstraintList.end();it++)		
		fL->ConstraintList.push_back(*it);		
}

void CChart::AppendSet(CFStructureItem *fL, CFStructureItem *fR)
{
	using namespace std;

	if(!fL||!fR)
		return;

	for(vector<CFStructureItem*>::iterator it=fR->SetElements.begin(); it!=fR->SetElements.end();it++)		
		fL->SetElements.push_back((*it)->GetClone());		
}

void CChart::AppendChilds(CFStructureItem *fL, CFStructureItem *fR)
{
	using namespace std;

	if(!fL||!fR)
		return;

	for(vector<CFStructureItem*>::iterator it=fR->ChildList.begin(); it!=fR->ChildList.end();it++)		
		fL->ChildList.push_back((*it)->GetClone());		
}


void CChart::PrintFDescW(CFDescription*fdesc, wstring title)
{
		using namespace std;
		
		if(!fdesc||!LexiconPtr)
			return;
		
		wstring output=L"";				

		output+=fdesc->Label->op;
		output+=L" ";

		for(vector<short>::iterator it1=fdesc->Label->list.begin();
			it1!=fdesc->Label->list.end();it1++)
			{
				output+=CSymbolTableManager::GetString(*it1);
				output+=L" ";
			}

		output+=fdesc->Operator;
		output+=L" ";
		
		if(fdesc->Value)
		{
			output+=fdesc->Value->op;
			output+=L" ";

			for(vector<short>::iterator it2=fdesc->Value->list.begin();
				it2!=fdesc->Value->list.end();it2++)
				{
					output+=CSymbolTableManager::GetString(*it2);
					output+=L" ";
				}
		}
		else
			output+=L" NULL";
		
		#ifdef GenLog
		CLogManager::AddMessage(L"Unification Failed: "+output+L"\r\n");
		#endif

		//MessageBox(NULL,output.c_str(),title.c_str(),MB_ICONINFORMATION);		
}


bool CChart::CheckForwardCompleteness(CFStructureItem *item)
{
	using namespace std;

	if(!item)
		return true;

	//CLexicon*lex=Grammar->GetLexicon();

	short predid=CSymbolTableManager::PredId;		
	short gfid=CSymbolTableManager::GFId;		

	bool gf_found=false;
	vector<short> gflist;
	wstring err_str=L"";

	vector<short> vlist;	

	CFStructureItem*child=NULL;
	CFStructureItem*gfnode=NULL;

	for(vector<CFStructureItem*>::iterator it=item->ChildList.begin();
		it!=item->ChildList.end();it++)
	{
		child=*it;		
		
		if(child->Label==predid)
		{				
			gfnode=child->SearchEx(gfid);

			if(gfnode)
			{
				gf_found=true;			
				gflist=gfnode->ValueList;
				//for(vector<short>::iterator t=child->ValueList.begin();	t!=child->ValueList.end();t++)
				//	pred_value.push_back(*t);			
			}
		}
		else
		{
			vlist.push_back(child->Label);
		}
	}
	
	//new code
	vector<short> * extracted=GetSetIntersection(&SubCatFrameList,&vlist);
		if (gf_found)
		{
			if(!IsSubSet(&gflist,extracted))
			{
				//by tahira for tracking error
				
				#ifdef GenLog
				CLogManager::AddMessage(L"Problem at "+CSymbolTableManager::GetString(item->Label)+L" Node...");
				CLogManager::AddMessage(L"GF contains:");

				for(int x=0;x<gflist.size();x++)
                    CLogManager::AddMessage(CSymbolTableManager::GetString(gflist[x]));
				#endif

				//////////////////////////////
				
				if(extracted)
				{
					//by tahira for tracking error
					#ifdef GenLog
					CLogManager::AddMessage(L"Found in FS:");
					for(int x=0;x<extracted->size();x++)
						CLogManager::AddMessage(CSymbolTableManager::GetString((*extracted)[x]));
					#endif
					//////////////////////////////

					extracted->clear();			
					try
					{
						delete extracted;
						extracted=NULL;
					}
					catch(...)
					{
						#ifdef GenLog
						CLogManager::AddMessage(L"#### extracted");
						#endif
					}
				}
				//Completeness Test Failed					
				return false;
			}

			if(!IsSubSet(extracted,&gflist))
			{
				//by tahira for tracking error
				#ifdef GenLog
				CLogManager::AddMessage(L"Problem at "+CSymbolTableManager::GetString(item->Label)+L" Node...");
				CLogManager::AddMessage(L"GF contains:");
				for(int x=0;x<gflist.size();x++)
                    CLogManager::AddMessage(CSymbolTableManager::GetString(gflist[x]));
				#endif
				//////////////////////////////
				if(extracted)
				{
					//by tahira for tracking error
					#ifdef GenLog
					CLogManager::AddMessage(L"Found in FS:");
					for(int x=0;x<extracted->size();x++)
						CLogManager::AddMessage(CSymbolTableManager::GetString((*extracted)[x]));
					#endif
					//////////////////////////////
					extracted->clear();
				
					try
					{
						delete extracted;
						extracted=NULL;
					}
					catch(...)
					{
						#ifdef GenLog
						CLogManager::AddMessage(L"#### extracted");
						#endif
					}
				}
				//Coherence Test Failed				
				return false;
			}
		}
		else if (extracted)
		{
			//by tahira for tracking error
			#ifdef GenLog
			CLogManager::AddMessage(L"Problem at"+CSymbolTableManager::GetString(item->Label)+L"Node...");
			for(int x=0;x<extracted->size();x++)
				CLogManager::AddMessage(CSymbolTableManager::GetString((*extracted)[x]));
			#endif
			//////////////////////////////
				extracted->clear();			
				try
				{
					delete extracted;
					extracted=NULL;
				}
				catch(...)
				{
					#ifdef GenLog
					CLogManager::AddMessage(L"#### extracted");
					#endif
				}
				return false;
		}

/*
	if(gf_found)
	{		
		vector<short> * extracted=GetSetIntersection(&SubCatFrameList,&vlist);

		if(!IsSubSet(&gflist,extracted))
		{
			if(extracted)
			{
				extracted->clear();			
				try
				{
					delete extracted;
					extracted=NULL;
				}
				catch(...)
				{
					CLogManager::AddMessage(L"#### extracted");
				}
			}
			//Completeness Test Failed					
			return false;
		}

		if(!IsSubSet(extracted,&gflist))
		{
			if(extracted)
			{
				extracted->clear();
				
				try
				{
					delete extracted;
					extracted=NULL;
				}
				catch(...)
				{
					CLogManager::AddMessage(L"#### extracted");
				}
			}
			//Coherence Test Failed				
			return false;
		}			
	}// if gf found
		
*/
	CFStructureItem*temp=NULL;
	for(vector<CFStructureItem*>::iterator it2=item->ChildList.begin();
		it2!=item->ChildList.end();it2++)
		{
			temp=*it2;
			if(temp->Label!=predid && temp->ChildList.size()!=0)
			{
				if(!CheckForwardCompleteness(temp))
					return false;
			}
		}
	
	return true;
}

/* REMOVED IN THIS (DOS/LINUX) VERSION
void CChart::Show_C_F_Structure(CTreeCtrl *fs_TreeCtrl, CTreeCtrl *cs_TreeCtrl)
{

	using namespace std;

	if(!fs_TreeCtrl || !cs_TreeCtrl)
		return;
	
	CChartItem*citem=NULL;
	
	short last_index=ChartSize-1;
	short csize=Chart[last_index].ItemVector.size();
	
	if(csize<=0)
	{		
		return;
	}				

	HTREEITEM hitem_c;
	HTREEITEM hitem_f;

	wstring fstr=L"F-Structure # ";
	wstring cstr=L"C-Structure # ";
	wstring str;
	
	short counter=1;
	wstring counter_str;
	CFStructureItem*nroot=NULL;
	short constraint_len=0;
	wstring constraint;
	bool passed=false;
	bool done=false;

	FSResults.clear();	
	
	for(vector<CChartItem*>::iterator it=Chart[last_index].ItemVector.begin();
		it!=Chart[last_index].ItemVector.end();it++)
		{			
			citem=*it;
			
			if(citem->Prod->LHS->Label==L"S" && citem->Prod->IsComplete())
			{																						
				nroot=citem->Prod->LHS->FSPtr->Root;
				passed=true;

				for(vector<wstring>::iterator it=nroot->ConstraintList.begin();
					it!=nroot->ConstraintList.end();it++)
				{
					constraint=*it;
					if(!EvaluateExpression(constraint,nroot,nroot))
					{				
						passed=false;
						break;
					}					
				}

				if(passed && CheckForwardCompleteness(nroot))
				{			
					if(SaveFStructure(nroot))
					{
						counter_str=GetWSTR(counter);
						
						str=fstr;
						str+=counter_str;					
						hitem_f=fs_TreeCtrl->InsertItem(str.c_str());
						ShowFStructureRec(nroot,fs_TreeCtrl,hitem_f,true);
						fs_TreeCtrl->Expand(hitem_f,TVE_EXPAND);			
						
						str=cstr;
						str+=counter_str;					
						hitem_c=cs_TreeCtrl->InsertItem(str.c_str());
						ShowCStructureRec(citem->Prod,cs_TreeCtrl,hitem_c,true);
						cs_TreeCtrl->Expand(hitem_c,TVE_EXPAND);											
						
						counter++;
						
						done=true;
					}
				}		

			}

		}		  

	if(!done)
		MessageBox(NULL,L"Sentence could not be parsed",L"Language Parser",MB_ICONINFORMATION);	

}
REMOVED IN THIS (DOS/LINUX) VERSION */

/* REMOVED IN THIS (DOS/LINUX) VERSION
vector<CFStructure*> * CChart::DisplayAndRetrieveFStructures(CTreeCtrl *fs_TreeCtrl)
{

	using namespace std;

	if(!fs_TreeCtrl)
		return NULL;
	
	CChartItem*citem=NULL;
	
	short last_index=ChartSize-1;
	short csize=Chart[last_index].ItemVector.size();
	
	if(csize<=0)
	{		
		return NULL;
	}				
	
	HTREEITEM hitem_f;

	wstring fstr=L"F-Structure # ";
	wstring str;
	
	short counter=1;
	wstring counter_str;
	
	CFStructureItem*nroot=NULL;
	short constraint_len=0;
	wstring constraint;
	bool passed=false;
	bool done=false;

	FSResults.clear();	
	
	for(vector<CChartItem*>::iterator it=Chart[last_index].ItemVector.begin();
		it!=Chart[last_index].ItemVector.end();it++)
		{			
			citem=*it;
			
			if(citem->Prod->LHS->Label==L"S" && citem->Prod->IsComplete())
			{																						
				nroot=citem->Prod->LHS->FSPtr->Root;
				passed=true;

				for(vector<wstring>::iterator it=nroot->ConstraintList.begin();
					it!=nroot->ConstraintList.end();it++)
				{
					constraint=*it;
					if(!EvaluateExpression(constraint,nroot,nroot))
					{				
						passed=false;
						break;
					}					
				}

				if(passed && CheckForwardCompleteness(nroot))
				{			
					if(SaveFStructure(nroot))
					{
						counter_str=GetWSTR(counter);
						
						str=fstr;
						str+=counter_str;					
						hitem_f=fs_TreeCtrl->InsertItem(str.c_str());
						ShowFStructureRec(nroot,fs_TreeCtrl,hitem_f,true);
						fs_TreeCtrl->Expand(hitem_f,TVE_EXPAND);															
						
						counter++;
						
						done=true;
					}
				}		

			}

		}		  

	if(!done)
		MessageBox(NULL,L"Sentence could not be parsed",L"Language Parser",MB_ICONINFORMATION);	

	return & FSResults;

}
REMOVED IN THIS (DOS/LINUX) VERSION */

vector<CFStructure*> * CChart::GetSourceFStructures(bool CheckCompletenessCoherence,CMapper*MapperInerface)
{

	using namespace std;
		
	CChartItem*citem=NULL;
	
	short last_index=ChartSize-1;
	short csize=Chart[last_index].ItemVector.size();
	
	if(csize<=0)	
		return NULL;						
	
	CFStructureItem*nroot=NULL;
	short constraint_len=0;
	wstring constraint;
	bool passed=false;
	bool done=false;

	FSResults.clear();	
	
	for(vector<CChartItem*>::iterator it=Chart[last_index].ItemVector.begin();
		it!=Chart[last_index].ItemVector.end();it++)
		{			
			citem=*it;
			
			if(citem->Prod->LHS->Label==L"S" && citem->Prod->IsComplete())
			{		
				if(citem->Prod->LHS->FSPtr)
				{
					nroot=citem->Prod->LHS->FSPtr->Root;
					passed=true;

					//CLogManager::AddMessage(L"Executing Constraints\n");

					for(vector<wstring>::iterator it=nroot->ConstraintList.begin();
						it!=nroot->ConstraintList.end();it++)
					{
						constraint=*it;						
						if(!EvaluateExpression(constraint,nroot,nroot))
						{	
							//CLogManager::AddMessage(constraint+L"-failed\n");
							passed=false;
							break;							
						}					
						//else	CLogManager::AddMessage(constraint+L"-passed\n");
					}

					if(passed)
					{	
						if(MapperInerface)
							MapperInerface->AdjustFStructure(citem->Prod->LHS->FSPtr);
						
						if(CheckCompletenessCoherence)
						{
							if(!CheckForwardCompleteness(nroot))
								continue;
						}

						SaveFStructure(nroot);
					}		
				}

			}

		}		  
	
	return & FSResults;
}

vector<CFStructure*> * CChart::GetGeneratedFStructures(bool TakeCurrentFSList)
{
	using namespace std;

	if(TakeCurrentFSList && FSResults.size()>0)
		return &FSResults;
		
	CChartItem*citem=NULL;
	
	short last_index=ChartSize-1;
	short csize=Chart[last_index].ItemVector.size();
	
	if(csize<=0)
	{		
		return NULL;
	}				
		
	CFStructureItem*nroot=NULL;
	
	wstring constraint;
	bool passed=false;
	bool done=false;

	FSResults.clear();	
	
	for(vector<CChartItem*>::iterator it=Chart[last_index].ItemVector.begin();
		it!=Chart[last_index].ItemVector.end();it++)
		{			
			citem=*it;
			
			if(citem->Prod->LHS->Label==L"S" && citem->Prod->IsComplete())
			{																						
				nroot=citem->Prod->LHS->FSPtr->Root;
				passed=true;

				for(vector<wstring>::iterator it=nroot->ConstraintList.begin();
					it!=nroot->ConstraintList.end();it++)
				{
					constraint=*it;
					if(!EvaluateExpression(constraint,nroot,nroot))
					{				
						passed=false;
						break;
					}					
				}

				if(passed && CheckForwardCompleteness(nroot))
				{			
					SaveFStructure(nroot);
				}		
			}
		}		  	

	if(FSResults.size()==0)
		return NULL;

	return &FSResults;
}

/*
CCStructure * CChart::RetrieveCStructure(CFStructure * FSPtr)
{
	using namespace std;		
	
	if(!FSPtr)
		return NULL;
	
	RetrieveCStructureRec(FS);

	CCStructure*cs=new CCStructure(
	
}*/

/* REMOVED IN THIS (DOS/LINUX) VERSION
void CChart::Generate_C_F_StructureXML(string fs_fname,string cs_fname)
{
	using namespace std;		
	
	CChartItem*citem=NULL;
	
	short last_index=ChartSize-1;
	short csize=Chart[last_index].ItemVector.size();
	
	if(csize<=0)
	{				
		return;
	}					

	wstring fstr=L"F-Structure-";
	wstring cstr=L"C-Structure-";
	wstring str;
	
	short counter=1;
	wstring counter_str;
	CFStructureItem*nroot=NULL;
	short constraint_len=0;
	wstring constraint;
	bool passed=false;
	bool done=false;

	FSResults.clear();	

	CFileWriter fs_fw(fs_fname);
	CFileWriter cs_fw(cs_fname);
	
	cs_fw.Write(L"<C-Structures>");	
	fs_fw.Write(L"<F-Structures>");	
	
	for(vector<CChartItem*>::iterator it=Chart[last_index].ItemVector.begin();
		it!=Chart[last_index].ItemVector.end();it++)
		{			
			citem=*it;
			
			if(citem->Prod->LHS->Label==L"S" && 
			   citem->Prod->IsComplete() && citem->Prod->LHS->FSPtr)
			{																						
				nroot=citem->Prod->LHS->FSPtr->Root;
				passed=true;

				for(vector<wstring>::iterator it=nroot->ConstraintList.begin();
					it!=nroot->ConstraintList.end();it++)
				{
					constraint=*it;
					if(!EvaluateExpression(constraint,nroot,nroot))
					{					
						passed=false;
						break;
					}					
				}

				if(passed && CheckForwardCompleteness(nroot))
				{			
					if(SaveFStructure(nroot))
					{
						counter_str=GetWSTR(counter);
						
						str=fstr;
						str+=counter_str;					
												
						fs_fw.Write(L"<"+str+L">");																	
						GenerateFStructureRecXML(nroot,&fs_fw);																																					
						fs_fw.Write(L"</"+str+L">");						
						
						str=cstr;
						str+=counter_str;					
						
						cs_fw.Write(L"<"+str+L">");	
						cs_fw.Write(L"<S>");	
						GenerateCStructureRecXML(citem->Prod,&cs_fw);
						cs_fw.Write(L"</S>");		
						cs_fw.Write(L"</"+str+L">");	
												
						counter++;
						
						done=true;
					}
				}		

			}

		}		  

		fs_fw.Write(L"</F-Structures>");	
		fs_fw.CloseFile();

		cs_fw.Write(L"</C-Structures>");	
		cs_fw.CloseFile();		
}
REMOVED IN THIS (DOS/LINUX) VERSION */

bool CChart::SaveFStructure(CFStructureItem *nroot)
{
	using namespace std;

	CFStructure*fs=NULL;

	if(FSResults.empty())
	{
		fs=new CFStructure(nroot->GetClone(),1.0);
		
		if(!fs)
			return false;

		fs->SetParentPointers();
		FSResults.push_back(fs);
		
		return true;
	}

	CFStructure*temp=NULL;

	for(vector<CFStructure*>::iterator it=FSResults.begin();it!=FSResults.end();it++)
	{
		temp=*it;	
		if(temp->Root->IsEqual(nroot))
			return false;
	}

	fs=new CFStructure(nroot->GetClone(),1.0);

	if(!fs)
		return false;

	fs->SetParentPointers();
		
	FSResults.push_back(fs);

	return true;
}

bool CChart::FS_ExistsInList(list<CFSCSPairInfo*> & fscs_list,CFStructureItem* fs)
{
	using namespace std;

	if(!fs)
		return false; 
	
	if(fscs_list.empty())
		return false;

	CFSCSPairInfo*tinfo=NULL;
	
	for(list<CFSCSPairInfo*>::iterator it=fscs_list.begin();it!=fscs_list.end();it++)
	{
		tinfo=*it;		
		if(tinfo->FSRef->Root->IsEqual(fs))
			return true;
	}
		
	return false;
}

CChart::~CChart()
{
	using namespace std;

	if(Chart)	
	{
		#ifdef GenLog
		CLogManager::AddMessage(L"Deleting Chart Array...\r\n");
		#endif
		
		try
		{
			delete []Chart;
			Chart=NULL;
		}
		catch(...)
		{
			#ifdef GenLog
			CLogManager::AddMessage(L"### Exception in Deleting Chart Array...\r\n");
			#endif

			Chart=NULL;
		}

		#ifdef GenLog
		CLogManager::AddMessage(L"Deleted Chart Array...\r\n");
		#endif
	}			
	
	SubCatFrameList.clear();
	FSResults.clear();		

	#ifdef GenLog
	CLogManager::AddMessage(L"Erasing Vectors...\r\n");
	CLogManager::AddMessage(L"Vectors Erased...\r\n");
	CLogManager::AddMessage(L"Successfull Exit...\r\n");
	#endif
	
}

bool CChart::LoadSubCatFrameList(string &fname)
{
	short id=0;

	CFileReader fr(256);

	wstring all=fr.ReadAll(fname);
	
	fr.CloseFile();

	vector<wstring> * vlist = CSentence::GetLexemeList(all,L",",false);	

	if(!vlist)
		return false;
	
	if(vlist->empty())
	{
		try{
		delete vlist;
		}
		catch(...)
		{
			#ifdef GenLog
			CLogManager::AddMessage(L"#### vlist");
			#endif
		}

		return false;
	}

	for(vector<wstring>::iterator it=vlist->begin();it!=vlist->end();it++)
	{		
		id=CSymbolTableManager::GetNum(*it,true);
		SubCatFrameList.push_back(id);
	}

	if(vlist)
	{
		try{
		vlist->clear(); /*ADDED IN THIS (LINUX) VERSION; MAY NOT BE NECESSARY*/
		delete vlist;
		vlist=NULL;
		}
		catch(...)
		{
			#ifdef GenLog
			CLogManager::AddMessage(L"#### vlist");
			#endif
		}

	}
	
	return true;
}

bool CChart::ClearChart()
{
	using namespace std;

	#ifdef GenLog
	CLogManager::AddMessage(L"In Func Clear Chart .....\r\n");
	#endif

	if(Chart)	
	{
		try
		{
			delete []Chart;
			Chart=NULL;
		}
		catch(...)
		{
			#ifdef GenLog
			CLogManager::AddMessage(L"##### Exception Caught(Del []Chart) .....\r\n");	
			#endif

			Chart=NULL;
		}

	}							

	#ifdef GenLog
	CLogManager::AddMessage(L"Out Func Clear Chart .....\r\n");
	#endif

	return true;
}

void CChart::Generate_C_F_StructureXMLEx(string fs_fname, string cs_fname)
{

}

bool CChart::CheckLexConstraints(CFStructureItem*nroot)
{
	wstring constraint;
	CFStructureItem * fsitem;
	
	for(vector<CFStructureItem *>::iterator it=nroot->ChildList.begin();
		it!=nroot->ChildList.end();it++)
	{
		fsitem = *it;
		for(vector<wstring>::iterator cit=fsitem->ConstraintList.begin();
			cit!=fsitem->ConstraintList.end();cit++)
		{
			constraint=*cit;						
			if(!EvaluateExpression(constraint,nroot,nroot))
			{	
				#ifdef GenLog
				CLogManager::AddMessage(L"Constraint : "+constraint+L" - failed\r\n");
				#endif
				return false;
			}
		}
		if (!CheckLexConstraints(fsitem))
			return false;
	}
					
	return true;
}

/* REMOVING FOR THIS (DOS/LINUX) VERSION
bool CChart::GenerateCFSXML(list<CFSCSPairInfo*> & SourceInfoList , const string & XMLFileName)
{
	if(	XMLFileName=="" || 
		SourceInfoList.size()==0)
		return false;

	CFileWriter fw(XMLFileName.c_str());

	wstring output_str=L"";
	wstring tag_str=L"";
	int counter=1;

	output_str=L"<C-Structures>";
	fw.Write(output_str);
	
	CFSCSPairInfo*sinfo=NULL;
	CCStructureItem*root=NULL;

	for(list<CFSCSPairInfo*>::iterator it=SourceInfoList.begin();
		it!=SourceInfoList.end();it++)
	{	
		sinfo=*it;

		tag_str=L"C-Structure-";
		tag_str+=GetWSTR(counter);
				
		fw.Write(L"<"+tag_str+L">");					
	
		root=sinfo->CSRef->Root;
		
		GenerateCStructuresXMLRec(root,fw);
		
		fw.Write(L"</"+tag_str+L">");
						
		counter++;				
	}
	
	output_str=L"</C-Structures>";
	fw.Write(output_str);

	return false;
}
REMOVING FOR THIS (DOS/LINUX) VERSION */

/* REMOVING FOR THIS (DOS/LINUX) VERSION
bool CChart::GenerateCStructuresXMLRec(CCStructureItem* CSPtr,
									   CFileWriter & FilePtr)
{
	if(!CSPtr)
		return false;
	
	FilePtr.Write(L"<"+CSPtr->Label+L">");
	wstring tstr=L"";
	
	if(CSPtr->ChildList.size()==0)
	{						
		tstr=MakeXMLCompatible(CSPtr->Value);
		FilePtr.Write(tstr);		
	}
	else
	{
		for(vector<CCStructureItem*>::iterator it=CSPtr->ChildList.begin();
			it!=CSPtr->ChildList.end();it++)
		{
			GenerateCStructuresXMLRec(*it,FilePtr);
		}
	}

	FilePtr.Write(L"</"+CSPtr->Label+L">");

return true;
}
REMOVING FOR THIS (DOS/LINUX) VERSION */