#include "stdafx.h"
#pragma warning(disable: 4786)


#ifndef CFSTRUCTUREITEM_H_HEADER_INCLUDED_BF6700FB
#define CFSTRUCTUREITEM_H_HEADER_INCLUDED_BF6700FB

#include <string>
#include <vector>
#include <map>
#include <stack>
#include <list>

//using std::string;
//using std::wstring;
//using std::vector;

using namespace std;


#include "CFDescription.h"
#include "CLexicon.h"
#include "global.h"


class CFStructureItem;

typedef vector< CFStructureItem* > FSItemList;

typedef pair<CFStructureItem*,CFStructureItem*> fs_pair;

//##ModelId=40626F3F0103
//##Documentation
//## It is the data structure that contains the information of node in a
//## F-Structure
class CFStructureItem
{

public:

	//added by nayyara
	bool IsVisited;
	static int counter;
	static int LastNodeId;

	vector<short> ValueList;	

	wstring GetValueString(CLexicon * lex);
	
	CFStructureItem* SearchItemInChilds(short id);
	bool eraseChild(short id);
	void AddElementInSet(CFStructureItem* fsitem);

	CFStructureItem* GetPurifiedForm(CFStructureItem*);
	
	CFStructureItem * SearchInSet(short id);
	CFStructureItem * SearchEx(short id);
	static CFStructureItem* GetNewFSItem(CFDescription*fdesc,bool assign_value=true);	
	
	bool insertCommon(hash_map<wstring,list<CFDescription*>*>* common);

	bool UpdateGenRefInfo(map<CFStructureItem*,CFStructureItem*> & MapTable);
	
	CFStructureItem * SearchAndCreate(vector<short>* vlist,char & created);

	CFStructureItem * GetRoot()
	{
		CFStructureItem * root=this;
		
		while(root->Parent)
			root=root->Parent;
		
		return root;
	}


	CFStructureItem * SearchAndCreate(short val,char & created)
	{
		vector<short> vlist;
		vlist.push_back(val);
		
		return SearchAndCreate(&vlist,created);
	}

	CFStructureItem * SearchInSet(vector<short>* vlist);
	CFStructureItem * Search(vector<short>* vlist,bool search_always = true);

	void AppendFSItem(CFStructureItem*fsitem,bool AddToSibling);
	
	//CFDescription * FDesc;   
	short Label;
	
	
    vector<wstring> ConstraintList;    
    vector<CFStructureItem*> ChildList;    
    vector<CFStructureItem*> SetElements;
	//map<CFStructureItem*,
	list<CFStructureItem*> LinkInfoList;	       
	
	//////////// New Additions ///////////////

	int ItemNo;	
	int Reference;	//Long Distance Dependency Pointer	
	bool IsReferred;		
	
	bool IsReferredForGen;
	CFStructureItem*GenRef;

	bool isMarked;
	bool isLinked;

	bool SaveInGC(CFStructureItem * node)
	{
		try
		{
			GarbageCollector.push_back(node);
		}
		catch(...)
		{
			#ifdef GenLog
			CLogManager::AddMessage(L"Error on adding F-Structure Item in GC");
			#endif

			return false;
		}
		
		return true;
	}
	////////////////////////////////////////
    
	CFStructureItem()
	{
		isMarked=false;
		isLinked=false;
		IsReferred=false;
		IsVisited = false;

		IsReferredForGen=false;
		GenRef=NULL;

		counter++;
		Label=-1;
		Parent=NULL;		
		Reference=0;
		ItemNo=++LastNodeId;

		SaveInGC(this);

	}

	CFStructureItem(short id,CFStructureItem * p=NULL)
	{
		isMarked=false;
		isLinked=false;
		IsReferred=false;
		IsVisited = false;

		counter++;
		Label=id;
		Parent=p;		
		Reference=0;
		ItemNo=++LastNodeId;

		IsReferredForGen=false;
		GenRef=NULL;

		SaveInGC(this);
	}

	void SetGenReference(CFStructureItem*Ref=NULL)
	{
		if(Ref)
			IsReferredForGen=true;
		else
			IsReferredForGen=false;

		//Get the top reference...if there is some
		//heirarchy of of references exists
		while(Ref && Ref->GenRef)
		{
			if (Ref == Ref->GenRef)
				break;
			Ref = Ref->GenRef;
		}

		GenRef=Ref;
	
		//un-comment for bi-direction gen linking...
		/*if(Ref)
		{
			Ref->IsReferredForGen = true;
			Ref->GenRef=this;
		}*/
	}

	static void ResetNodeId()
	{
		LastNodeId=0;
	}

	~CFStructureItem()
	{	
		Label=-1;		
		counter--;		
		Parent=NULL;		
	}

	static bool DeleteListFStructures(vector<CFStructureItem*>*fslist)
	{
		using namespace std;

		try
		{

			if(!fslist||fslist->empty())
				return false;

			CFStructureItem*temp;
			for(vector<CFStructureItem*>::iterator it=fslist->begin();it!=fslist->end();it++)
			{
				temp=*it;
				if(temp && temp->Label>0)
				{
					try{
					delete temp;
					}
		catch(...)
		{
			#ifdef GenLog
			CLogManager::AddMessage(L"#### ");
			#endif
		}

				}
			}

			fslist->clear();
		}
		catch(...) /* CHANGING FOR THIS (DOS/LINUX) VERSION */
		{
			/*exp->Delete(); REMOVING FOR THIS (DOS/LINUX VERSION */
			return false;
		}
		
		return true;
	}

	CFStructureItem*GetClone()
	{
		using namespace std;		

		CFStructureItem*temp=new CFStructureItem();

		if(!temp)
			return NULL;
						
		temp->Label = Label;
		temp->ValueList=ValueList;
		
		temp->IsReferred =IsReferred;
		temp->Reference=Reference;
		temp->ItemNo=ItemNo;

		temp->isMarked=isMarked;
		temp->isLinked=isLinked;
		temp->IsVisited=IsVisited;

		CFStructureItem*child=NULL;
		
		for(vector<CFStructureItem*>::iterator it1=ChildList.begin(); it1!=ChildList.end();it1++)
		{
			child=(*it1)->GetClone();
			child->Parent = temp;
			temp->ChildList.push_back(child);
		}

		for(vector<wstring>::iterator it2=ConstraintList.begin(); it2!=ConstraintList.end();it2++)		
			temp->ConstraintList.push_back(*it2);		

		temp->Parent=Parent;
		//temp->Parent=NULL;
		CFStructureItem * fsi=NULL;
		CFStructureItem * fsi_copy=NULL;
//		CFStructureItem * citem=NULL;

		for(vector<CFStructureItem*>::iterator it=SetElements.begin();
			it!=SetElements.end();it++)
		{
			fsi=*it;			
			if(fsi)
			{
				fsi_copy=fsi->GetClone();				
				temp->SetElements.push_back(fsi_copy);				
			}
		}
		
		////////////////////////////////////////////
		///added recently
		////////////////////////////////////////////
		
		temp->IsReferredForGen=IsReferredForGen;
		temp->GenRef=NULL;
		if(GenRef && temp->IsReferredForGen)
		{
			CFStructureItem*ref=GenRef;		
			while(ref->GenRef && ref!=ref->GenRef)
				ref=ref->GenRef;
			
			//temp->GenRef=ref;			
			//changed on 06.08.2005
			if(ref!=ref->GenRef)
				temp->GenRef=ref->GetClone();
			else
				temp->GenRef=NULL;			
		}
		
		////////////////////////////////////////////
										
		return temp;	
	}

	void ResolveReferencesForGenerator(map<CFStructureItem*,CFStructureItem*> & MapTable)
	{
		map<CFStructureItem*,CFStructureItem*>::iterator find;

		if(this->IsReferredForGen)
		{
			find=MapTable.find(this->GenRef);
			
			if(find!=MapTable.end())
				this->GenRef=find->second;						
			else
			{	
				#ifdef GenLog
				CLogManager::AddMessage(L"Generator Reference is not found. \r\n");			
				#endif
				//IsReferredForGen=false;
			}
			
		}

		for(vector<CFStructureItem*>::iterator it1=ChildList.begin(); it1!=ChildList.end();it1++)
		{
			(*it1)->ResolveReferencesForGenerator(MapTable);			
		}		
	
	}

	CFStructureItem * GetCloneWithTable(map<CFStructureItem*,CFStructureItem*> & MapTable)
	{
		using namespace std;		
	
		map<CFStructureItem*,CFStructureItem*>::iterator find_it=MapTable.find(this);

		if (find_it!=MapTable.end())
			return find_it->second;
		
		CFStructureItem*temp=new CFStructureItem(Label);

		MapTable.insert(fs_pair(this,temp));

		if(!temp)
			return NULL;
						
		temp->Label = Label;
		temp->ValueList.clear();
		//temp->ValueList.resize(ValueList.size());
		temp->ValueList = ValueList;
		
		temp->IsReferred =IsReferred;
		temp->Reference=Reference;
		temp->ItemNo=ItemNo;
		temp->isMarked=isMarked;
		temp->isLinked=isLinked;
		
		temp->IsReferredForGen=IsReferredForGen;
		temp->GenRef=GenRef;
		
		CFStructureItem*child=NULL;
		
		for(vector<CFStructureItem*>::iterator it1=ChildList.begin(); it1!=ChildList.end();it1++)
		{
			child=(*it1)->GetCloneWithTable(MapTable);
			child->Parent = temp;
			temp->ChildList.push_back(child);
		}		

		temp->LinkInfoList=LinkInfoList;

		for(vector<wstring>::iterator it2=ConstraintList.begin(); it2!=ConstraintList.end();it2++)		
			temp->ConstraintList.push_back(*it2);		

		//temp->Parent=Parent;
		temp->Parent=NULL;
		CFStructureItem * fsi=NULL;
		CFStructureItem * fsi_copy=NULL;
//		CFStructureItem * citem=NULL;

		for(vector<CFStructureItem*>::iterator it=SetElements.begin();
			it!=SetElements.end();it++)
		{
			fsi=*it;			
			if(fsi)
			{
				fsi_copy=fsi->GetCloneWithTable(MapTable);
				temp->SetElements.push_back(fsi_copy);				
			}
		}

		//temp->ResolveReferencesForGenerator(MapTable);
										
		return temp;
	}

	CFStructureItem*GetCloneEx(CFStructureItem * oldptr,CFStructureItem * * newptr)
	{
		using namespace std;
						
		CFStructureItem * temp=new CFStructureItem();

		if(!temp)
			return NULL;

		if(this==oldptr)
			*newptr=temp;
						
		temp->Label = Label;
		temp->ValueList= ValueList;

		temp->IsReferred =IsReferred;
		temp->Reference =Reference;
		temp->ItemNo=ItemNo;

		temp->isMarked=isMarked;
		temp->isLinked=isLinked;
		
		CFStructureItem*child=NULL;
		
		for(vector<CFStructureItem*>::iterator it1=ChildList.begin(); it1!=ChildList.end();it1++)
		{
			child=(*it1)->GetCloneEx(oldptr,newptr);
			child->Parent = temp;
			temp->ChildList.push_back(child);
		}

		for(vector<wstring>::iterator it2=ConstraintList.begin(); it2!=ConstraintList.end();it2++)		
			temp->ConstraintList.push_back(*it2);		

		//temp->Parent=Parent;
		CFStructureItem * fsi=NULL;
		CFStructureItem * fsi_copy=NULL;
//		CFStructureItem * citem=NULL;

		for(vector<CFStructureItem*>::iterator it=SetElements.begin();
			it!=SetElements.end();it++)
		{
			fsi=*it;			
			if(fsi)
			{
				fsi_copy=fsi->GetCloneEx(oldptr,newptr);				
				temp->SetElements.push_back(fsi_copy);				
			}
		}
										
		return temp;
	}


	CFStructureItem*GetCloneEx2(CFStructureItem * oldptr,
							    CFStructureItem * * newptr,
								stack<CFStructureItem*> * old_stk,		
								stack<CFStructureItem*> * new_stk
							   )
	{
		using namespace std;
						
		CFStructureItem * temp=new CFStructureItem();

		if(!temp)
			return NULL;

		if(this==oldptr)
			*newptr=temp;

		if(!old_stk->empty() && old_stk->top()==this)
		{
			old_stk->pop();
			new_stk->push(temp);
		}
						
		temp->Label = Label;
		temp->ValueList= ValueList;

		temp->IsReferred =IsReferred;
		temp->Reference =Reference;
		temp->ItemNo=ItemNo;

		temp->isMarked=isMarked;
		temp->isLinked=isLinked;
		
		CFStructureItem*child=NULL;
		
		for(vector<CFStructureItem*>::iterator it1=ChildList.begin(); it1!=ChildList.end();it1++)
		{
			child=(*it1)->GetCloneEx2(oldptr,newptr,old_stk,new_stk);
			child->Parent = temp;
			temp->ChildList.push_back(child);
		}

		for(vector<wstring>::iterator it2=ConstraintList.begin(); it2!=ConstraintList.end();it2++)		
			temp->ConstraintList.push_back(*it2);		

		//temp->Parent=Parent;
		CFStructureItem * fsi=NULL;
		CFStructureItem * fsi_copy=NULL;
	
		for(vector<CFStructureItem*>::iterator it=SetElements.begin();
			it!=SetElements.end();it++)
		{
			fsi=*it;			
			if(fsi)
			{
				fsi_copy=fsi->GetCloneEx2(oldptr,newptr,old_stk,new_stk);				
				temp->SetElements.push_back(fsi_copy);				
			}
		}
										
		return temp;
	}
	
	CFStructureItem * GetCloneSpecial(CFStructureItem*old)
	{
		using namespace std;		

		CFStructureItem * gparent=old;

		while(gparent->Parent!=NULL)
			gparent=gparent->Parent;

		CFStructureItem * newptr=NULL;
						
		gparent->GetCloneEx(old,&newptr);

		return newptr;
	}

	bool IsEqual(CFStructureItem*temp)
	{
		using namespace std;		

		if(!temp)		
			return false;
		
		if(temp->Label!= Label)
			return false;

		unsigned int len1=ValueList.size();
		unsigned int len2=temp->ValueList.size();

		if(len1!=len2)
			return false;

		unsigned short p=0;

		if(!IsSubSet(&temp->ValueList,&ValueList)||
		   !IsSubSet(&ValueList,&temp->ValueList))
		{		
			//new code
			wstring str = CSymbolTableManager::GetString(Label);
			if (str[0] != '_')
				return false;
		}

		len1=ChildList.size();
		len2=temp->ChildList.size();

		if(len1!=len2)
			return false;		

		CFStructureItem*t=NULL;
		CFStructureItem*found=NULL;
		
		for(vector<CFStructureItem*>::iterator it1=ChildList.begin();it1!=ChildList.end();it1++)
		{
			t=*it1;
			found=temp->SearchEx(t->Label);
			
			if(!found){
				//new code
				wstring str = CSymbolTableManager::GetString(Label);
				if (str[0] != '_')
					return false;

			}
			if(!t->IsEqual(found))
				return false;				
		}
		

		len1=ConstraintList.size();
		len2=temp->ConstraintList.size();

		if(len1!=len2)
			return false;		
		
		for(p=0;p<len1;p++)
		{
			if(ConstraintList.at(p)!=ConstraintList.at(p))
				return false;		
		}		

		/*len1=SetElements.size();
		len2=temp->SetElements.size();

		if(len1!=len2)
			return false;				
		
		for(vector<CFStructureItem*>::iterator it2=SetElements.begin();it2!=SetElements.end();it2++)
		{
			t=*it2;
			found=temp->SearchInSet(t->Label);
			
			if(!found)
				return false;

			if(!t->IsEqual(found))
				return false;				
		}*/
												
		return true;
	}

	CFStructureItem* GetReferencedNode(short NodeId);
	CFStructureItem* GetReferencedNodeRec(short NodeId);
	vector<CFStructureItem*> GetLinkedFSItems(CFStructureItem* FSPtr);
	bool AddNewLink(CFStructureItem* LinkedFSPtr);
	bool RemoveLink(CFStructureItem* LinkedFSPtr);
	bool CheckAndUpdateLinkedNodes(void);
	bool UpdateLinkedNodeInfo(map<CFStructureItem*,CFStructureItem*> & MapTable);
	bool UpdateLinkedNode(void);

		CFStructureItem * GetParent(bool SkipSpecialNodes=false)
	{				
		CFStructureItem * tparent = this->Parent;
		
		if(!tparent)
			return tparent;

		if(!SkipSpecialNodes)
			return this->Parent;

		//skipping SET and ELEMENT NODES...
		
		if(tparent->Label==CSymbolTableManager::ElementId)
		{
			tparent=tparent->Parent; //parent will be SET
			tparent=tparent->Parent; //parent will be required, XYZ
			return tparent;
		}
		
		if(tparent->Label==CSymbolTableManager::SetId)
		{				
			tparent=tparent->Parent; //parent will be required, XYZ
			return tparent;
		}
		
		return tparent;
	}

	void SetParent(CFStructureItem * p)
	{		
		this->Parent = p;		
	}

	static void CollectGarbage();

private:

	CFStructureItem * Parent;	
	static list<CFStructureItem*> GarbageCollector;
	

};

#define SEPARATOR NULL

#endif /* CFSTRUCTUREITEM_H_HEADER_INCLUDED_BF6700FB */
