
#include "stdafx.h"
#pragma warning(disable: 4786)

#include <iostream>
#include <stdlib.h>
#include "global.h" /*TO SMALL CASE FOR THIS (LINUX) VERSION*/
#include "CGrammar.h"
#include "PermutationMaker.h"
/*#include "Global.h" REMOVED IN THIS (LINUX) VERSION*/
#include "CSymbolTableManager.h"

#include "FileReader.h"
#include "FileWriter.h"

#include "RawProduction.h"
#include "Symbol.h" /*CASE MATCHED TO FILENAME IN THIS (LINUX) VERSION*/

#include "CLexicon.h"
#include "FDExpression.h"

#include <string>
#include <vector>
#include <stack>
#include <map>
/*#include ".\cgrammar.h" REMOVED IN THIS (LINUX) VERSION*/

using std::string;
using std::wstring;
using std::vector;
using std::stack;
using std::map;
using std::multimap;

using namespace std;

typedef pair<wstring,CProduction*> prod_pair;

//typedef vector<CSymbol*> symb_list;
//typedef vector<symbol>

//##ModelId=406125F601A9
CGrammar::CGrammar(const string & grules_fname, 
				   const string & macros_fname,
				   const string & lfg_fname,
				   const string & cfg_fname,
				   CLexicon*Lex,
				   bool ReplaceStarByRange,
				   bool IsGenerationGrammar)
	:LexiconPtr(Lex),ProdTable(500),
	RelevantProductionList(NULL)
{	
		
	PreprocessGrammar(grules_fname,macros_fname,lfg_fname,ReplaceStarByRange); //  1

	NewProdName=L"P_";	

	list<CRawProduction*> *vlist2=NULL;
	list<CRawProduction*> *vlist1;

	vlist1=ReadLFG(lfg_fname);	///2
	
	vlist2=ConvertLFG2CFG(vlist1);  //3	

	map<wstring,bool>* NULLS = Comp_Dir_Nullables( vlist2 ); // 4

	if( NULLS )
	{
		NULLS = Comp_Nullables( vlist2 , NULLS); // 5

		GenerateSelectionSet(vlist2,NULLS); // 6

		WriteCFGInFile(vlist2,cfg_fname);	//7

		try{
			delete NULLS;
		}
		catch(...)
		{
			#ifdef GenLog
			CLogManager::AddMessage(L"#### NULL");
			#endif
		}

	}

	//DeleteRawProdList(vlist1);	
	//DeleteRawProdList(vlist2);

	vlist2 = ReadCFG(cfg_fname); // 1a

	ProductionTable=PurifyProductions(vlist2);

	GetInfoFromProdTable();
	
	/*
	if(IsGenerationGrammar)
		WriteProductionsInFileExtended("data\\Urdu\\udebug.txt");
	else
		WriteProductionsInFileExtended("data\\English\\edebug.txt");
	*/

	//DeleteRawProdList(vlist2);

	if(IsGenerationGrammar)
	{
		RelevantProductionList=new deque<set<int>*>(IntSymbTab.GetSymbolCount());
		GenerateRelevantProductionTable();
		GenerateSelectionSetForGenertionGrammar();			
	}
}

CGrammar::~CGrammar()	
{				
	using namespace std;
	
	CProduction*tprod=NULL;

	if(ProductionTable)
	{	
		for(multimap<wstring,CProduction* >::iterator it = ProductionTable->begin(); 
			it!=ProductionTable->end();it++)
		{
			tprod=it->second;
			delete tprod;
		}		

		/*
		for (multimap<wstring,CProduction* >::iterator i = ProductionTable->begin(); 
			i!=ProductionTable->end();)
		{
			multimap<wstring,CProduction*>::iterator j = ProductionTable->upper_bound(i->first);						
			
			if(j!=)
			{
				do
				{			
					tprod=i->second;
					delete tprod;
					tprod=NULL;
					++i;

				} while(i != j && i != ProductionTable->end());						
			}
		}*/
		
		ProductionTable->clear();	
		try{
			delete ProductionTable;	
			ProductionTable=NULL;
		}
		catch(...)
		{
			#ifdef GenLog
			CLogManager::AddMessage(L"Error Deleting Production List #### ");
			#endif
		}
	}

	///////////////////////////////////////////////////
	/*deleting dynamically allocated data*/
	///////////////////////////////////////////////////

	if(RelevantProductionList)
	{
		int index=0;
		set<int> * set_ptr=NULL;
		int size=RelevantProductionList->size();

		for(index=0;index<size;index++)
		{
			set_ptr=RelevantProductionList->at(index);
			if(set_ptr)
			{
				set_ptr->clear();
				delete set_ptr;
			}
		}
		
		RelevantProductionList->clear();
		delete RelevantProductionList;
	}

	///////////////////////////////////////////////////
}

list<CRawProduction*>* CGrammar::GenerateSelectionSet(list<CRawProduction*> * vlist , map<wstring,bool>* nullables)
{

	using namespace std;
	
	/**************** GENERATION OF FIRST AND FOLLOW SETS *************************/
	bool SetChanged = false;
	
	typedef map<wstring,int>* SMapPtr;

	map< wstring,SMapPtr > FirstSet;
	map< wstring,SMapPtr >  FollowSet;

	map<wstring,int> * CurrFirst = NULL;
	map<wstring,int> * CurrFollow = NULL;

	typedef pair <wstring,int> StrIntPair;
	typedef pair <wstring,map<wstring,int>*> StrMapPair;
	do
	{
		SetChanged = false;
		/* CHANGED IN THIS (LINUX) VERSION
		vector<CSymbol*>::iterator RHS_it = NULL, End_Cond = NULL;
		*/
		vector<CSymbol*>::iterator RHS_it, End_Cond;
		list<CRawProduction*>::iterator prod_it;

		for( prod_it = vlist->begin() ; prod_it != vlist->end() ; prod_it++ )//each prduction x->y1 y2 ... yk )
		{
			CRawProduction * CurrProd = *prod_it;
			vector<CSymbol*>* RHS = &CurrProd->RHS; //RHS of Current Production

			RHS_it = RHS->begin();
			
			End_Cond = RHS->end();
			End_Cond--;
			

				
			for( ; RHS_it != End_Cond ; RHS_it++ ) //each i from 1 to k, each j from i+1 to k )
			{
				//if( all yi are nullabel ) already done in IsNull
				//{
					//then nullable(x) = true;
				//}

				if( IsNullRange( RHS->begin(), RHS_it, nullables) == true )//y1 ... yi-1 are all nullable )
				{
					//ten First(x) <- First[x] U First[yi];
					
					CSymbol *  Targ_NT = CurrProd->LHS;
					CSymbol *  Src_NT = *RHS_it;
				
					map<wstring,int> * Src_SET = NULL;

					map<wstring,map<wstring,int>*>::iterator find_it = NULL;
				
						
					find_it = FirstSet.find(Targ_NT->lexeme);

					bool new_create = false;
					if( find_it == FirstSet.end() )
					{
						CurrFirst =new map<wstring,int>;
						
						if(!CurrFirst)
							return NULL;

						new_create = true;
					}
					else
						CurrFirst = find_it->second;
			
					if( !(Src_NT->lexeme[0]>='a' && Src_NT->lexeme[0]<='z') ) //if not terminal
					{			
						find_it = FirstSet.find(Src_NT->lexeme);
						if( find_it != FirstSet.end() )
						{
							Src_SET = find_it->second;
							SetChanged |= OperateUnion(Src_SET,CurrFirst); //ten First(x) <- First[x] U First[yi];
						}	
					}
					else //it is a terminal
						CurrFirst->insert(StrIntPair(Src_NT->lexeme,1));
										
					if( new_create ) //no set for this NT existed before
					{
						SetChanged = true;
						FirstSet.insert(StrMapPair(Targ_NT->lexeme, CurrFirst));
					}
				}

				vector<CSymbol*>::iterator Temp_it = RHS_it;
				Temp_it++;
				if( IsNullRange( Temp_it, End_Cond, nullables) == true )//yi+1 ... yk are all nullable )
				{
					//then Follow+(yi) <- follow(yi) U follow(x);
					CSymbol *  Targ_NT = *RHS_it;
					CSymbol *  Src_NT = CurrProd->LHS;
				
					map<wstring,int> * Src_SET = NULL;

					map<wstring,map<wstring,int>*>::iterator find_it = NULL;
			
					if( (Targ_NT->lexeme[0]>='a' && Targ_NT->lexeme[0]<='z') )
						continue; // a terminal cannot have a follow set

					bool new_create = false;

					find_it = FollowSet.find(Targ_NT->lexeme);
					if( find_it != FollowSet.end() )
						CurrFollow = find_it->second;
					else
					{
						CurrFollow = new map<wstring,int>;
						new_create = true;
					}

					find_it = FollowSet.find(Src_NT->lexeme);
					if( find_it != FollowSet.end() )
					{
						Src_SET = find_it->second;
						if( CurrFollow == NULL )
							CurrFollow =new map<wstring,int>;

						if(!CurrFollow)
							return NULL;

						SetChanged |= OperateUnion(Src_SET,CurrFollow); //then Follow+(yi) <- follow(yi) U follow(x);
					}
					
					if( new_create == true )
					{
						SetChanged = true;
						FollowSet.insert( StrMapPair(Targ_NT->lexeme,CurrFollow) );
					}
				}
				
				////////////////////////////////////////////////
				/*vector<CSymbol*>::iterator Cond_it = RHS_it, Cond_it_Next , Union_it;
				Union_it = Cond_it; // j-1 , i
				Union_it++; //i+1
				Cond_it_Next = Union_it; //j

				for(  ; Cond_it!= End_Cond && Cond_it_Next != End_Cond; Cond_it++ , Cond_it_Next++ )
				{

					if( Union_it <= Cond_it && IsNullRange( Union_it , Cond_it, nullables) == true )//yi+1 ... yj-1 are all nullable)
					{
						//then Follow(yi) <- follow(yi) U first(yj)
						CSymbol * Targ_NT = *RHS_it;
						CSymbol * Src_NT = *Cond_it_Next;
						CSymbol * Src_NT1 = *Cond_it;
			
						map<wstring,int> * Src_SET = NULL;

						map<wstring,map<wstring,int>*>::iterator find_it = NULL;
			
						if( (Targ_NT->lexeme[0]>='a' && Targ_NT->lexeme[0]<='z') )
							continue; // a terminal cannot have a follow set
			
						bool new_create = false;
				
						find_it = FollowSet.find(Targ_NT->lexeme);
				
						if( find_it != FollowSet.end() )
							CurrFollow = find_it->second;
						else
						{
							CurrFollow = new map<wstring,int>;
							new_create = true;
						}

						
						if( !(Src_NT->lexeme[0]>='a' && Src_NT->lexeme[0]<='z') )
						{			
							find_it = FirstSet.find(Src_NT->lexeme);
							if( find_it != FirstSet.end() )
							{
								Src_SET = find_it->second;
								if( CurrFollow == NULL )
									CurrFollow =new map<wstring,int>;
	
								SetChanged |= OperateUnion(Src_SET,CurrFollow); //Follow(yi) <- follow(yi) U first(yj)
							}
							else SetChanged = true;
						}
						
						if( new_create )
						{
							SetChanged = true;
							FollowSet.insert( StrMapPair(Targ_NT->lexeme,CurrFollow) );
						}
					}
				}
				//end of nested for for Cond_it*/
				
				CSymbol * Targ_NT = *RHS_it;
				CSymbol * Src_NT = NULL;
				
				if( (Targ_NT->lexeme[0]>='a' && Targ_NT->lexeme[0]<='z') )
						continue; // a terminal cannot have a follow set
				
				map<wstring,int>* Src_Set;
				map<wstring,int>* Targ_Set;
				vector<CSymbol*>::iterator Union_it = RHS_it;
				map<wstring,map<wstring,int>*>::iterator find_it = FollowSet.find(Targ_NT->lexeme);
				bool new_create = false;
				
				if( find_it == FollowSet.end() )
				{
					Targ_Set = new map<wstring,int>;
					new_create = true;
				}
				else
					Targ_Set = find_it->second;

				for( Union_it++ ; Union_it != End_Cond ; Union_it++ )
				{
					Src_NT = *Union_it;
						
					if( !(Src_NT->lexeme[0]>='a' && Src_NT->lexeme[0]<='z') )
					{			
						find_it = FirstSet.find(Src_NT->lexeme);
						if( find_it != FirstSet.end() )
						{
							Src_Set = find_it->second;
							
							SetChanged |= OperateUnion(Src_Set,Targ_Set); //Follow(yi) <- follow(yi) U first(yj)
						}
						else SetChanged = true;
					}
					else
					{
						Targ_Set->insert( StrIntPair(Src_NT->lexeme,1) );
						break; //break in case of terminal
					}

					map<wstring,bool>::iterator find_it = nullables->find(Src_NT->lexeme);
					
					if( find_it == nullables->end() ) //it is not null NT break;
						break;
				}
				
				if( new_create )
				{
					FollowSet.insert( StrMapPair(Targ_NT->lexeme,Targ_Set));
					SetChanged = true;
				}

				new_create = false;

			}
		}//end of for

	}while(SetChanged);

	/* TESTING FIRST AND FOLLOW
	map<wstring,map<wstring,int>*>::iterator FirstIterator = FollowSet.begin();
	map<wstring,map<wstring,int>*>::iterator FollowIterator = FollowSet.begin();

	for( ; FirstIterator != FollowSet.end() ; FirstIterator++ )
	{
		wstring nt_name = FirstIterator->first;
		wstring nt_first = L"";
		
		map<wstring,int> * Cur_nt = FirstIterator->second;
		map<wstring,int>::iterator Cur_nt_it = Cur_nt->begin();

		for ( ; Cur_nt_it != Cur_nt->end() ; Cur_nt_it++ )
			nt_first += Cur_nt_it->first;
		
		MessageBox(NULL,nt_first.c_str(),nt_name.c_str(),MB_ICONINFORMATION);
	}*/

//	MessageBox(NULL, L"First & Follow",L"Complete",MB_ICONINFORMATION);

	typedef pair <short,short> ShShPair;

	list<CRawProduction*>::iterator prod_it;

	for( prod_it = vlist->begin() ; prod_it != vlist->end() ; prod_it++ )//each prduction x->y1 y2 ... yk 
	{
		CRawProduction * CurrProd = *prod_it;
		
		/* CHANGED IN THIS (LINUX) VERSION
		vector<CSymbol*>::iterator RHS_it = NULL, End_Cond = NULL;
		*/
		vector<CSymbol*>::iterator RHS_it, End_Cond;
		//CurrProd->SelectionSet = new map<short,short>;

		vector<CSymbol*>* RHS = &CurrProd->RHS; //RHS of Current Production

		RHS_it = RHS->begin();
		End_Cond = RHS->end();
		End_Cond --; //this is to exclude dot at end of each production
		
		for( ; RHS_it != End_Cond ; RHS_it++ ) //each i from 1 to k, each j from i+1 to k )
		{
			
			CSymbol * CurrRHS = * RHS_it;
			if( CurrRHS->lexeme[0]>='a' && CurrRHS->lexeme[0]<='z' )
			{
				short num = CSymbolTableManager::GetNum(CurrRHS->lexeme,false);

				CurrProd->SelectionSet.insert( ShShPair(num,num) );
				break;
			}
			else
			{
				map<wstring,map<wstring,int>*>::iterator Src_Set_it = FirstSet.find(CurrRHS->lexeme);
				if( FirstSet.end() != Src_Set_it )
				{
					map<wstring,int>* Src_Set = Src_Set_it->second;
					OperateUnion( Src_Set, &CurrProd->SelectionSet );
				}	

				map<wstring,bool>::iterator null_find_it = nullables->find(CurrRHS->lexeme);
				if( null_find_it == nullables->end() )
					break;
			}
		}
		if( RHS_it == End_Cond )
		{
			map<wstring,map<wstring,int>*>::iterator Src_Set_it = FollowSet.find(CurrProd->LHS->lexeme);
			if( Src_Set_it != FollowSet.end() )
			{
				map<wstring,int>* Src_Set = Src_Set_it->second;
				OperateUnion( Src_Set , &CurrProd->SelectionSet );
			}
		}

		/* TESTING SELECTION SETS (OUPUT CODE)
		wstring sel_str = L"";
		for( map<wstring,int>::iterator test_it_s = CurrProd->SelectionSet->begin(); test_it_s != CurrProd->SelectionSet->end() ; test_it_s++ )
		{
			sel_str += test_it_s->first;
			sel_str += L" ";
		}
		
		MessageBox(NULL, sel_str.c_str(), CurrProd->LHS->lexeme.c_str() ,MB_ICONINFORMATION);*/		

	}

	/* OPERATION CLEANUP: GETRID OF FIRST AND FOLLOW SETS */
	
	map<wstring,map<wstring,int>*>::iterator FirstIterator = FirstSet.begin();
		
	for( ; FirstIterator != FirstSet.end() ; FirstIterator++ )
	{
		wstring nt_first = L"";
	
		map<wstring,int> * Cur_nt = FirstIterator->second;	
		
		Cur_nt->clear(); //empty current set;

		try{
		delete Cur_nt;
		}
		catch(...)
		{
			#ifdef GenLog
			CLogManager::AddMessage(L"#### Cur_nt");
			#endif
		}

		Cur_nt=NULL;
	}

	FirstSet.clear();

	FirstIterator = FollowSet.begin();

	for( ; FirstIterator != FollowSet.end() ; FirstIterator++ )
	{
		wstring nt_first = L"";
	
		map<wstring,int> * Cur_nt = FirstIterator->second;	
		
		Cur_nt->clear(); //empty current set;
	
		try{
			delete Cur_nt;
		}
		catch(...)
		{
			#ifdef GenLog
			CLogManager::AddMessage(L"#### Cur_nt");
			#endif
		}

		Cur_nt=NULL;
	}

	FollowSet.clear();

	return vlist;
}

bool CGrammar::OperateUnion( map<wstring,int> * Src_Set , map<wstring,int> * Dest_Set )
{
	typedef pair <wstring,int> StrIntPair;
	bool ret_val = false;

	if(Src_Set->empty() )
	{
		return false; //the source follow set is empty;
	}

	/*else if( CurrFollow == NULL )
	{
		CurrFollow = new map<wstring,int>;
		map<wstring,int>::iterator cp_it = Src_SET->begin();
		while( cp_it != Src_SET->end() )
		{
			wstring te = cp_it->first;
			CurrFollow->insert(StrIntPair(te,1) );
			cp_it++;
		}
	} */
	else 
	{
		map<wstring,int>::iterator cp_it = Src_Set->begin();
		map<wstring,int>::iterator test_it = NULL;

		wstring test_str = L"";
		while( cp_it != Src_Set->end() )
		{
			wstring te = cp_it->first;
			
			test_it = Dest_Set->find(te);
			
			if( test_it == Dest_Set->end() ) //insert only if doesnot exist 
			{
				Dest_Set->insert(StrIntPair(te,1) );
				ret_val = true;
			}
			
			cp_it++;
		}
	}
	return ret_val;
}

bool CGrammar::OperateUnion( map<wstring,int> * Src_Set , map<short,short> * Dest_Set ) //second argument is different
{
	typedef pair <int,int> IntIntPair;
	bool ret_val = false;

	if(Src_Set->empty() )
	{
		return false; //the source follow set is empty;
	}

	/*else if( CurrFollow == NULL )
	{
		CurrFollow = new map<wstring,int>;
		map<wstring,int>::iterator cp_it = Src_SET->begin();
		while( cp_it != Src_SET->end() )
		{
			wstring te = cp_it->first;
			CurrFollow->insert(StrIntPair(te,1) );
			cp_it++;
		}
	} */
	else 
	{
		map<wstring,int>::iterator cp_it = Src_Set->begin();
		map<short,short>::iterator test_it = NULL;

		wstring test_str = L"";
		while( cp_it != Src_Set->end() )
		{
			wstring te = cp_it->first;
			
			short pos_num = CSymbolTableManager::GetNum(te,true);

			test_it = Dest_Set->find(pos_num);
									
			if( test_it == Dest_Set->end() ) //insert only if doesnot exist 
			{
				Dest_Set->insert(IntIntPair(pos_num,pos_num) );
				ret_val = true;
			}
			
			cp_it++;
		}
	}
	return ret_val;
}



bool CGrammar::IsNullRange( vector<CSymbol*>::iterator Start, vector<CSymbol*>::iterator End, map<wstring,bool>* nullables)
{
	bool IsNull = true;
	/*if( Start == End ) //to find out nullabliity of only one
	{
		CSymbol * Curr_Sym = * Start;
		wstring t = Curr_Sym->lexeme;
		map<wstring,bool>::iterator it = nullables->find(t);

		if( it == nullables->end() )
			IsNull=false;
	}*/

	for( ; Start != End && Start < End; Start++ )
	{
		CSymbol * Curr_Sym = * Start;
		wstring t = Curr_Sym->lexeme;

		if( t == L"." )
			continue;

		map<wstring,bool>::iterator it = nullables->find(t);

		if( it != nullables->end() )
			IsNull=true;
		else
		{
			IsNull = false;
			break;
		}



	}
	return IsNull;		
}

map<wstring,bool>* CGrammar::Comp_Dir_Nullables( list<CRawProduction*> * vlist )
{
	map<wstring,bool>* nullables = new map<wstring,bool>;
	
	typedef pair <wstring,bool> StrLexPtrPair;

	int count = 0;

	list<CRawProduction*>::iterator prod_iter = vlist->begin();

	for( ; prod_iter!=vlist->end() ; prod_iter++ )
	{
		CRawProduction * this_prod = * prod_iter;
		vector<CSymbol*>* RHS_list = &this_prod->RHS;

		if( RHS_list ->size() == 1 )
		{
			vector<CSymbol*>::iterator T = RHS_list->begin();
			CSymbol* this_sym = *T;
			if( this_sym->isop == true )
				nullables->insert(StrLexPtrPair( this_prod->LHS->lexeme ,true));
		}
	}

	return nullables;
}	

map<wstring,bool>* CGrammar::Comp_Nullables( list<CRawProduction*> * vlist , map<wstring,bool>* nullables)
{
	list<CRawProduction*>::iterator prod_iter = vlist->begin();

	typedef pair <wstring,bool> StrLexPtrPair;
	bool Change_Occured = false;
	
	do
	{
		Change_Occured = false;
		for( ; prod_iter!=vlist->end() ; prod_iter++ )
		{
			CRawProduction * this_prod = * prod_iter;

			map<wstring,bool>::iterator s_result = nullables->find(this_prod->LHS->lexeme);
			/*if( s_result != NULL ) CHANGED FOR THIS (LINUX) VERSION*/
			if( s_result != nullables->end() )
				continue; //already null
			else
			{
				vector<CSymbol*>* RHS_list = &this_prod->RHS;
	
				bool Not_Null = false;
	
				for( vector<CSymbol*>::iterator T = RHS_list->begin() ; T < RHS_list->end() ; T++ )
				{
					CSymbol* this_sym = *T;
					s_result = nullables->find( this_sym->lexeme );
					/*if( s_result != NULL  )CHANGED FOR THIS (LINUX) VERSION*/
					if( s_result != nullables->end()  )
					{
						Not_Null = true;
						break;
					}	
				}
				
				if( !Not_Null )
				{
					Change_Occured = true;
					nullables->insert( StrLexPtrPair( this_prod->LHS->lexeme , true) );
				}
			}
		}
	}while( Change_Occured );

	/* TEST WHICH NON-TERMINALS HAVE BEEN MARKED AS NULLABLE*/
	/*wstring allnulls = L"";

	for( map<wstring,bool>::iterator Tap = nullables->begin() ; Tap != nullables->end() ; Tap++ )
	{
		allnulls += Tap->first;
		allnulls += L"	";
	}

	MessageBox(NULL,allnulls.c_str(),L"NULLS",MB_ICONINFORMATION);*/

	return nullables;
}

list<CRawProduction*>* CGrammar::ConvertLFG2CFG(list<CRawProduction*> * vlist)
{	
	stack<CSymbol*> stk1;
	stack<CSymbol*> stk2;
	
	short uchar=0;
	short start_index=-1;
	short end_ndex=-1;
	short current_pno=0;
	short len=0;
	short new_prod_counter=0;	
	
	list<CRawProduction*>* rprod_list=new list<CRawProduction*>;

	if(!rprod_list)
		return NULL;

	CRawProduction*rprod=NULL;
	CRawProduction*tprod=NULL;
	
	CSymbol*symb=NULL;
	CSymbol*nsymb=NULL;
	

	short list_len=vlist->size();

	//for(short k=0;k<list_len;k++)
	for(list<CRawProduction*>::iterator rpit=vlist->begin();rpit!=vlist->end();rpit++)
	{
			rprod=*rpit;

			//rprod=vlist->at(k);

			//rprod->printW();

			len=rprod->RHS.size();			

			for(short p=0;p<len;p++)
			{
				symb=rprod->RHS.at(p);

				if(symb->lexeme[0]=='.')
					break;
				
				stk1.push(symb);				

				uchar=symb->lexeme[0];

				if(symb->isop==true && (uchar==']' || uchar==')'))
				{
					stk1.pop();	//pop ']' ')'
					
					short cbr=0;
					symb=stk1.top();
					cbr=symb->lexeme[0];
				
					while(cbr!='[' && cbr != '(')
					{
						stk2.push(symb);
						stk1.pop();
						
						symb=stk1.top();
						cbr=symb->lexeme[0];
					}

					stk1.pop();	//pop '[' '('

					symb=rprod->RHS.at(p+1);
					uchar=symb->lexeme[0];					

					if(cbr=='(')
					{
							//////////////////////////////
							// A -> B ( D | E ) F 
							// conversions:
							// A -> B X2 F
							// A -> B F
							// X2 -> . (NULL Production)
							// X2 -> D 
							// X2 -> E								
							////////////////////////////
							
							new_prod_counter++;

							nsymb=new CSymbol(NewProdName,L"^=!",false);
							
							if(!nsymb)
								break;

							nsymb->lexeme+=GetWSTR(new_prod_counter);
							
							vector<CSymbol*> * slist=GetStackList(&stk2);							
							
							tprod=new CRawProduction();
							
							if(!tprod)
								break;

							tprod->LHS=nsymb;
							AppendList(&tprod->RHS,slist);
							tprod->RHS.push_back(GetDotPtr());

							//tprod->print();
							rprod_list->push_back(tprod);

							//creating: X2 -> .
							tprod=new CRawProduction();

							if(!tprod)
								break;

							tprod->LHS=nsymb;							
							tprod->RHS.push_back(GetDotPtr());
							
							//tprod->print();
							rprod_list->push_back(tprod);

							stk1.push(nsymb);
						
					}
					else if(symb!=NULL && (uchar=='*'|| uchar=='+' || uchar=='#'))
					{
						p++;
					
						if(uchar=='*' || uchar=='+')
						{
							// A -> B [ D | E ]* F 
							// conversions:
							// A -> B X2 F
							// A -> B F
							// X2 -> [D | E] X2
							// X2 -> [D | E]
							////////////////////////////


							// A -> B [ D | E ]+ F 
							// conversions:
							// A -> B X2 F
							// X2 -> [D | E] X2
							// X2 -> [D | E]
														//creating X2
							new_prod_counter++;

							nsymb=new CSymbol();

							if(!nsymb)
								break;

							nsymb->condition=L"^=!";
							nsymb->isop =false;
							
							nsymb->lexeme=NewProdName;
							nsymb->lexeme+=GetWSTR(new_prod_counter);
							
							vector<CSymbol*> *slist=GetStackListRev(&stk2);							
							

							if(uchar=='+')
							{
								//creating: X2 -> [D | E]
								tprod=new CRawProduction();
								
								if(!tprod)
									break;

								tprod->LHS=nsymb;
								AppendList(&tprod->RHS,slist);
								tprod->RHS.push_back(GetDotPtr());

								//tprod->print();
								rprod_list->push_back(tprod);
							}

							//creating: X2 -> [D | E] X2
							tprod=new CRawProduction();

							if(!tprod)
								break;

							tprod->LHS=nsymb;
							AppendList(&tprod->RHS,slist);
							tprod->RHS.push_back(nsymb);
							tprod->RHS.push_back(GetDotPtr());
							
							//tprod->print();
							rprod_list->push_back(tprod);
						

							//creating: S -> B F
							//for * operator only
							if(uchar=='*')
							{
								CRawProduction * prod3=new CRawProduction();
								
								if(!prod3)
									break;

								prod3->LHS=nsymb;
								prod3->RHS.push_back(GetDotPtr());																								
								//prod3->print();
								rprod_list->push_back(prod3);
							}

							stk1.push(nsymb);
							
						}					
						else if(uchar=='#')
						{
								// A -> B [ D % E ]#1#3 F

								// conversions:
								// A -> B X2 F
								// A -> B X2 X2 F
								// A -> B X2 X2 X2 F
								// X2-> [D % E]

								p++; //at number1
								CSymbol * min=rprod->RHS.at(p);								
								//wprintf(L"%s",min->lexeme.c_str());

								p++; //at number2					
								p++; //at number2												
								
								CSymbol * max= rprod->RHS.at(p);
								//wprintf(L"%s",max->lexeme.c_str());										
								
								//p++; //creating X2

								new_prod_counter++;

								nsymb=new CSymbol(NewProdName,L"^=!",false);								
								
								if(!nsymb)
									break;

								nsymb->lexeme+=GetWSTR(new_prod_counter);

								new_prod_counter++;
								CSymbol * nsymb3=new CSymbol(NewProdName,L"^=!",false);								
								
								if(!nsymb3)
									break;

								nsymb3->lexeme+=GetWSTR(new_prod_counter);
															

								CRawProduction * prod3=new CRawProduction();								

								if(!prod3)
									break;

								prod3->LHS=nsymb3;								
															
								if(stk2.size()>0)
								{
									AppendList( &prod3->RHS,GetStackListRev(&stk2));															
								}
								
								prod3->RHS.push_back(GetDotPtr());															

								rprod_list->push_back(prod3);

								///////////////////////////added/////////////////////
								stk1.push(nsymb);								

								vector< vector<CSymbol*> * > * cases_list= GetAllCasesList(nsymb3,min,max);	

								for(unsigned short n=0;n<cases_list->size();n++)
								{
									CRawProduction*temp_prod=new CRawProduction();

									if(!temp_prod)
										break;

									temp_prod->LHS=nsymb;									
									
									AppendList(&temp_prod->RHS,cases_list->at(n));																										
									
									temp_prod->RHS.push_back(GetDotPtr());

									rprod_list->push_back(temp_prod);									
								}

								if(cases_list)
								{
									for(vector< vector<CSymbol*> * >::iterator it=cases_list->begin();
										it!=cases_list->end();it++)
										{
											try{
											delete *it;
											}
											catch(...)
											{
												#ifdef GenLog
												CLogManager::AddMessage(L"deleting cases_list ..\r\n");
												#endif
											}
										}
									try{
										delete cases_list;
									}
		catch(...)
		{
			#ifdef GenLog
			CLogManager::AddMessage(L"#### ");
			#endif
		}

									cases_list=NULL;
								}

							} //end else if
						}
						else 
						// productions enclosed in [XYZ DS DSD] with no
										// outer-operator *, + or #
						{
								///////////////////////////X->[[A%[A|B]]|D]
								//	X -> [A%B]
								//	X -> A B
								//	X -> B A

								new_prod_counter++;
								
								CSymbol * nsymb=new CSymbol(NewProdName,L"^=!",false);	
								
								if(!nsymb)
									break;

								nsymb->lexeme+=GetWSTR(new_prod_counter);

								tprod=new CRawProduction();
								
								if(!tprod)
									break;

								tprod->LHS=nsymb;

								while(!stk2.empty() && (stk2.top())->lexeme!=L"[")
								{
									tprod->RHS.push_back(stk2.top());
									stk2.pop();
								}

								list<CRawProduction*> * temp_prod_list=GetResolvedProductionList(tprod);
								AddProdList(rprod_list,temp_prod_list);										

								stk1.push(nsymb);
						}
				}
								
			}										

			if(stk1.size()>0)
			{
				vector<CSymbol*> * symb_list=GetStackList(&stk1);
								
				rprod->RHS=*symb_list;
				rprod->RHS.push_back(GetDotPtr());				
				rprod_list->push_back(rprod);	
				
				if(symb_list)
				{
					try{
						delete symb_list;
						symb_list=NULL;
						}
		catch(...)
		{
			#ifdef GenLog
			CLogManager::AddMessage(L"#### symb_list");
			#endif
		}

				}
								
			}

		}//end for					

return rprod_list;
}

//READS CFG WITH SELECTION SET
list<CRawProduction*>* CGrammar::ReadCFG(string CFGFileName)
{
	CFileReader fr( CFGFileName.c_str() , 1024 );
	
	wstring str=L"";
	short ch=0;
	bool Sel_set = false;
	int counter = 0;
	CSymbol * symbol;
	
	CRawProduction * prod=NULL;	
	
	list<CRawProduction*>*prod_list;	
	prod_list=new list<CRawProduction*>;

	//vector<CProduction*>*NullProdList;	
	//NullProdList = new vector<CProduction*>;
	
	if(!prod_list)
		return NULL;
	
	bool start=false;
	
	while((ch=fr.GetCurrentChar())!=0)
	{

		if(ch!=LF && ch!=CR)
			str+=ch;	

		//wprintf(L"%c: %s\n",ch, str.c_str());
		
		switch(ch)
		{			
			case '>' :
				str=L"";
				break;
			
			case '-' : 
				/*symbol=new CSymbol();
				str.erase(str.length()-1);
				symbol->lexeme=TrimString(str);
				prod=new CRawProduction();
				prod->LHS =symbol;*/
				break;

			case ':' :
				
				symbol=new CSymbol();
				
				if(!symbol)
					break;

				str.erase(str.length()-1);
				symbol->lexeme=TrimString(str);				
	
				str=L"";		
				fr.MoveNext();	
				while((ch=fr.GetCurrentChar())!=';')		
				{
					str+=ch;
					fr.MoveNext();
				}
				symbol->condition=TrimString(str);				
				prod->RHS.push_back(symbol);			
				symbol=NULL;
				str=L"";			
				break;			
		
			case '.' :	
	
	/*
				counter ++;
				prod->prodId = 	counter ;

				if(prod->RHS.size()==0)  //null production
					{
						NullProd.push_back(prod->LHS.Label)
					}

				else
				{
				wstring index = LHS.label;	
				searchingProdList[temp].push_back(prod->prodId);
				
										
				} //this "}" will cover more space
*/


				prod_list->push_back(prod);					
				
				symbol=NULL;
				str=L"";
				symbol=new CSymbol();

				if(!symbol)
					break;

				symbol->isop=true;
				symbol->lexeme=ch;
				symbol->condition=L"";
				prod->RHS.push_back(symbol);				
				prod=NULL;
				//prod=new CRawProduction();
				break;

			case '{' :	case '}' :	case '(' : 	
			case ')' :	case '*' :	case '+' : 	
			case '%' :	case '|' :	case '~' :
				symbol=new CSymbol();

				if(!symbol)
					break;

				symbol->isop=true;
				symbol->lexeme=ch;
				symbol->condition=L"";
				prod->RHS.push_back(symbol);
				str=str.erase(str.length()-1);						
				break;
			
			case ',' : //seperator for selection set
				typedef pair<short,short> ShSh_Pair;
				if( Sel_set )
				{
					/*
						if(!prod->SelectionSet.empty())
							prod->SelectionSet = new map<short,short>;
					*/
					
					str = str.erase( str.length()-1 );
					str = TrimString(str);
					int num = CSymbolTableManager::GetNum(str,true);

					prod->SelectionSet.insert( ShSh_Pair(num,num) );

					str.erase( str.begin() , str.end() );
				}
				break;
			case '[' : //new
				Sel_set = true;
				symbol=new CSymbol();

				if(!symbol)
					break;

				str.erase(str.length()-1);
				symbol->lexeme=TrimString(str);
				
				prod=new CRawProduction();

				if(!prod)
					break;

				prod->LHS =symbol;	
				str.erase(str.begin() ,str.end() );
				break;
			case ']' : //new
				Sel_set = false;
				break;

			case '#' :

				symbol=new CSymbol();
				
				if(!symbol)
					break;

				symbol->isop=true;
				symbol->lexeme=ch;
				symbol->condition=L"";
				prod->RHS.push_back(symbol);

				ch=fr.MoveNext();

				symbol=new CSymbol();
				
				if(!symbol)
					break;

				symbol->isop=true;
				symbol->lexeme=ch;
				symbol->condition=L"";
				prod->RHS.push_back(symbol);

				ch=fr.MoveNext();

				symbol=new CSymbol();
				
				if(!symbol)
					break;

				symbol->isop=true;
				symbol->lexeme=ch;
				symbol->condition=L"";
				prod->RHS.push_back(symbol);

				ch=fr.MoveNext();
				
				symbol=new CSymbol();
				
				if(!symbol)
					break;

				symbol->isop=true;
				symbol->lexeme=ch;
				symbol->condition=L"";				
				prod->RHS.push_back(symbol);
				
				str=L"";		
				break;

			case '/' :				
				
				while(true)
				{							
					ch=fr.GetCurrentChar();
						
					if(ch==LF||ch==CR||ch<=0)
						break;					

					str+=ch;
					fr.MoveNext();					
				}

				//MessageBox(NULL,str.c_str(),L"\nComment(Grammar)",MB_ICONINFORMATION);
				
				str=L"";

				break;

			}
	
		fr.MoveNext();
	}

	
	//////////////////////////////////////////////////////
	// Modification for Generator Grammar, that has list of NULL Production
	///////////////////////////////////////////////////////////////
	//For each prod in NULLprodList
	//	search prod->id in ProdList
	//	Access Siblingprod from Production list of found ithe
	//	Set Prod->NULLSibling to true for Each SiblingProd 
	//Next

	/*

	for(int nullIndex = 0 ; nullIndex < NullProdList.size(); nullIndex++ )
	{							//nullIndex is used to traverse NullProdList

		wstring searchindex = NullProdList[nullIndex];	
		if(searchingProdList[searchindex].size()!=0)	//if entry exists 
		{	
			for(int vecIndex = 0 ; vecIndex < searchingProdList[searchindex].size() ; vecIndex++)
			{
				//vecIndex is index to searchingProdList's vector(key,vector)
				//searchindex acts as key(index) to searchingProdList
				//prodIndex is index to prodList

				int prodIndex = searchingProdList[searchindex].at(vecIndex);
				prodList[prodIndex].siblingNull = true; 
			}
		}
	}*/

/*

================================================================ old version
	for(vector<wstring*>::iterator nullit=NullProd.begin();nullit!=NullProd.end();nullit++)
	{
		integer index=0;
		index = hashIndex(*nullit);	//to get the index where the particular production is present
		
		if(index)		//if nullprod with same production name is found
			{
				for(vector<integer*>::iterator searchit=searchProdList[index].begin();searchit!=searchProdList[index].end();searchit++)
				{
					prod_list[*searchit.ProductionNo].siblingNull = true;
				}
				
			}

			
		
	}
================================================================
	*/
	
	return prod_list;
	
}

//##ModelId=4088BB2F034B
list<CRawProduction*>* CGrammar::ReadLFG(string LFGFileName)
{
	CFileReader fr(LFGFileName.c_str(),1024);
	
	wstring str=L"";
	short ch=0;

	CSymbol * symbol;
	CRawProduction * prod=new CRawProduction();

	if(!prod)
		return NULL;
	
	list<CRawProduction*>*prod_list;	
	prod_list=new list<CRawProduction*>;

	if(!prod_list)
		return NULL;

	bool start=false;
	
	while((ch=fr.GetCurrentChar())!=0)
	{

		if(ch!=LF && ch!=CR)
			str+=ch;	

		switch(ch)
		{			
			case '>' :
				str=L"";
				break;
			
			case '-' :
				symbol=new CSymbol();
				
				if(!symbol)
					break;

				str.erase(str.length()-1);
				symbol->lexeme=TrimString(str);
				prod=new CRawProduction();

				if(!prod)
					break;

				prod->LHS =symbol;			
				break;

			case ':' :
				
				symbol=new CSymbol();
				
				if(!symbol)
					break;

				str.erase(str.length()-1);
				symbol->lexeme=TrimString(str);				
	
				str=L"";		
				fr.MoveNext();	
				while((ch=fr.GetCurrentChar())!=';')		
				{
					str+=ch;
					fr.MoveNext();
				}
				symbol->condition=TrimString(str);				
				prod->RHS.push_back(symbol);			
				symbol=NULL;
				str=L"";			
				break;			
		
			case '.' :											

				symbol=NULL;
				str=L"";
				symbol=new CSymbol();
				
				if(!symbol)
					break;

				symbol->isop=true;
				symbol->lexeme=ch;
				symbol->condition=L"";
				prod->RHS.push_back(symbol);				
				
				prod_list->push_back(prod);

				prod=new CRawProduction();

				if(!prod)
					break;

				break;

			case '[' :	case ']' :	case '{' :	
			case '}' :	case '(' : 	case ')' :	
			case '*' :	case '+' : 	case '%' :	
			case '|' :	case '~' :
				symbol=new CSymbol();
				
				if(!symbol)
					break;

				symbol->isop=true;
				symbol->lexeme=ch;
				symbol->condition=L"";
				prod->RHS.push_back(symbol);
				str=str.erase(str.length()-1);						
				break;
			
			case '#' :

				symbol=new CSymbol();
				
				if(!symbol)
					break;

				symbol->isop=true;
				symbol->lexeme=ch;
				symbol->condition=L"";
				prod->RHS.push_back(symbol);

				ch=fr.MoveNext();

				symbol=new CSymbol();
				
				if(!symbol)
					break;

				symbol->isop=true;
				symbol->lexeme=ch;
				symbol->condition=L"";
				prod->RHS.push_back(symbol);

				ch=fr.MoveNext();

				symbol=new CSymbol();
				
				if(!symbol)
					break;

				symbol->isop=true;
				symbol->lexeme=ch;
				symbol->condition=L"";
				prod->RHS.push_back(symbol);

				ch=fr.MoveNext();
				
				symbol=new CSymbol();
				
				if(!symbol)
					break;

				symbol->isop=true;
				symbol->lexeme=ch;
				symbol->condition=L"";				
				prod->RHS.push_back(symbol);
				
				str=L"";		
				break;

			case '/' :				
				
				while(true)
				{							
					ch=fr.GetCurrentChar();
						
					if(ch==LF||ch==CR||ch<=0)
						break;					

					str+=ch;
					fr.MoveNext();					
				}

				str=L"";

				break;

			}
	
		fr.MoveNext();
	}

	return prod_list;
	
}

//##ModelId=4088BB40029F
vector<CProduction*>* CGrammar::GenerateCFG(string CFGFileName)
{
	return NULL;
}

list<CRawProduction*>* CGrammar::GetResolvedProductionList(CRawProduction * prod)
{
	short len=0;		
	len=prod->RHS.size();

	short Operator=0;
	
	CSymbol*symb=NULL;
	
	vector<CSymbol*> symb_list;

	list<CRawProduction*> * prod_list=new list<CRawProduction*>;

	if(!prod_list)
		return NULL;
	
	for(short k=0;k<len;k++)
	{		
		symb=prod->RHS.at(k);

		if(symb->isop)
			Operator=symb->lexeme[0];
		else
			symb_list.push_back(symb);	

	}

	CProduction * nprod=NULL;

	if(Operator=='%')
	{
			short symb_count= symb_list.size();
			
			vector< vector<short> > perm_list;

			vector<short> perm_sub_list;
			
			//generate number permutations in the given range
			//returns vector<vector<int>>
			//for arguments min=0, max=2
			//result: [0,1,2],[0,2,1],[1,0,2],[1,2,0],[2,0,1],[2,1,0]

			GenerateNumberPermutations(0,symb_count-1,perm_list);

			short len1=perm_list.size();

			CRawProduction*nprod=NULL;

			for(short a=0;a<len1;a++)
			{
				nprod=new CRawProduction();

				if(!nprod)
					continue;
				
				nprod->LHS=prod->LHS;

				perm_sub_list=perm_list.at(a);
				
				short len2= perm_sub_list.size();				
				short  ind=-1;

				for(short b=0;b<len2;b++)
				{	
					ind=perm_sub_list.at(b);
					nprod->RHS.push_back(symb_list[ind]);
				}		

				nprod->RHS.push_back(GetDotPtr());
				//nprod->print();
				prod_list->push_back(nprod);			
			}	
		}
		else if(Operator=='|')
		{
			CRawProduction*tprod=NULL;

			for(unsigned short n=0;n<symb_list.size();n++)
			{		
				tprod = new CRawProduction();
				
				tprod->LHS = prod->LHS;
				tprod->RHS.push_back(symb_list[n]);
				tprod->RHS.push_back(GetDotPtr());				
				prod_list->push_back(tprod);
			}
		}
		else
		{			
			prod->RHS.push_back(GetDotPtr());
			prod_list->push_back(prod);
		}	

return prod_list;
	
}

multimap<wstring,CProduction*> * CGrammar::PurifyProductions(list<CRawProduction*> * rprod_list)
{
	using namespace std;

	multimap<wstring,CProduction*> * prod_table=NULL;
	//vector<CProduction*>*NullProdList;	
	//NullProdList = new vector<CProduction*>;
	

	if(!rprod_list)
		return prod_table;	

	prod_table=new multimap<wstring,CProduction*>;

	if(!prod_table)
		return NULL;

	CRawProduction*rprod=NULL;
	CProduction*temp=NULL;
	
	CSymbol*symb=NULL;
	CProductionSymbol*psymb=NULL;

	for(list<CRawProduction*>::iterator it=rprod_list->begin();
		it!=rprod_list->end();it++)	
	{			
		
		rprod=*it;		

		temp=new CProduction();
		
		if(!temp)
			continue;
			
		temp->LHS =new CProductionSymbol();

		if(!temp->LHS)
		{
			try{
				delete temp;
			}
			catch(...)
			{
				#ifdef GenLog
				CLogManager::AddMessage(L"#### ");
				#endif
			}
			continue;
		}

		temp->LHS->Label=rprod->LHS->lexeme.c_str();
		temp->LHS->isTerminal=false;

		temp->SelectionSet = rprod->SelectionSet;							
		
		for(vector<CSymbol*>::iterator it=rprod->RHS.begin();
			it!=rprod->RHS.end();it++)		
		{					
			symb=*it;
			
			psymb=new CProductionSymbol();
			
			if(!psymb)
				continue;

			psymb->Label=TrimString(symb->lexeme);
			
			if(psymb->Label[0]>='a' && psymb->Label[0]<='z')
				psymb->isTerminal=true;
	
			if(!symb->isop)
			{		
				ExtractConditionsAndConstraints(psymb,symb->condition);
			}

			temp->RHS.push_back(psymb);
		}

		prod_table->insert(prod_pair(temp->LHS->Label,temp));		
	}

	return prod_table;	
}



void CGrammar::PrintRawGrammar(list<CRawProduction*> * vlist)
{
	using namespace std;

	CRawProduction* rp=NULL;
	CSymbol * rs=NULL;
	vector<CSymbol*>*rp_rhs=NULL;

	for(list<CRawProduction*>::iterator it=vlist->begin();it!=vlist->end();it++)	
	{
		rp=*it;
		if(rp)
		{
			wprintf(L"\n%s ==> ",rp->LHS->lexeme.c_str());
			rp_rhs=&rp->RHS;
			for(vector<CSymbol*>::iterator it2= rp_rhs->begin(); it2!=rp_rhs->end();it2++)
			{
				rs=*it2;
				wprintf(L"		%s %s \n",rs->lexeme.c_str(),rs->condition.c_str()); 			
			}
					
		}		
	}		

}

bool CGrammar::PreprocessGrammar(const string & grules_fname, 
								 const string & macros_fname,
								 const string & cfg_fname,
								 bool ReplaceStarByRange)
{	
	using namespace std;	

	typedef pair<wstring,wstring> macro_pair;

	map<wstring,wstring> MacroList;

	CFileReader MacrosFReeader(macros_fname.c_str(),1024);

	short ch=0;
	
	wstring str=L"";
	wstring macro_name=L"";
	bool equal_came=false;

	while((ch=MacrosFReeader.GetCurrentChar())!=0)
	{
		if(ch=='=' && !equal_came)
		{
			macro_name=str;
			str=L"";
			equal_came=true;
		}
		else if(ch=='.')
		{
			macro_name=TrimString(macro_name);
			str=TrimString(str);
			MacroList.insert(macro_pair(macro_name,str));
			str=L"";
			equal_came=false;
		}
		else if (ch!=LF && ch!=CR)
			str+=ch;

		MacrosFReeader.MoveNext();
	}

	
	MacrosFReeader.CloseFile();

	/************************************************************/

	CFileReader RulesFReeader(grules_fname.c_str(),512);
	
	CFileWriter GRulesFWriter(cfg_fname.c_str());

	ch=0;
	
	str=L"";	
	int counter=0;
	
			macro_name  =L"";
	wstring macro_result=L"";	
	
	map<wstring,wstring>::iterator temp_it;

	while((ch=RulesFReeader.GetCurrentChar())!=0)
	{
		if(ch=='@')
		{
			macro_name=L"";
			
			while(ch!=' ' && ch!='.' && ch!=0)
			{
				macro_name+=ch;
				ch=RulesFReeader.MoveNext();
			}

			temp_it=MacroList.find(macro_name);			
			
			if(temp_it!=MacroList.end())
			{
				macro_result=temp_it->second;
			}
			else
			{				
				//macro_result=L"*** Macro not found for ";
				//macro_result+=macro_name;
				//macro_result+=L" ***";
				//wprintf(L"\n%s\n",macro_result.c_str());
			}
			
			str+=macro_result;			
			str+=ch;
		}
		else if(ReplaceStarByRange && ch=='*')
		{
			str+=L"#0#1";
		}
		else 		
			str+=ch;
		
		counter++;

		if(counter%256==0)
		{
			GRulesFWriter.Write(str);
			str=L"";
		}		

		RulesFReeader.MoveNext();

	}
	

	GRulesFWriter.Write(str);

	GRulesFWriter.CloseFile();


return true;
}


vector<CSymbol*> * CGrammar::GetStackList(stack<CSymbol*> *  stk1)
{
	stack<CSymbol*> stk2;

	vector<CSymbol*> * vlist=new vector<CSymbol*>;

	if(!vlist)
		return NULL;

	CSymbol* temp=NULL;

	while(!stk1->empty())
	{
		temp=stk1->top();
		stk2.push(temp);
		stk1->pop();
	}

	while(!stk2.empty())
	{
		temp=stk2.top();
		vlist->push_back(temp);		
		stk2.pop();
	}

	return vlist;
}


void CGrammar::AppendList(vector<CSymbol*> * first,vector<CSymbol*> * second/*Add second list in first*/)
{
	if(!first||!second)
		return;
	
	short len=second->size();

	for(short p=0;p<len;p++)	
		first->push_back(second->at(p));		
}

CSymbol* CGrammar::GetDotPtr()
{
	CSymbol* DotPtr=new CSymbol(L".",L"",true);
	return DotPtr;
}


void  CGrammar::AddProdList(list<CRawProduction*>*first,list<CRawProduction*>*second)
{
	if(!first||!second)
		return;
	
	//short len=second->size();
//	for(short p=0;p<len;p++)	
//		first->push_back(second->at(p));		
	
	for(list<CRawProduction*>::iterator rpit=second->begin();rpit!=second->end();rpit++)
		first->push_back(*rpit);
	
}

void CGrammar::GenerateNumberPermutations(short min, short max, vector<vector<short> > & vlist)
{
  using namespace std;

  vector<short> ca;

  for(short p=min;p<=max;p++)
	  ca.push_back(p);  
  
  CPermutationMaker maker; 
  
  vector<short> cnext;   

  maker.get_vector_permutation(ca,cnext,vlist); 
		
}

vector< vector<CSymbol*> * > * CGrammar::GetAllCasesList(CSymbol *symb, CSymbol *mins, CSymbol *maxs)
{
	vector< vector<CSymbol*> * > * case_list=NULL;

	if(!symb||!mins||!maxs)
		return case_list;

	case_list = new vector< vector<CSymbol*> * >;

	//..assuming #min#max ranges,,,,,min and max ranges from 0 to 9

	short min = mins->lexeme[0]-'0';
	short max = maxs->lexeme[0]-'0';

	
	vector<CSymbol*> * tlist=NULL;

	for(short k=min;k<=max;k++)
	{
		tlist=new vector<CSymbol*>;
		
		if(!tlist)
			continue;
		
		for(short m=0;m<k;m++)		
			tlist->push_back(symb);		
		
		case_list->push_back(tlist);
	}

	return case_list;	
}

void CGrammar::WriteCFGInFile(list<CRawProduction*> * vlist,string CFGFileName)
{
	using namespace std;

	CRawProduction* rp=NULL;
	CSymbol * rs=NULL;
	vector<CSymbol*>*rp_rhs=NULL;

	CFileWriter fw(CFGFileName);

	for(list<CRawProduction*>::iterator it=vlist->begin();it!=vlist->end();it++)	
	{
		rp=*it;
		if(rp)
		{
			fw.Write(rp->LHS->lexeme);

			/********** CHANGE BY OMAR FOR WRITING SELECTION SET TO FILE (START)********/
			
			fw.Write(L" [");

			map<short,short>::iterator sel_set_it = rp->SelectionSet.begin();
			wstring sel_str = L"";
			while( sel_set_it != rp->SelectionSet.end() )
			{
				//int num = sel_set_it->first;
				sel_str = CSymbolTableManager::GetString(sel_set_it->first);
				fw.Write(sel_str);
				fw.Write(L",");				
				
				sel_set_it++;
			}			

			fw.Write(L"]");

			/********** CHANGE BY OMAR FOR WRITING SELECTION SET TO FILE (END)********/

			fw.Write(L" -> ");			
			rp_rhs=&rp->RHS;
			for(vector<CSymbol*>::iterator it2= rp_rhs->begin(); it2!=rp_rhs->end();it2++)
			{
				rs=*it2;
				fw.Write(rs->lexeme);
				if(!rs->isop)
				{
					fw.Write(L":");
					fw.Write(rs->condition);
					fw.Write(L"; ");
				}				
			}			
		fw.Write(L"\r\n");
		}		
	}		

	fw.CloseFile();
}

void CGrammar::ExtractConditionsAndConstraints(CProductionSymbol*psymb, 
									 const std::wstring & condition)
{	
	wstring str=L"";
	wstring op=L"";	
	wstring left_str=L"";

	short bcount=0;
	short cond_len=0;
	short ch=0;
	
	bool bstarted=false;

	cond_len=condition.size();

	CFDescription * fdesc=NULL;

	if(condition[0]=='[')
		ch=0;

			
	for(short n=0;n<cond_len;n++)
	{
		ch=condition[n];
		switch(ch)
		{
			case '{':
				str+=ch;
				bstarted=true;
				break;

			case '}':
				str+=ch;
				bstarted=false;
				break;
			
			case '~':	
			case '[':	
								
				//constraint started
				
				bcount=1;

				if(ch=='~')				
					bcount=0;								
				
				str+=condition[n];
				n++;

				while(n<cond_len)
				{
					str+=condition[n];
					if(condition[n]=='[')
						bcount++;
					else if(condition[n]==']')
					{
						bcount--;
						if(bcount==0)
						{
							psymb->constraints.push_back(str);
							str=L"";
							break;			
						}
					}
					n++;
				}
				
				break;

			case '=':
							
				left_str=str;

				fdesc=new CFDescription();	

				if(!fdesc)
					break;
				
				if(condition[n+1]=='c')	
				{
					fdesc->Operator='c';													
					n++;
				}
				else
					fdesc->Operator=condition[n];					
				
				
				fdesc->Label=LexiconPtr->GenerateFDExpression(left_str);
				str=L"";

				break;

			case '$':
				
				left_str=str;

				fdesc=new CFDescription();									

				if(!fdesc)
					break;

				fdesc->Operator='$';													
				fdesc->Label=LexiconPtr->GenerateFDExpression(left_str);
				str=L"";

				break;
			
			case ',':			

				if(bstarted)
				{
					str+=condition[n];
					break;
				}
				
				if(str!=L"")
				{
					if(fdesc->Operator=='c')
					{
						wstring constraint=L"[";
						constraint+=left_str;						
						constraint+=L" =c ";
						constraint+=str;
						constraint+=L"]";
						psymb->constraints.push_back(constraint);
					}
					else
					{
						fdesc->Value=LexiconPtr->GenerateFDExpression(str);
						psymb->conditions.push_back(fdesc);
					}
				}
				
				str=L"";
				break;	
			
			case 32:	//spcae
			case 9:		//tab
							
				if(str!=L"")
				{
					str+=condition[n];
				}

				break;

			default:

				str+=condition[n];
			
				
		}//end switch
	}

	if(str!=L"")
	{
		if(fdesc->Operator=='c')
		{
			wstring constraint=L"[";
					constraint+=left_str;						
					constraint+=L" =c ";
					constraint+=str;
					constraint+=L"]";
			psymb->constraints.push_back(constraint);
		}
		else
		{
			fdesc->Value=LexiconPtr->GenerateFDExpression(str);
			psymb->conditions.push_back(fdesc);
		}
	}

}

void CGrammar::PrintProductions()
{
	using namespace std;

	if(!ProductionTable)
		return;

	for (multimap<wstring,CProduction* >::iterator i = ProductionTable->begin(); 
		i!=ProductionTable->end();)
	{
		multimap<wstring,CProduction*>::iterator j = ProductionTable->upper_bound(i->first);			

		wprintf(L"\n%s: %d\n",i->first.c_str(),ProductionTable->count(i->first));
		
		do
		{			
			//i->second->PrintProduction();
			++i;

		} while(i != j && i != ProductionTable->end());
		
		wprintf(L"\n");
	}

}
/////////////////////////////New start
bool CGrammar::GetProductionsExtended(short LabelID, 
									  vector<CProduction*> &ProdList,
									  CFStructureItem * GenFSPtr)  //new
{

	if(ProdTable.size() == 0)
		return false;

	list<CProduction*> Lst ;
	
	if(!GetProdList(LabelID , Lst))
		return false;
		
	list<CProduction*>::iterator LstIt = Lst.begin();

	CProduction * CProd = NULL;
	set<short>::iterator set_it;
	bool notfound=false;

	while(LstIt != Lst.end())
	{
		CProd = * LstIt ;
		if(!GenFSPtr || CProd->GenSelectionSet.size()==0)
		{
			ProdList.push_back(CProd->GetClone()) ;
		}
		else
		{			
			////////////////////recent changes////////////////////////
			notfound=false;
			for(set_it=CProd->GenSelectionSet.begin();
				set_it!=CProd->GenSelectionSet.end();set_it++)
			{
				if(!GenFSPtr->SearchEx(*set_it))
					notfound=true;
			}
			if(!notfound)
				ProdList.push_back(CProd->GetClone());
			//////////////////////////////////////////////////////////
		}
		

		LstIt++;
	}
	
	if(ProdList.size() != 0)
		return true;

	return false;
}
/////////////////////////////New end

vector<CProduction*> * CGrammar::GetProductions(wstring lhs)
{
	using namespace std;

	vector<CProduction*> * prod_list=NULL;

	if(!ProductionTable)
		return prod_list;

	lhs=TrimString(lhs);
	
	prod_list=new vector<CProduction*>;

	if(!prod_list)
		return NULL;

	multimap<wstring,CProduction*>::iterator it=ProductionTable->find(lhs);		

	if (it != ProductionTable->end()) 
	do
	{	
		prod_list->push_back(it->second/*->GetClone()*/);	
		++it;		
	} while (it != ProductionTable->upper_bound(lhs));


return prod_list;
}
/////////////////////////////New start
bool CGrammar::GetProductionsExExtended(short LabelID, vector<CLexicalEntry*>* lexent, vector<CProduction*> &prod_list)
{
	using namespace std;

	typedef pair <short,short> ShShPair;

	map<short,short> input_pos;
	map<short,short>::iterator pos_it;
	
	if( !lexent )
		return GetProductionsExtended(LabelID, prod_list,NULL);

	/* FORMATION OF MINIMAL LIST OF POS IN THE FOUND LEXICAL ENTRIES */
	vector<CLexicalEntry*>::iterator lex_it = lexent->begin() ;
	for( ; lex_it != lexent->end(); lex_it++ )
	{
		CLexicalEntry * this_ent = *lex_it;
		short pos = this_ent->POS;
		pos_it = input_pos.find( pos );
		if( pos_it == input_pos.end() )
			input_pos.insert( ShShPair( pos,pos ) );
	}

	if(ProdTable.size() == 0)
		return false;

	//if(!prod_list)
	//	return false;

	list<CProduction*> Lst; 
	
	if(!GetProdList(LabelID , Lst))
		return false;

	list<CProduction*>::iterator it = Lst.begin();	

	CProduction * Curr_prod = NULL;

	map<short,short>::iterator selset_it;

	while(it != Lst.end())
	{	//it->second->PrintProduction();
		Curr_prod = *it;
		for( pos_it = input_pos.begin() ; pos_it != input_pos.end() ; pos_it++) //match selection set with lexical entries (pos only)
		{
			if(!Curr_prod ->SelectionSet.empty())
			{
				selset_it = Curr_prod->SelectionSet.find(pos_it->first);
				if( selset_it != Curr_prod->SelectionSet.end() )
				{
					prod_list.push_back(*it/*->GetClone()*/);		
					break;
				}
			}
		}
		
		++it;		
	}


	//CLEANUP
	input_pos.clear();

	if(prod_list.size()!= 0 )
		return true;
	return false;
}
/////////////////////////////New end
vector<CProduction*> * CGrammar::GetProductionsEx(wstring lhs, vector<CLexicalEntry*>* lexent)
{
	using namespace std;

	typedef pair <short,short> ShShPair;

	map<short,short> input_pos;
	map<short,short>::iterator pos_it;
	
	if( !lexent )
		return GetProductions(lhs);

	/* FORMATION OF MINIMAL LIST OF POS IN THE FOUND LEXICAL ENTRIES */
	vector<CLexicalEntry*>::iterator lex_it = lexent->begin() ;
	for( ; lex_it != lexent->end(); lex_it++ )
	{
		CLexicalEntry * this_ent = *lex_it;
		short pos = this_ent->POS;
		pos_it = input_pos.find( pos );
		if( pos_it == input_pos.end() )
			input_pos.insert( ShShPair( pos,pos ) );
	}

	vector<CProduction*> * prod_list=NULL;

	if(!ProductionTable)
		return prod_list;

	lhs=TrimString(lhs);
	
	prod_list=new vector<CProduction*>;

	if(!prod_list)
		return NULL;

	multimap<wstring,CProduction*>::iterator it=ProductionTable->find(lhs);		

	CProduction * Curr_prod = NULL;

	map<short,short>::iterator selset_it;

	if (it != ProductionTable->end()) // we found some EECS
	do
	{	//it->second->PrintProduction();
		Curr_prod = it->second;
		for( pos_it = input_pos.begin() ; pos_it != input_pos.end() ; pos_it++) //match selection set with lexical entries (pos only)
		{
			if(!Curr_prod ->SelectionSet.empty())
			{
				selset_it = Curr_prod->SelectionSet.find(pos_it->first);
				if( selset_it != Curr_prod->SelectionSet.end() )
				{
					prod_list->push_back(it->second/*->GetClone()*/);		
					break;
				}
			}
		}
		
		++it;		
	} while (it != ProductionTable->upper_bound(lhs));


	//CLEANUP
	input_pos.clear();
return prod_list;
}

bool CGrammar::DeleteRawProdList(list<CRawProduction*>* plist)
{
	if(plist && !plist->empty())
	{
		for(list<CRawProduction*>::iterator it=plist->begin();it!=plist->end();it++)
		{
			try{
				delete *it;			
			}
			catch(...)
			{
				#ifdef GenLog
				CLogManager::AddMessage(L"Exception raised during the deletion of Raw Production...\r\n");
				#endif
			}
		}
		plist->clear();
		delete plist;
		plist=NULL;
	}

	return true;
}
///////////////////////////////////NEW start
bool CGrammar::RemoveORConstraintsEx(void)
{
	//currently dump constraints only
	
	#ifdef GenLog
	CLogManager::AddMessage(L"ONLY OR Grammar Constraints \r\n");
	#endif

	CProduction*tprod=NULL;
	CProductionSymbol*psymb=NULL;
	
	wstring tstr=L"";

	size_t k=0;
	size_t p=0;
	int index=0;

	list<CProduction*> Lst;

	//if(!GetProdList(1 , Lst) )
	//	return false;

	list<CProduction*>::iterator it ;//= Lst.begin();	//avoid oth index 

	for(unsigned int i=1 ; i<ProdTable.size() ; i++)
	{
		if(!GetProdList(i , Lst) )
			return false;

		it = Lst.begin();

		while(it != Lst.end())
		{
			tprod = *it;

			for(k=0; k<tprod->RHS.size(); k++)
			{
				psymb=tprod->RHS[k];

				for(p=0;p<psymb->constraints.size();p++)
				{
					tstr=TrimString(psymb->constraints[p]);
					index=tstr.find(L"|");
					if(index>0)
					{
						ReplicateProductionEx(tprod,p,k,tstr,index);
						
						//to break the outer loop
						k=tprod->RHS.size();
						
						break;					
					}				
				}
			}
			it++;
		}
	}	

	return true;
}

///////////////////////////////////NEW end

bool CGrammar::RemoveORConstraints(void)
{
	//currently dump constraints only
	
	#ifdef GenLog
	CLogManager::AddMessage(L"ONLY OR Grammar Constraints \r\n");
	#endif

	CProduction*tprod=NULL;
	CProductionSymbol*psymb=NULL;
	
	wstring tstr=L"";

	size_t k=0;
	size_t p=0;
	int index=0;

	for(multimap<wstring,CProduction*>::iterator it=ProductionTable->begin();
		it!=ProductionTable->end();it++)
	{
		tprod=it->second;
		
		for(k=0; k<tprod->RHS.size(); k++)
		{
			psymb=tprod->RHS[k];
			
			for(p=0;p<psymb->constraints.size();p++)
			{
				tstr=TrimString(psymb->constraints[p]);
				index=tstr.find(L"|");
				if(index>0)
				{
					ReplicateProduction(tprod,p,k,tstr,index);
					
					//to break the outer loop
					k=tprod->RHS.size();
					
					break;					
				}				
			}
		}
								
	}

	return true;
}

bool CGrammar::ReplicateProduction(CProduction* prod, 
								   int ConstraintIndex, 
								   int ProdSymbIndex, 
								   wstring & constraint,
								   int OrSymbIndex)
{
	if(!prod)
		return false;

	list<wstring> constr_list;
	wstring temp;	

	temp=constraint.substr(0,OrSymbIndex-1);
	constraint=constraint.substr(OrSymbIndex+1,constraint.size()-OrSymbIndex-2);
	
	temp=temp.substr(1);
	temp=TrimString(temp);

	if(temp[0]!='[' && temp[0]!='~')
		temp=L"["+temp;
	
	if(temp[temp.size()-1]!=']')
		temp=temp+L"]";
	
	prod->RHS[ProdSymbIndex]->constraints[ConstraintIndex]=temp;
	
	OrSymbIndex=constraint.find(L"|");

	while(OrSymbIndex>=0)
	{
		temp=constraint.substr(0,OrSymbIndex-1);
		temp=TrimString(temp);
		if(temp[0]!='[' && temp[0]!='~')
			temp=L"["+temp;
	
		if(temp[temp.size()-1]!=']')
			temp=temp+L"]";

		constr_list.push_back(temp);

		constraint=constraint.substr(OrSymbIndex+1);
		constraint=TrimString(constraint);
		OrSymbIndex=constraint.find(L"|");
	}

	//temp=constraint.substr(0,OrSymbIndex);

	constraint=TrimString(constraint);

	if(constraint[0]!='[' && constraint[0]!='~')
		constraint=L"["+constraint;

	if(constraint[constraint.size()-1]!=']')
		constraint=constraint+L"]";

	constr_list.push_back(constraint);

	
	CProduction*tprod=NULL;
	wstring str;

	for(list<wstring>::iterator lit=constr_list.begin();lit!=constr_list.end();lit++)
	{
		str=*lit;

		tprod=prod->GetClone();
				
		if(tprod)
		{
			tprod->RHS[ProdSymbIndex]->constraints[ConstraintIndex]=str;			
			ProductionTable->insert(prod_pair(tprod->LHS->Label,tprod));
		}
	}
	
	return true;
}
////////////////////////////////////////NEW start
bool CGrammar::ReplicateProductionEx(CProduction* prod, 
								   int ConstraintIndex, 
								   int ProdSymbIndex, 
								   wstring & constraint,
								   int OrSymbIndex)
{
	if(!prod)
		return false;

	list<wstring> constr_list;
	wstring temp;	

	temp=constraint.substr(0,OrSymbIndex-1);
	constraint=constraint.substr(OrSymbIndex+1,constraint.size()-OrSymbIndex-2);
	
	temp=temp.substr(1);
	temp=TrimString(temp);

	if(temp[0]!='[' && temp[0]!='~')
		temp=L"["+temp;
	
	if(temp[temp.size()-1]!=']')
		temp=temp+L"]";
	
	prod->RHS[ProdSymbIndex]->constraints[ConstraintIndex]=temp;
	
	OrSymbIndex=constraint.find(L"|");

	while(OrSymbIndex>=0)
	{
		temp=constraint.substr(0,OrSymbIndex-1);
		temp=TrimString(temp);
		if(temp[0]!='[' && temp[0]!='~')
			temp=L"["+temp;
	
		if(temp[temp.size()-1]!=']')
			temp=temp+L"]";

		constr_list.push_back(temp);

		constraint=constraint.substr(OrSymbIndex+1);
		constraint=TrimString(constraint);
		OrSymbIndex=constraint.find(L"|");
	}

	//temp=constraint.substr(0,OrSymbIndex);

	constraint=TrimString(constraint);

	if(constraint[0]!='[' && constraint[0]!='~')
		constraint=L"["+constraint;

	if(constraint[constraint.size()-1]!=']')
		constraint=constraint+L"]";

	constr_list.push_back(constraint);

	
	CProduction*tprod=NULL;
	wstring str;
	int index;

	for(list<wstring>::iterator lit=constr_list.begin();lit!=constr_list.end();lit++)
	{
		str=*lit;

		tprod=prod->GetClone();
				
		if(tprod)
		{
			tprod->RHS[ProdSymbIndex]->constraints[ConstraintIndex]=str;			
			index  = IntSymbTab.GetNum(tprod->LHS->Label);
			ProdTable[index].push_back(tprod);			
		}
	}		
	
	return true;
}

////////////////////////////////////////NEW end

bool CGrammar::WriteFullProductionsInFile(const string & file_name)
{
	CFileWriter fw(file_name);

	CProduction * tprod=NULL;
	CProductionSymbol * psymb=NULL;
	CFDescription* fdesc=NULL;

	size_t k=0;
	size_t p=0;
	wstring tstr;

	vector<CFDescription*>::iterator fdit;

	for(multimap<wstring,CProduction*>::iterator it=ProductionTable->begin();
		it!=ProductionTable->end();it++)
	{
		tprod=it->second;
		
		fw.Write(tprod->LHS->Label+L" ==> ");

		for(k=0; k<tprod->RHS.size()-1; k++)
		{
			psymb=tprod->RHS[k];		

			fw.Write(psymb->Label+L":");

			if(psymb->conditions.size()>0)
			{				
				fdit=psymb->conditions.begin();
				fdesc=*fdit;									
				tstr=fdesc->GetFDescriptionString();
				fw.Write(tstr);
				fdit++;

				for(;fdit!=psymb->conditions.end();fdit++)
				{
					fdesc=*fdit;						
					tstr=fdesc->GetFDescriptionString();
					fw.Write(L", "+tstr);
				}					
			}
			
			if(psymb->constraints.size()>0)
			{				
				fw.Write(L" , ");
				tstr=TrimString(psymb->constraints[0]);
				fw.Write(tstr);
			
				for(p=1;p<psymb->constraints.size();p++)
				{
					fw.Write(L" , ");
					tstr=TrimString(psymb->constraints[p]);				
					fw.Write(tstr);
				}				
			}
			
			fw.Write(L"; ");
		}
		
		fw.Write(L"\r\n");								
	}
	
	fw.CloseFile();

	return true;
}


void CGrammar::GetInfoFromProdTable()
{		
	multimap<wstring,CProduction*>::iterator it = ProductionTable->begin();

	int count=0;		
	int co=-1;
	
	int index;
	wstring CurrLabel;

	CProduction * tprod=NULL;

	wstring LHSLabel;
	
	int ProdCounter=1;

	while(it!=ProductionTable->end())
	{
		ProdCounter++;

		////////////////////code for checking purpose
		co++;		

		LHSLabel = it->first;		//label of LHS

		if(co==0)					//to count the total used indexes of vector			CurrLabel = LHSLabel;
			CurrLabel = LHSLabel;

		if(CurrLabel != LHSLabel)
		{
			CurrLabel = LHSLabel;
			co=-1;
			count++;
		}
		////////////////////////////
		index = IntSymbTab.GetNum(LHSLabel);	
		tprod = it->second;
		
		tprod->ProductionNo=ProdCounter;

		tprod->LHS->LabelID = index;

		int id ;
		wstring l;
		for(unsigned int k=0;k<tprod->RHS.size();k++)
		{			
			if(tprod->RHS[k]->isTerminal!=true && tprod->RHS[k]->Label != L".") //for non terminal symbol
			{				
				id = IntSymbTab.GetNum(tprod->RHS[k]->Label);
				l =	tprod->RHS[k]->Label;
				tprod->RHS[k]->LabelID = IntSymbTab.GetNum(tprod->RHS[k]->Label);
			}

			else if(tprod->RHS[k]->Label != L".")	
			{
				tprod->RHS[k]->LabelID = CSymbolTableManager::GetNum(tprod->RHS[k]->Label);
			}
		}
		
		ProdTable[index].push_back(tprod);	//because index returned start from 1
		 
		it++;		
	}
			
}


void CGrammar::WriteStackInFile()
{
	//GetInfoFromProdTable("from2.txt");

	CFileWriter fw("stackFile.txt");
	

	stack<CProduction*> CProdStack;				

	short LabelID = IntSymbTab.GetNum(L"S") ;

	list<CProduction*> ProdList ;
	
	if(!GetProdList(LabelID , ProdList))
		return ;

	list<CProduction*>::iterator ProdListit = ProdList.begin();	
	
	CProduction* CProd = NULL;

	CProduction* InnerCProd = NULL;

	list<CProduction*> InnerProdList ;
	
	list<CProduction*>::iterator InnerProdListit ;

	wstring temp;

	int size;

	while(ProdListit!=ProdList.end())
	{
		CProd = *ProdListit;
		CProdStack.push(CProd);
		ProdListit++;
	}
		
	short count = -1;
	while(!CProdStack.empty())
	{
		count++;

		if(count == 300)
			break;

		CProd = CProdStack.top();
		///////////////////////// file writing code
		temp = CProd->LHS->Label + L"(" + GetWSTR(CProd->LHS->LabelID) + L") -> " ;	//i.e. NP(12) ->
			fw.Write(temp);
			
			for(unsigned short i=0;i<CProd->RHS.size();i++)
			{			
				if(CProd->RHS[i]->Label != L".")	
				{
					temp = CProd->RHS[i]->Label +L"(" + GetWSTR(CProd->RHS[i]->LabelID) + L") " ; 
					fw.Write(temp);
				}
			}
			fw.Write(L"\r\n");
		////////////////////////
		CProdStack.pop();
		
		
		for(int i=CProd->RHS.size() - 1 ; i >= 0 ; i-- )
		{
			size = CProd->RHS.size() - 1;

			if(CProd->RHS[i]->isTerminal != true && CProd->RHS[i]->Label != L".")
			{
				if(CProd->RHS[i]->LabelID != CProd->LHS->LabelID)	//to avoid recursive infinite loop
				{
					LabelID = CProd->RHS[i]->LabelID;

					if(!GetProdList(LabelID , InnerProdList))
						return ;
					
					InnerProdListit = InnerProdList.begin();

					while(InnerProdListit!=InnerProdList.end())
					{
						
						InnerCProd = *InnerProdListit;						
						CProdStack.push(InnerCProd);
						InnerProdListit++;
					}
				}	
			}
		}
	}

	fw.CloseFile();

}


inline bool CGrammar::GetProdList(short LabelID , list<CProduction*> & ProdList)	//new
{
	ProdList = ProdTable[LabelID];

	if(ProdList.size() != 0)
		return true;
	return false;		
}


void CGrammar::temp()
{
	CFileWriter fw("temp.txt");
	multimap<wstring , CProduction*>::iterator q = ProductionTable->begin();
	CProduction* cprod = NULL;
	wstring t ;

	while(q!=ProductionTable->end())
	{		
		cprod = q->second;
		t = cprod->LHS->Label + L"	" + GetWSTR(cprod->LHS->LabelID) + L"\r\n";
		fw.Write(t);		
		q++;
	}
	fw.CloseFile();
}

short CGrammar::GetNum(wstring Label)
{
	return IntSymbTab.GetNum(Label);
}

vector<CSymbol*> * CGrammar::GetStackListRev(stack<CSymbol*> *  stk1)
{
	
	vector<CSymbol*> * vlist=new vector<CSymbol*>;

	if(!vlist)
		return NULL;

	CSymbol* temp=NULL;

	while(!stk1->empty())
	{
		temp=stk1->top();
		vlist->push_back(temp);
		stk1->pop(); 
	}

	
	return vlist;
}


bool CGrammar::GenerateRelevantProductionTable(void)
{
	typedef pair<int,int> IntPairs;

	list<CProduction*> alist;
		
	vector<list<CProduction*> >::iterator pit=ProdTable.begin();
	list<CProduction*>::iterator list_it;
	vector<CProductionSymbol*>::iterator psit;
	set<short>::iterator set_it;
	
	CProduction * prod=NULL;
	CProductionSymbol * prodsymb=NULL;
	short psid=0;		

	for(;pit!=ProdTable.end();pit++)
	{
		for(list_it=pit->begin();list_it!=pit->end();list_it++)
		{
			prod=*list_it;			 
						
			for(psit=prod->RHS.begin();psit!=prod->RHS.end();psit++)
			{
				prodsymb=*psit;
				if(!prodsymb->isTerminal && prodsymb->Label!=L".")
					AddUniqueProdNo(prodsymb->LabelID,prod->ProductionNo);
			}			
		}		
	}

	return true;
}

bool CGrammar::GetRelvantProductions(int ProductionSymbolId, 
									 list<int> & ProdNumberList)
{
	set<int> * pset=RelevantProductionList->at(ProductionSymbolId);

	if(!pset)
        return false;
	
	int pno=0;
	for(set<int>::iterator set_it=pset->begin();set_it!=pset->end();set_it++)	
	{
		pno=*set_it;
		ProdNumberList.push_back(*set_it);
	}
	
	if(ProdNumberList.size()==0)
		return false;

return true;
}

bool CGrammar::GenerateSelectionSetForGenertionGrammar(void)
{
	vector< list< CProduction *> >::iterator pit;
	list< CProduction *>::iterator lpit;
	vector<CProductionSymbol*>::iterator psit;
	vector<CFDescription*>::iterator fit;

	CProduction * prod=NULL;
	CProductionSymbol * prodsymb=NULL;	
	CFDescription * fdesc=NULL;		

	int CounterChecker=0;
	//short LabelId;

	pit=ProdTable.begin();
	pit++;//skipping first;

	for(;pit!=ProdTable.end(); pit++)
	{
		CounterChecker++;

		if(pit->size()==0)
			break;

		for(lpit=pit->begin();lpit!=pit->end();lpit++)
		{
			prod=*lpit;
			for(psit=prod->RHS.begin();psit!=prod->RHS.end();psit++)
			{
				prodsymb=*psit;
				if(prodsymb->conditions.size()>0)
				{
					fdesc=prodsymb->conditions[0];
					if(fdesc->Label->op==UP_OP && fdesc->Label->list.size()>0)						
					{
						//LabelId = fdesc->Label->list[0];
						prod->AddGenSelectionSetItem(fdesc->Label->list[0]);
					}
				}
			}
		}
	}

	return true;
}

bool CGrammar::WriteProductionsInFileExtended(const string & FileName)
{
	//file writing code to check the new production table values
	
	CFileWriter fw(FileName);
	
	vector<list<CProduction*> >::iterator qwe = ProdTable.begin();
	qwe++;			//because 0th index of vector is empty
	
	list<CProduction*> obj;

	wstring temp;

	int cnt=0;

	while(qwe!=ProdTable.end())
	{
		cnt++;
		obj=*qwe;

		list<CProduction*>::iterator tt = obj.begin();
				
		while(tt!=obj.end())
		{
			CProduction *CP = *tt;
					
			fw.Write(GetWSTR(cnt) + L". ");
			temp = CP->LHS->Label + L"(" + GetWSTR(CP->LHS->LabelID) + L") -> " ;
			fw.Write(temp);
			
			for(unsigned int i=0;i<CP->RHS.size();i++)
			{			
				if(CP->RHS[i]->Label != L".")	
				{
					temp = CP->RHS[i]->Label +L"(" + GetWSTR(CP->RHS[i]->LabelID) + L") " ; 
					fw.Write(temp);
				}
			}
			fw.Write(L"\r\n");
			tt++;
		}
		qwe++;
		
	}

	fw.CloseFile();

	return true;
}

bool CGrammar::AddUniqueProdNo(int ProdSymbId, int ProdId)
{
	set<int> * set_ptr=NULL;

	set_ptr=RelevantProductionList->at(ProdSymbId);
	
	if(!set_ptr)
	{
		set_ptr=new set<int>;
		(*RelevantProductionList)[ProdSymbId]=set_ptr;
	}

	set_ptr->insert(ProdId);

	return false;
}
