#include "stdafx.h"
#pragma warning(disable: 4786)

#ifndef CGRAMMAR_H_HEADER_INCLUDED_BF672D0D
#define CGRAMMAR_H_HEADER_INCLUDED_BF672D0D

#include "InternalSymbolTable.h"
#include "CProduction.h"
#include "RawProduction.h"
#include "CLexicon.h"

#include <string>
#include <vector>
#include <list>
#include <stack>
#include <map>

using std::string;
using std::vector; 
using std::stack; 
using std::map; 
using std::multimap;


//##ModelId=406112BB0250
//##Documentation
//## It is the data structure that contains the information of All Grammar
//## Rules. It also provides an API to get Productions from Grammar
class CGrammar
{
  public:

	CSymbol * GetDotPtr();

	bool ApplyConstraint(CFStructure * FSPtr1,
						 CFStructure * FSPtr2,
						 vector< FSItemList* > * ValueSet);

	vector<CProduction*> * GetProductions(wstring lhs);
	bool GetProductionsExtended(short LabelID, 
								vector<CProduction*> &,
								CFStructureItem*FSPtr=NULL);	//new

	vector<CProduction*> * GetProductionsEx(wstring lhs, vector<CLexicalEntry*>* lexent);
	bool GetProductionsExExtended(short LabelID, vector<CLexicalEntry*>* lexent, vector<CProduction*> &prod_list);	//new
private:
	void PrintProductions();
public:
    void ExtractConditionsAndConstraints(CProductionSymbol*psymb, 
									 const std::wstring & condition);
	void WriteCFGInFile(list<CRawProduction*> * prod_list,string CFGFileName);

	vector< vector<CSymbol*> * > *  GetAllCasesList(CSymbol*symb,CSymbol*min,CSymbol*max);

	void GenerateNumberPermutations(short min, short max, vector<vector<short> > & v_list);
	 	 
	void AddProdList(list<CRawProduction*>*frist,list<CRawProduction*>*second);
	
	void PrintRawGrammar(list<CRawProduction*> * vlist);
	
	multimap<wstring,CProduction*> * PurifyProductions(list<CRawProduction*> * rprod_list);
		
	list<CRawProduction*>* GetResolvedProductionList(CRawProduction * prod);
    
    CGrammar(const string & grules_fname, 
			 const string & macros_fname,
			 const string & lfg_fname,
			 const string & cfg_fname,
			 CLexicon*LexPtr,
			 bool ReplaceStarByRange=false,
			 bool IsGenerationGrammar=false);
	
	~CGrammar();

    //##ModelId=4063A158003B
    list<CRawProduction*>* ConvertLFG2CFG(list<CRawProduction*> * vlist);

    //##ModelId=4088BB2F034B
    list<CRawProduction*>* ReadLFG(string LFGFileName);

	list<CRawProduction*>* ReadCFG(string CFGFileName);

    //##ModelId=4088BB40029F
    vector<CProduction*>* GenerateCFG(string LFGFileName);

	vector<CSymbol*> * GetStackList(stack<CSymbol*> *  stk1);

	void AppendList(vector<CSymbol*> * first,vector<CSymbol*> * second);

	CLexicon * GetLexicon(){ return LexiconPtr;}
	
	wstring GetNewProdName(){return NewProdName;}

  private:
	vector<CSymbol*> * GetStackListRev(stack<CSymbol*> *  stk1);
	bool DeleteRawProdList(list<CRawProduction*>* plist);

	list<CRawProduction*>* GenerateSelectionSet(list<CRawProduction*> * vlist, map<wstring,bool>* nullables);

	map<wstring,bool>* Comp_Nullables( list<CRawProduction*> * vlist , map<wstring,bool>* nullables);

	map<wstring,bool>* Comp_Dir_Nullables( list<CRawProduction*> * vlist );

	bool IsNullRange(vector<CSymbol*>::iterator Start, vector<CSymbol*>::iterator End, map<wstring,bool>* nullables);
	//bool IsNullRange( CSymbol** Start, CSymbol** End, map<wstring,bool>* nullables);

	bool OperateUnion( map<wstring,int> * Src_Set , map<wstring,int> * Dest_Set );
	bool OperateUnion( map<wstring,int> * Src_Set , map<short,short> * Dest_Set ); //second argument is different used for selection set

	bool PreprocessGrammar(const string & grules_fname,
						   const string & macros_fname,
						   const string & cfg_fname,
						   bool ReplaceStarByRange=false);
    //##ModelId=406125D40294
    multimap<wstring,CProduction*> * ProductionTable;

	vector< list< CProduction *> > ProdTable;	//new

	//multimap<int,int> RelevantProductionList;
	
	deque<set<int>*> * RelevantProductionList;
	
	//Explanation:
	//1: S-1 -> A-31 B-32 C-33
	//2: A-31 -> M-34 N-35
	//3: B-32 -> C-33 N-35
	//RelevantProductionList will contain following:
	//ProductionSymbolId - ProductionNo
	//31 - 1
	//32 - 1
	//33 - 1
	//33 - 3
	//34 - 2
	//35 - 2
	//35 - 3 etc.

	void temp();	//new
	//to check whether new and old production tables point at same place or not
	

	wstring NewProdName;
//	CSymbol * DotPtr;
	CLexicon * LexiconPtr;

public:

	short GetNum(wstring Label);			//new
	void GetInfoFromProdTable();		//new
	void WriteStackInFile();				//new
	inline bool GetProdList(short LabelID , list<CProduction*> & InnerProdList);				//new
	bool RemoveORConstraints(void);
	bool RemoveORConstraintsEx(void);		//new

	bool ReplicateProduction(CProduction* prod, 
							int ConstraintIndex, 
							int ProdSymbIndex, 
							wstring & constraint,
							int OrSymbIndex);
	bool ReplicateProductionEx(CProduction* prod,	//new
							int ConstraintIndex, 
							int ProdSymbIndex, 
							wstring & constraint,
							int OrSymbIndex);
	
	bool WriteFullProductionsInFile(const string & file_name);

	bool GenerateRelevantProductionTable(void);
	bool GetRelvantProductions(int ProductionSymbolId, list<int> & ProdNumberList);
	bool GenerateSelectionSetForGenertionGrammar(void);

	CInternalSymbolTable IntSymbTab ;
	bool WriteProductionsInFileExtended(const string & FileName);
	bool AddUniqueProdNo(int ProdSymbId, int ProdId);
};




#endif /* CGRAMMAR_H_HEADER_INCLUDED_BF672D0D */
