// FileReader.cpp: implementation of the CFileReader class.
//
//////////////////////////////////////////////////////////////////////

#include "stdafx.h"
#pragma warning(disable: 4786)

#include "global.h"
#include "FileReader.h"

#include "LogManager.h"
/*#include ".\filereader.h" REMOVED IN THIS (LINUX) VERSION*/

//////////////////////////////////////////////////////////////////////
// Construction/Destruction
//////////////////////////////////////////////////////////////////////

int CFileReader::counter=0;

CFileReader::CFileReader(const char*FileName,short buffer_size)
:BufferSize(buffer_size)
{
	counter++;

	fptr=NULL;
	CurrentIndex=0;
	CurrentChar=0;	
	Buffer=new short[BufferSize];
	
	if(Buffer)
	{
		DataSize=0;
		OpenFile(FileName);
	}
}

CFileReader::CFileReader(short buffer_size)
:BufferSize(buffer_size)
{
	counter++;

	fptr=NULL;
	CurrentIndex=0;
	CurrentChar=0;	
	Buffer=new short[BufferSize];
	DataSize=0;	
}

CFileReader::~CFileReader()
{
	counter--;

	CloseFile();
	
	if(Buffer)
	{
		try{
		delete []Buffer;		
		Buffer=NULL;
		}
		catch(...)
		{
			#ifdef GenLog
			CLogManager::AddMessage(L"#### Buffer");
			#endif
		}

	}
}

short CFileReader::OpenFile(const char *FileName)
{
	if(fptr)
		fclose(fptr);

	fptr=fopen(FileName,"rb");
	
	if(!fptr)
	{
		wstring err=L"File '";		
		c2u(FileName,err);

		err+=L"' could not be opened....\n";
		/*MessageBox(NULL,err.c_str(),L"File Reader Component",1); REMOVING IN THIS (DOS/LINUX VERSION) */		

		return 0;
	}

	LoadBuffer();
	
	MoveNext();
	
return CurrentChar;
}


bool CFileReader::LoadBuffer()
{
	if(!fptr)
		return false;

	DataSize=fread(Buffer,sizeof(short),BufferSize,fptr);
	
	CurrentIndex=0;
	CurrentChar=0;
	
	if(DataSize>0)
		return true;	

return false;
}


short CFileReader::GetCurrentChar()
{
	return CurrentChar;
}


short CFileReader::MoveNext()
{
	CurrentIndex++;
	
	if(CurrentIndex<DataSize)
	{
		CurrentChar=Buffer[CurrentIndex];
		return CurrentChar;
	}

	if(LoadBuffer())
	{
		CurrentChar=Buffer[CurrentIndex];
		return CurrentChar;
	}

return 0;
}

wstring CFileReader::Read(int index, int size)
{
	fseek(fptr , index , SEEK_SET );
	
	wstring str=L"";
	short ch=-1;
	int buffindex = 0;

	int siz = fread( Buffer , sizeof(short) , BufferSize ,fptr);

	while( buffindex < size && buffindex < siz )
	{
		ch= Buffer[buffindex++];
		str+=ch;
	}
	
	return str;
}

wstring CFileReader::ReadAll(string FileName)
{
	this->OpenFile(FileName.c_str());
	
	wstring str=L"";
	short ch=-1;
	
	while((ch=GetCurrentChar())!=0)
	{
		str+=ch;
		this->MoveNext();
	}

	return str;
}

void CFileReader::CloseFile()
{
	if(fptr)
	{
		fclose(fptr);	
		fptr=NULL; /* ADDED IN THIS (LINUX) VERSION */
	}
}


bool CFileReader::ReadAndTokenize(const string & filename, 
								  const string & delimlist, 
								  list<wstring> & datalist)
{
	char DelimArr[256]={0};
	short delim=0;
	size_t len=delimlist.size();

	/* for(size_t k=0;k<len;k++) CHANGED IN THIS (LINUX) VERSION*/
	size_t k;
	for(k=0;k<len;k++)
	{
		delim=delimlist[k];
		if(delim>=0 && delim<=255)
			DelimArr[delim]=1;
	}

	wstring all=ReadAll(filename);
	wstring tok=L"";
	
	len=all.size();

	for(k=0;k<len;k++)
	{
		delim=all[k];
		
		if(delim>=0 && delim<=255 && DelimArr[delim] && tok!=L"")
		{
			tok=TrimString(tok);
			datalist.push_back(tok);
			tok=L"";
			continue;
		}	
		
		tok+=delim;
	}

	if(tok!=L"")
	{
		tok=TrimString(tok);
		datalist.push_back(tok);
	}
	
	return true;
}
