#include "stdafx.h"

#include "Generator.h"
#include "global.h"
#include "GenOutput.h"

#include "LogManager.h"

#include <algorithm>
#include "CSymbolTableManager.h"
#include "Generator.h"


using namespace std;

extern long EvaluateBooleanExpressionEx(string s);

typedef pair<CFStructureItem*,CFStructureItem*> fslinkpair;
typedef pair<int,CChartItem*> intcitem_pair;

#define QUOTE 39

CGenerator::CGenerator(CMapper*mapper, 
				   CLexicon * ULexicon, 
				   CGrammar * UGrammarMngr,
				   CMorphologicalGenerator * MorphGen)					   
{	
	
	if(1)
	{
	/*	ProductionMerger Merger;
		Merger.LoadProductions(CConfiguration::GetConfigInfo("UcfgFileName"));
		Merger.MergeProductions();
		Merger.WriteProductions();*/
	}


	an_attempt=NULL;
	UrduLexicon=ULexicon;
	UrduGrammarManager = UGrammarMngr;
	MapperInterface=mapper;
	UtilityInterface=mapper->UtilityInterface;
	MorphGenerator=MorphGen;

	typedef pair<short,short> IdPair;
	short id=0;

	///////////////////////////////////////////////////////////////////////////////////////
	// SpecialUnmarkedItemsFileName
	///////////////////////////////////////////////////////////////////////////////////////

	CFileReader fr(512);
	list<wstring> item_list;
	wstring str;

	fr.ReadAndTokenize(CConfiguration::GetConfigInfo("SpecialUnmarkedItemsFileName"),",",item_list);

	for(list<wstring>::iterator it=item_list.begin();it!=item_list.end();it++)
	{
		str=*it;
		id=CSymbolTableManager::GetNum(str,true);
		SpecialUnmarkedItems.insert(IdPair(id,id));
	}
		
	SpecialUnmarkedItems.insert(IdPair(CSymbolTableManager::PredId,CSymbolTableManager::PredId));
	SpecialUnmarkedItems.insert(IdPair(CSymbolTableManager::RootId,CSymbolTableManager::RootId));

	///////////////////////////////////////////////////////////////////////////////////////
	/// HEADLESS PREDICATES
	///////////////////////////////////////////////////////////////////////////////////////

	item_list.clear();
	fr.ReadAndTokenize(CConfiguration::GetConfigInfo("HeadlessPredicateItemsFileName"),",",item_list);

	for(list<wstring>::iterator it=item_list.begin();it!=item_list.end();it++)
	{
		str=*it;
		id=CSymbolTableManager::GetNum(str,true);
		HeadlessPredicateList.insert(IdPair(id,id));
	}		

	///////////////////////////////////////////////////////////////////////////////////////
	UrduGrammarManager->RemoveORConstraintsEx();
	///////////////////////////////////////////////////////////////////////////////////////

	SourceSymbolId=UrduGrammarManager->GetNum(L"S");

	//UrduGrammarManager.WriteProductionInFile("data\\UrduG.txt");	
}

CGenerator::~CGenerator()
{	
	CollectGarbage();	
}

list<wstring> CGenerator::GenerateSentencesEx(vector<CFStructure*> & TargetFSList)
{
	#ifdef GenLog
	CLogManager::AddMessage(L"Sentence Generator has started : \r\n");
	#endif
	
	list<CFStructure*> GeneratedFSList;
	list<wstring> SenList;

	CFStructure * fsptr=NULL;

	for(vector<CFStructure*>::iterator fit=TargetFSList.begin();fit!=TargetFSList.end();fit++)
	{
		fsptr=*fit;		
		Generator(fsptr,SenList,GeneratedFSList);
	}
	
	//LogManager.AddMessage(L"Block Count Before Generate Word : "+GetWSTR(FinalGenBlockList.size())+L"\r\n");
	
	//GenerateWords(FinalGenBlockList);		NEW
	
	//LogManager.AddMessage(L"Block Count After Generate Word : "+GetWSTR(FinalGenBlockList.size())+L"\r\n");
	
	//GenerateSentenceList(&FinalGenBlockList,&SenList);	NEW
	
	//LogManager.AddMessage(L"Block Count After Generate Sentence : "+GetWSTR(FinalGenBlockList.size())+L"\r\n");

	RefineSentences(&SenList);

	#ifdef GenLog
	CLogManager::AddMessage(L"Sentences Generated ("+GetWSTR(SenList.size())+L") :\r\n");
	#endif

	wstring sen;

	for(list<wstring>::iterator sit=SenList.begin();sit!=SenList.end();sit++)
	{
		sen=*sit;
		#ifdef GenLog
		CLogManager::AddMessage(sen+L" \r\n");
		#endif
	}

	/*REMOVED FOR THIS (DOS/LINUX) VERSION
	#ifdef GenLog
	GenerateFStructuresXML(GeneratedFSList);
	#endif
	REMOVED FOR THIS (DOS/LINUX) VERSION*/
	return SenList;
}
/*
bool CGenerator::ProduceGenerationBlocks(CFStructure * TargetFS, 								   
								   list<CGenerationBlock*> & FinalGenBlockList)
{
	using namespace std;			

	if(!TargetFS)
		return false;		
	
	vector<CProduction*> ProdList;			

	wstring label=L"ROOT";	
	wstring errmsg=L"";
	CCStructureItem * node=new CCStructureItem(label,L"",NULL,TargetFS->Root,NULL);
	CCStructure * tree=new CCStructure(node);

	CFStructureItem * tnode=NULL;
	CFStructureItem * gfnode=NULL;
	wstring tstr=L"";
	
	stack<CGenerationBlock* > BackupStack;
	CGenerationBlock*block=new CGenerationBlock(tree);			

	CProductionSymbol*tprod=NULL;
	CProductionSymbol*SProd=new CProductionSymbol();
	SProd->Label=L"S";
	SProd->isTerminal=false;
	
	CGenerationItem*temp=new CGenerationItem(TargetFS->Root,NULL,SProd,node);	
	block->GenStack.push(temp);
	BackupStack.push(block);
	

	bool BlockExecuted=true;
	int mycounter=0;
	short NodeId=0;

	while(!BackupStack.empty() )
	{
		//mycounter++;

		block=BackupStack.top();
		BackupStack.pop();

		while(!block->GenStack.empty() && BlockExecuted)	
		{
			temp=block->GenStack.top();
			block->GenStack.pop();
			
			if(temp->ProdSymbPtr)
			{				
				tprod=temp->ProdSymbPtr;				

				//---not---Evaluate Production Symbol here to limit the productions
				
				//tprod->conditions
				
				if(tprod->isTerminal)
				{	
					tstr=L"";	
					tnode=NULL;
					//generate a node and add this node to current tree
					if(temp->UpFSPtr)
					{						
						NodeId=CSymbolTableManager::GetNum(tprod->Label,true);
						
						if(HeadlessPredicateList.find(NodeId)==HeadlessPredicateList.end())
						{						
							tnode=temp->UpFSPtr->SearchEx(CSymbolTableManager::PredId);						
							if(tnode)
							{
								gfnode=tnode->SearchEx(CSymbolTableManager::GFId);
								tnode=tnode->SearchEx(CSymbolTableManager::ValueId);						

								if(tnode)
									tstr=CSymbolTableManager::GetStringByList(tnode->ValueList,L" ");
								if(gfnode)
								{	
									tstr=RemoveCharFromWSTRING(tstr,39);//single quote's ASCII							
									tstr=L"'"+tstr+L"<"+CSymbolTableManager::GetStringByList(gfnode->ValueList,L",")+L">'";
								}
							}
							else
								tstr=L"";
						}
						else
						{
							//tstr=MapperInterface->LexMngr->LexPtr->GetConstantName(temp->FSPtr->Label);
							tstr=tprod->Label;
						}
					
						node=new CCStructureItem(tprod->Label,tstr,NULL,temp->UpFSPtr,NULL);					
						temp->CurrentTreeNode->AddNewChild(node);	
						BlockExecuted=true;
					}
					else
					{
						BlockExecuted=false;
					}
                    			

					//CLogManager::AddMessage(L"Terminals: "+tstr+L"\r\n");
				}
				else
				{	
					ProdList=GetProductionListEx(tprod,temp->UpFSPtr);
					//ProdList=UrduGrammarManager.GetProductions((tprod->Label);
					if(ProdList.size()==0)
					{
						//No Production could be retrieved after matching the
						//conditions

						//errmsg=L"Grammar Manager could not find the Production for ";
						//errmsg+=tprod->Label;
						//AfxMessageBox(errmsg.c_str(),1,1);
						BlockExecuted=false;
						break;
					}
					else
					{	
						node=new CCStructureItem(tprod->Label,L"",NULL,temp->UpFSPtr,NULL);
						temp->CurrentTreeNode->AddNewChild(node);
						temp->CurrentTreeNode=node;						
						
						BlockExecuted=MakeGenerationBlockItem(&BackupStack,block,temp,&ProdList);
						
						//
						//if(BlockExecuted)
						//	CLogManager::AddMessage(L"MakeGenerationBlockItem("+tprod->Label+L"): TRUE \r\n");							
						//else
						//	CLogManager::AddMessage(L"MakeGenerationBlockItem("+tprod->Label+L"): FALSE \r\n");							
						//
						  
					}
				}
			}

			delete temp;

		}//end while(block)
		
		if(block->Tree->Root && BlockExecuted && !block->IsIncomplete)
		{
			FinalGenBlockList.push_back(block);
			//CLogManager::AddMessage(L"Added in Final Block List: "+block->Tree->Root->Label+L"\r\n");
		}
		else
			delete block;

		BlockExecuted=true;
	}
	
	return true;
}

bool CGenerator::MakeGenerationBlockItem(stack<CGenerationBlock*>* BkpStk, 
										 CGenerationBlock* GenBlock,
										 CGenerationItem*CurrentGenItem,
										 vector<CProduction*>* ProdList)
{
	CProduction*prod=NULL;
	CProductionSymbol*prodsymb=NULL;
	CGenerationItem*gitem=NULL;

	if(!ProdList || ProdList->size()==0)
		return false;

	if(ProdList->size()==1)
	{
		prod=ProdList->at(0);
		if(!LoadGenItemFromProduction(prod,GenBlock,CurrentGenItem))
			return false;
		return true;
	}	

	//production list has multiple productions for a sybmol so
	//1. Make N-1 Clones of Current Stack (GenBlock) and
	//2. Make Generation Items for each production in each block

	CGenerationBlock*GenBlockClone=NULL;
	CGenerationItem*CurrentGenItemClone=NULL;

	vector<CProduction*>::iterator it=ProdList->begin();
	CProduction*firstprod=*it;
	it++;
	
	CGenerationItem*NewCurrGenItem=NULL;

	for(;it!=ProdList->end();it++)
	{		
		prod=*it;
		
		GenBlockClone=GenBlock->GetClone(CurrentGenItem,&CurrentGenItemClone);
		
		if(!LoadGenItemFromProduction(prod,GenBlockClone,CurrentGenItemClone))
		{
			//de-allocate GenBlockClone
			delete GenBlockClone;
			continue;
		}
		
		//CLogManager::AddMessage(L"SYMBOL EXECUTED: " + CurrentGenItemClone->CurrentTreeNode->Label  );
		BkpStk->push(GenBlockClone);
	}

	if(!LoadGenItemFromProduction(firstprod,GenBlock,CurrentGenItem))
		return false;
    
	return true;
}


bool CGenerator::LoadGenItemFromProduction(CProduction * prod, 
										   CGenerationBlock* GenBlock,
										   CGenerationItem*CurrentGenItem)
{
	using namespace std;

	if(!prod || !GenBlock || !CurrentGenItem)
		return false;

	//CProductionSymbol * FSLHS=CurrentGenItem->ProdSymbPtr;
	
	CFStructureItem*FSPtr=CurrentGenItem->UpFSPtr;
	//CFStructureItem*tempFSPtr=NULL;

	CProductionSymbol*prodsymb=NULL;
	CGenerationItem*gitem=NULL;

	CFDescription*fdesc=NULL;
	wstring cond=L"";

	prod->PrintProductionF(CLogManager::GetLogFileRef(),L"Production Loaded : ");

	vector<CProductionSymbol*>::reverse_iterator rit=prod->RHS.rbegin(); //  Ending DOT
	rit++;

	//vector<CProductionSymbol*>::iterator simple_it=prod->RHS.begin();

	int found=-1;
	
	//
	//bool RightRecursive=false;
	//if(rit!=prod->RHS.rend())
	//{
	//	prodsymb=*rit;
	//	if(prodsymb->Label==prod->LHS->Label)
	//		RightRecursive=true;
	//}
	//

	vector<CFDescription*>::iterator fit;
	bool AllowLinking = false;

	for(;rit!=prod->RHS.rend();rit++)
	//for(;simple_it!=prod->RHS.end();simple_it++)
	{
			//prodsymb=*simple_it;
			
			//if(prodsymb->Label==L".")
			//	break;

			prodsymb=*rit;						
			AllowLinking = false;				

			// debugging
			if(prodsymb->Label==L"TenseAuxP" || prodsymb->Label==L"tense_aux")
			{
				//just to enter
				AllowLinking = false;			
			}
			
			if(prodsymb->conditions.size()==0)
			{
				CurrentGenItem->DownFSPtr=new CFStructureItem(-1);
			}
			else
			{
				fit=prodsymb->conditions.begin();
	
				//1- Pointer Setter //done
				//CurrentGenItem->FSPtr->isMarked=true;
			
				CurrentGenItem->DownFSPtr=GetRelevantFSPtr(CurrentGenItem->UpFSPtr,prodsymb);

				
				//if(RightRecursive && !tempFSPtr)
				//{
				//RightRecursive=false;
				//	continue;				
				//}
			
			
				if(!CurrentGenItem->DownFSPtr)
				{					
					//CLogManager::AddMessage(L"Relevant Pointer is not found for "+prodsymb->Label+L"\r\n");

					CurrentGenItem->DownFSPtr=new CFStructureItem(-1);
					AllowLinking = true;
					
					if(fit!=prodsymb->conditions.end())
					{							
						fdesc=*fit;
						if (!fdesc->Label || !fdesc->Value)
						{
							CLogManager::AddMessage(L"Why: ");
							LogUnificationError(fdesc);							
						}
						else if(!SetUpAndDownFStructures(fdesc,CurrentGenItem,AllowLinking))
							//if(!ApplyUnification(fdesc,CurrentGenItem->UpFSPtr,tempFSPtr))
						{
							LogUnificationError(fdesc);
							return false;
						}
					}
					else
					{
						CLogManager::AddMessage(L"No F-Description attached with Production Symbol : "+prodsymb->Label);
						return false;
					}
										
					//new things will be here.........
					//return false;
				}

				//first f-description is processed...
				fit++;
				/////////////////////////////////////

			}						
			

			
			//if(!tempFSPtr)			
			//	tempFSPtr=CurrentGenItem->FSPtr;
			
			
			if(CurrentGenItem->DownFSPtr)
				CurrentGenItem->DownFSPtr->isMarked=true;

			//tempFSPtr->isMarked=true;

			/////////////////EVALUATE F-DESCRIPTION and CONSTRAINTS//////////////////
			
            /////////////// APPLY CONSTRAINTS //////////////////
			
			//2- Apply Constraints
			
			for(vector<wstring>::iterator cit=prodsymb->constraints.begin();
				cit!=prodsymb->constraints.end();cit++)
			{
				cond=*cit;

				if(!EvaluateExpressionForGenerator(cond,CurrentGenItem->UpFSPtr,CurrentGenItem->DownFSPtr))
				{
					CLogManager::AddMessage(L"Constraint Failed: "+cond+L"\r\n");
					return false;
				}							
			}
			
			//apply unification			
			                        
			if(prodsymb->conditions.size()!=0)
			{				
				for(;fit!=prodsymb->conditions.end();fit++)
				{
					fdesc=*fit;
					if(!SetUpAndDownFStructures(fdesc,CurrentGenItem,AllowLinking))
					//if(!ApplyUnification(fdesc,CurrentGenItem->UpFSPtr,tempFSPtr))
					{
						LogUnificationError(fdesc);
						return false;
					}
				}
			}

												
			//3- Apply Unifications
			//UtilityInterface->UnifyForGenerator(
			//Evaluate F-DESCRIPTION			

			///////////////// EVALUATE F-DESCRIPTION and CONSTRAINTS //////////////////
			
			//tempFSPtr->isMarked=true;

			if(CurrentGenItem->DownFSPtr)
				CurrentGenItem->DownFSPtr->isMarked=true;

			gitem=new CGenerationItem(	CurrentGenItem->DownFSPtr,
										NULL,
										prodsymb,
										CurrentGenItem->CurrentTreeNode);
			
			//if(CurrentGenItem->DownFSPtr)
			//	CurrentGenItem->DownFSPtr->isMarked=true;

			GenBlock->GenStack.push(gitem);
	}

	return true;
}*/


vector<CProduction*> CGenerator::GetProductionListEx(CProductionSymbol* ProdSymb, 
													 CFStructureItem* FSPtr)
{
	using namespace std;

	vector<CProduction*> plist;	

	if(!ProdSymb||!FSPtr)
		return plist;
	
	vector<CProduction*>*list=UrduGrammarManager->GetProductions(ProdSymb->Label);

	if(!list)
		return plist;
		
	CProduction*prod=NULL;	

	for(vector<CProduction*>::iterator pit=list->begin();pit!=list->end();pit++)
	{
		prod=*pit;
		
		//ignoring NULL productions
		//if (prod->RHS.size()==1)
		//	continue;

		if(CheckProductionValidity(prod,FSPtr))		
			plist.push_back(prod);		
	}

	return plist;
}

bool CGenerator::CheckProductionValidity(CProduction* prod, CFStructureItem* FSPtr)
{
	using namespace std;

	if(!prod || !FSPtr)
		return false;

	if(prod->RHS.size()==1) // Acknowledging NULL Production
		return true;

	CProductionSymbol*psymb=NULL;
	CFDescription*fdptr=NULL;	
	CFStructureItem*fsitm=NULL;		

	for(vector<CProductionSymbol*>::iterator psit=prod->RHS.begin();psit!=prod->RHS.end();psit++)
		{			
			psymb=*psit;

			for(vector<CFDescription*>::iterator fdit=psymb->conditions.begin();
				fdit!=psymb->conditions.end();fdit++)
			{
				fdptr=*fdit;
				
				if(fdptr->Label && fdptr->Value)
				{					
					// Handling Cases:
					// ^   = !
					// ^___= !

					if(fdptr->Operator==UNIFY_OP && fdptr->Value->op==DOWN_OP && 
					   fdptr->Value->list.size()==0 && fdptr->Label->list.size()!=0)
					{
						if(!FSPtr->Search(&fdptr->Label->list))
							return false;
					}

					// Handling Cases:
					// ! $ ^
					// ! $ ^ ____

					if(fdptr->Operator==SET_OP && fdptr->Label->op==DOWN_OP && 
					   fdptr->Label->list.size()==0 && fdptr->Value->list.size()!=0)
					{
						if(!FSPtr->Search(&fdptr->Value->list))
							return false;
					}
													
				}				
			}		
		}

	return true;
}


CFStructureItem* CGenerator::GetRelevantFSPtr(CFStructureItem * FSPtr, 
											  CProductionSymbol* psymb)
{
	if(!FSPtr || !psymb)
		return NULL;

	//FSPtr->isMarked=true;
	//subject to change

	CFStructureItem * ReqPtr=NULL;
	CFDescription * fdptr =NULL;

	for(vector<CFDescription*>::iterator fdit=psymb->conditions.begin();
			fdit!=psymb->conditions.end();fdit++)
	{
			fdptr=*fdit;
			
			if(fdptr->Label && fdptr->Value)
			{					
				// Handling Cases:
				// ^   = !
				// ^___= !

				if(fdptr->Operator==UNIFY_OP && fdptr->Value->op==DOWN_OP && 
					fdptr->Value->list.size()==0)
				{
					if(fdptr->Label->list.size()!=0)
					{
						ReqPtr=FSPtr->Search(&fdptr->Label->list);
						if(ReqPtr)
						{
							ReqPtr->isMarked=true;
							return ReqPtr;
						}
					}
					else
					{
						FSPtr->isMarked=true;
						return FSPtr;
					}
				}

				// Handling Cases:
				// ! $ ^
				// ! $ ^ ____

				if(fdptr->Operator==SET_OP && fdptr->Label->op==DOWN_OP && 
					fdptr->Label->list.size()==0)
				{
					if(fdptr->Value->list.size()!=0)
					{						
						ReqPtr=FSPtr->Search(&fdptr->Value->list);
						
						if(ReqPtr)
						{
							ReqPtr->isMarked=true;
							ReqPtr=GetSetNextElement(ReqPtr);
							if(ReqPtr)
								ReqPtr->isMarked=true;
							return ReqPtr;
						}
					}
					else
					{
						FSPtr->isMarked=true;
						ReqPtr=GetSetNextElement(FSPtr);
						if(ReqPtr)
								ReqPtr->isMarked=true;
						return ReqPtr;
					}
				}
												
			}				
		}		

	return NULL;
}

/* REMOVED FROM THIS (DOS/LINUX) VERSION
void CGenerator::GenerateCStructuresXML(list<CGenerationBlock*> & BlockList)
{
	using namespace std;		
									
	wstring cstr=L"C-Structure-";
	wstring str;
	
	short counter=1;
	wstring counter_str;
	
	CCStructureItem*nroot=NULL;
				
	CFileWriter cs_fw("data\\cs1.xml");
	
	cs_fw.Write(L"<C-Structures>");	

	CGenerationBlock*GenBlock=NULL;
		
	for(list<CGenerationBlock*>::iterator it=BlockList.begin(); it!=BlockList.end();it++)
		{			
			GenBlock=*it;
			
			if(GenBlock && !GenBlock->IsIncomplete)
			{
				counter_str=GetWSTR(counter);
				
				str=cstr;
				str+=counter_str;					
				
				cs_fw.Write(L"<"+str+L">");	
				//cs_fw.Write(L"<S>");	
				GenerateCStructuresXMLRec(GenBlock->Tree->Root,&cs_fw);
				//cs_fw.Write(L"</S>");		
				cs_fw.Write(L"</"+str+L">");	
										
				counter++;								

			}

		}		  		

		cs_fw.Write(L"</C-Structures>");	
		cs_fw.CloseFile();		
}
REMOVED FROM THIS (DOS/LINUX) VERSION*/

/*REMOVED FROM THIS (DOS/LINUX) VERSION
bool CGenerator::GenerateCStructuresXMLRec(CCStructureItem* CSPtr,CFileWriter * FilePtr)
{
	if(!CSPtr || !FilePtr)
		return false;
	
	FilePtr->Write(L"<"+CSPtr->Label+L">");
	wstring tstr=L"";
	
	if(CSPtr->ChildList.size()==0)
	{
		//FilePtr->Write(L"<VALUE>"+CSPtr->Value+L"</VALUE>");
		tstr=MakeXMLCompatible(CSPtr->Value);
		FilePtr->Write(tstr);
	}
	else
	{
		for(vector<CCStructureItem*>::iterator it=CSPtr->ChildList.begin();
			it!=CSPtr->ChildList.end();it++)
		{
			GenerateCStructuresXMLRec(*it,FilePtr);
		}
	}

	FilePtr->Write(L"</"+CSPtr->Label+L">");

return true;
}
*/
/*
void CGenerator::GenerateWords(list<CGenerationBlock*> & BlockList)
{
	using namespace std;													
	CGenerationBlock*GenBlock=NULL;

	int junk=1;
	
	list<CGenerationBlock*>::iterator it=BlockList.begin(); 	

	//while(it!=BlockList.end())
	
	for(;it!=BlockList.end();it++)
	{			
		junk++;
		
		if(junk==500)
			break;

		GenBlock=*it;			

		if(GenBlock->IsIncomplete)
			continue;
				
		if(!CheckFStructureIsMarked(GenBlock->Tree->Root->UpFSLink))
		{
			GenBlock->IsIncomplete=true;						
			continue;
		}
		
		if(!GenerateWordsRec(GenBlock->Tree->Root,GenBlock,BlockList))
		{
			GenBlock->IsIncomplete=true;					
			continue;
		}


	}		
}*/

/*bool CGenerator::GenerateWordsRec(CCStructureItem* CSPtr,CGenerationBlock*GenBlock,list<CGenerationBlock*> & BlockList)
{		
	if(GenBlock->IsIncomplete)
		return false;

	CCStructureItem*csptr=NULL;

	/*if(CSPtr->Label==L"VerbalP")
	{
		CSPtr->Label=L"VerbalP";
	}*/		
	
	/*if(CSPtr->ChildList.size()>0)
	{
		for(vector<CCStructureItem*>::iterator it=CSPtr->ChildList.begin();
			it!=CSPtr->ChildList.end();it++)
		{
			csptr=*it;			
			if(!GenerateWordsRec(csptr,GenBlock,BlockList))
				return false;
		}
	}	
	else 
	{	
		/*
		Temporary productions or Null productions may have empty values
		if(CSPtr->Value==L"")
		{
			CLogManager::AddMessage(L"Word Generation: Node '" + CSPtr->Label + L"' has empty value?\r\n");
			return false;
		}*/
		
		/*
		if(CSPtr->Label==L"tense_aux")
		{
			CSPtr->Label=L"tense_aux";
		}
		*/

		/*if (!CSPtr->Visited && CSPtr->Value!=L"")
		{
			if(!GenerateWord(CSPtr,GenBlock,BlockList))
				return false;
		}
				
		CSPtr->Visited=true;
	}
	
return true;
}*/


/*bool CGenerator::GenerateWord(CFStructure* GenFS, list<CFStructure*> & WordList, wstring POS)
//bool CGenerator::GenerateWord(CCStructureItem* OrigCSPtr,
//							  CGenerationBlock*GenBlock,
//							  list<CGenerationBlock*> & BlockList)
{
	CCStructureItem* CSPtr=OrigCSPtr;

	if(!CSPtr)
		return false;
	
	vector<CLexicalEntry*> * vlist=NULL;
	list<CLexicalEntry*> * vlist2=NULL;

	//temporary
	wstring tofind=TrimString(CSPtr->Value);	

	///added just for debugging............
	//if(tofind==L"asp_aux")
	//{
	//	tofind=tofind;
	//}
	
	if(tofind==L"")
	{
		CSPtr->Visited=true;		
		return true;
	}
	
	//if(tofind==L"'pro'")
	//	tofind=L"pro";
	
	bool EntriesFound=false;
	//vlist=UrduLexicon.GetLexicalEntryList(tofind);
	vlist=UrduLexicon.GetLexicalEntryListUsingPOS(tofind,CSPtr->Label,EntriesFound);
	
	//if no lexical entry found 
 	if(!vlist || vlist->size()==0)
	//No Lexical Entry Found, Using Morphological Generator
	{		
		if(EntriesFound)
			return false;

		//MorphGenerator.GenerateForms(CSPtr->Value,CSPtr->Label,NULL);
		vlist=this->GenerateLexEntriesByMorphGenerator(CSPtr);
		CSPtr->Visited=true;
		
		if(!vlist)
			return true;
	}

	///added just for debugging............
	//if(tofind==L"asp_aux")
	//{
	//	tofind=tofind;
	//}
	///////////////////////////////////////

	size_t ListSize=vlist->size();	
	
	CLexicalEntry*le=NULL;
	wstring constraint;
	bool stopped=false;
	
	CFDescription * fdesc=NULL;

	if(ListSize==1)
	{
		le=vlist->front();
		stopped=false;
		
		//checking constraints
		for(list<wstring>::iterator sit=le->ConstraintList.begin();
			sit!=le->ConstraintList.end();sit++)
		{
			constraint=*sit;		
			if(!UtilityInterface->EvaluateExpression(constraint,CSPtr->UpFSLink,CSPtr->UpFSLink))
			{
				stopped=true;
				break;
			}
		}
		
		if(stopped)
		{
			GenBlock->IsIncomplete=true;		
			return false;
		}
			
		for(list<CFDescription*>::iterator cit=le->ConditionList.begin();
			cit!=le->ConditionList.end();cit++)
		{
			fdesc=*cit;					
			if(fdesc->Label->list.at(0)!=CSymbolTableManager::PredId)
			{
				if(!ApplyUnification(fdesc,CSPtr->UpFSLink,CSPtr->UpFSLink))
				{								
					stopped=true;
					break;
				}
			}
		}
		
		if(stopped)
		{
			GenBlock->IsIncomplete=true;		
			return false;
		}
				
		CSPtr->LexEntry=le;		
		CSPtr->Visited=true;

		return true;
	}
	
	CGenerationBlock * OrigBlk=NULL;
	CGenerationBlock * NewBlk=NULL;
	CCStructureItem * NewCSPtr=NULL;
	CCStructureItem * temp=NULL;

	for(vector<CLexicalEntry*>::iterator it=vlist->begin();it!=vlist->end();it++)
	{ 
		le=*it;
		stopped=false;
		//checking constraints
		for(list<wstring>::iterator sit=le->ConstraintList.begin();sit!=le->ConstraintList.end();sit++)
		{
			constraint=*sit;		
			if(!UtilityInterface->EvaluateExpression(constraint,CSPtr->UpFSLink,CSPtr->UpFSLink))
			{
				stopped=true;
				break;
			}
		}
		
		if(stopped)
		{			
			//return false;
			GenBlock->IsIncomplete=true;
			continue;
		}

		if(!OrigBlk)
		{
			OrigBlk=GenBlock->GetCloneWithClonedFSInfo(CSPtr,&NewCSPtr);
			//OrigBlk=GenBlock->GetCloneEx(GetCloneWithClonedFSInfo(CSPtr,&NewCSPtr);
			
			//swapping....pointer addresses...
			temp=CSPtr;
			CSPtr=NewCSPtr;
			NewCSPtr=temp;

			NewBlk=GenBlock;			
		}
		else
		{
			NewBlk=OrigBlk->GetCloneWithClonedFSInfo(CSPtr,&NewCSPtr);				
		}
		
		if(!NewBlk)
			continue;

		NewBlk->IsIncomplete=true;
		NewCSPtr->Visited=true;
					
		for(list<CFDescription*>::iterator cit=le->ConditionList.begin();cit!=le->ConditionList.end();cit++)
		{
			fdesc=*cit;		
			
			if(fdesc->Label->list.at(0)!=CSymbolTableManager::PredId)
			{
				if(!ApplyUnification(fdesc,NewCSPtr->UpFSLink,NewCSPtr->UpFSLink))
				{								
					stopped=true;
					break;
				}
			}
		}
		
		if(stopped)
		{
			if(NewBlk!=GenBlock)
				delete NewBlk;
			else
				GenBlock->IsIncomplete=true;
			continue;
		}
		
		NewCSPtr->Visited=true;
		NewBlk->IsIncomplete=false;
		NewCSPtr->LexEntry=le;		

		//CLogManager::AddMessage(tofind+L" : "+le->Word+L"\r\n");

		NewCSPtr->Visited=true;
		
		if(NewBlk!=GenBlock)
		{
			BlockList.push_back(NewBlk);
		}		
	}

	if(OrigBlk)
		delete OrigBlk;

	delete vlist;	//deleting dynamic vector only..not the contents in it.      			
		
return true;*/


///////////////////////////////////////////////////			NEW CODE
bool CGenerator::GenerateWord(CChartItem * CurrentChartItem, 							  
							  CProductionSymbol*ProdSymb)
{
	vector<CLexicalEntry*> * vlist=NULL;
	list<CLexicalEntry*> * vlist2=NULL;
	
	bool check=false;
	CFStructureItem* Temp = NULL;

	CFStructureItem* GenFS=ProdSymb->GenFSPtr;
	wstring tofind=L"";
	bool EntriesFound=false;

	GenFS->isMarked=true;
	CFStructureItem *parent;

				

	if(HeadlessPredicateList.find(ProdSymb->LabelID)==HeadlessPredicateList.end())
	{
		GenFS->isMarked=true;

		CFStructureItem* pred_ptr = GenFS->SearchEx(CSymbolTableManager::PredId);
		
		if(!pred_ptr)// || pred_ptr->isMarked)
			return false;

		pred_ptr->isMarked=true;

		pred_ptr=pred_ptr->SearchEx(CSymbolTableManager::ValueId); 

		if(!pred_ptr)
			return false;

		pred_ptr->isMarked=true;

		tofind = CSymbolTableManager::GetString(pred_ptr->ValueList.at(0));				
		for(int n=1;n<pred_ptr->ValueList.size();n++)
		{
			tofind = tofind+L" "+CSymbolTableManager::GetString(pred_ptr->ValueList.at(n));
		}
		
		if(tofind[0]==QUOTE)
			tofind=tofind.substr(1,tofind.size()-2);
	}
	else
		tofind=ProdSymb->Label;
		
	vlist=UrduLexicon->GetLexicalEntryListUsingPOS(tofind,ProdSymb->Label,EntriesFound);			

	#ifdef GenLog
	CLogManager::AddMessage(L"Searched Lexical Entries for : "+tofind+L" as "+ProdSymb->Label+L" in "+CSymbolTableManager::GetString(GenFS->Label)+L" Node.");		
	#endif

	if(vlist && vlist->size()>0 && HeadlessPredicateList.find(ProdSymb->LabelID)==HeadlessPredicateList.end())
	{
		vector<CLexicalEntry*> * good_vlist=new vector<CLexicalEntry*>;
		CFStructureItem* temp=GenFS->SearchEx(CSymbolTableManager::PredId);
		temp=temp->SearchEx(CSymbolTableManager::GFId);
		if(temp)
		{
			vector<short> one;
			one.resize(1);
			bool gf_found=true;
			for(int lenum=0;lenum<vlist->size();lenum++)
			{
				bool gf_found=true;
				for(int x=1;x<(*(*vlist).at(lenum)->ConditionList.begin())->Value->list.size();x++)
				{
					one[0]=(*(*vlist).at(lenum)->ConditionList.begin())->Value->list[x];
					if(!IsSubSet(&one,&temp->ValueList))
					{
						gf_found=false;
						break;
					}
				}
				if(gf_found && IsSubSet(&temp->ValueList,&(*(*vlist).at(lenum)->ConditionList.begin())->Value->list))
					good_vlist->push_back(vlist->at(lenum));

				//temp->IsEqual(*vlist->at(lenum)->ConditionList[i]->Value->list[x]);
			}
			temp=NULL;
			vlist->clear();
			vlist=good_vlist;
		}
	}	

	//if no lexical entry found 
 	if(!vlist || vlist->size()==0)
	//No Lexical Entry Found, Using Morphological Generator
	{		
		if(vlist)
		{
			delete vlist;
			vlist=NULL;
		}

		if(EntriesFound)
			return false;

		//vlist=this->GenerateLexEntriesByMorphGenerator(tofind,CSymbolTableManager::GetNum(ProdSymb->Label),GenFS);
		
		vlist=this->GenerateLexEntriesByMorphGenerator(tofind,ProdSymb->Label,GenFS);
		
		if(!vlist)
		{
			if(GenFS)
			{
				parent = GenFS->GetParent();
				while(parent && !parent->isMarked)
				{
					parent->isMarked = true;
					parent = parent->GetParent();
				}
			}
			return true;
		}
	}
	

	size_t ListSize=vlist->size();	
	
	CLexicalEntry*le=NULL;
	wstring constraint;
	bool stopped=false;
	
	CFDescription * fdesc=NULL;
	CProduction       * prod=NULL;
	CProduction       * new_prod=NULL;
		
	CChartItem * NewChartItem=NULL;
	stopped=false;

	if(ListSize==1)
	{
		le=vlist->front();
		//checking constraints
		for(list<wstring>::iterator sit=le->ConstraintList.begin();
			sit!=le->ConstraintList.end();sit++)
		{
			constraint=*sit;		
			if(!UtilityInterface->EvaluateExpression(constraint,GenFS,GenFS))
			{
				stopped=true;
				break;
			}
		}
	
		if(stopped)
		{	
			vlist->clear();
			delete vlist;
			vlist=NULL;
			return false;
		}
		
		for(list<CFDescription*>::iterator cit=le->ConditionList.begin();
			cit!=le->ConditionList.end();cit++)
		{
			fdesc=*cit;					
 			if(fdesc->Label->list.at(0)!=CSymbolTableManager::PredId)
			{
				if(!ApplyUnification(fdesc,GenFS,GenFS))
				{		
					LogUnificationError(fdesc);
					stopped=true;
					break;
				}
			}
		}		
	
		if(stopped)
		{
/*			if(OrigChartItem)
				delete OrigChartItem;*/
			vlist->clear();
			delete vlist;
			vlist=NULL;
			return false;			
		}
	
		prod=CurrentChartItem->Prod;
		if(ApplyConstraints(CurrentChartItem->FSPtr,GenFS,ProdSymb,false))
		{
			vector<CFDescription*>::iterator fdit=ProdSymb->conditions.begin();
			CFDescription*fdptr=NULL;
			//fdit++; //ignoring first pointer setter condition
			bool condition_failed=false;
			for(;fdit!=ProdSymb->conditions.end();fdit++)
			{
				fdptr=*fdit;
				if(!ApplyUnification(fdptr,CurrentChartItem->FSPtr,GenFS))
				{
					LogUnificationError(fdptr);
					condition_failed=true;
					break;
				}					
			}

			if(!condition_failed)
			{
				new_prod = prod->GetClone();
				ProdSymb = new_prod->GetCurrentSymbol();
				new_prod->MoveDot();
				ProdSymb->SenWord=le->Word;
				NewChartItem = new CChartItem(CurrentChartItem->FSPtr,new_prod,CurrentChartItem->StartIndex,CurrentChartItem->CurrentIndex + 1,true);
				Enqueue(&GenerationChart,NewChartItem,true);
			}
		}
		vlist->clear();
		delete vlist;
		vlist=NULL;
		
		if(GenFS)
		{
			parent = GenFS->GetParent();
			while(parent && !parent->isMarked)
			{
				parent->isMarked = true;
				parent = parent->GetParent();
			}
		}
		return true;
	}				
	else
	{

		CChartItem * OrigChartItem=NULL;
		OrigChartItem=CurrentChartItem->GetClone();
		CProductionSymbol * psymb=NULL;

		for(vector<CLexicalEntry*>::iterator it=vlist->begin();it!=vlist->end();it++)
		{ 
			le=*it;
			stopped=false;
		
			NewChartItem = OrigChartItem->GetClone();
			psymb=NewChartItem->Prod->GetCurrentSymbol();
			
			psymb->GenFSPtr=NULL;
			UpdateProductionFSPointers(NewChartItem);   //?????						
			GenFS=psymb->GenFSPtr;			
			
			if(GenFS)
			{
				GenFS->isMarked=true;
				parent = GenFS->GetParent();
				while(parent && !parent->isMarked)
				{
					parent->isMarked = true;
					parent = parent->GetParent();
				}
			}
			for(list<wstring>::iterator sit=le->ConstraintList.begin();sit!=le->ConstraintList.end();sit++)
			{
				constraint=*sit;		
				if(!UtilityInterface->EvaluateExpression(constraint,GenFS,GenFS))
				{
					stopped=true;
					break;
				}
			}
		
		if(stopped)
		{			
			NewChartItem->FSPtr=NULL;
			NewChartItem->Prod=NULL;
			delete NewChartItem;
			continue;
		}
		
		for(list<CFDescription*>::iterator cit=le->ConditionList.begin();cit!=le->ConditionList.end();cit++)
		{
			fdesc=*cit;		
			
			if(fdesc->Label->list.at(0)!=CSymbolTableManager::PredId)
			{
				if(!ApplyUnification(fdesc,GenFS,GenFS))
				{			
					LogUnificationError(fdesc);
					stopped=true;
					break;
				}
			}
		}
		
		if(stopped)
		{	
			NewChartItem->FSPtr=NULL;
			NewChartItem->Prod=NULL;		
			delete NewChartItem;
			continue;
		}
				
		prod=NewChartItem->Prod;
		psymb=prod->GetCurrentSymbol();

		if(ApplyConstraints(NewChartItem->FSPtr,GenFS,psymb,false))
		{
			vector<CFDescription*>::iterator fdit=psymb->conditions.begin();
			CFDescription*fdptr=NULL;
			//fdit++; //ignoring first pointer setter condition
			bool condition_failed=false;
			for(;fdit!=psymb->conditions.end();fdit++)
			{
				fdptr=*fdit;
				if(!ApplyUnification(fdptr,NewChartItem->FSPtr,GenFS))
				{
					LogUnificationError(fdptr);
					condition_failed=true;
					break;
				}					
			}

			if(!condition_failed)
			{
				prod->MoveDot();
				psymb->SenWord=le->Word;
				NewChartItem->StartIndex = CurrentChartItem->StartIndex;
				NewChartItem->CurrentIndex = CurrentChartItem->CurrentIndex + 1;
				Enqueue(&GenerationChart,NewChartItem,true);
			}
			else
			{
				NewChartItem->FSPtr=NULL;
				NewChartItem->Prod=NULL;		
				delete NewChartItem;
			}
		}
	}
	
	if(OrigChartItem)
	{
		OrigChartItem->FSPtr=NULL;
		OrigChartItem->Prod=NULL;		
		delete OrigChartItem;	
		OrigChartItem=NULL;
	}

	}
	
	if(vlist)
	{
		vlist->clear();
		delete vlist;
		vlist=NULL;
	}

	if(GenFS)
	{
		parent = GenFS->GetParent();
		while(parent && !parent->isMarked)
		{
			parent->isMarked = true;
			parent = parent->GetParent();
		}
	}

	return true;
}

/*REMOVED FROM THIS (DOS/LINUX) VERSION
void CGenerator::GenerateFStructuresXML(list<CFStructure*> & FSList)
{
	using namespace std;		
									
	wstring cstr=L"F-Structure-";
	wstring str;
	
	short counter=1;
	wstring counter_str;
	

	CFileWriter cs_fw("data\\fs1.xml");
	
	cs_fw.Write(L"<F-Structures>");	

	CFStructure* FS = NULL;

	for(list<CFStructure*>::iterator it=FSList.begin(); it!=FSList.end();it++)
		{			
			FS = *it;
			
			if(FS /*&& !FS->IsIncomplete)
/*			{
				counter_str=GetWSTR(counter);
				
				str=cstr;
				str+=counter_str;									
				cs_fw.Write(L"<"+str+L">");	
				//cs_fw.Write(L"<S>");	
				GenerateFStructuresXMLRec(FS->Root,&cs_fw);
				//cs_fw.Write(L"</S>");		
				cs_fw.Write(L"</"+str+L">");	
										
				counter++;								

			}

		}		  		

		cs_fw.Write(L"</F-Structures>");	
		cs_fw.CloseFile();		
}
REMOVED FROM THIS (DOS/LINUX) VERSION*/

/*REMOVED FROM THIS (DOS/LINUX) VERSION
bool CGenerator::GenerateFStructuresXMLRec(CFStructureItem* FSPtr,
										   CFileWriter * FilePtr)
{
	if(!FSPtr || !FilePtr)
		return false;
	
	wstring labname=CSymbolTableManager::GetString(FSPtr->Label);
		
	labname=MakeXMLCompatible(labname);

	FilePtr->Write(L"<"+labname+L">");

	wstring str;
	
	if(FSPtr->ChildList.size()==0)
	{		
		str=L"";

		vector<short>::iterator it=FSPtr->ValueList.begin();
		size_t vlen=FSPtr->ValueList.size();
				
		if(vlen==1)
			str=CSymbolTableManager::GetString(*it);
		else if(vlen>1)
		{
			str=CSymbolTableManager::GetString(*it);
			it++;
			for(;it!=FSPtr->ValueList.end();it++)
			{
				str+=L" , ";
				str+=CSymbolTableManager::GetString(*it);
			}
		}		

		str=MakeXMLCompatible(str);
		FilePtr->Write(str);
		//FilePtr->Write(L"<VALUE>"+str+L"</VALUE>");

	}
	else
	{
		for(vector<CFStructureItem*>::iterator it=FSPtr->ChildList.begin();
			it!=FSPtr->ChildList.end();it++)
		{
			GenerateFStructuresXMLRec(*it,FilePtr);
		}
	}

	FilePtr->Write(L"</"+labname+L">");

return true;
}
REMOVED FROM THIS (DOS/LINUX) VERSION*/

bool CGenerator::ApplyUnification(CFDescription   * fdesc,
								  CFStructureItem * fsup,
								  CFStructureItem * fsdown,
								  bool ReadOnlyMode)
{	
	if(!fdesc || !fsup || !fsdown)
		return false;
	
	if(!ReadOnlyMode)
	{
		fsup->isMarked=true;
		fsdown->isMarked=true;
	}

	CFStructureItem*fstemp1=NULL;
	CFStructureItem*fstemp2=NULL;
	char created1=0;
	char created2=0;
	if(fdesc->Operator==SET_OP)
	{
		fstemp1=fstemp2=NULL;

		if(fdesc->Label->op==UP_OP)
		{
			if(fdesc->Label->list.size()==0)
				fstemp1=fsup;
			else if(!ReadOnlyMode)
				fstemp1=fsup->SearchAndCreate(&fdesc->Label->list,created1);
		}
		else if(fdesc->Label->op==DOWN_OP)
		{
			if(fdesc->Label->list.size()==0)
				fstemp1=fsdown;
			else if(!ReadOnlyMode)
				fstemp1=fsdown->SearchAndCreate(&fdesc->Label->list,created1);
		}

		if(fdesc->Value->op==UP_OP)
		{
			if(fdesc->Value->list.size()==0)
			{
				fstemp2=fsup;fstemp2->isMarked=true;
				fstemp2=fstemp2->SearchEx(CSymbolTableManager::SetId);fstemp2->isMarked=true;
				fstemp2=fstemp2->SearchEx(fsdown->Label);fstemp2->isMarked=true;
			}
			else if(!ReadOnlyMode)
			{
				fstemp2=fsup->SearchAndCreate(&fdesc->Value->list,created2);fstemp2->isMarked=true;
				fstemp2=fstemp2->SearchAndCreate(CSymbolTableManager::SetId,created2);fstemp2->isMarked=true;
				fstemp2=fstemp2->SearchAndCreate(fsdown->Label,created2);fstemp2->isMarked=true;
			}
		}
		else if(fdesc->Value->op==DOWN_OP)
	    {
			if(fdesc->Value->list.size()==0)
			{
				fstemp2=fsdown;fstemp2->isMarked=true;
				fstemp2=fstemp2->SearchEx(CSymbolTableManager::SetId);fstemp2->isMarked=true;
				fstemp2=fstemp2->SearchEx(fsup->Label);fstemp2->isMarked=true;
			}
			else if(!ReadOnlyMode)
			{
				fstemp2=fsdown->SearchAndCreate(&fdesc->Value->list,created2);fstemp2->isMarked=true;
				fstemp2=fstemp2->SearchAndCreate(CSymbolTableManager::SetId,created2);fstemp2->isMarked=true;
				fstemp2=fstemp2->SearchAndCreate(fsup->Label,created2);fstemp2->isMarked=true;
			}
		}
		if(fstemp1 && fstemp2)
		{
			fstemp1->isMarked = true;
			fstemp2->isMarked = true;
			//CHANGE HERE, COMMENT LINE BELOW. LINK ALL.
			if(created1 && created2)
			{
				//linking two empty nodes..			
				fstemp1->AddNewLink(fstemp2);
				fstemp2->AddNewLink(fstemp1);
				#ifdef GenLog
				wstring labname=CSymbolTableManager::GetStringByList(fdesc->Label->list,L".");
				CLogManager::AddMessage(L"Linking Empty Nodes: " + labname + L"\r\n");
				#endif
			}
			else
			{				
				if(!UnifyForGenerator(fstemp2,fstemp1,NULL,false,ReadOnlyMode))
					return false;
			}
		}
		else if (fstemp1 && fdesc->Value->op==VALUE_OP)
		{
			if(fstemp1->Label==CSymbolTableManager::PredId)
			{
				fstemp1->isMarked=true;

				fstemp1=fstemp1->SearchEx(CSymbolTableManager::ValueId);				
				
				if(!fstemp1)
					return false;	
				
				fstemp1->isMarked = true;

				if(fstemp1->ValueList.size()!=0)
				{
					//CLogManager::AddMessage(L"SUPPRESSING OVERWRITING of PRED...\r\n");
					return true;
				}				
			}
			
			if(!UnifyForGenerator(fstemp1,NULL,&fdesc->Value->list,false,ReadOnlyMode))
				return false;
		}
	}

	if(fdesc->Operator==UNIFY_OP)
	{
		fstemp1=fstemp2=NULL;

		if(fdesc->Label->op==UP_OP)
		{
			if(fdesc->Label->list.size()==0)
				fstemp1=fsup;
			else if(!ReadOnlyMode)
				fstemp1=fsup->SearchAndCreate(&fdesc->Label->list,created1);
		}
		else if(fdesc->Label->op==DOWN_OP)
		{
			if(fdesc->Label->list.size()==0)
				fstemp1=fsdown;
			else if(!ReadOnlyMode)
				fstemp1=fsdown->SearchAndCreate(&fdesc->Label->list,created1);
		}

		if(fdesc->Value->op==UP_OP)
		{
			if(fdesc->Value->list.size()==0)
				fstemp2=fsup;
			else if(!ReadOnlyMode)
				fstemp2=fsup->SearchAndCreate(&fdesc->Value->list,created2);
		}
		else if(fdesc->Value->op==DOWN_OP)
	    {
			if(fdesc->Value->list.size()==0)
				fstemp2=fsdown;
			else if(!ReadOnlyMode)
				fstemp2=fsdown->SearchAndCreate(&fdesc->Value->list,created2);
		}
		
		if(fstemp1 && fstemp2)
		{
			fstemp1->isMarked = true;
			fstemp2->isMarked = true;
			//CHANGE HERE, COMMENT LINE BELOW. LINK ALL.
			if(created1 && created2)
			{
				//linking two empty nodes..			
				fstemp1->AddNewLink(fstemp2);
				fstemp2->AddNewLink(fstemp1);
				#ifdef GenLog
				wstring labname=CSymbolTableManager::GetStringByList(fdesc->Label->list,L".");
				CLogManager::AddMessage(L"Linking Empty Nodes: " + labname + L"\r\n");
				#endif
			}
			else
			{				
				if(!UnifyForGenerator(fstemp1,fstemp2,NULL,false,ReadOnlyMode))
					return false;
			}
		}
		else if (fstemp1 && fdesc->Value->op==VALUE_OP)
		{
			if(fstemp1->Label==CSymbolTableManager::PredId)
			{
				fstemp1->isMarked=true;

				fstemp1=fstemp1->SearchEx(CSymbolTableManager::ValueId);				
				
				if(!fstemp1)
					return false;	
				
				fstemp1->isMarked = true;

				if(fstemp1->ValueList.size()!=0)
				{
					//CLogManager::AddMessage(L"SUPPRESSING OVERWRITING of PRED...\r\n");
					return true;
				}				
			}
			
			if(!UnifyForGenerator(fstemp1,NULL,&fdesc->Value->list,false,ReadOnlyMode))
				return false;
		}
	}	
	
	return true;
}
CFStructureItem* CGenerator::GetSetNextElement(CFStructureItem* set_item, 
											   bool ReadOnlyMode)
{
	if(!set_item)
		return NULL;

	CFStructureItem*fs_set=NULL;
	CFStructureItem*element=NULL;

	//set_item->isMarked=true;

	fs_set=set_item->SearchEx(CSymbolTableManager::SetId);
	
	if(fs_set)
	{	
		if(!ReadOnlyMode)
			fs_set->isMarked=true;

		for(vector<CFStructureItem*>::iterator it=fs_set->ChildList.begin();
			it!=fs_set->ChildList.end();it++)
		{
			element=*it;

		//	if(ReadOnlyMode)
		//		return element;
			
			if(!element->SearchEx(CSymbolTableManager::PredId)->isMarked)
			{
				
				if(!ReadOnlyMode)
				{
					element->isMarked=true;
					
				}
				return element;
			}			
		}
		element=NULL;
		CLogManager::AddMessage(L"No unmarked ELEMENT left");
	}
	else
	{
		CLogManager::AddMessage(L"I am not SET");
	}
			
return element;
}

CFDescription* CGenerator::GenerateFDescription(wstring &  fdstr)
{
	CFDescription*fdesc=NULL;	
	
	int ind=fdstr.find(L"=c");

	if(ind<0)
		return fdesc;
	
	wstring left;
	wstring right;
	
	int remove_brakets=0;

	fdstr=TrimString(fdstr);

	if(fdstr.at(0)=='[')
		remove_brakets=1;

	//left.replace(L"[",L"");
	//right.replace(L"]",L"");
	int len=fdstr.length();

	left=fdstr.substr(remove_brakets,ind-1);	
	len=len-left.length()-2;
	right=fdstr.substr(ind+2,len-remove_brakets-1);
	
	fdesc=new CFDescription();
	fdesc->Label=UtilityInterface->LexiconPtr->GenerateFDExpression(left);
	fdesc->Value=UtilityInterface->LexiconPtr->GenerateFDExpression(right);
	fdesc->Operator=UNIFY_OP;

	return fdesc;
}

bool CGenerator::UnifyForGenerator(CFStructureItem * & fL, 
						  CFStructureItem *fR, 
						  vector<short> *vR,
						  bool ApplySetUnification,
						  bool ReadOnlyMode)
{
	using namespace std;
	wstring msg;

	CFStructureItem*temp = NULL;
	CFStructureItem*fL_element = NULL;
	CFStructureItem*fR_element = NULL;
	vector<short> *result=NULL;	

	//CHANGE HERE, INSERT SOME CODE THAT WILL 
	//RESET THE REF TO LINKED MASTER CELL.

	//////////////////////////////////////////////
	//code above it is new implementation
	////////////////////////////////////////

 	if(!fR && fL && vR)
	{		
		if(fL->GenRef)
			fL=fL->GenRef;

		if(fL->ChildList.size()!=0)
			return false; 
						
		if(!fL->ValueList.empty())
		{						
			result=UtilityInterface->GetSetIntersection(&fL->ValueList,vR);			

			if(!result)			
				return false;
						
			if(ReadOnlyMode)
			{
				result->clear();
				delete result;
				result=NULL;
				return true;
			}
			
			fL->ValueList=*result;
		
			if(!fL->UpdateLinkedNode())
				return false;
									
			result->clear();

			try
			{
				delete result;
				result=NULL;
			}
			catch(...)
			{
				//CLogManager::AddMessage(L"#### ");
			}
		}		
		else
		{
			if(ReadOnlyMode)
				return true;

			for(vector<short>::iterator it=vR->begin();it!=vR->end();it++)
				fL->ValueList.push_back(*it);
			
			if(!fL->UpdateLinkedNode())
				return false;						
		}		

	}
	else if(!vR)
	{
		short len=fR->ChildList.size();
 		short size=fR->SetElements.size();

		char created=0;		
				
		//calculate size of Set
		//check len==0 and size==0
		
		if( len==0 && ( (ApplySetUnification && size==0) || !ApplySetUnification ))
		{		
			if(fL->GenRef)
				fL=fL->GenRef;					
			
			if(fR->GenRef)
				fR=fR->GenRef;

			fL->isMarked =true;
			fR->isMarked =true;

			//Generator Rule: if one is empty fill it by other's value
			if(fL->ValueList.empty())
			{				
				////fL->ValueList= fR->ValueList;
				fL->SetGenReference(fR);
							
				if(!fL->UpdateLinkedNode())
					return false;
			}
			else if((fR->ValueList.empty())) 
			{				
				////fR->ValueList= fL->ValueList; 
				fR->SetGenReference(fL);

				if(!fR->UpdateLinkedNode())
					return false;
			}
			else
			{
				result=UtilityInterface->GetSetIntersection(&fR->ValueList,&fL->ValueList);
				
				if(!result)				
					return false;

				if(ReadOnlyMode)
				{
					result->clear();
					delete result;
					result=NULL;
					return true;
				}
				
				fL->ValueList=*result;
				
				if(!fL->UpdateLinkedNode())
					return false;			

				result->clear();
				
				try{					
					delete result;
					result=NULL;
				}
				catch(...)
				{
					//CLogManager::AddMessage(L"#### result");
				}

			}			
		}
		else
		{		
			if(ApplySetUnification && size!=0)
			{
				CFStructureItem*nfs=new CFStructureItem();
				
				if(!nfs)
					return false;

				nfs->Label=CSymbolTableManager::GetNum(L"SpecialSet",true);
				CFStructureItem*rs=NULL;
				
				for(vector<CFStructureItem*>::iterator it=fR->SetElements.begin();
					it!=fR->SetElements.end();it++)
					{
						temp=*it;
						rs=fL->GetClone();
						
						if(!UnifyForGenerator(rs,temp,NULL))
						{		
							try{
								delete rs;
								rs=NULL;
							}
							catch(...)
							{
								//CLogManager::AddMessage(L"#### rs");
							}

							//MessageBox(NULL,L"Set Unification Failed",L"Set Unification Failed",MB_ICONERROR);
							return false;					
						}
					
						UtilityInterface->AppendConstraints(rs,temp);							
						nfs->SetElements.push_back(rs);																
					}
							
				*fL=*nfs;
			}
			
			if(fL==fR)
				return true;

			for(short p=0;p<len;p++)
			{
				fR_element=fR->ChildList.at(p);				

				fL_element=fL->SearchEx(fR_element->Label);			
				
				if(fL_element)
				{
					if(!UnifyForGenerator(fL_element,fR_element,NULL))
					{
						//msg=L"!UnifyForGenerator(fL_element,fR_element,NULL)";
						//MessageBox(NULL,msg.c_str(),msg.c_str(),2);						
						
						#ifdef GenLog
						CLogManager::AddMessage(L"UnifyForGenerator() Failed : \r\n"+CSymbolTableManager::GetString(fL_element->Label)+
												L" : "+
												CSymbolTableManager::GetStringByList(fL_element->ValueList,L",")+
												L" <-> "+CSymbolTableManager::GetStringByList(fR_element->ValueList,L",")+L"\r\n");
						#endif

						return false;				
					}
					fL_element->isMarked=fR_element->isMarked;
				}
				else				
					fL->AppendFSItem(fR_element->GetClone(),false);
			}				
		}
	}		
		
	if(!ApplySetUnification && fR && !ReadOnlyMode)
	{			
		for(vector<CFStructureItem*>::iterator it=fR->SetElements.begin();
			it!=fR->SetElements.end();it++)
			{
				temp=*it;
				fL->SetElements.push_back(temp->GetClone());
			}
		
		UtilityInterface->AppendConstraints(fL,fR);		
	}			
	
return true;
}

bool CGenerator::GenerateSentenceList(list<CGenerationBlock*> * BlockList, 
									  list<wstring> * SenList)
{
	if(!BlockList || !SenList)
		return false;
	
	using namespace std;													
	CGenerationBlock*GenBlock=NULL;
	wstring sentence;

	CCStructureItem*cnode=NULL;		
		
	for(list<CGenerationBlock*>::iterator it=BlockList->begin(); 
		it!=BlockList->end();it++)
	{			
		GenBlock=*it;	
		
		if(GenBlock->IsIncomplete)
			continue;		
		
		cnode=GenBlock->Tree->Root;
		
		sentence=L"";

		if(GenerateSentenceListRec(cnode,sentence))
			SenList->push_back(sentence);
		
	}	
	
	//CLogManager::AddMessage(L"Total Sentences: "+GetWSTR(SenList->size())+L"\r\n");

	return false;
}

bool CGenerator::GenerateSentenceListRec(CCStructureItem* cnode, 
										 std::wstring & sentence)
{
	if(!cnode)
		return false;

	CLexicalEntry*le=NULL;
	
	if(cnode->ChildList.size()==0)	//leaf node
	{
		if(cnode->LexEntry)
		{			
			sentence=sentence + L" " + cnode->LexEntry->Word;			
		}
		else if(cnode->Value.size()!=0)
		{
			short found=-1;
			found=cnode->Value.find('<');
			
			if(found>0)
				sentence=sentence + L" " + cnode->Value.substr(0,found);
			else
				sentence=sentence + L" " + cnode->Value;
		}
	}
	else
	{
		for(vector<CCStructureItem*>::iterator cit=cnode->ChildList.begin();cit!=cnode->ChildList.end();cit++)
			if(!GenerateSentenceListRec(*cit,sentence))
				return false;
	}

	return true;
}

bool CGenerator::RefineSentences(list<wstring> * SenList)
{
	if(!SenList)
		return false;		

	if(SenList->size()==0)
		return false;
	
	SenList->sort();

	//sort(SenList->begin(),SenList->end());

	list<wstring> TempList;

	wstring prev=L"";
	wstring curr=L"";
	
	short quote=39; //single quote
	wstring alif=L" ";
	/*alif+=_TCHAR(0x0627);CHANGED IN THIS (LINUX ONLY) VERSION*/
	/**/
	wchar_t a= 0x0627 ;
	alif+=a;
	/**/	
	alif+=L" ";
	wstring space=L" ";

	list<wstring>::iterator sit=SenList->begin();
	curr=prev=*sit;
	if(curr[0]==0x0627 && curr[1]==' ')
		curr=L" " +curr;

	curr=ReplaceWSTR(prev,alif,space);
	TempList.push_back(RemoveCharFromWSTRING(curr,quote));

	for(;sit!=SenList->end();sit++)
	{
		if(*sit!=prev)
		{
			curr=prev=*sit;
			if(curr[0]==0x0627 && curr[1]==' ')
				curr=L" " +curr;
			curr=ReplaceWSTR(curr,alif,space);
			TempList.push_back(RemoveCharFromWSTRING(curr,quote));
		}
	}		

	#ifdef GenLog
	CLogManager::AddMessage(L"Refined Sentences from "+GetWSTR(SenList->size())+L" to "+GetWSTR(TempList.size())+L"\r\n");
	#endif

	*SenList=TempList;

	return false;
}



bool CGenerator::CheckFStructureIsMarked(CFStructureItem* FSPtr)
{
	if(!FSPtr)
		return false;

	if(FSPtr->ChildList.size()==0)
		return true;	

	/*
	if(!FSPtr->isMarked)
	{
		map<short,short>::iterator it=SpecialUnmarkedItems.find(FSPtr->Label);
		
		if(it==SpecialUnmarkedItems.end())//if not found
		{
			wstring str=MapperInterface->LexPtr->GetConstantName(FSPtr->Label);
			CLogManager::AddMessage(L"Unmarked F-Structure Node Found i.e. "+str+L"\r\n");
			return false;
		}		
	}
	*/
	
	CFStructureItem*child=NULL;

	if(!FSPtr->isMarked && !FSPtr->IsVisited)
	{		
		map<short,short>::iterator found_it=SpecialUnmarkedItems.find(FSPtr->Label);
		
		if(found_it==SpecialUnmarkedItems.end())//if not found
		{
			wstring str=CSymbolTableManager::GetString(FSPtr->Label);
			
			#ifdef GenLog
			CLogManager::AddMessage(L"Unmarked F-Structure Node Found i.e. "+str+L" : \r\n");
			#endif
			return false;
			for(vector<CFStructureItem*>::iterator fsit=FSPtr->ChildList.begin();
				fsit!=FSPtr->ChildList.end();fsit++)
			{
				child=*fsit;
				if(child->ChildList.size()!=0 && !child->isMarked)
				{
					#ifdef GenLog
					str=CSymbolTableManager::GetString(child->Label);
					if(child->Label==CSymbolTableManager::PredId)
					{
						str+=L"[";
						child=child->SearchEx(CSymbolTableManager::ValueId);
						str+=CSymbolTableManager::GetString( *(child->ValueList.begin()) );
						str+=L"]";
					}
					CLogManager::AddMessage(L"Unmarked F-Structure Node Found i.e. "+str+L"\r\n");
					#endif
					return false;			
				}
			}
			
		}		
		
		FSPtr->isMarked=true;
		return true;
	}
	
	//*/

	for(vector<CFStructureItem*>::iterator it=FSPtr->ChildList.begin();
		it!=FSPtr->ChildList.end();it++)
	{
		if(!CheckFStructureIsMarked(*it))
			return false;
	}

	//if all the chilren are marked then mark the parent too.
	//FSPtr->isMarked=true;


	return true;
}

vector<CLexicalEntry*> * CGenerator::GenerateLexEntriesByMorphGenerator(wstring word, wstring POS, CFStructureItem* fs)
{
	vector<CLexicalEntry*> * LEList = NULL ;

	if(!fs)
		return LEList;

	CLexicalEntry*le=NULL;

	PROPVECT fdesclist;
	CFDescription*fdesc=NULL;

	CFStructureItem*fsitem=fs;
	CFStructureItem*temp=NULL;

	if(fsitem /*&& fsitem->Parent && fsitem->Parent->Parent*/)
	{
		//fsitem=fsitem->Parent->Parent;

		for(vector<CFStructureItem*>::iterator fsit=fsitem->ChildList.begin();fsit!=fsitem->ChildList.end();fsit++)
		{
			temp=*fsit;
			if(temp->ChildList.size()==0)
			{
				if(temp->ValueList.size())
				{
					fdesc=new CFDescription();				
					fdesc->Operator=UNIFY_OP;
				
					fdesc->Label=new CFDExpression();
					fdesc->Label->list.push_back(temp->Label);
					fdesc->Label->op=UP_OP;

					fdesc->Value=new CFDExpression();
					fdesc->Value->list=temp->ValueList;
					fdesc->Value->op=VALUE_OP;
				
					fdesclist.push_back(fdesc);
				}
			}
		}	
	}
	CFDescription* Worddescptr=NULL;
	short WordProplabel;

	//WSTRVECTOR * StrList=MorphGenerator->GenerateForms(word,POS,&fdesclist);
	LEList=MorphGenerator->GenerateForms(word,POS,&fdesclist);

	//DeleteFDescList(fdesclist);

	//if(!LEList)
	//	return LEList;

	/*LEList=new vector<CLexicalEntry*>;

	for(vector<wstring>::iterator sit=StrList->begin();sit!=StrList->end();sit++)
	{
		le=new CLexicalEntry();
		le->Word=*sit;
		le->Predicate=word;
		le->POS=POS[0];
		le->IsCache=true;
	//	list<CFDescription*>::iterator fsit=le->ConditionList.begin();
	//	le->ConditionList.insert(fsit, fdesclist.at(0));
        LEList->push_back(le);
		
		////////////////////////////////		
		//UrduLexicon->SaveLexEntry(le);
		////////////////////////////////

	}*/

	return LEList;
}

bool CGenerator::EvaluateExpressionForGenerator(wstring &wstr_in, 
												CFStructureItem * FSPtr_up,
												CFStructureItem * FSPtr_down,
												bool ReadOnlyMode)
{
	if(!FSPtr_up)
		return false;

	/* REPLACED IN THIS (DOS/LINUX) VERSION
	CString cstr=wstr_in.c_str();			
	cstr.Replace(L"=c",L"=="); 
	REPLACED IN THIS (DOS/LINUX) VERSION*/

	/*WITH*/
	wstring wstr=L"";
	wstring temp=wstr_in;
	/*WITH END*/
	
	/* REPLACED IN THIS (DOS/LINUX) VERSION
	wstring wstr=cstr.GetBuffer(512);
	REPLACED IN THIS (DOS/LINUX) VERSION*/
	
	/*WITH*/
	if(temp.find(L"=c")!= wstring::npos)
	{
		while(temp.find(L"=c")!= wstring::npos)
		{
			wstr=temp.replace(temp.find(L"=c"),2,L"==");
			temp=wstr;
		}
	}
	else
		wstr=temp;
	/*WITH END*/

	//MessageBox(NULL,wstr.c_str(),L"In Evaluate Expression",MB_ICONINFORMATION);
	
	wstring	left=L"";
	wstring	right=L"";
	wstring	str=L"";	

	string bool_exp="";	
	
	short cond_len= wstr.size();		
	short last_op=0;
	short result=0;	
	bool LocalReadOnlyMode=ReadOnlyMode;
	
	for(short n=0;n<cond_len;n++)
	{
		switch(wstr[n])
		{
		   case ']':

				if(left==L"")
				{
					bool_exp+=")";
					last_op=')';
					break;
				}
								
				right=str;
				result=CheckConstraintForGenerator(FSPtr_up,FSPtr_down,left,right,LocalReadOnlyMode);				
				
				bool_exp += (result+48);
				
				bool_exp+=")";
				last_op=')';

				left=right=str=L"";
				LocalReadOnlyMode=ReadOnlyMode;								
				break;

		   case '&':
		   case '|':

				if(last_op==wstr[n])
					break;

				if(last_op==')' && left==L"")
				{					
					last_op= wstr[n];	
					bool_exp +=(char) wstr[n];
					break;
				}

				last_op= wstr[n];

				right=str;

				result=CheckConstraintForGenerator(FSPtr_up,FSPtr_down,left,right,LocalReadOnlyMode);
										
				bool_exp += (result+48);
				bool_exp += (char)wstr[n];
				left=right=str=L"";
				
				break;

		   case '[':
				
				bool_exp+="(";		
				break;
   		   			
		   case '=':

				if(last_op=='=')
					break;

				last_op='=';
				left=str;
				str=L"";
				break;
					
			case '~':
				LocalReadOnlyMode=true;
				bool_exp += "!";	
				break;
					
  			default:
		
				str+= wstr[n];					

		}//end switch					
	
	}//end for
	
	long rs=EvaluateBooleanExpressionEx(bool_exp);

	if(rs>0)
		return true;
	
	return false;
}

//CHANGE REQUIRED HERE
short CGenerator::CheckConstraintForGenerator(	CFStructureItem*up,
												CFStructureItem*down,
												wstring &left,
												wstring &right,
												bool ReadOnlyMode)
{
	
///////////////////////////////////////////////////////
////write new code above it 
//////////////////////////////////////////////////////
	
	if(!down || !up)
		return 0;

	left=TrimString(left);
	right=TrimString(right);

	if(left==L"^ TNS_ASP PERF")
		int test=0;

	CFDExpression * left_exp=MapperInterface->LexPtr->GenerateFDExpression(left);
	CFDExpression * right_exp=MapperInterface->LexPtr->GenerateFDExpression(right);

	if(!left_exp||!right_exp)
	{
		CFDExpression::DeleteFDExpressions(left_exp,right_exp);
		return 0;
	}

	CFStructureItem*fs1=NULL;
	CFStructureItem*fs2=NULL;
	CFStructureItem*it=NULL;
	
	char created1=0;
	char created2=0;

	if(left_exp->op==UP_OP)
	{
		if(ReadOnlyMode)
		{
			fs1=up->Search(&left_exp->list);			
			if(!fs1)
			{
				CFDExpression::DeleteFDExpressions(left_exp,right_exp);
				return 0;
			}
		}
		else
		{
			fs1=up->SearchAndCreate(&left_exp->list,created1);
			it=fs1;
			while(it && it!=up)
			{
				it->isMarked=true;
				it=it->GetParent();
			}
		}			
				
		if(right_exp->op==DOWN_OP)
		{
			if(ReadOnlyMode)
			{
				fs2=down->Search(&right_exp->list);				
			}
			else
			{
				fs2=down->SearchAndCreate(&right_exp->list,created2);
				it=fs2;
				while(it && it!=down)
				{
					it->isMarked=true;
					it=it->GetParent();
				}
			}

			if(!fs2)
			{
				CFDExpression::DeleteFDExpressions(left_exp,right_exp);
				return 1;
			}
			
			//if both are created
			if(created1 && created2)
			{
				//fs1->LinkInfoList.push_back(fs2);
				fs1->AddNewLink(fs2);
				fs2->AddNewLink(fs1);
				#ifdef GenLog
				CLogManager::AddMessage(L"EE Gen: Both Expression Sides are created...having no value \r\n");
				CFDExpression::DeleteFDExpressions(left_exp,right_exp);
				#endif
				return 1;						
			}

			//if LHS is created
			if(created1 && !created2)
			{
				if(fs1->GenRef)
					fs1=fs1->GenRef;

				////fs1->ValueList=fs2->ValueList;
				if(fs2->GenRef)
					fs2=fs2->GenRef;
				fs1->SetGenReference(fs2);

				if(!fs1->UpdateLinkedNode())
				{
					CFDExpression::DeleteFDExpressions(left_exp,right_exp);
					return false;
				}

				CFDExpression::DeleteFDExpressions(left_exp,right_exp);
				return 1;
			}

			//if RHS is created
			if(!created1 && created2)
			{
				if(fs2->GenRef)
					fs2=fs2->GenRef;
				
				////fs2->ValueList=fs1->ValueList;	
				if(fs1->GenRef)
					fs1=fs1->GenRef;

				fs2->SetGenReference(fs1);

				if(!fs2->UpdateLinkedNode())
				{
					CFDExpression::DeleteFDExpressions(left_exp,right_exp);
					return false;
				}

				CFDExpression::DeleteFDExpressions(left_exp,right_exp);
				return 1;
			}
		
			
			if(!CChart::IsIntersectionEmpty(&fs2->ValueList,&fs1->ValueList)) 
			{
				CFDExpression::DeleteFDExpressions(left_exp,right_exp);
				return 1;
			}
			else
			{
				wstring str=L"Left side:\n";
				for(vector<short>::iterator i=fs1->ValueList.begin();i!=fs1->ValueList.end();i++)
				{
					str+=CSymbolTableManager::GetString(*i);
					str+=L",";
				}
				str+=L"\nRight side:\n";
				for(vector<short>::iterator i=fs2->ValueList.begin();i!=fs2->ValueList.end();i++)
				{
					str+=CSymbolTableManager::GetString(*i);
					str+=L",";
				}
				CLogManager::AddMessage(str);
			}

			CFDExpression::DeleteFDExpressions(left_exp,right_exp);
			return 0;
		}
	
		if(fs1->GenRef)
			fs1=fs1->GenRef;

		if(fs1 && fs1->ValueList.size()==0)
		{		
			fs1->ValueList=right_exp->list;
			
			if(!fs1->UpdateLinkedNode())
			{
				CFDExpression::DeleteFDExpressions(left_exp,right_exp);
				return false;
			}

			CFDExpression::DeleteFDExpressions(left_exp,right_exp);
			return 1;
		}
								
		if(fs1 && !CChart::IsIntersectionEmpty(&right_exp->list,&fs1->ValueList))	
		{
			CFDExpression::DeleteFDExpressions(left_exp,right_exp);
			return 1;
		}
	}
	else if(left_exp->op==DOWN_OP)
	{
		if(ReadOnlyMode)
		{
			fs2=down->Search(&left_exp->list);
			if(!fs2)
			{
				CFDExpression::DeleteFDExpressions(left_exp,right_exp);
				return 1;
			}
		}
		else
			fs2=down->SearchAndCreate(&left_exp->list,created2);	
	
		if(right_exp->op==UP_OP)
		{
			if(ReadOnlyMode)
			{
				fs1=up->Search(&right_exp->list);	
				if(!fs1)
				{
					CFDExpression::DeleteFDExpressions(left_exp,right_exp);
					return 1;
				}
			}
			else
				fs1=up->SearchAndCreate(&right_exp->list,created1);	

			if(created1 && created2)
			{
				fs1->AddNewLink(fs2);
				fs2->AddNewLink(fs1);
				
				#ifdef GenLog
				CLogManager::AddMessage(L"EE Gen: Both Expression Sides are created...having no value \r\n");
				CFDExpression::DeleteFDExpressions(left_exp,right_exp);
				#endif

				return 1;						
			}		

			if(created2)
			{
				if(fs2->GenRef)
					fs2=fs2->GenRef;

				////fs2->ValueList=fs1->ValueList;
				if(fs1->GenRef)
					fs1=fs1->GenRef;

				fs2->SetGenReference(fs1);
				
				if(!fs2->UpdateLinkedNode())
				{
					CFDExpression::DeleteFDExpressions(left_exp,right_exp);
					return false;
				}

				CFDExpression::DeleteFDExpressions(left_exp,right_exp);
				return 1;
			}

			if(created1)
			{
				if(fs1->GenRef)
					fs1=fs1->GenRef;
				////fs1->ValueList=fs2->ValueList;
				
				if(fs2->GenRef)
					fs2=fs2->GenRef;
				fs1->SetGenReference(fs2);

				if(!fs1->UpdateLinkedNode())
				{
					CFDExpression::DeleteFDExpressions(left_exp,right_exp);
					return false;
				}

				CFDExpression::DeleteFDExpressions(left_exp,right_exp);
				return 1;
			}
			
			if(!CChart::IsIntersectionEmpty(&fs2->ValueList,&fs1->ValueList)) 
			{
				CFDExpression::DeleteFDExpressions(left_exp,right_exp);
				return 1;
			}

			CFDExpression::DeleteFDExpressions(left_exp,right_exp);
			return 0;
		}
	
		if(fs2)
		{		
			if(fs2->GenRef)
				fs2=fs2->GenRef;		

			if(fs2->ValueList.size()==0)
			{
				fs2->ValueList=right_exp->list;
				
				if(!fs2->UpdateLinkedNode())
					return false;

				CFDExpression::DeleteFDExpressions(left_exp,right_exp);
				return 1;
			}

			if(!CChart::IsIntersectionEmpty(&right_exp->list,&fs2->ValueList))
			{
				CFDExpression::DeleteFDExpressions(left_exp,right_exp);
				return 1;
			}
		}
	}

	CFDExpression::DeleteFDExpressions(left_exp,right_exp);	
	return 0;	
}

bool CGenerator::SetUpAndDownFStructures(CFDescription*fdptr,
										 CGenerationItem*GenItem,
										 bool AllowLinking,
										 bool ReadOnlyMode)
{
	if(!fdptr || !fdptr->Label || !fdptr->Value || !GenItem || !GenItem->UpFSPtr)
		return false;

	size_t len1=fdptr->Label->list.size();
	size_t len2=fdptr->Value->list.size();

	//CFStructureItem*FSPtr=GenItem->UpFSPtr;	
	//CFStructureItem*ReqPtr=NULL;

	// Handling Cases:
	// ^   = !
	// ^___= !

	if(fdptr->Operator==UNIFY_OP && fdptr->Value->op==DOWN_OP && len2==0)
	{
		if(len1!=0)
		{
			GenItem->DownFSPtr=GenItem->UpFSPtr->Search(&fdptr->Label->list);
			if(GenItem->DownFSPtr)
			{
				GenItem->DownFSPtr->isMarked=true;
				return true;				
			}
			else
			{
				#ifdef GenLog
				CLogManager::AddMessage(L"SetUpAndDownFStructures- Not Found\r\n");
				#endif
				return false;
			}
		}
		else
		{
			GenItem->DownFSPtr=GenItem->UpFSPtr;
			GenItem->DownFSPtr->isMarked=true;
			return true;				
		}
	}

	// Handling Cases:
	// ! $ ^
	// ! $ ^ ____

	if(fdptr->Operator==SET_OP && fdptr->Label->op==DOWN_OP && len1==0)
	{
		if(len2!=0)
		{						
			GenItem->DownFSPtr=GenItem->UpFSPtr->Search(&fdptr->Value->list);
			
			if(GenItem->DownFSPtr)
			{
				GenItem->DownFSPtr->isMarked=true;
				GenItem->DownFSPtr=GetSetNextElement(GenItem->DownFSPtr);

				if(GenItem->DownFSPtr)
				{
					if(!ReadOnlyMode)
						GenItem->DownFSPtr->isMarked=true;
					return true;
				}
				else
				{
					//error
					return false;
				}
			}
			else
			{
				//error
				return false;
			}
		}
		else
		{
			GenItem->UpFSPtr->isMarked=true;
			GenItem->DownFSPtr=GetSetNextElement(GenItem->UpFSPtr);
			if(GenItem->DownFSPtr)
			{
				GenItem->DownFSPtr->isMarked=true;
				return true;
			}
			else
			{
				//error
				return false;
			}
			
		}
	}

	// Handling Cases:
	// ^ ____ = value-list
	// ! ____ = value-list
	
	if(fdptr->Operator==UNIFY_OP && fdptr->Value->op==VALUE_OP && len1!=0 && len2!=0)
	{	
		char created1=0;
		CFStructureItem*fsitem=NULL;

		if(!GenItem->DownFSPtr)
			GenItem->DownFSPtr=new CFStructureItem(-1);
        
		if(fdptr->Label->op==UP_OP)
			fsitem=GenItem->UpFSPtr;
		else
			fsitem=GenItem->DownFSPtr;

		fsitem=fsitem->SearchAndCreate(&fdptr->Label->list,created1);

		if(created1)
		{	
			////fsitem->ValueList=fdptr->Value->list;

			if(fsitem->GenRef)
				fsitem=fsitem->GenRef;

            fsitem->ValueList=fdptr->Value->list;

			fsitem->isMarked=true;
			return true;
		}

		vector<short>*result=UtilityInterface->GetSetIntersection(&fsitem->ValueList,
																  &fdptr->Value->list);
			
		if (!result)
			return false;
		
        //not decided yet.........
		if(fsitem->GenRef)
			fsitem=fsitem->GenRef;

		fsitem->ValueList=*result;
		
		fsitem->isMarked=true;

		result->clear();
		delete result;

		return true;
	}

	//EXCEPTIONS
	// Handling Cases:
	// ^ ____ = ^ _____
	/*
		
		Under Discussion....

	*/

	//EXCEPTIONS
	// Handling Cases:
	// ^ ____ = ! _____

	vector<short> * result=NULL;

	if(fdptr->Operator==UNIFY_OP && fdptr->Label->op==UP_OP && fdptr->Value->op==DOWN_OP && len1!=0 && len2!=0)
	{					
		char created1=0;
		char created2=0;

		//////////////////////////////////////////////////////////////////////////////
		//up and down pointers are set already so 
		//just unifying 
		//////////////////////////////////////////////////////////////////////////////
		if(GenItem->UpFSPtr && GenItem->DownFSPtr)
		{
			CFStructureItem * up = NULL;
			CFStructureItem * down = NULL;

			up=GenItem->UpFSPtr->SearchAndCreate(&fdptr->Label->list,created1);
			down=GenItem->DownFSPtr->SearchAndCreate(&fdptr->Value->list,created2);

			if(created1 && created2)
			{
				//if (AllowLinking)		//COMMENTED ON  MAY 20, 11:46am

				down->SetGenReference(up);			

				up->isMarked=true;
				down->isMarked=true;
				return true;
			}		

			if(created1)
			{
				////if(down->ValueList.size()==0)
				////{
					down->SetGenReference(up);
				////}
				////else
				////{
					////up->ValueList=down->ValueList;

					////if (AllowLinking)
					////	down->SetGenReference(up);			
				
					
				////}

				up->isMarked=true;
				down->isMarked=true;
				return true;
			}		

			if(created2)
			{				
				////if(up->ValueList.size()==0)
				////{
					down->SetGenReference(up);
				////}
				////else
				////{
				////	down->ValueList=up->ValueList;
				
				////	if (AllowLinking)
				////		down->SetGenReference(up);			
				////}

				up->isMarked=true;
				down->isMarked=true;
				
				return true;
			}		
	            
			result=UtilityInterface->GetSetIntersection(&up->ValueList,&down->ValueList);
			
			if (!result)
				return false;
			
			//not decided yet.........
			
			if(up->GenRef)
				up=up->GenRef;

			up->ValueList=*result;					
			
			if (up != down)
			{
				////newcode
				down->ValueList.clear();
			}
			down->SetGenReference(up);
			/////
			delete result;

			//if(UnifyForGenerator(up,down,NULL))
			//	return true;

			up->isMarked=true;
			down->isMarked=true;
			
			return true;	
		}
		//////////////////////////////////////////////////////////////////////////////

		if(!GenItem->DownFSPtr)
		{
			GenItem->DownFSPtr=new CFStructureItem(-1);
		}
		
		CFStructureItem * up=GenItem->UpFSPtr;
		CFStructureItem * down=GenItem->DownFSPtr;			

		up=up->SearchAndCreate(&fdptr->Label->list,created1);
		down=down->SearchAndCreate(&fdptr->Value->list,created2);				

		if(created1 || created2)
		{
			if(AllowLinking)
				down->SetGenReference(up);			
			up->isMarked=true;
			down->isMarked=true;
			return true;
		}		
            
		result=UtilityInterface->GetSetIntersection(&up->ValueList,&down->ValueList);
		
		if (!result)
			return false;
		
        //not decided yet.........
		
		if(up->GenRef)
			up=up->GenRef;

		up->ValueList=*result;					

		delete result;

		up->isMarked=true;
		down->isMarked=true;
		
		return true;		
	}
		
												
	return false;
}

void CGenerator::LogUnificationError(CFDescription*fdesc)
{
	#ifdef GenLog

	wstring gstr=L"Unification failed : ";

	if(!fdesc)
	{
		#ifdef GenLog
		gstr+=L"NULL FDescription provided..\r\n";
		CLogManager::AddMessage(gstr);
		#endif

		return;
	}

	if(fdesc->Label)
	{
		gstr+=L" ";
		gstr+=fdesc->Label->op;		
		if(fdesc->Label->list.size()!=0)
			gstr+=CSymbolTableManager::GetStringByList(fdesc->Label->list,L" ");
	}
	else
		gstr+=L" NULL ";

	gstr+=L" ";
	gstr+=fdesc->Operator;
	gstr+=L" ";
	
	if(fdesc->Value)
	{
		gstr+=L" ";
		gstr+=fdesc->Value->op;
		if(fdesc->Value->list.size()!=0)
			gstr+=CSymbolTableManager::GetStringByList(fdesc->Value->list,L" ");
	}
	else
		gstr+=L" NULL ";
		
	gstr+=L"\r\n";
	
	CLogManager::AddMessage(gstr);

	#endif
}


void CGenerator::Generator(CFStructure* FSPtr,
						   list<wstring> & SenList,
						   list<CFStructure*> & FSList)
{	
	an_attempt=NULL;
	
	marking_failed = 0;

	vector<CProduction*> SourceProdList;
	UrduGrammarManager->GetProductionsExtended(SourceSymbolId,SourceProdList,FSPtr->Root);
	
	CChartItem  * ChartItem = NULL;
	CProduction * tprod = NULL;

	for(vector<CProduction*>::iterator pit=SourceProdList.begin();
		pit!=SourceProdList.end();pit++)
	{
		tprod=*pit;
		ChartItem =new CChartItem(FSPtr->Root,tprod, 0, 0, false);	
		Enqueue(&GenerationChart,ChartItem,true);
	}
	
	int i= 0;
	int index=0;
	int size=0;
	wstring POS;

	CChartItem * citem=NULL;
	CProduction * cprod=NULL;
	CProductionSymbol * cprodsymb=NULL;
	bool Execute=true;

	deque<CChartItem*>::reverse_iterator rit;	

	deque<CChartItem*>::iterator cit=GenerationChart.chart.begin();
	deque<CChartItem*>::iterator cit_LookAhead;	
	
	for(int index=0;index<GenerationChart.chart.size();index++)
	{		
		citem = GenerationChart.chart[index];	
		cprod = citem->Prod;					

		if(!cprod->IsComplete())//if production is incomplete
		{
			cprodsymb = cprod->GetCurrentSymbol();
			
			if(cprodsymb->isTerminal)
			{					
				Scanner(citem,cprodsymb);
			}
			else
			{
				Predictor(citem,cprodsymb);
			}
		}
		else
		{
			Completer(citem,SenList,FSList);
		}		
		
		if(	index+1==GenerationChart.chart.size()&& 
			SenList.size()==0 && 
			BackupGenerationChart.chart.size()>0 )			
		{			
			rit=BackupGenerationChart.chart.rbegin();
			citem=*rit;		
			BackupGenerationChart.chart.pop_back();			
		
			//GenerationChart.chart.push_back(citem);
			Enqueue(&GenerationChart,citem,true);
		}
	}		
	if(SenList.empty() && an_attempt)
	{
		wstring sent;
		MakeSentencesByWords(an_attempt,sent);
		FSList.push_back(new CFStructure(an_attempt->FSPtr,1.0));
		SenList.push_back(sent);
	}
	////////////////////////
	//Look for the longest(???)// All completed production of S, 
	//check completeness, coherence(from  chart parser), 
	//marking. Retrieve sentence from S productions. 
	//By changing  S->NP VP well traverse and retrieve the leaf nodes from these. 
	////////////////////////

	//return Chart.SentenceList	//c	
}

void CGenerator::Scanner(CChartItem* ChartItem,
						 CProductionSymbol * ProdSymb)
{	
	//list<CFStructureItem*> FSList;
	/*Comments N: Unification part: First have to trace down the proper fstructure according to fdecription writen
	if there is any ! at right hand side, trace down according to its left hand side. if there is none, assume ^ = !
	*/

	if(ProdSymb->conditions.size())
	{
	vector<CFDescription*>::iterator first=ProdSymb->conditions.begin();
	CFDescription*first_fdptr = *first;
	
	SetUpAndDownFStructuresEx(first_fdptr,ChartItem,ProdSymb,true,false);

	if(ProdSymb->GenFSPtr)
	{
		ProdSymb->GenFSPtr->isMarked=true;
		if(!GenerateWord(ChartItem,ProdSymb));
			//ProdSymb->GenFSPtr->isMarked=false;
	}
	}
}

void CGenerator::Predictor(CChartItem* CurrentChartItem,
						   CProductionSymbol*CurrentProdSymb)
{
	//int index = CurrentChartItem->Prod->curr_index;
	
	//CProductionSymbol * psymb=CurrentChartItem->Prod->GetCurrentSymbol();
	vector<CFDescription*>::iterator first=CurrentProdSymb->conditions.begin();	
	CFDescription*first_fdptr = *first;
			
	SetUpAndDownFStructuresEx(first_fdptr,CurrentChartItem,CurrentProdSymb,true,true); 

	vector<CProduction*> ProdList;

	UrduGrammarManager->GetProductionsExtended(	CurrentProdSymb->LabelID,
												ProdList,
												CurrentProdSymb->GenFSPtr);//c

	if(ProdList.size()==0)
	{
		#ifdef GenLog
		CLogManager::AddMessage(L"No Production Found for "+CurrentProdSymb->Label+L"\r\n");
		#endif
		return;
	}
		
	

	///////////////////marking////////////////////////////////
	//if(CurrentProdSymb->GenFSPtr)
	//{		
	//	CurrentChartItem->FSPtr->isMarked=true;
	//	CurrentProdSymb->GenFSPtr->isMarked=true;
	//}
	//////////////////////////////////////////////////////////

	/*****checking constraints and executing unification statements in ReadOnly Mode.*****/
	
	wstring aconstraint=L"";

	for(vector<wstring>::iterator constr_it=CurrentProdSymb->constraints.begin();
		constr_it!=CurrentProdSymb->constraints.end();constr_it++)
	{
		aconstraint=*constr_it;

		if(!EvaluateExpressionForGenerator(aconstraint,CurrentChartItem->FSPtr,CurrentProdSymb->GenFSPtr,true))
		{	
			#ifdef GenLog
			CLogManager::AddMessage(L"Constraint Failed in Gen Predictor: "+aconstraint+L" \r\n");
			#endif
			return;
		}	
	}

	CFDescription * fdescptr=NULL;

	for(vector<CFDescription*>::iterator fdit=CurrentProdSymb->conditions.begin();
		fdit!=CurrentProdSymb->conditions.end();fdit++)
	{
		fdescptr=*fdit;
		if(!ApplyUnification(fdescptr,CurrentChartItem->FSPtr,CurrentProdSymb->GenFSPtr,true))
		{
			LogUnificationError(fdescptr);
			return;
		}
	}
	
	/**********/

	CProduction* prodptr = NULL;

	for(vector<CProduction*>::iterator pit=ProdList.begin();pit!=ProdList.end();pit++)
	{
		prodptr=*pit;
		
		if(!EvaluateConstraintsAndConditionsInReadOnlyMode(prodptr,CurrentProdSymb->GenFSPtr))
		{
			#ifdef GenLog
			CLogManager::AddMessage(L"Production Failed in Pre-Evaluation: ");
			prodptr->PrintProductionF(CLogManager::GetLogFileRef(),L"Production Failed in Pre-Evaluation: ");
			#endif

			continue;
		}
		
		//Evaluate production.....constraints.....and unification....

		//if(ApplyConstraintsAll(CurrentChartItem.FS,  Prod, CAN_NOT_CREATE_FEATURES))//c
		//{			
			//CProduction *prod = new CProduction();
			//prod->LHS = new CProductionSymbol();
			CChartItem* NewChartItem = new CChartItem(prodptr,0,0,false);

			//NewChartItem->Prod->ProductionNo = prodptr->ProductionNo;
			//NewChartItem->Prod = prodptr;
			//NewChartItem.Production = Prod
			
			NewChartItem->StartIndex = CurrentChartItem->CurrentIndex;
			NewChartItem->CurrentIndex = CurrentChartItem->CurrentIndex ;
			NewChartItem->FSPtr = CurrentProdSymb->GenFSPtr;
						
		//	CurrentProdSymb->ChartItemPtr = NewChartItem;
			//NewChartItem.FS = CurrentChartItem.FS  to implement now	
			/*
			Comments N: accoring to conditions pick up the right FS
			*/
			//NewChartItem.Production.NULLConnectedFS = CurrentChartItem.FS

			if(prodptr->IsNullProduction())
				Enqueue(&BackupGenerationChart,NewChartItem);
			else
				Enqueue(&GenerationChart,NewChartItem,true);			
		//}
	}

}
bool CGenerator::Completer(CChartItem* ParsedChartItem,
						   list<wstring> & SenList,
						   list<CFStructure*> & FSList)
{
	/*
	Make a Clone F-Structure for each Prod waiting for Parsed Symbol, 
	Apply constraint and if successful insert new chart item, 
	with parsedFS and parsedChartItemPos on the Chart. //c
	if(ParsedChartItem.Production.ConnetedNullProduction !=NULL)
	BkupChart.Remove(ParsedChartItem.Production.ConnetedNullProduction, 
	ParsedChartItem.Production.NULLConnectedFS)//c*/

	/////////////////new code
	wstring sent;

	if(ParsedChartItem->Prod->LHS->LabelID == SourceSymbolId) 
	{
		an_attempt = ParsedChartItem;

		if(CheckFStructureIsMarked(ParsedChartItem->FSPtr))
		{			
			MakeSentencesByWords(ParsedChartItem,sent);
			//GenerateXMLForCStructures(ParsedChartItem);
			FSList.push_back(new CFStructure(ParsedChartItem->FSPtr,1.0));
			SenList.push_back(sent);					
			return true;
		}
		else
		{
			marking_failed++;
			if(marking_failed >=3)
			{
				MakeSentencesByWords(an_attempt,sent);
				//GenerateXMLForCStructures(ParsedChartItem);
				FSList.push_back(new CFStructure(an_attempt->FSPtr,1.0));
				SenList.push_back(sent);					
				return true;
			}
		}
		return false;			
	}

	/////////////////	

	//hash_multimap<int,CChartItem*>::iterator cit=LookUpTable.find(ParsedChartItem->Prod->ProductionNo);
	//int FoundCount=LookUpTable.count(ParsedChartItem->Prod->ProductionNo);

	CFStructureItem * newfs = NULL;
	//CChartItem*citem=NULL;

	CChartItem * NewChartItem=NULL;	

	CChartItem * temp_citem=NULL;
	CProduction* tprod=NULL;
	CProductionSymbol*psymb=NULL;

	/////////////////recent changes/////////////////////////////////

	list<int> ProdNumList;
	deque< CChartItem* > RequiredChartItems;

	UrduGrammarManager->GetRelvantProductions(
		ParsedChartItem->Prod->LHS->LabelID,
		ProdNumList);	

	int prod_id=0;	
	hash_multimap<int,CChartItem*>::iterator find_prod;

	for(list<int>::iterator pit=ProdNumList.begin();pit!=ProdNumList.end();pit++)
	{
		prod_id=*pit;
		
		find_prod=LookUpTable.find(prod_id);
		
		while(find_prod!=LookUpTable.end() && find_prod->first==prod_id)
		{
			RequiredChartItems.push_back(find_prod->second);
			find_prod++;
		}
	}

	///////////////////////////////////////////////////////////////	
	
	deque<CChartItem*> * ListPtr=NULL;
	
	if(RequiredChartItems.size()!=0)
		ListPtr=&RequiredChartItems;
	else
		ListPtr=&GenerationChart.chart;
	
	CFDescription * fdescptr=NULL;
	vector<CFDescription*>::iterator fdit;
	bool unif_failed=false;
	bool first = true;

	for(deque<CChartItem*>::iterator git=ListPtr->begin();git!=ListPtr->end();git++)
	{
		temp_citem=*git;
		tprod=temp_citem->Prod;
		psymb=tprod->GetCurrentSymbol();
		
		if(temp_citem->CurrentIndex == ParsedChartItem->StartIndex)
		{
			if(psymb && !psymb->isTerminal && psymb->LabelID==ParsedChartItem->Prod->LHS->LabelID)
			{
				/*if( (*psymb->conditions.begin())->Label->list.size()==0)
				{
					wstring temp = CSymbolTableManager::GetString(ParsedChartItem->FSPtr->Label);
					if(temp.find(L"ELEMENT") != wstring.npos && (*psymb->conditions.begin())->Value->list.size()>0)
						continue;
				}*/
				//else if(ParsedChartItem->FSPtr->Label!=*((*psymb->conditions.begin())->Label->list.end()-1))
				//	continue;

			
				if(ParsedChartItem->FSPtr && psymb->GenFSPtr )
				{
					if(ParsedChartItem->FSPtr->ItemNo!=psymb->GenFSPtr->ItemNo)
						continue;
					if(ParsedChartItem->Prod->IsNullProduction() && ParsedChartItem->FSPtr->GetParent()&& psymb->GenFSPtr->GetParent())/*UPDATE*/
					{
						if(ParsedChartItem->FSPtr->GetParent()->ItemNo!=psymb->GenFSPtr->GetParent()->ItemNo)
							continue;
					}
					else
						if(ParsedChartItem->FSPtr->GetParent()!=psymb->GenFSPtr->GetParent())
							continue;
				}
				if(temp_citem->FSPtr && ( !(ParsedChartItem->Prod->IsNullProduction()) || (temp_citem->Prod->curr_index==0) ) )
						newfs=temp_citem->FSPtr->GetClone();		
					else
						newfs=temp_citem->FSPtr; 
			
			
			//SetUpAndDownFStructuresEx(psymb->
			
			//If ApplyConstraints(NewFS,ChartItem.Prod.CurrentSymbol, CAN_CREATE_FEATURE) 						

					CFStructureItem * tempParsedChartItemFS=ParsedChartItem->FSPtr->GetClone();

				if(ApplyConstraints(newfs,tempParsedChartItemFS,psymb,false))
				{
					if(!ParsedChartItem->Prod->IsNullProduction())
					{
						fdit=psymb->conditions.begin();
						//fdit++; //skipping pointer setter unification statement.
				
						unif_failed=false;

						for(;fdit!=psymb->conditions.end();fdit++)
						{
							fdescptr=*fdit;
							if(!ApplyUnification(fdescptr,newfs,tempParsedChartItemFS,false))
							{
								#ifdef GenLog
								CLogManager::AddMessage(L"Completer: Unification Failed.");
								LogUnificationError(fdescptr);
								#endif
								unif_failed=true;
								break;
							}
						}
					}
					
					if(!unif_failed)
					{
						if(tprod->extended_by == ParsedChartItem->Prod->ProductionNo)
							if(ParsedChartItem->Prod->IsNullProduction())
								continue;
						tprod->extended_by = ParsedChartItem->Prod->ProductionNo;
						tprod=tprod->GetClone();
						tprod->GetCurrentSymbol()->ChartItemPtr=ParsedChartItem;
						if(ParsedChartItem->Prod->IsNullProduction())
						{
							if(temp_citem->first_null_prod)
							{
								temp_citem->first_null_prod = false;
								temp_citem->StackSize = BackupGenerationChart.Size();
							}
						}
						tprod->MoveDot();
						if(!ParsedChartItem->Prod->IsNullProduction() || (temp_citem->Prod->curr_index==1) )
						{
							newfs->isMarked=true;
							newfs->IsVisited = true;
							CFStructureItem *parent;
							parent = newfs->GetParent();
							while(parent && !parent->IsVisited)
							{
								parent->IsVisited = true;
								parent = parent->GetParent();
							}
					
							parent = newfs->GetParent();
							while(parent && !parent->isMarked)
							{
								parent->isMarked = true;
								parent = parent->GetParent();
							}
						}
						if(tprod->IsComplete() && !tprod->IsNullProduction())
						{
							if(ParsedChartItem->Prod->IsNullProduction())
							{
								int size = BackupGenerationChart.Size();
								while(temp_citem->StackSize < size)
								{
									#ifdef GenLog
									deque<CChartItem*>::reverse_iterator rit;
									rit = BackupGenerationChart.chart.rbegin();
									CChartItem * citem = *rit;
								
									citem->Prod->PrintProductionF(CLogManager::GetLogFileRef(),L"Production Removed from Backup Chart : ");		
									#endif
									BackupGenerationChart.chart.pop_back();
									size--;
								}
							}
							else
							{
								fdit=psymb->conditions.begin();
								fdescptr=*fdit;
								bool res = true;
								if(fdescptr->Operator == UNIFY_OP && fdescptr->Label->list.size())
									res = CheckFStructureIsMarked(tempParsedChartItemFS);
								if(res)
								{
									int size = BackupGenerationChart.Size();
									if(temp_citem->StackSize > -1)
									{
										while(temp_citem->StackSize < size)
										{
											#ifdef GenLog
											deque<CChartItem*>::reverse_iterator rit;
											rit = BackupGenerationChart.chart.rbegin();
											CChartItem * citem = *rit;
								
											citem->Prod->PrintProductionF(CLogManager::GetLogFileRef(),L"Production Removed from Backup Chart : ");		
											#endif
											BackupGenerationChart.chart.pop_back();
											size--;
										}
									}
								}
								else
								{
									temp_citem->Prod->PrintProductionF(CLogManager::GetLogFileRef(),L"Production's dot could not be moved : ");		
									continue;
								}
							}
						}
					
						NewChartItem=new CChartItem(newfs,
												tprod,
												temp_citem->StartIndex,
												ParsedChartItem->CurrentIndex,false);

						NewChartItem->first_null_prod =	temp_citem->first_null_prod;
						NewChartItem->StackSize =	temp_citem->StackSize;
						Enqueue(&GenerationChart,NewChartItem,true);
						first = false;
					}
					
				}
			}
		}
	}
	
	return true;
}

void CGenerator::Enqueue(CGenChart * GenChart,
						 CChartItem * ChartItem,
						 bool AddInLookupTable)	
{	
	if(!FoundInChart(GenChart,ChartItem))
	{
		GenChart->chart.push_back(ChartItem); 
		
		#ifdef GenLog
		ChartItem->Prod->PrintProductionF(CLogManager::GetLogFileRef(),L"Production Loaded into Chart : ");		
		#endif

		if(AddInLookupTable)
			LookUpTable.insert(intcitem_pair(ChartItem->Prod->ProductionNo,ChartItem));
	}
	else
	{	
		#ifdef GenLog
		ChartItem->Prod->PrintProductionF(CLogManager::GetLogFileRef(),L"Not Loaded : ");		
		#endif
		
		//deleting ChartItem;
		ChartItem->FSPtr=NULL;
		ChartItem->Prod=NULL;
		delete ChartItem;
		ChartItem=NULL;
	}
}
// Comments N: function needs review. unintialized pointer (tprod) being used
bool CGenerator::FoundInChart(CGenChart * GenChart,
							  CChartItem* NewChartItem)//if loop still to modify
{
	bool match=false;
	int index=0;
		
	CChartItem * citem=NULL;
	CProduction * tprod=NULL;
	
	int ProdNumberToFind=NewChartItem->Prod->ProductionNo;
	
	hash_multimap<int,CChartItem*>::iterator cit=LookUpTable.find(ProdNumberToFind);

	for(;cit!=LookUpTable.end() && cit->first==ProdNumberToFind ; cit++)        	
	{			
		citem=cit->second;
		
		if(citem->FSPtr && NewChartItem->FSPtr &&
		   citem->FSPtr==NewChartItem->FSPtr)//citem->FSPtr->GetRoot()==NewChartItem->FSPtr->GetRoot())
		{
			match=true;
			
			tprod=citem->Prod;

			if(citem->Prod->curr_index==NewChartItem->Prod->curr_index  )		
			{															
				for(index=0 ; index<tprod->curr_index ; index++)
				{				
					if (tprod->RHS[index]->ChartItemNo != NewChartItem->Prod->RHS[index]->ChartItemNo)
					{
						match=false;
						break;
					}				
				}
				//Changes for set        start here
				for(index=0 ; index<tprod->RHS.size() ; index++)
				{				
					if ( tprod->RHS[index]->Label == tprod->LHS->Label)
					{
						match=false;
						break;
					}				
				}
				/*CFStructureItem * temp=citem->FSPtr->SearchEx(CSymbolTableManager::SetId);
				if(temp)
				{
					vector<CFStructureItem *>::iterator child_it=temp->ChildList.begin();
					while( child_it!=temp->ChildList.end() )
					{
						CFStructureItem * pred=(*child_it)->SearchEx(CSymbolTableManager::PredId);
						if( pred && !(pred->isMarked==false) )
						{
							match=false;
							break;
						}
						child_it++;
					}
				}*/
				//Changes for set        end here
			}
			else
				match=false;
	
			if(match)
				break;
		}
	}

return match;
}

void CGenerator::MakeSentencesByWords(CChartItem *ParsedChartItem,
									  wstring &sent)
{
	CProduction * prod =NULL;
	CProductionSymbol * psymb=NULL;
	bool skip = false;

	prod=ParsedChartItem->Prod;

	for(vector<CProductionSymbol*>::iterator psit=prod->RHS.begin();
		psit!=prod->RHS.end();psit++)
	{
		//if(((*psit)->conditions.begin())->Label->list.size()>0)
	/*	if((*psit)->conditions.size() && (*(*psit)->conditions.begin())->Label->list.size()>0)
		{
			for(vector<CProductionSymbol*>::iterator temp_it=prod->RHS.begin();
				temp_it!=psit;temp_it++)
			{
				if((*psit)->GenFSPtr && (*psit)->GenFSPtr->ItemNo == (*temp_it)->GenFSPtr->ItemNo)
				{
					skip = true;
					break;
				}
			}
		}
		if(skip)
		{
			skip = false;
			continue;
		}*/
		psymb=*psit;
	
		if(psymb->isTerminal /*&& !psymb->GenFSPtr->IsVisited*/)
		{
			//search psymb->SenWord from Lexicon
			//
			sent = sent + L" " + psymb->SenWord;
			//psymb->GenFSPtr->IsVisited = true;
		}
		else
		{				
			if(psymb->Label!=L"." && psymb->ChartItemPtr)
			{
				MakeSentencesByWords(psymb->ChartItemPtr,sent);
				//break;
			}			
		}
	}

}

/*REMOVED FROM THIS (DOS/LINUX) VERSION
bool CGenerator::GenerateXMLForCStructures(CChartItem *ParsedChartItem)
{
	if(!ParsedChartItem)
		return false;

	CFileWriter XMLWriter("data\\cs1.xml");
	XMLWriter.Write(L"<C-Structure>");
	GenerateXMLForCStructuresRec(ParsedChartItem,XMLWriter);
	XMLWriter.Write(L"</C-Structure>");

	return true;
}
REMOVED FROM THIS (DOS/LINUX) VERSION*/
/*
bool CGenerator::GenerateXMLForCStructuresRec(CChartItem *ParsedChartItem,
										  CFileWriter & xml_writer)
{
	CProduction * prod =NULL;
	CProductionSymbol * psymb=NULL;

	prod=ParsedChartItem->Prod;

	wstring NodeName=MakeXMLCompatible(prod->LHS->Label);
	
	xml_writer.Write(L"<"+NodeName+L">");

	for(vector<CProductionSymbol*>::iterator psit=prod->RHS.begin();
		psit!=prod->RHS.end();psit++)
	{
		psymb=*psit;
		
		if(psymb->isTerminal)
		{								
			xml_writer.Write(L"<"+psymb->Label+L">");
			xml_writer.Write(psymb->SenWord);
			xml_writer.Write(L"</"+psymb->Label+L">");
		}
		else
		{				
			if(psymb->Label!=L"." && psymb->ChartItemPtr)
			{
				GenerateXMLForCStructuresRec(psymb->ChartItemPtr,xml_writer);				
			}			
		}
	}

	xml_writer.Write(L"</"+NodeName+L">");
	
	return true;

}
*/
bool CGenerator::ApplyConstraints(CFStructureItem* UpFSPtr, 
								  CFStructureItem* DownFSPtr, 
								  CProductionSymbol* ProdSymb, 
								  bool ReadOnlyFlag)
{
	if(!UpFSPtr || !DownFSPtr || !ProdSymb)
		return false;
	
	wstring constr;
	for(vector<wstring>::iterator cit=ProdSymb->constraints.begin();
		cit!=ProdSymb->constraints.end();cit++)
	{
		constr=*cit;
		if(!EvaluateExpressionForGenerator(constr,UpFSPtr,DownFSPtr,ReadOnlyFlag))
		{
			#ifdef GenLog
			CLogManager::AddMessage(L"Constraint Failed: "+constr+L"\r\n");
			#endif
			return false;
		}
	}

	return true;
}

bool CGenerator::SetUpAndDownFStructuresEx(	CFDescription*fdptr,
											CChartItem * ChartItem,
											CProductionSymbol * ProdSymb,
											bool AllowLinking,
											bool ReadOnlyMode)
{
	if(!fdptr || !fdptr->Label || !fdptr->Value || 
		!ChartItem || !ChartItem->FSPtr || !ProdSymb)
		return false;

	size_t len1=fdptr->Label->list.size();
	size_t len2=fdptr->Value->list.size();

	//CFStructureItem*FSPtr=GenItem->UpFSPtr;	
	//CFStructureItem*ReqPtr=NULL;

	// Handling Cases:
	// ^   = !
	// ^___= !

	if(fdptr->Operator==UNIFY_OP && fdptr->Value->op==DOWN_OP && len2==0)
	{
		if(len1!=0)
		{					
			ProdSymb->GenFSPtr = ChartItem->FSPtr->Search(&fdptr->Label->list);

			if(ProdSymb->GenFSPtr)
				return true;				
		
			#ifdef GenLog
			CLogManager::AddMessage(L"SetUpAndDownFStructures- Not Found\r\n");
			#endif

			return false;		
		}
		else
		{
			ProdSymb->GenFSPtr=ChartItem->FSPtr;									
			return true;				
		}
	}

	// Handling Cases:
	// ! $ ^
	// ! $ ^ ____

	if(fdptr->Operator==SET_OP && fdptr->Label->op==DOWN_OP && len1==0)
	{
		if(len2!=0)
		{						
			ProdSymb->GenFSPtr = ChartItem->FSPtr->Search(&fdptr->Value->list);
			
			if(!ProdSymb->GenFSPtr)
			{
				#ifdef GenLog
				CLogManager::AddMessage(L"SetUpAndDownFStructures - Not Found\r\n");			
				#endif
				return false;
			}

			ProdSymb->GenFSPtr=GetSetNextElement(ProdSymb->GenFSPtr,ReadOnlyMode);

			if(!ProdSymb->GenFSPtr)
			{
				#ifdef GenLog
				CLogManager::AddMessage(L"SetUpAndDownFStructures - Set Element - Not Found1\r\n");			
				#endif
				return false;
			}
			else
			{
				vector<short> temp;
				for(int b=0;b<fdptr->Value->list.size();b++)
				{
					temp.push_back(fdptr->Value->list[b]);
					ChartItem->FSPtr->Search(&temp)->isMarked=true;
				}
			}						
			return true;

		}
		else
		{			
			ProdSymb->GenFSPtr = GetSetNextElement(ChartItem->FSPtr,ReadOnlyMode);
			if(!ProdSymb->GenFSPtr)
			{
				#ifdef GenLog
				CLogManager::AddMessage(L"SetUpAndDownFStructures - Set Element - Not Found2\r\n");			
				#endif
				return false;
			}
				
			return true;						
		}
	}

	// Handling Cases:
	// ^ ____ = value-list
	// ! ____ = value-list
	
	if(fdptr->Operator==UNIFY_OP && fdptr->Value->op==VALUE_OP && len1!=0 && len2!=0)
	{	
		ProdSymb->GenFSPtr = ChartItem->FSPtr ;

		char created1=0;
		CFStructureItem*fsitem=NULL;

		if(!ChartItem->FSPtr)
			ChartItem->FSPtr=new CFStructureItem(-1);       		
		
		ProdSymb->GenFSPtr = ChartItem->FSPtr ;

		fsitem=ChartItem->FSPtr;

		fsitem=fsitem->SearchAndCreate(&fdptr->Label->list,created1);

		if(created1)
		{	
			////fsitem->ValueList=fdptr->Value->list;

			if(fsitem->GenRef)
				fsitem=fsitem->GenRef;

            fsitem->ValueList=fdptr->Value->list;
			
			return true;
		}

		vector<short>*result=UtilityInterface->GetSetIntersection(&fsitem->ValueList,
																  &fdptr->Value->list);
			
		if (!result)
			return false;
		
        //not decided yet.........
		if(fsitem->GenRef)
			fsitem=fsitem->GenRef;

		fsitem->ValueList=*result;
		
		fsitem->isMarked=true;

		delete result;

		return true;
	}

	//EXCEPTIONS
	// Handling Cases:
	// ^ ____ = ^ _____
	/*
		
		Under Discussion....

	*/

	//EXCEPTIONS
	// Handling Cases:
	// ^ ____ = ! _____


	vector<short> * result=NULL;

	if(fdptr->Operator==UNIFY_OP && fdptr->Label->op==UP_OP && fdptr->Value->op==DOWN_OP && len1!=0 && len2!=0)
	{					
		char created1=0;
		char created2=0;

		//////////////////////////////////////////////////////////////////////////////
		//up and down pointers are set already so 
		//just unifying 
		//////////////////////////////////////////////////////////////////////////////

		if(ChartItem->FSPtr && ProdSymb->GenFSPtr)
		{
			CFStructureItem * up = NULL;
			CFStructureItem * down = NULL;

			up=ChartItem->FSPtr->SearchAndCreate(&fdptr->Label->list,created1);
			down=ProdSymb->GenFSPtr->SearchAndCreate(&fdptr->Value->list,created2);

			if(created1 && created2)
			{
				//if (AllowLinking)		//COMMENTED ON  MAY 20, 11:46am

				down->SetGenReference(up);			

				up->isMarked=true;
				down->isMarked=true;
				return true;
			}		

			if(created1)
			{
				////if(down->ValueList.size()==0)
				////{
					down->SetGenReference(up);
				////}
				////else
				////{
					////up->ValueList=down->ValueList;

					////if (AllowLinking)
					////	down->SetGenReference(up);			
				
					
				////}

				up->isMarked=true;
				down->isMarked=true;
				return true;
			}		

			if(created2)
			{				
				////if(up->ValueList.size()==0)
				////{
					down->SetGenReference(up);
				////}
				////else
				////{
				////	down->ValueList=up->ValueList;
				
				////	if (AllowLinking)
				////		down->SetGenReference(up);			
				////}

				up->isMarked=true;
				down->isMarked=true;
				
				return true;
			}		
	            
			result=UtilityInterface->GetSetIntersection(&up->ValueList,&down->ValueList);
			
			if (!result)
				return false;
			
			//not decided yet.........
			
			if(up->GenRef)
				up=up->GenRef;

			up->ValueList=*result;					
			
			if (up != down)
			{
				////newcode
				down->ValueList.clear();
			}
			down->SetGenReference(up);
			/////
			delete result;

			//if(UnifyForGenerator(up,down,NULL))
			//	return true;

			up->isMarked=true;
			down->isMarked=true;
			
			return true;	
		}
		//////////////////////////////////////////////////////////////////////////////

		if(!ProdSymb->GenFSPtr)
		{
			ProdSymb->GenFSPtr=new CFStructureItem(-1);
		}
		
		CFStructureItem * up   = ChartItem->FSPtr;
		CFStructureItem * down = ProdSymb->GenFSPtr;			

		up=up->SearchAndCreate(&fdptr->Label->list,created1);
		down=down->SearchAndCreate(&fdptr->Value->list,created2);				

		if(created1 || created2)
		{
			if(AllowLinking)
				down->SetGenReference(up);			
			up->isMarked=true;
			down->isMarked=true;
			return true;
		}		
            
		result=UtilityInterface->GetSetIntersection(&up->ValueList,&down->ValueList);
		
		if (!result)
			return false;
		
        //not decided yet.........
		
		if(up->GenRef)
			up=up->GenRef;

		up->ValueList=*result;					

		delete result;

		up->isMarked=true;
		down->isMarked=true;
		
		return true;		
	}

	return false;
}

bool CGenerator::UpdateProductionFSPointers(CChartItem* ChartItem)
{
	if(!ChartItem)
		return false;

	CProductionSymbol * psymb = ChartItem->Prod->GetCurrentSymbol();
	
	if(psymb->conditions.size()==0)
	{
		psymb->GenFSPtr=ChartItem->FSPtr;
		return true;
	}

	vector<CFDescription*>::iterator first=psymb->conditions.begin();
	
	return SetUpAndDownFStructuresEx(*first,ChartItem,psymb,false,true);

}


bool CGenerator::CollectGarbage(void)
{
	CChartItem*citem=NULL;
	deque<CChartItem*>::iterator cit;
	
	int index=0;
	int size=GenerationChart.chart.size();

	for(index=0;index<size;index++)
	{
		citem=GenerationChart.chart.at(index);
		delete citem;
	}			
	
	size=BackupGenerationChart.chart.size();

	for(index=0;index<size;index++)
	{
		citem=BackupGenerationChart.chart[index];
		delete citem;
	}
	
	GenerationChart.ClearAll();
	BackupGenerationChart.ClearAll();
	LookUpTable.clear();

	return true;
}

bool CGenerator::EvaluateConstraintsAndConditionsInReadOnlyMode(CProduction* prod, 
																CFStructureItem* FSPtr)
{
	if(!prod || !FSPtr)
		return false;

	vector<CProductionSymbol*>::iterator pit=prod->RHS.begin();
	vector<CFDescription*>::iterator first;
	
	CProductionSymbol*psymb=NULL;
	CFDescription*first_fdptr=NULL;	
	CFDescription * fdescptr=NULL;
	wstring aconstraint=L"";

	CChartItem TempChartItem(FSPtr,prod,0,0,0);
	
	for(int pindex=0; pindex<prod->RHS.size()-1; pindex++)	
	{	
		psymb = prod->RHS[pindex];
		
		first = psymb->conditions.begin();	
		if(first!=psymb->conditions.end())
		{
			first_fdptr = *first;

			SetUpAndDownFStructuresEx(first_fdptr,&TempChartItem,psymb,true,true); 
		}
		
		for(vector<wstring>::iterator constr_it=psymb->constraints.begin();
			constr_it!=psymb->constraints.end();constr_it++)
		{
			aconstraint=*constr_it;

			if(!EvaluateExpressionForGenerator(aconstraint,FSPtr,psymb->GenFSPtr,true))
			{	
				#ifdef GenLog
				CLogManager::AddMessage(L"Constraint Failed in Gen Predictor: "+aconstraint+L" \r\n");
				#endif

				TempChartItem.FSPtr=NULL;
				TempChartItem.Prod=NULL;
				psymb->GenFSPtr=NULL;
				psymb->FSPtr=NULL;
				return false;
			}	
		}
	
		for(vector<CFDescription*>::iterator fdit=psymb->conditions.begin();
			fdit!=psymb->conditions.end();fdit++)
		{
			fdescptr=*fdit;
			if(!ApplyUnification(fdescptr,FSPtr,psymb->GenFSPtr,true))
			{
				LogUnificationError(fdescptr);
				TempChartItem.FSPtr=NULL;
				TempChartItem.Prod=NULL;
				psymb->GenFSPtr=NULL;
				psymb->FSPtr=NULL;
				return false;
			}
		}

		psymb->GenFSPtr=NULL;
		psymb->FSPtr=NULL;

	}

	/////////// Setting NULL to avoid de-allocation ////////
	TempChartItem.FSPtr=NULL;
	TempChartItem.Prod=NULL;
	///////////////////////////////////////////////////////
			
	return true;
}

bool CGenerator::DeleteFDescList(vector<CFDescription*> & FDescList)
{
	CFDescription * fdesc=NULL;

	for(vector<CFDescription*>::iterator fit=FDescList.begin();
		fit!=FDescList.end();fit++)
	{
		fdesc=*fit;
		delete fdesc;
	}	
	
	FDescList.clear();

	return true;
}
