/*REMOVED COMMENTS DUE TO PROBLEM IN LINUX ONLY VERSION*/
#include "stdafx.h"

#include "global.h"
#include "MorphologicalGenerator.h"
#include "LogManager.h"

#include <iostream>
#include <string>

#include "FileReader.h"
#include "FileWriter.h"

//////////////////////////////////////////////////////////////////////
// Construction/Destruction
//////////////////////////////////////////////////////////////////////

CMorphologicalGenerator::CMorphologicalGenerator(CLexicon * LexPtr)
{
	UrduLexicon=LexPtr;

	string file_name=CConfiguration::GetConfigInfo("NounRules");
	
	if(file_name!="")
		ReadRules((char*)file_name.c_str(), &nounRules);	
	else
		CLogManager::AddMessage(L"Noun Rules File Not found");
		
	file_name=CConfiguration::GetConfigInfo("VerbRules");

	if(file_name!="")
		ReadRules((char*)file_name.c_str(), &verbRules);	
	else
		CLogManager::AddMessage(L"Verb Rules File Not found");
	
	file_name=CConfiguration::GetConfigInfo("AdjRules");

	if(file_name!="")
		ReadRules((char*)file_name.c_str(), &adjRules);	
	else
		CLogManager::AddMessage(L"Adjective Rules File Not found");

	//initiallize Days[]
	Days[1]=L"پیر، ";	Days[2]=L"منگل، ";	Days[3]=L"بدہ، ";	Days[4]=L"جمعرات، ";	Days[5]=L"جمعہ، ";
	Days[6]=L"ہفتہ، ";	Days[7]=L"اتوار، ";
	
	//initialize Months[]
	Months[1]=L"جنوری";	Months[2]=L"فروری";	Months[3]=L"مارچ";	Months[4]=L"اپریل";	Months[5]=L"مئی";
	Months[6]=L"جون";	Months[7]=L"جولائی";	Months[8]=L"اگست";	Months[9]=L"ستمبر";	Months[10]=L"اکتوبر";
	Months[11]=L"نومبر";	Months[12]=L"دسمبر";

	//initialize features' numbers from symbol table
	//^DAY_IN_DIGIT=31st,^DAY_IN_WORD=NULL,^MONTH_IN_DIGIT=FALSE,^MONTH_VALUE=6,^Year=2005.
	DayInDigits=CSymbolTableManager::GetNum(L"DAY_IN_DIGIT");
	DayInWords=CSymbolTableManager::GetNum(L"DAY_IN_WORD");
	Month=CSymbolTableManager::GetNum(L"MONTH_VALUE");
	Year=CSymbolTableManager::GetNum(L"Year");
	Type=CSymbolTableManager::GetNum(L"TIMETYPE");
	Hours=CSymbolTableManager::GetNum(L"HR");
	Minutes=CSymbolTableManager::GetNum(L"MIN");
	Seconds=CSymbolTableManager::GetNum(L"SEC");

	//initialize Numbers[]
	Numbers[0]=L"";		Numbers[1]=L"ایک";	Numbers[2]=L"دو";	Numbers[3]=L"تین";	Numbers[4]=L"چار";	Numbers[5]=L"پانچ";
	Numbers[6]=L"چھ";	Numbers[7]=L"سات";	Numbers[8]=L"آٹھ";	Numbers[9]=L"نو";	Numbers[10]=L"دس";	Numbers[11]=L"گیارہ";
	Numbers[12]=L"بارہ";	Numbers[13]=L"تیرہ";	Numbers[14]=L"چودہ";	Numbers[15]=L"پندرہ";	Numbers[16]=L"سولہ";	Numbers[17]=L"سترہ";
	Numbers[18]=L"اٹھارہ";	Numbers[19]=L"انیس";	Numbers[20]=L"بیس";	Numbers[21]=L"اکیس";	Numbers[22]=L"بائیس";
	Numbers[23]=L"تئیس";	Numbers[24]=L"چوبیس";	Numbers[25]=L"پچیس";	Numbers[26]=L"چھبیس";	Numbers[27]=L"ستائیس";
	Numbers[28]=L"اٹھائیس";	Numbers[29]=L"انتیس";	Numbers[30]=L"تیس";	Numbers[31]=L"اکتیس";	Numbers[32]=L"بتیس";
	Numbers[33]=L"تینتیس";	Numbers[34]=L"چونتیس";	Numbers[35]=L"پینتیس";	Numbers[36]=L"چھتیس";	Numbers[37]=L"سینتیس";
	Numbers[38]=L"اڑتیس";	Numbers[39]=L"انتالیس";	Numbers[40]=L"چالیس";	Numbers[41]=L"اکتالیس";	Numbers[42]=L"بیالیس";
	Numbers[43]=L"تینتالیس";	Numbers[44]=L"چوالیس";	Numbers[45]=L"پینتالیس";	Numbers[46]=L"چھیالیس";
	Numbers[47]=L"سینتالیس";	Numbers[48]=L"اڑتالیس";	Numbers[49]=L"انچاس";	Numbers[50]=L"پچاس";
	Numbers[51]=L"اکاون";	Numbers[52]=L"باون";	Numbers[53]=L"ترپن";	Numbers[54]=L"چون";	Numbers[55]=L"پچپن";
	Numbers[56]=L"چھپن";	Numbers[57]=L"ستاون";	Numbers[58]=L"اٹھاون";	Numbers[59]=L"انسٹھ";	Numbers[60]=L"ساٹھ";
	Numbers[61]=L"اکسٹھ";	Numbers[62]=L"باسٹھ";	Numbers[63]=L"تریسٹھ";	Numbers[64]=L"چونسٹھ";	Numbers[65]=L"پینسٹھ";
	Numbers[66]=L"چھیاسٹھ";	Numbers[67]=L"سڑسٹھ";	Numbers[68]=L"اڑسٹھ";	Numbers[69]=L"انہتر";	Numbers[70]=L"ستر";
	Numbers[71]=L"اکہتر";	Numbers[72]=L"بہتر";	Numbers[73]=L"تہتر";	Numbers[74]=L"چوہتر";	Numbers[75]=L"پچھتر";	Numbers[76]=L"چھہتر";
	Numbers[77]=L"ستتر";	Numbers[78]=L"اٹھتر";	Numbers[79]=L"اناسی";	Numbers[80]=L"اسی";	Numbers[81]=L"اکاسی";
	Numbers[82]=L"بیاسی";	Numbers[83]=L"تراسی";	Numbers[84]=L"چوراسی";	Numbers[85]=L"پچاسی";	Numbers[86]=L"چھیاسی";
	Numbers[87]=L"ستاسی";	Numbers[88]=L"اٹھاسی";	Numbers[89]=L"نواسی";	Numbers[90]=L"نوے";	Numbers[91]=L"اکانوے";
	Numbers[92]=L"بانوے";	Numbers[93]=L"ترانوے";	Numbers[94]=L"چورانوے";	Numbers[95]=L"پچانوے";	Numbers[96]=L"چھیانوے";
	Numbers[97]=L"ستانوے";	Numbers[98]=L"اٹھانوے";	Numbers[99]=L"ننانوے";
}

CMorphologicalGenerator::~CMorphologicalGenerator()
{
	using namespace std;

	for(vector<CMorphologicalRule*>::iterator it=nounRules.begin();it!=nounRules.end();it++)
	{
		CMorphologicalRule* ruleptr=*it;
		delete ruleptr;
	}

	for(vector<CMorphologicalRule*>::iterator it1=verbRules.begin();it1!=verbRules.end();it1++)
	{
		CMorphologicalRule* ruleptr=*it1;
		delete ruleptr;
	}

	for(vector<CMorphologicalRule*>::iterator it2=adjRules.begin();it2!=adjRules.end();it2++)
	{
		CMorphologicalRule* ruleptr=*it2;
		delete ruleptr;
	}

	this->nounRules.clear();
	this->verbRules.clear();
	this->adjRules.clear();

}

vector<CLexicalEntry*> * CMorphologicalGenerator::GenerateForms(wstring word, wstring pos, PROPVECT* props)
{	
	//get the appropriate rule map and apply each rule..
	// if the rule applies then place result in a WSTRVECTOR	
	
	WSTRVECTOR* wordForms=NULL;
	RULEVECTOR* ruleVectPtr;
	wstring form=L"";
	CLexicalEntry*le=NULL;
	vector<CLexicalEntry*> * LEList = NULL ;
	LEList=new vector<CLexicalEntry*>;
	CFDescription* Worddescptr=NULL;
	
	if (pos!=L"n" && pos!=L"v" && pos!=L"adj")
	{
		if (pos==L"date")//date
		{
			wstring final=getDate(props);
			if(final!=L"")
			{
				le = new CLexicalEntry();
				le->Word=final;
				le->Predicate=final;
			
				for(vector<CFDescription*>::iterator it2=props->begin();it2!=props->end();it2++)
				{
					Worddescptr=*it2;
					le->ConditionList.push_back(Worddescptr);
				}

				le->POS=pos[0];
				LEList->push_back(le);
			}
			return LEList;
		}
		else if (pos==L"time")//time
		{
			wstring final=getTime(props);
			if(final!=L"")
			{
				le = new CLexicalEntry();
				le->Word=final;
				le->Predicate=final;
			
				for(vector<CFDescription*>::iterator it2=props->begin();it2!=props->end();it2++)
				{
					Worddescptr=*it2;
					le->ConditionList.push_back(Worddescptr);
				}

				le->POS=pos[0];
				LEList->push_back(le);
			}
			return LEList;
		}
		else if (pos==L"ordinal")//ordinal
		{
			wstring final=this->getOrdinal(word,props);
			if(final!=L"")
			{
				le = new CLexicalEntry();
				le->Word=final;
				le->Predicate=final;
			
				for(vector<CFDescription*>::iterator it2=props->begin();it2!=props->end();it2++)
				{
					Worddescptr=*it2;
					le->ConditionList.push_back(Worddescptr);
				}

				le->POS=pos[0];
				LEList->push_back(le);
			}
			return LEList;
		}
		else if (pos==L"cardinal")//cardinal
		{
			wstring final=this->getCardinal(word,props);
			if(final!=L"")
			{
				le = new CLexicalEntry();
				le->Word=final;
				le->Predicate=final;
			
				for(vector<CFDescription*>::iterator it2=props->begin();it2!=props->end();it2++)
				{
					Worddescptr=*it2;
					le->ConditionList.push_back(Worddescptr);
				}

				le->POS=pos[0];
				LEList->push_back(le);
			}
			return LEList;
		}
		else if (pos==L"alphanum")
		{
			wstring final=L"";
			for(PROPVECT::iterator i=props->begin(); i!=props->end(); i++)
			{	
				if((*i)->Label->list[0]==CSymbolTableManager::GetNum(L"ANUM"))
				{
					final=CSymbolTableManager::GetString((*i)->Value->list[0]);
				}
			}

			if(final!=L"")
			{
				le = new CLexicalEntry();
				le->Word=final;
				le->Predicate=final;
			
				for(vector<CFDescription*>::iterator it2=props->begin();it2!=props->end();it2++)
				{
					Worddescptr=*it2;
					le->ConditionList.push_back(Worddescptr);
				}

				le->POS=pos[0];
				LEList->push_back(le);
			}
			return LEList;
		}
		else
		{
            return NULL;
		}
	}
	else
	{
		if(pos==L"n")
		{
			ruleVectPtr=&this->nounRules;
		}
		else if(pos==L"v")
		{
			ruleVectPtr=&this->verbRules;
		}
		else if(pos==L"adj")
		{
			ruleVectPtr=&this->adjRules;
		}
		else
			return NULL;//wordForms; // NULL

		wordForms=new WSTRVECTOR;
		

		for(vector<CMorphologicalRule*>::iterator it=ruleVectPtr->begin();it!=ruleVectPtr->end();it++)
		{
			CMorphologicalRule* ruleptr=*it;
			le=new CLexicalEntry();
			if(ruleptr->applyRule(word,props, le)==true)
			{
				for(vector<CFDescription*>::iterator it2=props->begin();it2!=props->end();it2++)
				{
					Worddescptr=*it2;
					le->ConditionList.push_back(Worddescptr);
				}

				le->POS=pos[0];
				LEList->push_back(le);
				//wordForms->push_back(form);
			}
			else
				delete le;

		}
		return LEList;	
	}
}

void ReplaceSpaceWithinQ(wstring & A, short To, short By)
{
	bool QStart = false;

	for(int i=0;i<A.size();i++)
	{
		if(A[i]=='\'')
		{	
			QStart = !QStart;
			continue;
		}
		if(QStart && A[i]==To)
		{
			A[i] = By;
		}
	}
}
void ReplaceChar(wstring & A, short To, short By)
{
	for(int i=0;i<A.size();i++)
	{
		if(A[i]==To)
		{
			A[i] = By;
		}
	}
}

void CMorphologicalGenerator::ReadRules(char* fname, RULEVECTOR * ruleVectPtr)
{
	wstring affix=L"";
	wstring endCondStr=L"";
	wstring propString=L"";
	wstring replacementsString=L"";
	PROPVECT* propVectPtr=NULL;
	REPVECT* repVectPtr=NULL;
	CONDITIONVECTOR* endCondVect=NULL;

	CMorphologicalRule* rulePtr=NULL;

	/*short ch=L'';CHANGED IN THIS (LINUX ONLY) VERSION - will be ok in Win*/
	short ch;
	CFileReader fr(fname,256);
	ch=fr.GetCurrentChar();

	while(ch!=0)
	{
		affix=L"";
		endCondStr=L"";
		propString=L"";
		replacementsString=L"";

		//affix
		ReadFileTill(affix, L';', &fr);
		RemoveEndSpaces(affix);
		
		ReplaceSpaceWithinQ(affix,' ','$');

		ExtractStr(affix);

		ReplaceChar(affix,'$',' ');

		//ending conditions
		ReadFileTill(endCondStr, L';', &fr);
		RemoveEndSpaces(endCondStr);
		endCondVect=GetEndCondVect(endCondStr);
		
		//properties string
		ReadFileTill(propString, L';', &fr);
		RemoveEndSpaces(propString);
		propVectPtr=GetPropVect(propString);

		//replacements string
		ReadFileTill(replacementsString, L'.', &fr);
		RemoveEndSpaces(replacementsString);
		repVectPtr=GetRepVect(replacementsString);

		rulePtr=new CMorphologicalRule;	

		rulePtr->properties=propVectPtr;
		rulePtr->replacements=repVectPtr;
		rulePtr->end_conditions=endCondVect;
		rulePtr->affix=affix;	
		
		ruleVectPtr->push_back(rulePtr);
		ch=fr.GetCurrentChar();
	}
}

CFDescription* CMorphologicalGenerator::GetCFDescription(wstring str)
{
	RemoveEndSpaces(str);
	if(str.size()==0)
	{
		return NULL;
	}
	CFDescription* descPtr=new CFDescription;
	descPtr->Operator=L'=';
	short i;
	wstring labelStr=L"";
	wstring valueStr=L"";
	bool operatorfound=false;

	for(i=0;i<str.size(); i++)
	{
		if(str.at(i)==L'=')
		{
			operatorfound=true;
		}

		if(operatorfound==false)
		{
			labelStr+=str.at(i);
		}
		else if(operatorfound==true && str.at(i)!=L'=')
		{
			valueStr+=str.at(i);
		}

	}

	descPtr->Label=UrduLexicon->GenerateFDExpressionEx2(labelStr); ///changed
	descPtr->Operator='=';
 	RemoveEndSpaces(labelStr);
	RemoveEndSpaces(valueStr);
	descPtr->Value=UrduLexicon->GenerateFDExpressionEx2(valueStr); ///changed
	
	return descPtr;
}

WSTRPAIR* CMorphologicalGenerator::GetPair(wstring pairStr)
{
	RemoveEndSpaces(pairStr);
	
	WSTRPAIR* pairPtr=new WSTRPAIR;
	wstring rem=L"";
	wstring add=L"";

	if(pairStr.size()==0)
	{
		return NULL;
	}

	/*for(int i=0;pairStr.at(i)!=L'(';i++); CHANGED IN THIS (LINUX ONLY) VERSION - ok in Win*/
	int i;
	for(i=0;pairStr.at(i)!=L'(';i++);

	i=i+1;

	for(;pairStr.at(i)!=L',';i++)
	{
		rem+=pairStr.at(i);
	}

	i=i+1;

	for(; pairStr.at(i)!=L')';i++)
	{
		add+=pairStr.at(i);
	}

	RemoveEndSpaces(rem);
	RemoveEndSpaces(add);

	if(rem.at(0)==L'@')
	{
		rem=L"";
	}
	
	if(add.at(0)==L'@')
	{
		add=L"";
	}

	pairPtr->first=rem;
	pairPtr->second=add;

	return pairPtr;
}

REPVECT* CMorphologicalGenerator::GetRepVect(wstring repStr)
{
	RemoveEndSpaces(repStr);

	REPVECT* repVectPtr=new REPVECT;
	WSTRPAIR* pairPtr=NULL;
	wstring pairStr=L"";
	int i=0;
	bool start=false;
	/*short ch=L''; CHANGED IN THIS (LINUX ONLY) VERSION - ok in Win*/
	short ch;
	if(repStr.size()==0)
	{
		return repVectPtr;
	}
		
	ch=repStr.at(i);
	while(i<repStr.size())
	{
		ch=repStr.at(i);
		switch(ch)
		{
		case L'(':
			start=true;
			pairStr+=ch;
			break;
		case L')':
			if(start==true)
			{
				pairStr+=ch;
				pairPtr=GetPair(pairStr);
				if(pairPtr!=NULL)
				{
					repVectPtr->push_back(pairPtr);
				}
				pairStr=L"";
				start=false;				
			}
			else
			{
				cerr << "error in replacement pair's bracketing!";
			}
				break;
		default:
			if(start==true)
			{
				pairStr+=ch;
			}
		}
		i++;
	}
	return repVectPtr;
}

PROPVECT* CMorphologicalGenerator::GetPropVect(wstring propStr)
{
	RemoveEndSpaces(propStr);
	propStr+=L',';		//add delimeter to last property as well.
	PROPVECT* propVectPtr=new PROPVECT;
	CFDescription* cfdescPtr=NULL;
	short len=propStr.size();
	wstring descStr=L"";
	int p;
	bool inBracket=false;
	
	for(p=0; p<len; p++)
	{
		short ch=propStr.at(p);
		if(propStr.at(p)==L'{')
		{
			inBracket=true;
		}
		else if(propStr.at(p)==L'}')
		{
			if(!inBracket)
			{
				cerr<< "error with rule(properties) bracketing!";
			}
			inBracket=false;
		}

		if(propStr.at(p)==L',') // the desc string has completed!
		{
			if(!inBracket)
			{
				RemoveEndSpaces(descStr);
				cfdescPtr=GetCFDescription(descStr);
				if(cfdescPtr!=NULL)
				{
					propVectPtr->push_back(cfdescPtr);
				}
				descStr=L"";
			}
			else
			{
				descStr+=propStr.at(p);
			}
		}

		else // add to the current desc string
		{
			descStr+=propStr.at(p);
		}
	}
	return propVectPtr;

}

void CMorphologicalGenerator::ExtractStr(wstring& wstr)
{
	wstring ans=L"";
	TrimString(wstr);
	if(wstr.at(0)==L'@')
	{
		wstr=L"";
		return;
	}
	short len=wstr.size();

	for(short p=0; p<len; p++)
	{
		if(wstr.at(p)!=L'\'' && wstr.at(p)!=L' ')
		{
			ans+=wstr.at(p);
		}
	}
	wstr=ans;
	if(wstr.at(0)==L'@')
	{
		wstr=L"";
	}
	return;
}

CONDITIONVECTOR* CMorphologicalGenerator::GetEndCondVect(wstring str)
{
	CONDITIONVECTOR* CondVect= new CONDITIONVECTOR;
	CEndCondition* endCond;
	wstring condStr=L"";
	RemoveEndSpaces(str);
	str+=L',';
	short ch;
	int i=0;
	while(i<str.size())
	{
		ch=str.at(i);
		switch(ch)
		{
		case L',':
			ExtractStr(condStr);
			if(condStr.size()>0)
			{
				if(condStr.at(0)==L'~')
				{
					endCond=new CEndCondition;
					condStr=condStr.substr(1,condStr.size()-1);
					endCond->sequence=condStr;
					endCond->present=false;
				}
				else
				{
					endCond=new CEndCondition;
					endCond->sequence=condStr;
					endCond->present=true;
				}
				CondVect->push_back(endCond);
			}
			condStr=L"";
			break;
		default:
			condStr+=ch;
			break;
		}
		i++;
	}
	return CondVect;
}

wstring CMorphologicalGenerator::getDate(PROPVECT* props)
{
	wstring final;
	wstring daydigit, daywords, month, year, yeartype;
	for(PROPVECT::iterator i=props->begin(); i!=props->end(); i++)
	{
		if((*i)->Label->list[0]==DayInDigits)
		{
			daydigit=CSymbolTableManager::GetString((*i)->Value->list[0]);
		}
		else if((*i)->Label->list[0]==DayInWords)
		{
			daywords=CSymbolTableManager::GetString((*i)->Value->list[0]);
		}
		else if((*i)->Label->list[0]==Month)
		{
			month=CSymbolTableManager::GetString((*i)->Value->list[0]);
		}
		else if((*i)->Label->list[0]==Year)
		{
			year=CSymbolTableManager::GetString((*i)->Value->list[0]);
		}
		else if((*i)->Label->list[0]==Type)
		{
			yeartype=CSymbolTableManager::GetString((*i)->Value->list[0]);
		}
	}

	if(year==L"" && month==L"")
		return L"";

    if(daywords!=L"")
	{
		/* CHANGED IN THIS (LINUX ONLY) VERSION
		if(_wtoi(daywords.data())>0 && _wtoi(daywords.data())<8)
			final+=Days[_wtoi(daywords.data())];
		CHANGED IN THIS (LINUX ONLY) VERSION: */
		if(string2int(Convert_wstr2str(daywords.data()))>0 && string2int(Convert_wstr2str(daywords.data()))<8)
			final+=Days[string2int(Convert_wstr2str(daywords.data()))];		
		else
			final+=daywords;		
	}
	if(daydigit!=L"" && month!=L"")
	{
		if(daydigit!=L"NULL")
		{
			final+=daydigit;
			final+=L" ";
		}
		/* CHANGED IN THIS (LINUX ONLY) VERSION
		if(_wtoi(month.data())>0 && _wtoi(month.data())<13)
			final+=Months[_wtoi(month.data())];
		CHANGED IN THIS (LINUX ONLY) VERSION: */
		if(string2int(Convert_wstr2str(month.data()))>0 && string2int(Convert_wstr2str(month.data()))<13)
			final+=Months[string2int(Convert_wstr2str(month.data()))];
		else
			final+=month;
	}
	if(final!=L"")
		final+=L"، ";
	if(year!=L"")
	{
		final+=year;
		final+=L"";
	}
	if(yeartype!=L"")
	{
		final+=L" ";
		if(yeartype==L"bc")
			final+=L"قبل مسیح ";		
	}
	
	return final;
}

wstring CMorphologicalGenerator::getTime(PROPVECT* props)
{
	wstring final;
	wstring hr, min, sec, type;
	for(PROPVECT::iterator i=props->begin(); i!=props->end(); i++)
	{
		if((*i)->Label->list[0]==Hours)
		{
			hr=CSymbolTableManager::GetString((*i)->Value->list[0]);
		}
		else if((*i)->Label->list[0]==Minutes)
		{
			min=CSymbolTableManager::GetString((*i)->Value->list[0]);
		}
		else if((*i)->Label->list[0]==Seconds)
		{
			sec=CSymbolTableManager::GetString((*i)->Value->list[0]);
		}
		else if((*i)->Label->list[0]==Type)
		{
			type=CSymbolTableManager::GetString((*i)->Value->list[0]);
		}
	}

	if(hr==L"" && min==L"")
		return L"";

	if(hr!=L"")
	{
		if(type!=L"")
		{
			if (type==L"am" || type==L"AM" || type==L"a.m")
			{
				if(atoi(Convert_wstr2str(hr).c_str())<=2 || atoi(Convert_wstr2str(type).c_str())==12)
				{
					final+=L"رات ";
				}
				else if(atoi(Convert_wstr2str(hr).c_str())>=3 && atoi(Convert_wstr2str(type).c_str())<=11)
				{
					final+=L"صبح ";
				}
			}
			else if(type==L"pm" ||type==L"PM" ||type==L"p.m")
			{
				if(atoi(Convert_wstr2str(hr).c_str())<=2 || atoi(Convert_wstr2str(type).c_str())==12)
				{
					final+=L"دوپہر ";
				}
				else if(atoi(Convert_wstr2str(hr).c_str())==3)
				{
					final+=L"سہ پہر ";
				}
				else if(atoi(Convert_wstr2str(hr).c_str())>=4 && atoi(Convert_wstr2str(hr).c_str())<=7)
				{
					final+=L"شام ";
				}
				else if(atoi(Convert_wstr2str(hr).c_str())>=8 && atoi(Convert_wstr2str(hr).c_str())<=12)
				{
					final+=L"رات ";
				}
			}
		}
		final+=hr;
	}
	if(atoi(Convert_wstr2str(min).c_str())>0 || atoi(Convert_wstr2str(sec).c_str())>0)
	{
		final+=L"بجکر ";
		if(min!=L"")
		{
			final+=min;
			final+=L" منٹ ";
		}
		if(sec!=L"")
		{
			final+=sec;
			final+=L" سکنڈ";
		}
	}
	else
	{
		final+=L" بجے";
	}
	return final;
}

wstring CMorphologicalGenerator::getOrdinal(wstring word, PROPVECT* props)
{
	wstring final;
	wstring gend;
	wstring num_value;
	wstring flag_value;
	for(PROPVECT::iterator i=props->begin(); i!=props->end(); i++)
	{
		if(CSymbolTableManager::GetString((*i)->Label->list[0])==L"GEND")
		{
			gend=CSymbolTableManager::GetString((*i)->Value->list[0]);
			
		}
		else if(CSymbolTableManager::GetString((*i)->Label->list[0])==L"ORD_NUMBER_VAL")
		{
			num_value=CSymbolTableManager::GetString((*i)->Value->list[0]);
			
		}
		else if(CSymbolTableManager::GetString((*i)->Label->list[0])==L"IN_WORDS")
		{
			flag_value=CSymbolTableManager::GetString((*i)->Value->list[0]);
			
		}
	}

	if(flag_value!=L"NEG")
		final=this->getUrduNumberinWords(num_value);
	else
		final=num_value;

	//if(final==L"")
	//{
	//	final= L"صفر";
	//}

	int num;
	bool isfull=false;
	wstring a;
	//modification to make it an ordinal
	if(word.size()>=2)
	{
		a=word.substr(word.size()-2,2);
	}
	else
	{
		a=word;
		isfull=true;
	}
	num=atoi(Convert_wstr2str(a).c_str());
	int full=atoi(Convert_wstr2str(word).c_str());
	if (gend==L"M")
	{
		if(full==1 && isfull)
		{
			final=L"پہلا";
		}
		else if(full==2 && isfull)
		{
			final=L"دوسرا";
		}
		else if(full==3 && isfull)
		{
			final=L"تیسرا";
		}
		else if(full==4 && isfull)
		{
			final=L"چوتھا";
		}
		else if (full==6 && isfull)
		{
			final=L"چھٹا";
		}
		else if(num>=79 && num<=90)
		{
			final=final + L" واں";
		}
		else if(num>=91 && num<=99)
		{
			final=final.substr(0,final.size()-1)+L"اں";
		}
		else
		{
			final+=L"واں";
		}
	}
	else if(gend==L"F")
	{
		if(full==1 && isfull)
		{
			final=L"پہلی";
		}
		else if(full==2 && isfull)
		{
			final=L"دوسری";
		}
		else if(full==3 && isfull)
		{
			final=L"تیسری";
		}
		else if(full==4 && isfull)
		{
			final=L"چوتھی";
		}
		else if (full==6 && isfull)
		{
			final=L"چھٹی";
		}
		else if(num>=79 && num<=90)
		{
			final=final + L" ویں";
		}
		else if(num>=91 && num<=99)
		{
			final=final.substr(0,final.size()-1)+L"یں";
		}
		else
		{
			final+=L"ویں";
		}
	}
	return final;
}

wstring CMorphologicalGenerator::getCardinal(wstring word, PROPVECT* props)
{
	wstring final;
	wstring inwords, hascomma, unit, number;
	for(PROPVECT::iterator i=props->begin(); i!=props->end(); i++)
	{
		if(CSymbolTableManager::GetString((*i)->Label->list[0])==L"IN_WORDS")
		{
			inwords=CSymbolTableManager::GetString((*i)->Value->list[0]);
		}
		else if(CSymbolTableManager::GetString((*i)->Label->list[0])==L"HAS_COMMA")
		{
			hascomma=CSymbolTableManager::GetString((*i)->Value->list[0]);
		}
		else if(CSymbolTableManager::GetString((*i)->Label->list[0])==L"UNIT")
		{
			unit=CSymbolTableManager::GetString((*i)->Value->list[0]);
		}
		else if(CSymbolTableManager::GetString((*i)->Label->list[0])==L"NUMBER_VAL")
		{
			number=CSymbolTableManager::GetString((*i)->Value->list[0]);
		}
	}
	if(inwords==L"NEG")
	{
		final=number;
		if(hascomma==L"POS")
		{
			int index=number.size()-4;
			int dotat=number.find_last_of(L".");
			if(dotat!=-1)
			{
				index=dotat-4;
			}
			while(index>=0)
			{
				appendAt(final,index,L",");
				index=index-2;
			}
		}
		return final;
	}
	else
	{
		wstring w=this->getUrduNumberinWords(word);
		if(w==L"")
		{
			return L"صفر";
		}
		else
		{
			return w;
		}
	}
}

wstring CMorphologicalGenerator::getUrduNumberinWords(wstring eng)
{
	wstring final;
	wstring tens;
	wstring hundreds;
	wstring thousands;
	wstring laakh;
	wstring karor;
	wstring arab;

	//tens
	if(eng.size()>=2)
	{
		tens=getinWords(eng.substr(eng.size()-2,2));
	}
	else if(eng.size()==1)
	{
		return getinWords(eng);
	}
	else if(eng.size()==0)
	{
		return L"";
	}
	final=tens;
	
	//hundreds
	if(eng.size()>=3)
	{
        hundreds=getinWords(eng.substr(eng.size()-3,1));
	}
	else
	{
		return final;
	}
	if(hundreds!=L"")
	{
		final=hundreds + L" سو " + final;
	}
	
	//thousands
	if(eng.size()>=5)
	{
        thousands=getinWords(eng.substr(eng.size()-5,2));
	}
	else if(eng.size()==4)
	{
		thousands=getinWords(eng.substr(eng.size()-4,1));
		return thousands + L" ہزار " + final;
	}
	if(thousands!=L"")
	{
		final=thousands + L" ہزار " + final;
	}

	//laakh
	if(eng.size()>=7)
	{
        laakh=getinWords(eng.substr(eng.size()-7,2));
	}
	else if(eng.size()==6)
	{
		laakh=getinWords(eng.substr(eng.size()-6,1));
		return laakh + L" لاکھ " + final;
	}
	if(laakh!=L"")
	{
		final=laakh + L" لاکھ " + final;
	}

	//karor
	if(eng.size()>=9)
	{
        karor=getinWords(eng.substr(eng.size()-9,2));
	}
	else if(eng.size()==6)
	{
		karor=getinWords(eng.substr(eng.size()-8,1));
		return karor + L" کروڑ " + final;
	}
	if(karor!=L"")
	{
		final=karor + L" کروڑ " + final;
	}

	//arab
	if(eng.size()>=11)
	{
        arab=getUrduNumberinWords(eng.substr(0,eng.size()-9));
	}
	else if(eng.size()==10)
	{
		arab=getinWords(eng.substr(eng.size()-10,1));
		if(arab!=L"")
		{
			final= arab + L" ارب " + final;
		}
		return final;
	}
	if(arab!=L"")
	{
		final=arab + L" ارب " + final;
	}

	return final;
}

wstring CMorphologicalGenerator::getinWords(wstring eng) //for two words
{
	int num=atoi(Convert_wstr2str(eng).c_str());
	if(num>=0 && num<100)
		return Numbers[num];
	else
		return L"";
}
