// MorphologicalRule.cpp: implementation of the CMorphologicalRule class.
//
//////////////////////////////////////////////////////////////////////

#include "stdafx.h"

#include "MorphologicalRule.h"
#include "global.h"
#include "CChart.h"

//////////////////////////////////////////////////////////////////////
// Construction/Destruction
//////////////////////////////////////////////////////////////////////

int CMorphologicalRule::ObjectCounter=0;

CMorphologicalRule::CMorphologicalRule()
{
	ObjectCounter++;
	affix=L"";
}

CMorphologicalRule::~CMorphologicalRule()
{
	ObjectCounter--;

	for(vector<CFDescription*>::iterator it=this->properties->begin();it!=this->properties->end();it++)
	{
		CFDescription* descPtr=*it;
		delete descPtr;
	}
	this->properties->clear();
	delete this->properties;
	properties=NULL;

	for(vector<CEndCondition*>::iterator it2=this->end_conditions->begin();it2!=this->end_conditions->end();it2++)
	{
		CEndCondition* cond=*it2;
		delete cond;
	}
	this->end_conditions->clear();
	delete this->end_conditions;
	end_conditions=NULL;

	for(vector<WSTRPAIR*>::iterator it3=this->replacements->begin();it3!=this->replacements->end();it3++)
	{
		WSTRPAIR* wstrpair=*it3;
		delete wstrpair;
	}
	this->replacements->clear();
	delete this->replacements;
}

bool CMorphologicalRule::applyRule(wstring word, PROPVECT* props, CLexicalEntry* le)
{
	CFDescription* Ruledescptr=NULL;
	CFDescription* Worddescptr=NULL;
	short Rulelabel;
	short WordProplabel;
	wstring wordForm;
	//short RuleVal, WordVal;
	CFDescription* matchedWordProp=NULL;
	bool valueMatched=false;
	list<CFDescription*> ConditionList;

	if(word.size()<=0)
	{
		return false;
	}


	for(vector<CEndCondition*>::iterator i=this->end_conditions->begin(); i!=this->end_conditions->end(); i++)
	{
		CEndCondition* cond=*i;
		if(cond->present==false)
		{
			if(MatchEndSeq(word, cond->sequence) == true)
			{
				return false;
			}
		}
		else
		{
			if(MatchEndSeq(word, cond->sequence) ==false)
			{
				return false;
			}
		}
	}

	bool matched = false;
	//chk if the properties match
	for(vector<CFDescription*>::iterator it=this->properties->begin();it!=this->properties->end();it++)
	{
		Ruledescptr=*it;
		Rulelabel=Ruledescptr->Label->list.front();
		matchedWordProp=NULL;
		matched = false;
		for(vector<CFDescription*>::iterator it2=props->begin();it2!=props->end();it2++)
		{
			Worddescptr=*it2;
			WordProplabel=Worddescptr->Label->list.front();
			if(WordProplabel==Rulelabel && Ruledescptr->Value->list.size()!=0)
			{
				//matchedWordProp=Worddescptr;
				if(Worddescptr->Value->list.size())
				{
					if(CChart::IsIntersectionEmpty(&Ruledescptr->Value->list,&Worddescptr->Value->list))
					{
						return false;	
					}
					matched = true;
					break;
				}
				else
				{
					matched = true;
					break;
				}
			}
        }
		if (!matched)
		{
			Ruledescptr->Label->op = UP_OP;
			Ruledescptr->Value->op = VALUE_OP;
			ConditionList.push_back(Ruledescptr);
    	}
	}

	for(list<CFDescription*>::iterator it2=ConditionList.begin();it2!=ConditionList.end();it2++)
	{
		Worddescptr=*it2;
		le->ConditionList.push_back(Worddescptr);
	}
	
	//apply any replacement in the vector that is applicable
	WSTRPAIR* currRep=NULL;
	bool match=true;

	for(vector<WSTRPAIR*>::iterator it5=this->replacements->begin();it5!=this->replacements->end();it5++)
	{
		currRep=*it5;
		match=MatchEndSeq(word,currRep->first);
		if(match==true)
		{
			//apply the replacement!
			wordForm=word.substr(0,word.size()-currRep->first.size());
			wordForm+=currRep->second;
			wordForm=wordForm+this->affix; //apply the affix and return
			
			le->Word=wordForm;
			le->Predicate=word;
			le->IsCache=true;
			return true;
		}
	}

	//apply the affix
	wordForm=word+this->affix;

	le->Word=wordForm;
	le->Predicate=word;
	le->IsCache=true;

	return true;
}
