#include "stdafx.h"
#include "P_DisqualifierElement.h" /*CHANGED IN THIS (LINUX) VERSION*/

P_DisqualifierElement::P_DisqualifierElement(void)
: lexeme_pos(0)
, disqualifier_pos(0)
, is_lexeme(false)
, disqualifier_position(0)
, is_plus(false)
// DONE 23-08-06
{
}

P_DisqualifierElement::P_DisqualifierElement(wstring element_string)
: lexeme_pos(0)
, disqualifier_pos(0)
, is_lexeme(0)
, disqualifier_position(0)
, is_plus(false)
// DONE 23-08-06
{
	// string example:
	// a_art+1

	// first copy word till u get to the ':'
	int i = 0;
	wstring str;
	while (element_string[i] != ':')
	{
		str.push_back(element_string[i]);
		i++;
	}
	
	// get the pos (or lack of one)
	wstring pos_str;
	i++;
	while (element_string[i] != '+' && element_string[i] != '-')
	{
		pos_str.push_back(element_string[i]);
		i++;
	}
	
	if (pos_str == L"#")	// means that the indicator itself is a pos
	{
		disqualifier_pos = POSTable::getPOSNum(Convert_wstr2str(str));
	}
	else
	{
		is_lexeme = true;
		disqualifier_lexeme = str;
		lexeme_pos = POSTable::getPOSNum(Convert_wstr2str(pos_str));
	}

	// now get direction and position
	if (element_string[i] == '+')
		is_plus = true;

	i++;
	char *position = new char[2];
	position[0] = element_string[i];
	position[1] = '\0';
	disqualifier_position = atoi(position);
}

P_DisqualifierElement::~P_DisqualifierElement(void)
// DONE 23-08-06
{
}

bool P_DisqualifierElement::disqualified(vector<vector<P_LexicalEntry*> > &lexical_entry_list, vector<Token*> &token_list, short segment_index)
// DONE 24-08-06
{
	short disqualification_index = segment_index;
	vector<vector<P_LexicalEntry*> >::iterator lex_entries_it = lexical_entry_list.begin();
	vector<Token*>::iterator tokens_it = token_list.begin();
	
	// first get the index of the lexical entry and token that need to be examined
	if (is_plus)
		disqualification_index = segment_index + disqualifier_position;
	else
		disqualification_index = segment_index - disqualifier_position;

	// ensure that the disqualification index is within sentence range
	if (disqualification_index < token_list.size())
	{
		// position iterators accordingly
		for (short i = 0; i < disqualification_index; i++)
		{
			lex_entries_it++;
			tokens_it++;
		}

		
		vector<P_LexicalEntry*>::iterator it;
		// now check if the current lexical entry matches the disqualification element
		if (is_lexeme)
		{
			// check if lexeme matches
			if (disqualifier_lexeme == (*tokens_it)->lexeme)
			{
				// check if there is a matching pos
				vector<P_LexicalEntry*>::iterator lex_entry_it;
				for (lex_entry_it = (*lex_entries_it).begin(); lex_entry_it != (*lex_entries_it).end(); lex_entry_it++)
				{
					if ((*lex_entry_it)->POS == lexeme_pos)
						return true;		
				}
			}
		}
		else
		{
			// select highest probability POS and match that
			float prob = 0;
			P_LexicalEntry* highest_prob_entry = NULL;
			it = (*lex_entries_it).begin();
			prob = (*it)->probability;
			highest_prob_entry = (*it);
			for (it = (*lex_entries_it).begin(); it != (*lex_entries_it).end(); it++)
			{
				if (prob < (*it)->probability)
				{
					prob = (*it)->probability;
					highest_prob_entry = (*it);
				}
			}
			if (disqualifier_pos == highest_prob_entry->POS)
				return true;
		}
	}
	return false;
}

void P_DisqualifierElement::removeInvalidEntries(vector<vector<P_LexicalEntry*> >& lexical_entry_list, P_SegmentMatch* match)
// DONE 24-08-06
{
	// identify the lexical entry that needs the clean-up operation
	vector<vector<P_LexicalEntry*> >::iterator lex_entries_it = lexical_entry_list.begin();

	short ident = match->match_index;
	if (is_plus)
		ident = ident + disqualifier_position;
	else
		ident = ident - disqualifier_position;

	short i = 0;
	while (i < ident)
	{
		lex_entries_it++;		
		i++;
	}
	// ensure that the disqualification index is within sentence range
	if (ident < lexical_entry_list.size())
	{
		// have reached the lexical entry; remove the pos
		if (!is_lexeme)
		{
			if ((*lex_entries_it).size() > 1)
			{
				vector<P_LexicalEntry*> temp; 
				while (!(*lex_entries_it).empty())
				{
					temp.push_back((*lex_entries_it).back());
					(*lex_entries_it).pop_back();
				}
			
				P_LexicalEntry* entry;
				while (!temp.empty())
				{
					entry = temp.back();
					if (entry->POS != disqualifier_pos)
						(*lex_entries_it).push_back(entry);
					temp.pop_back();
				}
			}
		}
		else
		{
			if ((*lex_entries_it).size() > 1)
			{
				vector<P_LexicalEntry*> temp; 
				while (!(*lex_entries_it).empty())
				{
					temp.push_back((*lex_entries_it).back());
					(*lex_entries_it).pop_back();
				}
			
				P_LexicalEntry* entry;
				while (!temp.empty())
				{
					entry = temp.back();
					if (entry->POS != lexeme_pos)
						(*lex_entries_it).push_back(entry);
					temp.pop_back();
				}
			}
		}
	}
}
