#pragma once

/*ADDED IN THIS (LINUX) VERSION*/
#ifndef P_FSINITIALIZER_H
#define P_FSINITIALIZER_H
/*ADDED IN THIS (LINUX) VERSION*/

#include "fsglobal.h"
#include "P_FSItem.h"
#include "P_FSItemSimple.h"
#include "P_FSItemComplex.h"
#include "P_TempFSItemArray.h"
#include "P_ConValPlainValList.h"
#include "P_ConValFDLink.h"
#include "P_FSSymbolTable.h"

#include <vector>
#include <string>
/*#include <hash_map>REMOVED IN THIS (LINUX ONLY) VERSION*/
#include <iostream>
#include <fstream>

using namespace std;
/*using namespace stdext;REMOVED IN THIS (LINUX ONLY) VERSION*/

struct P_TempFDStringItem
{
	string f_desc_string;
	P_FSItem* f_item;
};

class P_FSInitializer
{
public:
	P_FSInitializer(void);
	~P_FSInitializer(void);

	static list<string> fsTypeAttr;
	static vector<PFSitemID> getFDVector(string f_desc, contextByte &max_context);
	static vector< vector<PFSitemID> > getFDVectors(vector< vector<string> > f_descriptions, contextByte &max_context);
	static P_FSItem* getFSItem(PFSitemID itemID);
	static void printDataStructures(ofstream &myfile);	// for testing
	static void init(void);	// instead of constructor because this will be a static class
	static void deleteExtraInfo(void);	// to be called after all major f-description processing is complete
	static void endInitialization(void);
	static void collectSelectiveGarbage(void);
	
private:
    // primary data structures
	static P_FSItem** fsitem_lookup_table;	// keeps generic fs items for instantiation
											// their ID is the index of the array
	static PFSitemID fsitem_insertion_index;
	static vector<PFSitemID>* fsitem_insert_acc;	// to help speedup insertion
	static PFSitemID lookup_table_temp_index;	// indicates the point from where the temporary entries start in fsitem_lookup_table
	
	// temporary data structures
	static P_TempFSItemArray temp_fsitems;	// keeps temporary versions of P_FSItems that need to be merged
									// and may or may not be inserted into fsitem_lookup_table;
									// exists for construction of 1 FDVector
	
	static P_TempFDStringItem* temp_FD_string_item;	// keeps single f-descriptions with their 
												// corresponding P_FSItem so that copies can
												// be made to put in temp_fsitems 
												// (saves construction cost);
												// exists for construction of all FDVectors
	static PFSitemID temp_FD_si_index;	// insertion index for above array
	static hash_map<string, PFSitemID> dummy_hash;	// hash table for efficient insertion into above array	
									
	static PFSitemID* fdesc_numeric;	// to store f-description in numeric form
	static int fdesc_num_counter;	// to keep track of insertions
	static list<contextByte>* fdesc_num_contexts;	// to temporarily keep contexts
	static list< list<contextByte> > prod_context;	// to keep overall production context
	static list< list<contextByte> > prepareProductionContextList(byte no_of_fdescs); // to calculate overall production context
	static contextByte max_assigned_context;

	static bool preprocess(string f_desc);	// makes a numeric array of the f-description and fills the temp data structs
	static string getNextToken(string input, int &index);	// e.g. NUM =c SG, SUBJ NUM = PL, ADV_TYPE = {V_MOD, N_MOD} etc.
															// or operators: [ ] , | ~
	static bool fdNumInsert(PFSitemID op);	// inserts an element into fdesc_numeric
	static string cleanUpToken(string token);	// removes useless spaces
	
	static PFSitemID getTempFDStringItemID(string fdesc);	// inserts the string (with the P_FSItem) if it doesn't
													// already exist and returns its ID
	static P_FSItem* getTempFDStringItemCopy(PFSitemID item_id);	// returns the item corresponding to the ID
	
	static string* reGetNextToken(string input);	// e.g. "SUBJ", "=c", "NUM"

	static list<PFSsymID> reReGetNextToken(string input);	// retrieves individual value IDs from a list e.g., {SG, PL}
	static list<string> reReGetNextTokenString(string input);	// retrieves individual value strings from a list e.g., {SG, PL}

	static bool resolveNots(void);	// resolves all the nots in fdesc_numeric
	static bool applyContexts(void);	// adds contexts in fdesc_num_contexts
	static bool buildTempFSItemList(void);	// put initial fs item versions (no contexts) in list
	static PFSitemID getFSItemID(P_FSItem* fs_item);
	static PFSitemID getCorrespondingOr(int index);
	static bool initialization_complete;
	static void fixFormat(string& fdesc);

	// to speed up insertion into fsitem table
	static PFSitemID findFSItem(P_FSItem* fs_item);
	static void updateFSItemInsertAcc(PFSsymID attr, PFSitemID insert_index);
	static void removeFromAcc(PFSsymID attr, PFSitemID insert_index);	// to remove indices of temp entries
	static bool internal_call;	// to check whether the call to getFDVector is external or from getFDVectors
	static void changeSetToOrs(string& fdesc);
public:
	static void DumpP_FSInitializer();
	static void LoadP_FSInitializer();
};
#endif /*ADDED IN THIS (LINUX) VERSION*/
