#include "stdafx.h"
#include "P_Grammar.h"
#include "RawProduction.h"
#include "StringFunc.h" /*CHANGED IN THIS (LINUX) VERSION*/
#include "P_FSInitializer.h"
#include "FileWriter.h" /*CHANGED IN THIS (LINUX) VERSION*/

#include<math.h>

/*
//	Some utility operators & functions

bool operator || (const CSymbol & A, const CSymbol & B)
bool operator ||(const vector<CSymbol*> & A, const vector<CSymbol*> & B)
bool operator == (const CSymbol & A, const CSymbol & B)
bool operator ==(const vector<CSymbol*> & A, const vector<CSymbol*> & B)
bool operator == (const CRawProduction& A, const CRawProduction& B)
void ReplaceSymbols(list<CRawProduction*> * vlist,CRawProduction* Prod1,CRawProduction* Prod2,CRawProduction* ProdBy)
CRawProduction* MergeDifferentLHS(CRawProduction* Prod1,CRawProduction* Prod2)
CRawProduction* MergeSameLHS(CRawProduction* Prod1,CRawProduction* Prod2)

*/

P_Grammar::P_Grammar(const string & grules_fname, 
				   const string & macros_fname,
				   const string & lfg_fname,
				   const string & cfg_fname,
				   bool ReplaceStarByRange)	:ProdTable(400)
	{
		
		string Compile_Option = CConfiguration::GetConfigInfo("CompileP_Grammar" , true);
		if(Compile_Option=="yes")
		{
				PreprocessGrammar(grules_fname,macros_fname,lfg_fname);
			
				NewProdName=L"P_";	

				list<CRawProduction*>*vlist2 = ConvertLFG2CFG(this->ReadLFG(lfg_fname));
				
				WriteCFGInFile(vlist2,"debugCFG.cfg");	//7

				map<wstring,bool>* NULLS = Comp_Dir_Nullables( vlist2 ); // 4
			
				if( NULLS )
				{
					NULLS = Comp_Nullables( vlist2 , NULLS); // 5

					vlist2=GenerateSelectionSet(vlist2,NULLS); // 6
			
					insertPOSInPOSMap(CConfiguration::GetConfigInfo("POSs"));
					insertProdInProdMap(vlist2);

					PurifyProductions(vlist2);

					AddInProductionsMap();

					WriteCFGInFile(vlist2,cfg_fname);	//7
		            
					this->WriteGrammarInFile(this->AllProductions,CConfiguration::GetConfigInfo("ncfgFileName"));

					try{
						delete NULLS;
					}
					catch(...)
					{
						#ifdef GenLog
						CLogManager::AddMessage(L"#### NULL");
						#endif
					}

				}

				DumpGrammartoDumpFile();
		}
		else
		{
			LoadGrammarFromDumpFile();
		}
}
/*****************************************************************/
bool operator == (const CSymbol & A, const CSymbol & B)
{
	if(A.condition==B.condition && A.lexeme == B.lexeme)
	{
		return true;
	}
	return false;
}
/*****************************************************************/
/*****************************************************************/

bool operator ==(vector<CSymbol*> & A, vector<CSymbol*> & B)
{
	if(A.size()!=B.size())
	{
		return false;
	}
	for(int i=0;i<A.size();i++)
	{
		if(!(*A[i]==*B[i]))
		{
			return false;
		}
	}
	return true;
}
/*****************************************************************/

/*****************************************************************/

bool operator && (vector<CRawProduction*> & tempMerger1 , vector<CRawProduction*> & tempMerger2)
{
	if(tempMerger1.size()!=tempMerger2.size())
	{
		return false;
	}
	else
	{
		for(int i=0;i<tempMerger1.size();i++)
		{
			if(!(tempMerger1[i]->RHS==tempMerger2[i]->RHS))
			{
				return false;
			}
		}
		return true;
	}
}


bool operator || (const CSymbol & A, const CSymbol & B)
{
	if(A.lexeme == B.lexeme)
	{
		return true;
	}
	return false;
}
/*****************************************************************/
/*****************************************************************/

bool operator ||(const vector<CSymbol*> & A, const vector<CSymbol*> & B)
{
	if(A.size()!=B.size())
	{
		return false;
	}
	for(int i=0;i<A.size();i++)
	{
		if(!(*A[i]||*B[i]))
		{
			return false;
		}
	}
	return true;
}
/*****************************************************************/
/*****************************************************************/

void ReplaceSymbols(list<CRawProduction*> * vlist,CRawProduction* Prod1,CRawProduction* Prod2,CRawProduction* ProdBy)
{
	list<CRawProduction*>::iterator iter = vlist->begin();

	for(;iter!=vlist->end();iter++)
	{
		CRawProduction* p = *iter;

		for(int i=0;i<p->RHS.size();i++)
		{
			if(p->RHS[i]->lexeme == Prod1->LHS->lexeme || p->RHS[i]->lexeme == Prod2->LHS->lexeme)
			{
				p->RHS[i]->lexeme = ProdBy->LHS->lexeme;
			}
		}
	}
 }
/*****************************************************************/
/*****************************************************************/

CRawProduction* MergeDifferentLHS(CRawProduction* Prod1,CRawProduction* Prod2)
{
	CRawProduction* temp = new CRawProduction();
    
	wstring tempwstring = Prod1->LHS->lexeme;
	temp->LHS = new CSymbol(tempwstring,L"",false);

	for(int i=0;i<Prod1->RHS.size();i++)
	{
		temp->RHS.push_back(Prod1->RHS[i]->GetClone());
	}

	return temp;
}
/*
CRawProduction* MergeSameLHS(CRawProduction* Prod1,CRawProduction* Prod2)
{
	CRawProduction* temp = new CRawProduction();
    
	wstring tempwstring = Prod1->LHS->lexeme+Prod2->LHS->lexeme;
	temp->LHS = new CSymbol(tempwstring,L"",false);

	for(int i=0;i<Prod1->RHS.size();i++)
	{
		temp->RHS.push_back(Prod1->RHS[i]->GetClone());
	}

	return temp;
}    */ //  

/*****************************************************************/
/*****************************************************************/
/*****************************************************************/

bool operator == (const CRawProduction& A, const CRawProduction& B)
{
	if(A.RHS == B.RHS)
	{
		return true;
	}
	return false;
}
/*****************************************************************/
/*****************************************************************/
/**						Utility Functions Ends					**/
/*****************************************************************/
/*****************************************************************/

void P_Grammar::WriteCFGInFile(list<CRawProduction*> * vlist,string CFGFileName)
{
	using namespace std;

	CRawProduction* rp=NULL;
	CSymbol * rs=NULL;
	vector<CSymbol*>*rp_rhs=NULL;

	CFileWriter fw(CFGFileName);

	for(list<CRawProduction*>::iterator it=vlist->begin();it!=vlist->end();it++)	
	{
		rp=*it;
		if(rp)
		{
			fw.Write(rp->LHS->lexeme);

			/********** CHANGE BY OMAR FOR WRITING SELECTION SET TO FILE (START)********/
			
			fw.Write(L" [");

			map<short,short>::iterator sel_set_it = rp->SelectionSet.begin();
			wstring sel_str = L"";
			while( sel_set_it != rp->SelectionSet.end() )
			{
				//int num = sel_set_it->first;
				sel_str = CSymbolTableManager::GetString(sel_set_it->first);
				fw.Write(sel_str);
				fw.Write(L",");				
				
				sel_set_it++;
			}			

			fw.Write(L"]");

			/********** CHANGE BY 'GREAT *** OMAR *** [May Allah place him in Heaven]' FOR WRITING SELECTION SET TO FILE (END)********/

			fw.Write(L" -> ");			
			rp_rhs=&rp->RHS;
			for(vector<CSymbol*>::iterator it2= rp_rhs->begin(); it2!=rp_rhs->end();it2++)
			{
				rs=*it2;
				fw.Write(rs->lexeme);
				if(!rs->isop)
				{
					fw.Write(L":");
					fw.Write(rs->condition);
					fw.Write(L"; ");
				}				
			}			
		fw.Write(L"\r\n");
		}		
	}		

	fw.CloseFile();
}


list<CRawProduction*>* P_Grammar::ConvertLFG2CFG(list<CRawProduction*> * vlist)
{
	stack<CSymbol*> stk1;
	stack<CSymbol*> stk2;

	short uchar=0;
	short start_index=-1;
	short end_ndex=-1;
	short current_pno=0;
	short len=0;
	short new_prod_counter=0;	
	int prodNoForDebug =0;
	list<CRawProduction*>* rprod_list=new list<CRawProduction*>;

	if(!rprod_list)
		return NULL;

	CRawProduction*rprod=NULL;
	CRawProduction*tprod=NULL;

	CSymbol*symb=NULL;
	CSymbol*nsymb=NULL;

	short list_len=vlist->size();

	int debugVal=126;
	//for(short k=0;k<list_len;k++)
	for(list<CRawProduction*>::iterator rpit=vlist->begin();rpit!=vlist->end();rpit++)
	{

		if(new_prod_counter>=256)
		{
			new_prod_counter=new_prod_counter;
		}

		prodNoForDebug ++;
		if(prodNoForDebug ==debugVal)
		{
			prodNoForDebug =prodNoForDebug ;
		}
		rprod=*rpit;

		//rprod=vlist->at(k);

		//rprod->printW();

		len=rprod->RHS.size();			

		for(short p=0;p<len;p++)
		{
			symb=rprod->RHS.at(p);

			if(symb->lexeme[0]=='.')
				break;

			stk1.push(symb);				

			uchar=symb->lexeme[0];

			if(symb->isop==true && (uchar==']' || uchar==')'))
			{
				stk1.pop();	//pop ']' ')'

				short cbr=0;
				symb=stk1.top();
				cbr=symb->lexeme[0];

				while(cbr!='[' && cbr != '(')
				{
					stk2.push(symb);
					stk1.pop();
					
					symb=stk1.top();
					cbr=symb->lexeme[0];
				}

				stk1.pop();	//pop '[' '('

				symb=rprod->RHS.at(p+1);
				uchar=symb->lexeme[0];					

				if(cbr=='(')
				{
					//////////////////////////////
					// A -> B ( D | E ) F 
					// conversions:
					// A -> B X2 F
					// A -> B F
					// X2 -> D 
					// X2 -> E								
					
					// X2 -> . (NULL Production) now this is removed


					////////////////////////////

					new_prod_counter++;

					nsymb=new CSymbol(NewProdName,L"^=!",false);

					if(!nsymb)
						break;

					nsymb->lexeme+=GetWSTR(new_prod_counter);

					vector<CSymbol*> * slist=GetStackList(&stk2);							

					tprod=new CRawProduction();

					if(!tprod)
						break;

					tprod->LHS=nsymb;
					AppendList(&tprod->RHS,slist);
					tprod->RHS.push_back(GetDotPtr());

					//tprod->print();
					vlist->push_back(tprod);

/****************************************************/
			/*
			 *	Change Start here
			 *	By Sarfraz
			 *	NULL Production shdn't be produced
			 *	A -> B F is creating
			 */

			tprod=new CRawProduction();

					if(!tprod)
						break;

			tprod->LHS = rprod->LHS;

			slist=GetStackList2WithCopy(&stk1);
			
			//tprod->RHS = *slist;
			
			AppendList(&tprod->RHS,slist);

			for(int iindex = p+1;iindex<len;iindex++)
			{
				tprod->RHS.push_back(rprod->RHS[iindex]);
			}
		
			vlist->push_back(tprod);
/****************************************************/
			/*
			 *	Change End here
			 *	By Sarfraz
			 *	NULL Production shdn't be produced
			 */
/****************************************************/
					stk1.push(nsymb);
				}
				else if(symb!=NULL && (uchar=='*'|| uchar=='+' || uchar=='#'))
				{
					p++;

					if(uchar=='*' || uchar=='+')
					{
						// A -> B [ D | E ]* F 
						// conversions:
						// A -> B X2 F
						// A -> B F
						// X2 -> [D | E] X2
						// X2 -> [D | E]
						////////////////////////////


						// A -> B [ D | E ]+ F 
						// conversions:
						// A -> B X2 F
						// X2 -> [D | E] X2
						// X2 -> [D | E]
						//creating X2
						new_prod_counter++;

						nsymb=new CSymbol();

						if(!nsymb)
							break;

						nsymb->condition=L"^=!";
						nsymb->isop =false;

						nsymb->lexeme=NewProdName;
						nsymb->lexeme+=GetWSTR(new_prod_counter);

						vector<CSymbol*> *slist=GetStackListRev(&stk2);							

						//creating: X2 -> [D | E]
						tprod=new CRawProduction();

						if(!tprod)
							break;

						tprod->LHS=nsymb;
						AppendList(&tprod->RHS,slist);
						tprod->RHS.push_back(GetDotPtr());

						//tprod->print();
						rprod_list->push_back(tprod);
						vlist->push_back(tprod);
						
						//creating: X2 -> [D | E] X2
						tprod=new CRawProduction();

						if(!tprod)
							break;

						tprod->LHS=nsymb;
						AppendList(&tprod->RHS,slist);
						tprod->RHS.push_back(nsymb);
						tprod->RHS.push_back(GetDotPtr());

						//tprod->print();
						rprod_list->push_back(tprod);
						vlist->push_back(tprod);
						//creating: S -> B F
						//for * operator only
						if(uchar=='*')
						{
/*****************************************************/
/*			// Ignored Again
//----------------------------------------------------/
				CRawProduction * prod3=new CRawProduction();

				if(!prod3)
					break;

				prod3->LHS=nsymb;
				prod3->RHS.push_back(GetDotPtr());																								
				//prod3->print();
				rprod_list->push_back(prod3);
*/
/*****************************************************/
			int nothingstar = 0;
/**************************************************/
			/*
			 *	Change Start here
			 *	By Sarfraz
			 *	NULL Production shdn't be produced
			 *	A -> B F is creating
			 */
/**************************************************/
			tprod=new CRawProduction();

					if(!tprod)
						break;

			tprod->LHS = rprod->LHS;

			slist=GetStackList2WithCopy(&stk1);

			AppendList(&tprod->RHS,slist);

			for(int iindex = p+1;iindex<len;iindex++)
			{
				tprod->RHS.push_back(rprod->RHS[iindex]);
			}
		
			vlist->push_back(tprod);
			rprod_list->push_back(tprod);
/****************************************************/
			/*
			 *	Change End here
			 *	By Sarfraz
			 *	NULL Production shdn't be produced
			 */
/****************************************************/
						}

						stk1.push(nsymb);

					}					
					else if(uchar=='#')
					{
						// A -> B [ D % E ]#1#3 F

						// conversions:
						// A -> B X2 F
						// A -> B X2 X2 F
						// A -> B X2 X2 X2 F
						// X2-> [D % E]

						p++; //at number1
						CSymbol * min=rprod->RHS.at(p);								
						//wprintf(L"%s",min->lexeme.c_str());

						p++; //at number2					
						p++; //at number2												

						CSymbol * max= rprod->RHS.at(p);
						//wprintf(L"%s",max->lexeme.c_str());										

						//p++; //creating X2

						new_prod_counter++;

						nsymb=new CSymbol(NewProdName,L"^=!",false);								

						if(!nsymb)
							break;

						nsymb->lexeme+=GetWSTR(new_prod_counter);

						new_prod_counter++;
						CSymbol * nsymb3=new CSymbol(NewProdName,L"^=!",false);								

						if(!nsymb3)
							break;

						nsymb3->lexeme+=GetWSTR(new_prod_counter);

						CRawProduction * prod3=new CRawProduction();								

						if(!prod3)
							break;

						prod3->LHS=nsymb3;								

						if(stk2.size()>0)
						{
							AppendList( &prod3->RHS,GetStackListRev(&stk2));															
						}

						prod3->RHS.push_back(GetDotPtr());															

						rprod_list->push_back(prod3);

						///////////////////////////added/////////////////////
						stk1.push(nsymb);								

						vector< vector<CSymbol*> * > * cases_list= GetAllCasesList(nsymb3,min,max);	

						for(unsigned short n=0;n<cases_list->size();n++)
						{
							CRawProduction*temp_prod=new CRawProduction();

							if(!temp_prod)
								break;

							temp_prod->LHS=nsymb;									

							AppendList(&temp_prod->RHS,cases_list->at(n));																										

							temp_prod->RHS.push_back(GetDotPtr());

							rprod_list->push_back(temp_prod);									
						}

						if(cases_list)
						{
							for(vector< vector<CSymbol*> * >::iterator it=cases_list->begin();
								it!=cases_list->end();it++)
							{
								try{
									delete *it;
								}
								catch(...)
								{
									#ifdef GenLog
									CLogManager::AddMessage(L"deleting cases_list ..\r\n");
									#endif
								}
							}
							try{
								delete cases_list;
							}
							catch(...)
							{
								#ifdef GenLog
								CLogManager::AddMessage(L"#### ");
								#endif
							}

							cases_list=NULL;
						}

					} //end else if
				}
				else 
					// productions enclosed in [XYZ DS DSD] with no
					// outer-operator *, + or #
				{
					///////////////////////////X->[[A%[A|B]]|D]
					//	X -> [A%B]
					//	X -> A B
					//	X -> B A

					new_prod_counter++;

					CSymbol * nsymb=new CSymbol(NewProdName,L"^=!",false);	

					if(!nsymb)
						break;

					nsymb->lexeme+=GetWSTR(new_prod_counter);

					tprod=new CRawProduction();

					if(!tprod)
						break;

					tprod->LHS=nsymb;

					while(!stk2.empty() && (stk2.top())->lexeme!=L"[")
					{
						tprod->RHS.push_back(stk2.top());
						stk2.pop();
					}

					list<CRawProduction*> * temp_prod_list=GetResolvedProductionList(tprod);
					AddProdList(rprod_list,temp_prod_list);
					AddProdList(vlist,temp_prod_list);
					stk1.push(nsymb);
				}
			}
		}										

		if(stk1.size()>0)
		{
			vector<CSymbol*> * symb_list=GetStackList(&stk1);

			rprod->RHS=*symb_list;
			rprod->RHS.push_back(GetDotPtr());				
			rprod_list->push_back(rprod);	

			if(symb_list)
			{
				try{
					delete symb_list;
					symb_list=NULL;
				}
				catch(...)
				{
					#ifdef GenLog
					CLogManager::AddMessage(L"#### symb_list");
					#endif
				}

			}

		}

	}//end for					



// Removing All NULL productions prodeced if all optionals in the productions

//return vlist;

/*
///////////////////////////////////////////////////
//	This is the code for removing NULL Productions
///////////////////////////////////////////////////
*/

	WriteCFGInFile(vlist,"debug.cfg");	//7


CRawProduction* debugp ;
list<CRawProduction*>::iterator debugiter = vlist->begin();
bool changes;
do
{
	changes = false;

	for(list<CRawProduction*>::iterator i=vlist->begin();i!=vlist->end();i++)
	{
		CRawProduction* p = *i;

		// If the production is Null-Production

		if(p->RHS.size()==1 && p->RHS[0]->lexeme==L".")
		{

			bool AnyotherProdExist = false;
			for(list<CRawProduction*>::iterator ni=vlist->begin();ni!=vlist->end();ni++)
			{
				if(ni==i)
					continue;

				if((*ni)->LHS->lexeme == (*i)->LHS->lexeme)
				{
					AnyotherProdExist = true;
					break;
				}
			}

			list<CRawProduction*>::iterator itemp = i;
			i--;
			vlist->erase(itemp);
			
//			vlist->remove(*i);

			for(list<CRawProduction*>::iterator i2=vlist->begin();i2!=vlist->end();i2++)
			{
				CRawProduction* pp = *i2;
				
				// If the production is Null-Production
				
				for(int index=0;index<pp->RHS.size();index++)
				{
					if(pp->RHS[index]->lexeme==p->LHS->lexeme)
					{
						if(!AnyotherProdExist)
						{
							pp->RHS.erase(pp->RHS.begin()+index);
							changes = true;
							break;
						}

						CRawProduction* ppclone = new CRawProduction();
						ppclone->LHS = pp->LHS->GetClone();
							
							
						for(int iii=0;iii<pp->RHS.size();iii++)
						{
							ppclone->RHS.push_back(pp->RHS[iii]->GetClone());
						}
							
						changes = true;
						ppclone->counter = p->counter;
							
						pp->RHS.erase(pp->RHS.begin()+index);
							
						vlist->insert(i2,ppclone);
							
						i2++;
							
						break;
					}
				}
			}
		}
	}
}
while(changes);
/*
//////////////////////////////////////////////////////////////////////////
//	This is the Merging of the Productions having 
//	different left Hand side & same Right Hand side
/////////////////////////////////////////////////////////////////////////
*/
	int debuglisti = 117;


	vlist = SortProductions(vlist);

	WriteCFGInFile(vlist,"debugB4NULLRemovalCFG.cfg");	//7

	vector<CRawProduction*> MHead;
	vector<CRawProduction*>* MRemove;

	vector<vector<CRawProduction*>* > AllRemovers;


	wstring lhs=L"";
	wstring lhsNext=L"";

	CRawProduction* curr;

	for(list<CRawProduction*>::iterator i=vlist->begin();i!=vlist->end();i++)
	{
		curr = *i;

		lhsNext = curr->LHS->lexeme;

		string tnlhs = wstringToString(lhsNext);
		string tlhs = wstringToString(lhs);

		if(lhsNext == L"P_121")
		{
			lhsNext  =  lhsNext ;
		}

		if(lhs==L"")
		{
			lhs = lhsNext;
			MHead.push_back(curr);
			continue;
		}


		if(lhsNext == lhs)
		{
			MHead.push_back(curr);
			continue;
		}
		else
		{
			
			//	lhsNext Already Assigned to the Next 
			//	Change it to lhs
			lhs = lhsNext;
			MRemove = new vector<CRawProduction*>;
			list<CRawProduction*>::iterator i2 = i;
			i--;
		//	i2++;
			for( ; i2!=vlist->end();i2++)
			{
				CRawProduction* curr = *i2;
				lhsNext = curr->LHS->lexeme;

				if(lhsNext == lhs)
				{
					MRemove->push_back(curr);
					continue;
				}
				else
				{
					// Check for the Equality and Merge if possible

					if(MHead && *MRemove)
					{
						CRawProduction* Debugcurr = MRemove->at(0);
						AllRemovers.push_back(MRemove);
						
						MRemove = new vector<CRawProduction*>;
						
						MRemove->push_back(curr);
						lhs = lhsNext;

					}
					else
					{
						MRemove->clear();
						MRemove->push_back(curr);
						lhs = lhsNext;
					}
				}
				
			}
			/*
				Removing is Here
			*/

			if(AllRemovers.size()>0)
			{
						lhs = lhs;
			}

			for(int t1=0;t1<AllRemovers.size();t1++)
			{
				for(int t2=0;t2<AllRemovers[t1]->size();t2++)
				{
					vlist->remove(AllRemovers[t1]->at(t2));
				}
			}

			/*************************************************/
			if(AllRemovers.size())
			{
				if(MHead.at(0)->LHS->lexeme==L"P_121")
				{
					lhs = lhs;
				}
				CRawProduction * tcurr;
				for(list<CRawProduction*>::iterator ti = vlist->begin();ti!=vlist->end();ti++)
				{
					
					tcurr = *ti;
					if(tcurr->LHS->lexeme == L"Simp")
					{
						string temp = wstringToString(tcurr->RHS[0]->lexeme);
						if(temp=="P_218")
						{
							temp = temp;
						}
					}

					for(int t3=0;t3<tcurr->RHS.size();t3++)
					{
						for(int t4=0;t4<AllRemovers.size();t4++)
						{
								string t0 = wstringToString(tcurr->RHS[t3]->lexeme);
								string t00 =wstringToString(AllRemovers[t4]->at(0)->LHS->lexeme);

							if(tcurr->RHS[t3]->lexeme==AllRemovers[t4]->at(0)->LHS->lexeme)
							{
								tcurr->RHS[t3] = MHead.at(0)->LHS;
								break;
							}
						}
					}
				}
			}
			
			lhs = L"";
			MHead.clear();
			MRemove->clear();
			AllRemovers.clear();

			if(lhs!=L"")
			{
				lhs = lhs; 
			}
		}
	}
	return vlist;
}

string P_Grammar::GiveProdSym(int i)
{
	return ProdSymbols[i];
}
string P_Grammar::GivePOSSym(int i)
{
	return Terminals[i];
}
vector<P_Production*>* P_Grammar::GetProduction(short LHS_Symbol, P_FirstSet FirstSet)
{
	vector<P_Production*>* temp = new vector<P_Production*>;
	string pname = ProdSymbols[LHS_Symbol];

	hash_multimap<short,P_Production*>::iterator i = ProductionsMap.find(LHS_Symbol);
	P_Production* tempProd;

	while(i!=ProductionsMap.end() && i->first == LHS_Symbol)
	{
		tempProd = i->second;

		if(tempProd->SelectionSet & FirstSet)
		{
			temp->push_back(tempProd);
		}
		i++;
	}
 return temp;
}
vector<P_Production*>* P_Grammar::GetProduction(short LHS_Symbol)
{
	vector<P_Production*>* temp = new vector<P_Production*>;
	string pname = ProdSymbols[LHS_Symbol];

	hash_multimap<short,P_Production*>::iterator i = ProductionsMap.find(LHS_Symbol);
	P_Production* tempProd;

	while(i!=ProductionsMap.end() && i->first == LHS_Symbol)
	{
		tempProd = i->second;

		temp->push_back(tempProd);
		
		i++;
	}
 return temp;
}


P_Production* P_Grammar::GetProduction(int ProdId)
{
	return AllProductions->at(ProdId);
}

short P_Grammar::GetStartSymbolID()
{
	return StartSymbolID;
}

void P_Grammar::init()
{
}

void P_Grammar::DumpGrammartoDumpFile()
{
	P_Production* temp;
	int ChunkSize = 50;		// At Max Production Size
	int NoOfProductions = AllProductions->size();
	short *Buffer = new short[NoOfProductions*ChunkSize];

	int Bindex=-1;

	Buffer[++Bindex]= StartSymbolID;		// Start Symbol
	Buffer[++Bindex]= NoOfProductions;
	

	FILE * fwriter=fopen(CConfiguration::GetConfigInfo("DumpedEgrammar").c_str(),"wb");
	size_t bytes_written=fwrite((void*)Buffer,2,Bindex+1,fwriter);

	
/*	fclose(fwriter);

	Buffer = new short[2];	
	
	// one is for StartSymbolID
	// 2nd is for Number of Productions


	FILE *fr=fopen(CConfiguration::GetConfigInfo("DumpedEgrammar").c_str(),"rb");
	size_t bytes_read=fread((void*)Buffer,2,2,fr);

*/

	Bindex=-1;

	for(int i=0;i<NoOfProductions;i++)
	{
		temp = AllProductions->at(i);
		Buffer[++Bindex]=temp->LHS;		// LHS
		Buffer[++Bindex]=temp->ProdID;
		Buffer[++Bindex]=temp->MAXCONTEXT;
		*(int*)(&Buffer[++Bindex])=temp->SelectionSet.FirstSet1;
		++Bindex;
		*(int*)(&Buffer[++Bindex])=temp->SelectionSet.FirstSet2;
		++Bindex;
		*(float*)(&Buffer[++Bindex])=temp->probability;
		++Bindex;
////////////////////////////////////////////
		Buffer[++Bindex]=temp->RHS.size();

		for(int j=0;j<temp->RHS.size();j++)
		{
			Buffer[++Bindex]= temp->RHS[j]->SymbolId;
		
			Buffer[++Bindex]= temp->RHS[j]->Terminal;

			Buffer[++Bindex]= temp->RHS[j]->FDescriptionIndices.size();

			for(int k=0;k<temp->RHS[j]->FDescriptionIndices.size();k++)
			{
				Buffer[++Bindex]= temp->RHS[j]->FDescriptionIndices[k];
			}
		}
	}

	bytes_written=fwrite((void*)Buffer,2,Bindex+1,fwriter);

	fclose(fwriter);



	////////////////////////////////////
	//		Writing Productions Symbol
	////////////////////////////////////
	
	fstream symbolsWriter(CConfiguration::GetConfigInfo("DumpedEgrammarSymbols").c_str(),ios::out);

	symbolsWriter<<Terminals.size()<<endl;
	symbolsWriter<<ProdSymbols.size()<<endl;
	
	/* CHANGED IN THIS (LINUX) VERSION
	for(int i=1;i<Terminals.size();i++)//because Terminals[0] is empty entry
	CHANGED IN THIS (LINUX) VERSION */
	int i;
	for(i=1;i<Terminals.size();i++)//because Terminals[0] is empty entry
	{		
		symbolsWriter<<Terminals[i]<<endl;
	}

	for(i=0;i<ProdSymbols.size();i++)
	{		
		symbolsWriter<<ProdSymbols[i]<<endl;
	}

	symbolsWriter.close();



	//Buffer = new short[2];	
	//
	//// one is for StartSymbolID
	//// 2nd is for Number of Productions


	//FILE *fr=fopen(CConfiguration::GetConfigInfo("DumpedEgrammar").c_str(),"rb");
	//size_t bytes_read=fread((void*)Buffer,2,2,fr);


}

void P_Grammar::LoadGrammarFromDumpFile()
{
	P_Production* temp;
	int ChunkSize = 50;		// At Max Production Size
	int NoOfProductions ;//= AllProductions->size();
	short *Buffer = new short[2];	
	
	// one is for StartSymbolID
	// 2nd is for Number of Productions

	int Bindex=-1;

	FILE *fr=fopen(CConfiguration::GetConfigInfo("DumpedEgrammar").c_str(),"rb");
	size_t bytes_read=fread((void*)Buffer,2,2,fr);

	StartSymbolID= Buffer[++Bindex];		// Start Symbol
	NoOfProductions=Buffer[++Bindex] ;

	AllProductions = new vector<P_Production*>(NoOfProductions);
	
	delete []Buffer;



	Buffer = new short[NoOfProductions*ChunkSize];

	bytes_read=fread(Buffer,2,NoOfProductions*ChunkSize,fr);

	Bindex = -1 ;

	for(int i=0;i<NoOfProductions;i++)
	{
		temp = new P_Production();
		temp->LHS=Buffer[++Bindex];		// LHS
		temp->ProdID=Buffer[++Bindex];
		temp->MAXCONTEXT=Buffer[++Bindex];
		temp->SelectionSet.FirstSet1=*(int*)(&Buffer[++Bindex]);
		++Bindex;
		temp->SelectionSet.FirstSet2=*(int*)(&Buffer[++Bindex]);
		++Bindex;
		temp->probability=*(float*)(&Buffer[++Bindex]);
		++Bindex;
////////////////////////////////////////////
		temp->RHS.resize(Buffer[++Bindex]);

		for(int j=0;j<temp->RHS.size();j++)
		{
			
			
			temp->RHS[j] = new P_Symbol;
			
			temp->RHS[j]->SymbolId=Buffer[++Bindex];
			
			
			temp->RHS[j]->Terminal=Buffer[++Bindex];
			
			temp->RHS[j]->FDescriptionIndices.resize(Buffer[++Bindex]);
						
			for(int k=0;k<temp->RHS[j]->FDescriptionIndices.size();k++)
			{
				temp->RHS[j]->FDescriptionIndices[k]=Buffer[++Bindex];
				
			}
		}

		(*AllProductions)[i]=temp;
		ProductionsMap.insert(ProdMapEntry(temp->LHS,temp));
	}
	
	////////////////////////////////////
	//		Writing Productions Symbol
	////////////////////////////////////
	
	fstream symbolsReader(CConfiguration::GetConfigInfo("DumpedEgrammarSymbols").c_str(),ios::in);
	int TerminalSize;
	int ProdSymbolsSize;
	symbolsReader>>TerminalSize;
	symbolsReader>>ProdSymbolsSize;

	Terminals.resize(TerminalSize);
	ProdSymbols.resize(ProdSymbolsSize);

	
	/* CHANGED IN THIS (LINUX) VERSION
	for(int i=1;i<Terminals.size();i++)//because Terminals[0] is empty entry
	CHANGED IN THIS (LINUX) VERSION */
	int i;
	for(i=1;i<Terminals.size();i++)//because Terminals[0] is empty entry
	{		
		symbolsReader>>Terminals[i];		
		TerSymbolsMap.insert(PosSymMapEntry(Terminals[i],i));	
	}

	for(i=0;i<ProdSymbols.size();i++)
	{
		symbolsReader>>ProdSymbols[i];
		ProdSymbolsMap.insert(ProdSymMapEntry(ProdSymbols[i],i));	
	}

	symbolsReader.close();

}

void P_Grammar::ReadGrammar()
{
	
}
/************************************************************/
	/****		Wht should i do Yaar	****/
/************************************************************/
vector<CSymbol*> * P_Grammar::GetStackListRev(stack<CSymbol*> *  stk1)
{
	
		vector<CSymbol*> * vlist=new vector<CSymbol*>;

		if(!vlist)
			return NULL;

		CSymbol* temp=NULL;

		while(!stk1->empty())
		{
			temp=stk1->top();
			vlist->push_back(temp);
			stk1->pop(); 
		}
		
	return vlist;
}
/*****************************************************************/
bool P_Grammar::IsNonTerminal(wstring & TerOrNTer)
{
	if( TerOrNTer[0]>='a' && TerOrNTer[0]<='z' )
	{
		return false;
	}
	else
	{
		return true;
	}
}
/*****************************************************************/
	int P_Grammar::GiveSizeOfNT(list<CRawProduction*> * vlist ,vector<wstring> & fillNT)
	{
		bool Ispresent = false;
		CRawProduction* p=0;
		for(list<CRawProduction*>::iterator iter = vlist->begin();iter!=vlist->end();iter++)
		{
			Ispresent = false;
			p=*iter;
			for(int i=0;i<fillNT.size();i++)
			{
				if(fillNT[i]==p->LHS->lexeme )
				{
					Ispresent = true;
					break;
				}
			}
			if(Ispresent==false)
			{
				fillNT.push_back(p->LHS->lexeme);
			}
		}
		return fillNT.size();
	}
/*****************************************************************/
	int P_Grammar::GiveNTIndex(vector<wstring>& A,wstring Astring)
	{
		for(int i=0;i<A.size();i++)
		{
			if(Astring==A[i])
			{
				return i;
			}
		}
	//temp change
		return -1;
		// shd never come here

		throw ("\n\nRUN TIME ERROR ILLEGAL");
	}
/*****************************************************************/
bool P_Grammar::Add(vector<wstring>& SetA, vector<wstring>& SetB)
{
	bool tbool=false;
	for(int i=0;i<SetB.size();i++)
	{
			tbool|=Add(SetA,SetB[i]);
	}
	return tbool;
}

//************************************************//
bool P_Grammar::Add(vector<wstring>& Set, wstring toAdd)
{
	int size=Set.size();
	for(int i=0;i<size;i++)
	{
		if(Set[i]==toAdd)
			return false;
	}
	Set.push_back(toAdd);
	return true;
}

/*****************************************************************/
	list<CRawProduction*>* P_Grammar::GenerateSelectionSet(list<CRawProduction*> * vlist, map<wstring,bool>* nullables)
	{
	
	bool SetChanged = false;

	typedef map<wstring,int>* SMapPtr;

	map< wstring,SMapPtr > FirstSet;
	map< wstring,SMapPtr > FollowSet;

	map<wstring,int> * CurrFirst = NULL;
	map<wstring,int> * CurrFollow = NULL;

	typedef pair <wstring,int> StrIntPair;
	typedef pair <wstring,map<wstring,int>*> StrMapPair;


	vector<wstring> SelectionSetwithProd(vlist->size());
	
	vector<wstring> ProdsOnlyNT;
						
	int NonTerminalSize = GiveSizeOfNT(vlist,ProdsOnlyNT);
	vector<vector<wstring> > SSProdsOnlyNT(NonTerminalSize);
	
	vector<vector<wstring> > SSwithAllProds(vlist->size());

	bool changes;
	
	
	list<CRawProduction*>::iterator piter = vlist->begin();

	///**********************************///
	do
	{
		changes=false;
		CRawProduction* p;
		
		int ProdIndex=0;
		for(list<CRawProduction*>::iterator piter = vlist->begin();piter!=vlist->end();piter++)
		{
			p=*piter;
			
			for(int rhs=0;rhs<p->RHS.size();rhs++)
			{
				wstring temp=p->RHS[rhs]->lexeme;
				if(IsNonTerminal(temp))  //Non Terminal
				{
					int ind2=GiveNTIndex(ProdsOnlyNT,temp);
					//temp change
					if(ind2 != -1)
					{
						changes|=Add(SSwithAllProds[ProdIndex],SSProdsOnlyNT[ind2]);
						int ind3 = GiveNTIndex(ProdsOnlyNT,p->LHS->lexeme);
						if(ind3 != -1)
							changes|=Add(SSProdsOnlyNT[ind3],SSProdsOnlyNT[ind2]);
					}
					break;
					
				}
				else	// if(IsTerminal(temp))
				{
					changes|=Add(SSwithAllProds[ProdIndex],temp);
					
					int ind3 = GiveNTIndex(ProdsOnlyNT,p->LHS->lexeme);

					changes|=Add(SSProdsOnlyNT[ind3],temp);
					break;
				}
			}				
			
		ProdIndex++;
		}
	}
	while(changes);



	typedef pair <short,short> ShShPair;

	list<CRawProduction*>::iterator prod_it;

	vector<vector<wstring> >::iterator ProdIterator;


// I m here

	for( prod_it = vlist->begin(),ProdIterator= SSwithAllProds.begin(); prod_it != vlist->end() ; prod_it++,ProdIterator++ )//each prduction x->y1 y2 ... yk 
	{
		CRawProduction * CurrProd = *prod_it;

		vector<wstring> FSetOfCurrProd = *ProdIterator;
		
		for( int fsi=0;fsi<FSetOfCurrProd.size() ; fsi++ ) //each i from 1 to k, each j from i+1 to k )
		{
			
			wstring FSstring  = FSetOfCurrProd.at(fsi);
			short num = CSymbolTableManager::GetNum(FSstring,true);
			CurrProd->SelectionSet.insert( ShShPair(num,num) );
		}
	}
	return vlist;
}

	map<wstring,bool>* P_Grammar::Comp_Nullables( list<CRawProduction*> * vlist , map<wstring,bool>* nullables)
	{
	list<CRawProduction*>::iterator prod_iter = vlist->begin();

	typedef pair <wstring,bool> StrLexPtrPair;
	bool Change_Occured = false;
	
	do
	{
		Change_Occured = false;
		for( ; prod_iter!=vlist->end() ; prod_iter++ )
		{
			CRawProduction * this_prod = * prod_iter;

			map<wstring,bool>::iterator s_result = nullables->find(this_prod->LHS->lexeme);
			/*if( s_result != NULL )CHANGED IN THIS (LINUX) VERSION*/
			if( s_result != nullables->end() )
				continue; //already null
			else
			{
				vector<CSymbol*>* RHS_list = &this_prod->RHS;
	
				bool Not_Null = false;
	
				for( vector<CSymbol*>::iterator T = RHS_list->begin() ; T < RHS_list->end() ; T++ )
				{
					CSymbol* this_sym = *T;
					s_result = nullables->find( this_sym->lexeme );
					/*if( s_result != NULL  ) CHANGED IN THIS (LINUX ONLY) VERSION - ok in Win*/
					if (s_result !=  nullables->end())
					{
						Not_Null = true;
						break;
					}	
				}
				
				if( !Not_Null )
				{
					Change_Occured = true;
					nullables->insert( StrLexPtrPair( this_prod->LHS->lexeme , true) );
				}
			}
		}
	}
	while( Change_Occured );

	/* TEST WHICH NON-TERMINALS HAVE BEEN MARKED AS NULLABLE*/
	/*wstring allnulls = L"";

	for( map<wstring,bool>::iterator Tap = nullables->begin() ; Tap != nullables->end() ; Tap++ )
	{
		allnulls += Tap->first;
		allnulls += L"	";
	}

	MessageBox(NULL,allnulls.c_str(),L"NULLS",MB_ICONINFORMATION);*/

	return nullables;
}

	map<wstring,bool>* P_Grammar::Comp_Dir_Nullables( list<CRawProduction*> * vlist )
	{
	map<wstring,bool>* nullables = new map<wstring,bool>;
	
	typedef pair <wstring,bool> StrLexPtrPair;

	int count = 0;

	list<CRawProduction*>::iterator prod_iter = vlist->begin();

	for( ; prod_iter!=vlist->end() ; prod_iter++ )
	{
		CRawProduction * this_prod = * prod_iter;
		vector<CSymbol*>* RHS_list = &this_prod->RHS;

		if( RHS_list ->size() == 1 )
		{
			vector<CSymbol*>::iterator T = RHS_list->begin();
			CSymbol* this_sym = *T;
			if( this_sym->isop == true )
				nullables->insert(StrLexPtrPair( this_prod->LHS->lexeme ,true));
		}
	}

	return nullables;
}


	
	vector< vector<CSymbol*> * > * P_Grammar::GetAllCasesList(CSymbol*symb,CSymbol*mins,CSymbol*maxs)
	{
		vector< vector<CSymbol*> * > * case_list=NULL;

		if(!symb||!mins||!maxs)
			return case_list;

		case_list = new vector< vector<CSymbol*> * >;

		//..assuming #min#max ranges,,,,,min and max ranges from 0 to 9

		short min = mins->lexeme[0]-'0';
		short max = maxs->lexeme[0]-'0';


		vector<CSymbol*> * tlist=NULL;

		for(short k=min;k<=max;k++)
		{
			tlist=new vector<CSymbol*>;

			if(!tlist)
				continue;

			for(short m=0;m<k;m++)		
				tlist->push_back(symb);		

			case_list->push_back(tlist);
		}

		return case_list;	
	}

	void P_Grammar::GenerateNumberPermutations(short min, short max, vector<vector<short> > & vlist)
	{
		using namespace std;

		vector<short> ca;

		for(short p=min;p<=max;p++)
			ca.push_back(p);  

		CPermutationMaker maker; 

		vector<short> cnext;   

		maker.get_vector_permutation(ca,cnext,vlist); 

	}

	void P_Grammar::AddProdList(list<CRawProduction*>*first,list<CRawProduction*>*second)
	{
		if(!first||!second)
			return;

		//short len=second->size();
		//	for(short p=0;p<len;p++)	
		//		first->push_back(second->at(p));		

		for(list<CRawProduction*>::iterator rpit=second->begin();rpit!=second->end();rpit++)
			first->push_back(*rpit);
	}

	

	list<CRawProduction*>* P_Grammar::GetResolvedProductionList(CRawProduction * prod)
	{
		short len=0;		
		len=prod->RHS.size();

		short Operator=0;
		
		CSymbol*symb=NULL;
		
		vector<CSymbol*> symb_list;

		list<CRawProduction*> * prod_list=new list<CRawProduction*>;

		if(!prod_list)
			return NULL;
		
		for(short k=0;k<len;k++)
		{		
			symb=prod->RHS.at(k);

			if(symb->isop)
				Operator=symb->lexeme[0];
			else
				symb_list.push_back(symb);	

		}

		CProduction * nprod=NULL;

		if(Operator=='%')
		{
				short symb_count= symb_list.size();
				
				vector< vector<short> > perm_list;

				vector<short> perm_sub_list;
				
				//generate number permutations in the given range
				//returns vector<vector<int>>
				//for arguments min=0, max=2
				//result: [0,1,2],[0,2,1],[1,0,2],[1,2,0],[2,0,1],[2,1,0]

				GenerateNumberPermutations(0,symb_count-1,perm_list);

				short len1=perm_list.size();

				CRawProduction*nprod=NULL;

				for(short a=0;a<len1;a++)
				{
					nprod=new CRawProduction();

					if(!nprod)
						continue;
					
					nprod->LHS=prod->LHS;

					perm_sub_list=perm_list.at(a);
					
					short len2= perm_sub_list.size();				
					short  ind=-1;

					for(short b=0;b<len2;b++)
					{	
						ind=perm_sub_list.at(b);
						nprod->RHS.push_back(symb_list[ind]);
					}		

					nprod->RHS.push_back(GetDotPtr());
					//nprod->print();
					prod_list->push_back(nprod);			
				}	
			}
			else if(Operator=='|')
			{
				CRawProduction*tprod=NULL;

				for(unsigned short n=0;n<symb_list.size();n++)
				{		
					tprod = new CRawProduction();
					
					tprod->LHS = prod->LHS;
					tprod->RHS.push_back(symb_list[n]);
					tprod->RHS.push_back(GetDotPtr());				
					prod_list->push_back(tprod);
				}
			}
			else
			{			
				prod->RHS.push_back(GetDotPtr());
				prod_list->push_back(prod);
			}	

	return prod_list;
		
	}


	vector<CSymbol*> * P_Grammar::GetStackList2WithCopy(stack<CSymbol*> *  stk1)
	{
		stack<CSymbol*> stk2;

		vector<CSymbol*> * vlist=new vector<CSymbol*>;

		if(!vlist)
			return NULL;

		CSymbol* temp=NULL;

		while(!stk1->empty())
		{
			temp=stk1->top();
			stk2.push(temp);
			stk1->pop();
		}

		while(!stk2.empty())
		{
			temp=stk2.top();
			stk1->push(temp);
			vlist->push_back(temp);		
			stk2.pop();
		}

		return vlist;
	}


	vector<CSymbol*> * P_Grammar::GetStackList(stack<CSymbol*> *  stk1)
	{
		stack<CSymbol*> stk2;

		vector<CSymbol*> * vlist=new vector<CSymbol*>;

		if(!vlist)
			return NULL;

		CSymbol* temp=NULL;

		while(!stk1->empty())
		{
			temp=stk1->top();
			stk2.push(temp);
			stk1->pop();
		}

		while(!stk2.empty())
		{
			temp=stk2.top();
			vlist->push_back(temp);		
			stk2.pop();
		}

		return vlist;
	}

	void P_Grammar::AppendList(vector<CSymbol*> * first,vector<CSymbol*> * second)
	{
		
		if(!first||!second)
			return;
		
		short len=second->size();

		for(short p=0;p<len;p++)	
			first->push_back(second->at(p));		

	}

	wstring P_Grammar::GetNewProdName()
	{
		return NewProdName;
	}



	list<CRawProduction*>* P_Grammar::ReadLFG(string LFGFileName)
	{
	CFileReader fr(LFGFileName.c_str(),1024);
	
	wstring str=L"";
	short ch=0;

	CSymbol * symbol;
	CRawProduction * prod=new CRawProduction();

	if(!prod)
	{
		fr.CloseFile(); /*ADDED IN THIS (LINUX) VERSION*/
		return NULL;
	}
	
	list<CRawProduction*>*prod_list;	
	prod_list=new list<CRawProduction*>;

	if(!prod_list)
		return NULL;

	bool start=false;
	
	while((ch=fr.GetCurrentChar())!=0)
	{

		if(ch!=LF && ch!=CR)
			str+=ch;	

		switch(ch)
		{			
			case '>' :
				str=L"";
				break;
			
			case '-' :
				symbol=new CSymbol();
				
				if(!symbol)
					break;

				str.erase(str.length()-1);
				symbol->lexeme=TrimString(str);
				prod=new CRawProduction();

				if(!prod)
					break;

				prod->LHS =symbol;			
				break;

			case ':' :
				
				symbol=new CSymbol();
				
				if(!symbol)
					break;

				str.erase(str.length()-1);
				symbol->lexeme=TrimString(str);				
	
				str=L"";		
				fr.MoveNext();	
				while((ch=fr.GetCurrentChar())!=';')		
				{
					str+=ch;
					fr.MoveNext();
				}
				symbol->condition=TrimString(str);				
				prod->RHS.push_back(symbol);			
				symbol=NULL;
				str=L"";			
				break;			
		
			case '.' :											

				symbol=NULL;
				str=L"";
				symbol=new CSymbol();
				
				if(!symbol)
					break;

				symbol->isop=true;
				symbol->lexeme=ch;
				symbol->condition=L"";
				prod->RHS.push_back(symbol);				
				
				prod_list->push_back(prod);

				prod=new CRawProduction();

				if(!prod)
					break;

				break;

			case '[' :	case ']' :	case '{' :	
			case '}' :	case '(' : 	case ')' :	
			case '*' :	case '+' : 	case '%' :	
			case '|' :	case '~' :
				symbol=new CSymbol();
				
				if(!symbol)
					break;

				symbol->isop=true;
				symbol->lexeme=ch;
				symbol->condition=L"";
				prod->RHS.push_back(symbol);
				str=str.erase(str.length()-1);						
				break;
			
			case '#' :

				symbol=new CSymbol();
				
				if(!symbol)
					break;

				symbol->isop=true;
				symbol->lexeme=ch;
				symbol->condition=L"";
				prod->RHS.push_back(symbol);

				ch=fr.MoveNext();

				symbol=new CSymbol();
				
				if(!symbol)
					break;

				symbol->isop=true;
				symbol->lexeme=ch;
				symbol->condition=L"";
				prod->RHS.push_back(symbol);

				ch=fr.MoveNext();

				symbol=new CSymbol();
				
				if(!symbol)
					break;

				symbol->isop=true;
				symbol->lexeme=ch;
				symbol->condition=L"";
				prod->RHS.push_back(symbol);

				ch=fr.MoveNext();
				
				symbol=new CSymbol();
				
				if(!symbol)
					break;

				symbol->isop=true;
				symbol->lexeme=ch;
				symbol->condition=L"";				
				prod->RHS.push_back(symbol);
				
				str=L"";		
				break;

			case '/' :				
				
				while(true)
				{							
					ch=fr.GetCurrentChar();
						
					if(ch==LF||ch==CR||ch<=0)
						break;					

					str+=ch;
					fr.MoveNext();					
				}

				str=L"";

				break;

			}
	
		fr.MoveNext();
	}
	fr.CloseFile(); /*ADDED IN THIS (LINUX) VERSION*/
	return prod_list;
}

void P_Grammar::AddInProductionsMap()
{
	typedef pair<short,P_Production*> ProdEntryinHashMap;
	
	P_Production* temp;

	for(int i=0;i<AllProductions->size();i++)
	{
		temp = AllProductions->at(i);
		if(temp->LHS==128)
		{
			temp=temp;
		}
		ProductionsMap.insert(ProdEntryinHashMap(temp->LHS,temp));
	}
}
void P_Grammar::WriteGrammarInFile(vector<P_Production*> * prod_list,string CFGFileName)
{
	CFileWriter fw;

	fw.OpenFile(CFGFileName);

	

	P_Production* rp=0;

	int debug=0;

	for(int it = 0;it< prod_list->size();it++)
	{
		wstring AProductionToWrite=L"";
		if(it==debug)
		{
			debug=debug;
		}
		AProductionToWrite=L"";
		rp=prod_list->at(it);
		if(rp)
		{
			AProductionToWrite=stringToWstring(GiveprodAtIndex(rp->LHS));

			/********** CHANGE BY OMAR [May Allah Give him the higher place in Haven] FOR WRITING SELECTION SET TO FILE (START)********/
			/*
				Modified By Sarfraz Raza
			*/
			/*************************************************************************************************************************/
			AProductionToWrite+=L" [";

			int i= rp->SelectionSet.FirstSet1;

			/*
				still Now i m dumping all the Selection Set as the 
				short Value of the Selection Set
			*/

			string FS="";

			char A[20];

			itoa(i,A,10);

			AProductionToWrite+= stringToWstring(A);

			AProductionToWrite+=L" ";

			i= rp->SelectionSet.FirstSet2;

			itoa(i,A,10);

			AProductionToWrite+= stringToWstring(A);
			
			AProductionToWrite+=L"]";

			/********** CHANGE BY 'GREAT *** OMAR *** [May Allah place him in Heaven]' FOR WRITING SELECTION SET TO FILE (END)********/

			AProductionToWrite+=L" -> ";
			
			for(int i= 0; i<rp->RHS.size();i++)
			{
				if(rp->RHS[i]->Terminal)
				{
				  	AProductionToWrite+=stringToWstring(GiveposAtIndex(rp->RHS[i]->SymbolId));
				}
				else
				{
					if(GiveprodAtIndex(rp->LHS)=="Simp")
					{
						debug = debug;
					}

					AProductionToWrite+=stringToWstring(GiveprodAtIndex(rp->RHS[i]->SymbolId));
				}
				
				//Now Writing Condition

					AProductionToWrite+=L":";

					short shortVec;
					for(int fdi=0;fdi<rp->RHS[i]->FDescriptionIndices.size();fdi++)
					{
						shortVec = rp->RHS[i]->FDescriptionIndices[fdi];
							char A[10];
							itoa(shortVec,A,10);
							string temp =A;

							AProductionToWrite+=stringToWstring(temp)+L" ";
						if(rp->RHS[i]->FDescriptionIndices.size()-1!=fdi)
						{
							AProductionToWrite+=L"|";
						}
					}
					AProductionToWrite+=L"; ";
			}			
			AProductionToWrite+=L"\r\n";
			fw.Write(AProductionToWrite);
		}		
	}
	fw.CloseFile();
}

string P_Grammar::GiveprodAtIndex(int index)
{
	return (ProdSymbols[index]);
}
string P_Grammar::GiveposAtIndex(int index)
{
	return (Terminals[index]);
}

list<CRawProduction*> * P_Grammar::SortProductions(list<CRawProduction*> * vlist2)
{
	hash_multimap<wstring,CRawProduction*> SortedProdList;
	typedef pair<wstring,CRawProduction*> SortedEntry;

	for(list<CRawProduction*>::iterator ii = vlist2->begin(); ii!=vlist2->end();ii++)
	{
		CRawProduction* curr = *ii;
		SortedProdList.insert(SortedEntry(curr->LHS->lexeme,curr));
	}
	vlist2->clear();
	for(hash_multimap<wstring,CRawProduction*>::iterator si = SortedProdList.begin();si!=SortedProdList.end();si++)
	{
		vlist2->push_back(si->second);
	}
	return vlist2;
}

/****************************************************************/

int indexReturned(CRawProduction* AProd,vector<CRawProduction*> & ProdsVec)
{
	for(int i=0;i<ProdsVec.size();i++)
	{
		if(*(ProdsVec[i])==*(AProd))
		{
			return i;
		}
	}

	throw "Exception Comes with Not Equal CRawProduction";
}

void Push_Unique(vector<CRawProduction*> & My,CRawProduction* A)
{
	for(int i=0;i<My.size();i++)
	{
		if(My[i]==A)
		{
			return ;
		}
	}
	My.push_back(A);
}

void P_Grammar::PurifyProductions(list<CRawProduction*> * vlist2)
{
	/*ADDED IN THIS (LINUX ONLY) VERSION*/	
	string objstr1;
	string objstr2;	
	/*ADDED IN THIS (LINUX ONLY) VERSION*/

	/*
		Loading Terminals
	*/
/*
	string TerPath = CConfiguration::GetConfigInfo("POSs");
	fstream fin(TerPath.c_str(),ios::in);

	string pos="";

	while(!fin.eof())
	{
		fin>>pos;
		int index = Terminals.size();
		Terminals.push_back(pos);
		TerSymbolsMap.insert(PosSymMapEntry(pos,index));
	}
*/
	/*
		Mapping Productions To NEW Data Structures
	*/

	list<P_Production*> * ProdTable= new list<P_Production*> ;

	P_Production* ATempProd=NULL;

	//P_FSInitializer FSinit;


	vector<string> OneCondition;
	vector<vector<string> > AllMergedConditions;

	/*
		// Sort the productions with respect to RHS	
	*/

	vlist2 = SortProductions(vlist2);

	vector<CRawProduction*> MHead;

	wstring lhs=L"";
	wstring lhsNext=L"";
	
	list<CRawProduction*>::iterator i = vlist2->begin();

	CRawProduction* curr;// = *i;

//	lhs = curr->LHS->lexeme;

	/*
		Mergin the productions having Same Left Hand Side 
		& Right Hand Side but different FDescriptions
	*/

	int count = 0;
	int debugCount =262;

	bool Iscontinue; /*ADDED IN THIS (LINUX ONLY) VERSION - check for Windows*/
	vector<CRawProduction* >iVecs;
	bool EOFplist = false;
	for(; i!=vlist2->end() || (!EOFplist);i++)
	{
		
		if(i==vlist2->end())
		{
			EOFplist = true;
			goto Again;
		}

		/*bool Iscontinue = false; CHANGED IN THIS (LINUX ONLY) VERSION - check for Win*/
		Iscontinue = false;
		for(int ivecI =0 ;ivecI<iVecs.size();ivecI++)
		{
			if(*i==iVecs[ivecI])
			{
				Iscontinue = true;
				break;
			}
		}
		if(Iscontinue)
		{
			continue;
		}
		
		OneCondition.clear();
		AllMergedConditions.clear();

		ATempProd = new P_Production;
		curr = *i;

		if(curr->LHS->lexeme==L"S'")
		{
			curr->LHS = curr->LHS;
		}

		lhsNext = curr->LHS->lexeme;

		if(lhsNext == lhs)
		{
			MHead.push_back(curr);
			continue;
		}
		else if(lhs==L"")
		{
			lhs = lhsNext;
			MHead.push_back(curr);
			continue;
		}

		else
		{
			//	lhsNext Already Assigned to the Next 
			//	Change it to lhs
			lhs = lhsNext;
			i--;

Again:
			
		/*
			Merging start here	Merging start here	Merging start here
		*/

			if(MHead.size()>1)
			{
				vector<int> MergeWithPrev;
				vector<int> MergeWithCurr;
				for(int mi =0 ;mi<MHead.size();mi++)
				{
					MergeWithCurr.clear();
					bool IsContinue = false;
					
					for(int mwi=0;mwi<MergeWithPrev.size();mwi++)
					{
						if(mi == MergeWithPrev[mwi])
						{
							IsContinue = true;
							break;
						}
					}
					
					if(IsContinue == true)
					{
						continue;
					}

					for(int mj=mi+1;mj<MHead.size();mj++)
					{
						if(MHead[mi]->RHS||MHead[mj]->RHS)
						{
							MergeWithCurr.push_back(mj);
						}
					}
					
					ATempProd = new P_Production;	
					CRawProduction* curr = MHead[mi];
					
					/* CHANGED IN THIS (LINUX ONLY) VERSION
					ATempProd->LHS = GiveprodIndex(wstringToString(curr->LHS->lexeme));
					CHANGED IN THIS (LINUX ONLY) VERSION */
					objstr1 = wstringToString(curr->LHS->lexeme) ;
					ATempProd->LHS = GiveprodIndex(objstr1);
					
					ATempProd->probability = 0;	// curr->prob;
					
					ATempProd->ProdID = ProdTable->size();

						for(map<short,short>::iterator ssi= curr->SelectionSet.begin();ssi!=curr->SelectionSet.end();ssi++)
						{
							short di= ssi->first;
							string a = wstringToString(CSymbolTableManager::GetString(di));
							ATempProd->SelectionSet |= GiveposIndex(a);//(int)pow(2,GiveposIndex(a));
						}
						P_Symbol* ASYmbol;
						vector<wstring> temp;
						string posORnt = "";

						for(int ri=0;ri<curr->RHS.size()-1;ri++)
						{
							temp.clear();
							ASYmbol = new P_Symbol;

							posORnt = wstringToString(curr->RHS[ri]->lexeme);

							ASYmbol->Terminal =(!IsNonTerminal(curr->RHS[ri]->lexeme));
							
							if(ASYmbol->Terminal)
							{
								ASYmbol->SymbolId = GiveposIndex(posORnt);
							}
							else
							{
								ASYmbol->SymbolId = GiveprodIndex(posORnt);
							}
							string cond = wstringToString(curr->RHS[ri]->condition);
							OneCondition.push_back(cond);
							ATempProd->RHS.push_back(ASYmbol);
						}
						AllMergedConditions.push_back(OneCondition);

						for(int rri=0;rri<MergeWithCurr.size();rri++)
						{
							OneCondition.clear();
							curr = MHead.at(MergeWithCurr[rri]);
							for(int tempi=0;tempi<curr->RHS.size();tempi++)
							{
								string cond = wstringToString(curr->RHS[tempi]->condition);
								OneCondition.push_back(cond);
							}
							AllMergedConditions.push_back(OneCondition);
						}

						OneCondition.clear();

						vector<vector<short> > MyAllConditions = P_FSInitializer::getFDVectors(AllMergedConditions,ATempProd->MAXCONTEXT);  

						AllMergedConditions.clear();
						
						
						// this will be Uncomment


						
						for(int i=0;i<ATempProd->RHS.size();i++)
						{
							ATempProd->RHS[i]->FDescriptionIndices = MyAllConditions[i];
						}
							
						for(int mwi =0; mwi<MergeWithCurr.size();mwi++)
						{
							MergeWithPrev.push_back(MergeWithCurr[mwi]);
						}

						for(int t1=0;t1<MergeWithCurr.size();t1++)
						{	
							// This is only for checking the No of Productions Remaing
						Push_Unique(iVecs,MHead[MergeWithCurr[t1]]);
						//	vlist2->remove(MHead[MergeWith[t1]]);
						}
						ProdTable->push_back(ATempProd);
				}
				
			  }
			  else
			  {
					ATempProd = new P_Production;	
					CRawProduction* curr = MHead[0];
					
					/* CHANGED IN THIS (LINUX ONLY) VERSION - ok in Win
					ATempProd->LHS = GiveprodIndex(wstringToString(curr->LHS->lexeme));
					CHANGED IN THIS (LINUX ONLY) VERSION */
					objstr2 = wstringToString(curr->LHS->lexeme) ;
					ATempProd->LHS = GiveprodIndex(objstr2);
					
					ATempProd->probability = 0;	// curr->prob;
					
					ATempProd->ProdID = ProdTable->size();

						for(map<short,short>::iterator ssi= curr->SelectionSet.begin();ssi!=curr->SelectionSet.end();ssi++)
						{
							short di= ssi->first;
							string a = wstringToString(CSymbolTableManager::GetString(di));
							ATempProd->SelectionSet |= GiveposIndex(a);
						}
						P_Symbol* ASYmbol;
						vector<wstring> temp;
						string posORnt = "";

						for(int ri=0;ri<curr->RHS.size()-1;ri++)	// Ignoring " . "
						{
							temp.clear();
							ASYmbol = new P_Symbol;

							posORnt = wstringToString(curr->RHS[ri]->lexeme);

							ASYmbol->Terminal =(!IsNonTerminal(curr->RHS[ri]->lexeme));
							
							if(ASYmbol->Terminal)
							{
								ASYmbol->SymbolId = GiveposIndex(posORnt);
							}
							else
							{
								ASYmbol->SymbolId = GiveprodIndex(posORnt);
							}
							string cond = wstringToString(curr->RHS[ri]->condition);
							OneCondition.push_back(cond);
							ATempProd->RHS.push_back(ASYmbol);
						}
						AllMergedConditions.push_back(OneCondition);

						OneCondition.clear();

						vector<vector<short> > MyAllConditions = P_FSInitializer::getFDVectors(AllMergedConditions,ATempProd->MAXCONTEXT);  
						
						// this will be Uncomment
						
						for(int i=0;i<ATempProd->RHS.size();i++)
						{
							ATempProd->RHS[i]->FDescriptionIndices = MyAllConditions[i];
						}
							
						ProdTable->push_back(ATempProd);

			  }
			
			lhs = L"";
			}
			MHead.clear();

			if(EOFplist)
			{
				break;
			}
		}
	////////////////////////////////////////////////////////////////
	
	AllProductions = new vector<P_Production*>(ProdTable->size());

	list<P_Production*>::iterator li = ProdTable->begin();

	P_Production* temp;
	int size = AllProductions->size();
	for(int i=0;i<size;i++)
	{
		temp = *li;
		AllProductions->at(i)=temp;
		li++;
	}

	string SD = "S'";
	this->StartSymbolID = this->GiveprodIndex(SD);
}

int P_Grammar::insertProdInProdMap(string ProdName)
{
		int index;
		if(Inserted(ProdSymbols,ProdName,index))
		{
			ProdSymbolsMap.insert(ProdSymMapEntry(ProdName,index));
		}

		return index;
}

void P_Grammar::insertProdInProdMap(list<CRawProduction*> * vlist2)
{
	string prodLHS;
	for(list<CRawProduction*>::iterator i= vlist2->begin();i!=vlist2->end();i++)
	{
		int index;
		prodLHS = wstringToString((*i)->LHS->lexeme);

		if(Inserted(ProdSymbols,prodLHS,index))
		{
			ProdSymbolsMap.insert(ProdSymMapEntry(prodLHS,index));
		}
	}
}

bool P_Grammar::Inserted(vector<string> & A,string & b,int & index)
{
	/*for(int i=0;i<A.size();i++) CHANGED IN THIS (LINUX) VERSION*/
	int i;
	for(i=0;i<A.size();i++)
	{
		if(A[i]==b)
		{
			index =i;
			return false;
		}
	}

	index = i;
	A.push_back(b);
	return true;
}

void P_Grammar::insertPOSInPOSMap(string Path)
{
	fstream reader(Path.c_str(),ios::in);

	string apos = "";
	Terminals.push_back(apos);
	while(!reader.eof())
	{
		apos="";
		reader>>apos;
		short POSindex = Terminals.size();
		Terminals.push_back(apos);
		TerSymbolsMap.insert(PosSymMapEntry(apos,POSindex));
	}
	
/*	string terminal;
	for(list<CRawProduction*>::iterator i= vlist2->begin();i!=vlist2->end();i++)
	{
		for(int ti=0;ti<(*i)->RHS.size();ti++)
		{
			if(!IsNonTerminal((*i)->RHS[ti]->lexeme))
			{
				int index ;
				string pos = wstringToString((*i)->RHS[ti]->lexeme);
				if(pos=="quant")
				{
					pos = pos;
				}
				if(Inserted(Terminals,pos,index))
				{
					TerSymbolsMap.insert(PosSymMapEntry(pos,index));
				}
			}
		}
	}


	string TerPath = CConfiguration::GetConfigInfo("Terminals");
	fstream fout(TerPath.c_str(),ios::out);

	for(int i=0;i<Terminals.size();i++)
	{
		fout<<Terminals[i]<<endl;
	}
*/
}

int P_Grammar::GiveposIndex(string & temp)
{
	hash_map<string,int>::iterator i = TerSymbolsMap.find(temp);	

	if(i==TerSymbolsMap.end())
	{
		short POSindex = Terminals.size();
		Terminals.push_back(temp);
		TerSymbolsMap.insert(PosSymMapEntry(temp,POSindex));
		
		return POSindex;

/*
		MessageBox(NULL,L"ERROR",L"ERROR",MB_ICONINFORMATION);

		throw ("RUN TIME ERROR");
*/
	}

	int ii = i->second;

	return i->second;
}

int P_Grammar::GiveprodIndex(string & temp)
{
	hash_map<string,int>::iterator i = ProdSymbolsMap.find(temp);	

	return i->second;
}


	bool P_Grammar::PreprocessGrammar(const string & grules_fname,
							const string & macros_fname,
							const string & cfg_fname,
							bool ReplaceStarByRange)
	{	
	typedef pair<wstring,wstring> macro_pair;

	map<wstring,wstring> MacroList;

	CFileReader MacrosFReeader(macros_fname.c_str(),1024);

	short ch=0;
	
	wstring str=L"";
	wstring macro_name=L"";
	bool equal_came=false;

	while((ch=MacrosFReeader.GetCurrentChar())!=0)
	{
		if(ch=='=' && !equal_came)
		{
			macro_name=str;
			str=L"";
			equal_came=true;
		}
		else if(ch=='.')
		{
			macro_name=TrimString(macro_name);
			str=TrimString(str);
			MacroList.insert(macro_pair(macro_name,str));
			str=L"";
			equal_came=false;
		}
		else if (ch!=LF && ch!=CR)
			str+=ch;

		MacrosFReeader.MoveNext();
	}
	
	MacrosFReeader.CloseFile();

	/************************************************************/

	CFileReader RulesFReeader(grules_fname.c_str(),512);
	
	CFileWriter GRulesFWriter(cfg_fname.c_str());

	ch=0;
	
	str=L"";	
	int counter=0;
	
			macro_name  =L"";
	wstring macro_result=L"";	
	
	map<wstring,wstring>::iterator temp_it;

	while((ch=RulesFReeader.GetCurrentChar())!=0)
	{
		if(ch=='@')
		{
			macro_name=L"";
			
			while(ch!=' ' && ch!='.' && ch!=0)
			{
				macro_name+=ch;
				ch=RulesFReeader.MoveNext();
			}

			temp_it=MacroList.find(macro_name);			
			
			if(temp_it!=MacroList.end())
			{
				macro_result=temp_it->second;
			}
			else
			{				
				//macro_result=L"*** Macro not found for ";
				//macro_result+=macro_name;
				//macro_result+=L" ***";
				//wprintf(L"\n%s\n",macro_result.c_str());
			}
			
			str+=macro_result;			
			str+=ch;
		}
		else if(ReplaceStarByRange && ch=='*')
		{
			str+=L"#0#1";
		}
		else 		
			str+=ch;
		
		counter++;

		if(counter%256==0)
		{
			GRulesFWriter.Write(str);
			str=L"";
		}		

		RulesFReeader.MoveNext();

	}
	

	GRulesFWriter.Write(str);

	GRulesFWriter.CloseFile();


	return true;
}

bool P_Grammar::GetALine(CFileReader & FHandler,string & A)
{
	A="";
	short ch=0;
	
	while((ch=FHandler.GetCurrentChar())!=0)
	{
start:
		switch(ch)
		{
				case LF:
				case CR:
//				case ' ':
//				case '\t':
					A="";

					break;
				case '@':
					A+=ch;
					FHandler.MoveNext();
					while((ch=FHandler.GetCurrentChar())!='@')
					{
						A+=ch;
						FHandler.MoveNext();
					}
					A+=ch;
					break;
				case '/' :				
						
						while(true)
						{						
							ch=FHandler.GetCurrentChar();
								
							if(ch==LF||ch==CR||ch<=0)
								break;					

							A+=ch;
							FHandler.MoveNext();					

						}
						break;

				case '.':
				{
					FHandler.MoveNext();
					return true;	// Line Ends
				}

				default:		
						A+=ch;
					
			}//end switch
	
		
		FHandler.MoveNext();
	}
	return false;
}

//
//bool P_Grammar::LoadProductions()
//{
//	string Path = CConfiguration::GetConfigInfo("ncfgFileName");
//	CFileReader FReader(1024);
//	string ALine="";
//	
//	while(GetALine(FReader,ALine))
//	{
//		
//	}
//}
