#include "stdafx.h"
#include "P_TempFSItemArray.h" /*CHANGED IN THIS (LINUX) VERSION*/

P_TempFSItemArray::P_TempFSItemArray(void)
{
	// a new item will be stored in the vector that is at 
	// the index number that is the symbolID for it's attribute
	temp_fsitem_array = new vector<P_FSItem*>[FSSYMBOLTABLESIZE + 1];	
}

P_TempFSItemArray::~P_TempFSItemArray(void)
{
	delete [] temp_fsitem_array;
}

bool P_TempFSItemArray::insertFSItem(P_FSItem *fs_item)
{
	temp_fsitem_array[fs_item->attribute].push_back(fs_item);
	return false;
}

bool P_TempFSItemArray::merge(void)
// will be merged such that each item's attribute is it's index in the array;
// items with different operators will be in different columns (separate vector elements)
{
	// all of these will always point to a P_FSItemComplex; their attribute byte will be partially 
	// initialized to indicate the information shown pertaining to the following variables:
	P_FSItemComplex* merged_eq = NULL;		// for all with operator = (^attribute)
	P_FSItemComplex* merged_set = NULL;	// for all wth operator $ (^attribute)	
	P_FSItemComplex* merged_eqc = NULL;	// for all with operator =c (^attribute)
	P_FSItemComplex* merged_noteq = NULL;	// for all with operator != (^attribute)
	P_FSItemComplex* merged_eqc_down = NULL;	// for all with operator =c (!attribute)
	P_FSItemComplex* merged_noteq_down = NULL;	// for all with operator != (!attribute)

	P_FSItem* fs_item_eq = NULL;
	P_FSItem* fs_item_set = NULL;
	P_FSItem* fs_item_eqc = NULL;
	P_FSItem* fs_item_noteq = NULL;
	P_FSItem* fs_item_eqc_down = NULL;
	P_FSItem* fs_item_noteq_down = NULL;
	
	vector<P_FSItem*>::iterator it;
	
	for (int i = 0; i < (FSSYMBOLTABLESIZE+1); i++)
	{
		if (temp_fsitem_array[i].size() > 1)
		{
			merged_eq = new P_FSItemComplex();
			merged_set = new P_FSItemComplex();
			merged_eqc = new P_FSItemComplex();
			merged_noteq = new P_FSItemComplex();
			merged_eqc_down = new P_FSItemComplex();
			merged_noteq_down = new P_FSItemComplex();

			// match the attribute_type to the particular attribute that is being catered to
			// in this iteration, e.g., an attribute like "SUBJ NUM" needs to have it's second
			// bit on; therefore equate to attribute_type of first item in the vector
			// (all items in the vector have the same attribute)
			it = temp_fsitem_array[i].begin();	// to get first element of vector

			merged_eq->attribute_type = OPEQUAL;
			merged_eq->attribute_type = merged_eq->attribute_type | (*it)->getAttributeType();
			merged_eq->attribute = i;

			merged_set->attribute_type = OPSET;
			merged_set->attribute_type = merged_set->attribute_type | (*it)->getAttributeType();
			merged_set->attribute = i;

			merged_eqc->attribute_type = OPEQUALC;
			merged_eqc->attribute_type = merged_eqc->attribute_type | (*it)->getAttributeType();
			merged_eqc->attribute = i;

			merged_noteq->attribute_type = OPNOT;
			merged_noteq->attribute_type = merged_noteq->attribute_type | (*it)->getAttributeType();
			merged_noteq->attribute = i;

			merged_eqc_down->attribute_type = OPEQUALC | DOWNARROWATTR;
			merged_eqc_down->attribute_type = merged_eqc_down->attribute_type | (*it)->getAttributeType();
			merged_eqc_down->attribute = i;

			merged_noteq_down->attribute_type = OPNOT | DOWNARROWATTR;
			merged_noteq_down->attribute_type = merged_noteq_down->attribute_type | (*it)->getAttributeType();
			merged_noteq_down->attribute = i;
			
			for (it = temp_fsitem_array[i].begin(); it != temp_fsitem_array[i].end(); it++)
			{
				// attempt to merge all the items but only those that can be validly merged 
				// into that item will actually be  merged
				fs_item_eq = merged_eq;
				(*it)->eqMergeInto(fs_item_eq);	// ensure that correct type of item is sent as the argument

				fs_item_set = merged_set;
				(*it)->setMergeInto(fs_item_set);	// ensure that correct type of item is sent as the argument

				fs_item_eqc = merged_eqc;
				(*it)->eqcMergeInto(fs_item_eqc);	// ensure that correct type of item is sent as the argument

				fs_item_noteq = merged_noteq;
				(*it)->noteqMergeInto(fs_item_noteq);	// ensure that correct type of item is sent as the argument

				fs_item_eqc_down = merged_eqc_down;
				(*it)->eqcDownMergeInto(fs_item_eqc_down);	// ensure that correct type of item is sent as the argument

				fs_item_noteq_down = merged_noteq_down;
				(*it)->noteqDownMergeInto(fs_item_noteq_down);	// ensure that correct type of item is sent as the argument
			}	

			// clear the vector and fill in new merged items
			temp_fsitem_array[i].clear();
				
			if ((fs_item_eq->numberOfValues()) > 0)
				temp_fsitem_array[i].push_back(fs_item_eq);
			if ((fs_item_set->numberOfValues()) > 0)
				temp_fsitem_array[i].push_back(fs_item_set);
			if ((fs_item_eqc->numberOfValues()) > 0)
				temp_fsitem_array[i].push_back(fs_item_eqc);
			if ((fs_item_noteq->numberOfValues()) > 0)
				temp_fsitem_array[i].push_back(fs_item_noteq);
			if ((fs_item_eqc_down->numberOfValues()) > 0)
				temp_fsitem_array[i].push_back(fs_item_eqc_down);
			if ((merged_noteq_down->numberOfValues()) > 0)
				temp_fsitem_array[i].push_back(merged_noteq_down);	
		}
	}
	return false;
}

void P_TempFSItemArray::clear(void)
{
	for (int i = 0; i < FSSYMBOLTABLESIZE+1; i++)
		temp_fsitem_array[i].clear();
}
